/*
Copyright 2013-present Barefoot Networks, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#ifndef LIB_NULLSTREAM_H_
#define LIB_NULLSTREAM_H_

#include <filesystem>
#include <iostream>
#include <ostream>
#include <streambuf>

namespace P4 {

template <class cT, class traits = std::char_traits<cT>>
class basic_nullbuf final : public std::basic_streambuf<cT, traits> {
    typename traits::int_type overflow(typename traits::int_type c) {
        return traits::not_eof(c);  // indicate success
    }
};

template <class cT, class traits = std::char_traits<cT>>
class onullstream final : public std::basic_ostream<cT, traits> {
 public:
    onullstream() : std::basic_ios<cT, traits>(&m_sbuf), std::basic_ostream<cT, traits>(&m_sbuf) {
        this->init(&m_sbuf);
    }

 private:
    basic_nullbuf<cT, traits> m_sbuf;
};

typedef onullstream<char> nullstream;

// If nullOnError is 'true', on error a nullstream is returned
// otherwise a nullptr is returned
// FIXME: This should return unique_ptr instead to track lifetime
std::ostream *openFile(const std::filesystem::path &name, bool nullOnError);

}  // namespace P4

#endif /* LIB_NULLSTREAM_H_ */
