#include "ir/ir-generated.h"    // IWYU pragma: keep

#include "ir/ir-inline.h"       // IWYU pragma: keep
#include "ir/json_generator.h"  // IWYU pragma: keep
#include "ir/json_loader.h"     // IWYU pragma: keep
#include "ir/visitor.h"         // IWYU pragma: keep
#include "lib/algorithm.h"      // IWYU pragma: keep
#include "lib/log.h"            // IWYU pragma: keep

using namespace P4;

std::map<cstring, NodeFactoryFn> IR::unpacker_table = {
{"Type_Unknown"_cs, NodeFactoryFn(&IR::Type_Unknown::fromJSON)},
{"Path"_cs, NodeFactoryFn(&IR::Path::fromJSON)},
{"NamedExpression"_cs, NodeFactoryFn(&IR::NamedExpression::fromJSON)},
{"AnnotationToken"_cs, NodeFactoryFn(&IR::AnnotationToken::fromJSON)},
{"Annotation"_cs, NodeFactoryFn(&IR::Annotation::fromJSON)},
{"Argument"_cs, NodeFactoryFn(&IR::Argument::fromJSON)},
{"Type_Any"_cs, NodeFactoryFn(&IR::Type_Any::fromJSON)},
{"Type_Fragment"_cs, NodeFactoryFn(&IR::Type_Fragment::fromJSON)},
{"Type_Type"_cs, NodeFactoryFn(&IR::Type_Type::fromJSON)},
{"Type_Boolean"_cs, NodeFactoryFn(&IR::Type_Boolean::fromJSON)},
{"Type_State"_cs, NodeFactoryFn(&IR::Type_State::fromJSON)},
{"Type_Bits"_cs, NodeFactoryFn(&IR::Type_Bits::fromJSON)},
{"Type_Varbits"_cs, NodeFactoryFn(&IR::Type_Varbits::fromJSON)},
{"Parameter"_cs, NodeFactoryFn(&IR::Parameter::fromJSON)},
{"ParameterList"_cs, NodeFactoryFn(&IR::ParameterList::fromJSON)},
{"Type_Var"_cs, NodeFactoryFn(&IR::Type_Var::fromJSON)},
{"Type_InfInt"_cs, NodeFactoryFn(&IR::Type_InfInt::fromJSON)},
{"Type_Dontcare"_cs, NodeFactoryFn(&IR::Type_Dontcare::fromJSON)},
{"Type_Void"_cs, NodeFactoryFn(&IR::Type_Void::fromJSON)},
{"Type_MatchKind"_cs, NodeFactoryFn(&IR::Type_MatchKind::fromJSON)},
{"TypeParameters"_cs, NodeFactoryFn(&IR::TypeParameters::fromJSON)},
{"StructField"_cs, NodeFactoryFn(&IR::StructField::fromJSON)},
{"Type_Struct"_cs, NodeFactoryFn(&IR::Type_Struct::fromJSON)},
{"Type_UnknownStruct"_cs, NodeFactoryFn(&IR::Type_UnknownStruct::fromJSON)},
{"Type_HeaderUnion"_cs, NodeFactoryFn(&IR::Type_HeaderUnion::fromJSON)},
{"Type_Header"_cs, NodeFactoryFn(&IR::Type_Header::fromJSON)},
{"Type_Set"_cs, NodeFactoryFn(&IR::Type_Set::fromJSON)},
{"Type_List"_cs, NodeFactoryFn(&IR::Type_List::fromJSON)},
{"Type_Tuple"_cs, NodeFactoryFn(&IR::Type_Tuple::fromJSON)},
{"Type_P4List"_cs, NodeFactoryFn(&IR::Type_P4List::fromJSON)},
{"Type_Package"_cs, NodeFactoryFn(&IR::Type_Package::fromJSON)},
{"Type_Parser"_cs, NodeFactoryFn(&IR::Type_Parser::fromJSON)},
{"Type_Control"_cs, NodeFactoryFn(&IR::Type_Control::fromJSON)},
{"Type_Name"_cs, NodeFactoryFn(&IR::Type_Name::fromJSON)},
{"Type_Array"_cs, NodeFactoryFn(&IR::Type_Array::fromJSON)},
{"Type_Specialized"_cs, NodeFactoryFn(&IR::Type_Specialized::fromJSON)},
{"Type_SpecializedCanonical"_cs, NodeFactoryFn(&IR::Type_SpecializedCanonical::fromJSON)},
{"Declaration_ID"_cs, NodeFactoryFn(&IR::Declaration_ID::fromJSON)},
{"Type_String"_cs, NodeFactoryFn(&IR::Type_String::fromJSON)},
{"Type_Enum"_cs, NodeFactoryFn(&IR::Type_Enum::fromJSON)},
{"SerEnumMember"_cs, NodeFactoryFn(&IR::SerEnumMember::fromJSON)},
{"Type_SerEnum"_cs, NodeFactoryFn(&IR::Type_SerEnum::fromJSON)},
{"Type_Table"_cs, NodeFactoryFn(&IR::Type_Table::fromJSON)},
{"Type_ActionEnum"_cs, NodeFactoryFn(&IR::Type_ActionEnum::fromJSON)},
{"Type_Method"_cs, NodeFactoryFn(&IR::Type_Method::fromJSON)},
{"ArgumentInfo"_cs, NodeFactoryFn(&IR::ArgumentInfo::fromJSON)},
{"Type_MethodCall"_cs, NodeFactoryFn(&IR::Type_MethodCall::fromJSON)},
{"Type_Action"_cs, NodeFactoryFn(&IR::Type_Action::fromJSON)},
{"Method"_cs, NodeFactoryFn(&IR::Method::fromJSON)},
{"Type_Typedef"_cs, NodeFactoryFn(&IR::Type_Typedef::fromJSON)},
{"Type_Newtype"_cs, NodeFactoryFn(&IR::Type_Newtype::fromJSON)},
{"Type_Extern"_cs, NodeFactoryFn(&IR::Type_Extern::fromJSON)},
{"Neg"_cs, NodeFactoryFn(&IR::Neg::fromJSON)},
{"UPlus"_cs, NodeFactoryFn(&IR::UPlus::fromJSON)},
{"Cmpl"_cs, NodeFactoryFn(&IR::Cmpl::fromJSON)},
{"LNot"_cs, NodeFactoryFn(&IR::LNot::fromJSON)},
{"Mul"_cs, NodeFactoryFn(&IR::Mul::fromJSON)},
{"MulAssign"_cs, NodeFactoryFn(&IR::MulAssign::fromJSON)},
{"Div"_cs, NodeFactoryFn(&IR::Div::fromJSON)},
{"DivAssign"_cs, NodeFactoryFn(&IR::DivAssign::fromJSON)},
{"Mod"_cs, NodeFactoryFn(&IR::Mod::fromJSON)},
{"ModAssign"_cs, NodeFactoryFn(&IR::ModAssign::fromJSON)},
{"Add"_cs, NodeFactoryFn(&IR::Add::fromJSON)},
{"AddAssign"_cs, NodeFactoryFn(&IR::AddAssign::fromJSON)},
{"Sub"_cs, NodeFactoryFn(&IR::Sub::fromJSON)},
{"SubAssign"_cs, NodeFactoryFn(&IR::SubAssign::fromJSON)},
{"AddSat"_cs, NodeFactoryFn(&IR::AddSat::fromJSON)},
{"AddSatAssign"_cs, NodeFactoryFn(&IR::AddSatAssign::fromJSON)},
{"SubSat"_cs, NodeFactoryFn(&IR::SubSat::fromJSON)},
{"SubSatAssign"_cs, NodeFactoryFn(&IR::SubSatAssign::fromJSON)},
{"Shl"_cs, NodeFactoryFn(&IR::Shl::fromJSON)},
{"ShlAssign"_cs, NodeFactoryFn(&IR::ShlAssign::fromJSON)},
{"Shr"_cs, NodeFactoryFn(&IR::Shr::fromJSON)},
{"ShrAssign"_cs, NodeFactoryFn(&IR::ShrAssign::fromJSON)},
{"Equ"_cs, NodeFactoryFn(&IR::Equ::fromJSON)},
{"Neq"_cs, NodeFactoryFn(&IR::Neq::fromJSON)},
{"Lss"_cs, NodeFactoryFn(&IR::Lss::fromJSON)},
{"Leq"_cs, NodeFactoryFn(&IR::Leq::fromJSON)},
{"Grt"_cs, NodeFactoryFn(&IR::Grt::fromJSON)},
{"Geq"_cs, NodeFactoryFn(&IR::Geq::fromJSON)},
{"BAnd"_cs, NodeFactoryFn(&IR::BAnd::fromJSON)},
{"BAndAssign"_cs, NodeFactoryFn(&IR::BAndAssign::fromJSON)},
{"BOr"_cs, NodeFactoryFn(&IR::BOr::fromJSON)},
{"BOrAssign"_cs, NodeFactoryFn(&IR::BOrAssign::fromJSON)},
{"BXor"_cs, NodeFactoryFn(&IR::BXor::fromJSON)},
{"BXorAssign"_cs, NodeFactoryFn(&IR::BXorAssign::fromJSON)},
{"LAnd"_cs, NodeFactoryFn(&IR::LAnd::fromJSON)},
{"LOr"_cs, NodeFactoryFn(&IR::LOr::fromJSON)},
{"Dots"_cs, NodeFactoryFn(&IR::Dots::fromJSON)},
{"NamedDots"_cs, NodeFactoryFn(&IR::NamedDots::fromJSON)},
{"Constant"_cs, NodeFactoryFn(&IR::Constant::fromJSON)},
{"BoolLiteral"_cs, NodeFactoryFn(&IR::BoolLiteral::fromJSON)},
{"StringLiteral"_cs, NodeFactoryFn(&IR::StringLiteral::fromJSON)},
{"PathExpression"_cs, NodeFactoryFn(&IR::PathExpression::fromJSON)},
{"TypeNameExpression"_cs, NodeFactoryFn(&IR::TypeNameExpression::fromJSON)},
{"Slice"_cs, NodeFactoryFn(&IR::Slice::fromJSON)},
{"PlusSlice"_cs, NodeFactoryFn(&IR::PlusSlice::fromJSON)},
{"Member"_cs, NodeFactoryFn(&IR::Member::fromJSON)},
{"Concat"_cs, NodeFactoryFn(&IR::Concat::fromJSON)},
{"ArrayIndex"_cs, NodeFactoryFn(&IR::ArrayIndex::fromJSON)},
{"Range"_cs, NodeFactoryFn(&IR::Range::fromJSON)},
{"Mask"_cs, NodeFactoryFn(&IR::Mask::fromJSON)},
{"Mux"_cs, NodeFactoryFn(&IR::Mux::fromJSON)},
{"DefaultExpression"_cs, NodeFactoryFn(&IR::DefaultExpression::fromJSON)},
{"This"_cs, NodeFactoryFn(&IR::This::fromJSON)},
{"Cast"_cs, NodeFactoryFn(&IR::Cast::fromJSON)},
{"SelectCase"_cs, NodeFactoryFn(&IR::SelectCase::fromJSON)},
{"SelectExpression"_cs, NodeFactoryFn(&IR::SelectExpression::fromJSON)},
{"MethodCallExpression"_cs, NodeFactoryFn(&IR::MethodCallExpression::fromJSON)},
{"ConstructorCallExpression"_cs, NodeFactoryFn(&IR::ConstructorCallExpression::fromJSON)},
{"BaseListExpression"_cs, NodeFactoryFn(&IR::BaseListExpression::fromJSON)},
{"ListExpression"_cs, NodeFactoryFn(&IR::ListExpression::fromJSON)},
{"P4ListExpression"_cs, NodeFactoryFn(&IR::P4ListExpression::fromJSON)},
{"StructExpression"_cs, NodeFactoryFn(&IR::StructExpression::fromJSON)},
{"Invalid"_cs, NodeFactoryFn(&IR::Invalid::fromJSON)},
{"InvalidHeader"_cs, NodeFactoryFn(&IR::InvalidHeader::fromJSON)},
{"InvalidHeaderUnion"_cs, NodeFactoryFn(&IR::InvalidHeaderUnion::fromJSON)},
{"HeaderStackExpression"_cs, NodeFactoryFn(&IR::HeaderStackExpression::fromJSON)},
{"ListCompileTimeValue"_cs, NodeFactoryFn(&IR::ListCompileTimeValue::fromJSON)},
{"P4ListCompileTimeValue"_cs, NodeFactoryFn(&IR::P4ListCompileTimeValue::fromJSON)},
{"StructCompileTimeValue"_cs, NodeFactoryFn(&IR::StructCompileTimeValue::fromJSON)},
{"CompileTimeMethodCall"_cs, NodeFactoryFn(&IR::CompileTimeMethodCall::fromJSON)},
{"SymbolicVariable"_cs, NodeFactoryFn(&IR::SymbolicVariable::fromJSON)},
{"ParserState"_cs, NodeFactoryFn(&IR::ParserState::fromJSON)},
{"P4Parser"_cs, NodeFactoryFn(&IR::P4Parser::fromJSON)},
{"P4Control"_cs, NodeFactoryFn(&IR::P4Control::fromJSON)},
{"P4Action"_cs, NodeFactoryFn(&IR::P4Action::fromJSON)},
{"Type_Error"_cs, NodeFactoryFn(&IR::Type_Error::fromJSON)},
{"Declaration_MatchKind"_cs, NodeFactoryFn(&IR::Declaration_MatchKind::fromJSON)},
{"ExpressionValue"_cs, NodeFactoryFn(&IR::ExpressionValue::fromJSON)},
{"ExpressionListValue"_cs, NodeFactoryFn(&IR::ExpressionListValue::fromJSON)},
{"ActionListElement"_cs, NodeFactoryFn(&IR::ActionListElement::fromJSON)},
{"ActionList"_cs, NodeFactoryFn(&IR::ActionList::fromJSON)},
{"KeyElement"_cs, NodeFactoryFn(&IR::KeyElement::fromJSON)},
{"Key"_cs, NodeFactoryFn(&IR::Key::fromJSON)},
{"Entry"_cs, NodeFactoryFn(&IR::Entry::fromJSON)},
{"EntriesList"_cs, NodeFactoryFn(&IR::EntriesList::fromJSON)},
{"Property"_cs, NodeFactoryFn(&IR::Property::fromJSON)},
{"TableProperties"_cs, NodeFactoryFn(&IR::TableProperties::fromJSON)},
{"P4Table"_cs, NodeFactoryFn(&IR::P4Table::fromJSON)},
{"P4ValueSet"_cs, NodeFactoryFn(&IR::P4ValueSet::fromJSON)},
{"Declaration_Variable"_cs, NodeFactoryFn(&IR::Declaration_Variable::fromJSON)},
{"Declaration_Constant"_cs, NodeFactoryFn(&IR::Declaration_Constant::fromJSON)},
{"Declaration_Instance"_cs, NodeFactoryFn(&IR::Declaration_Instance::fromJSON)},
{"P4Program"_cs, NodeFactoryFn(&IR::P4Program::fromJSON)},
{"ExitStatement"_cs, NodeFactoryFn(&IR::ExitStatement::fromJSON)},
{"ReturnStatement"_cs, NodeFactoryFn(&IR::ReturnStatement::fromJSON)},
{"EmptyStatement"_cs, NodeFactoryFn(&IR::EmptyStatement::fromJSON)},
{"AssignmentStatement"_cs, NodeFactoryFn(&IR::AssignmentStatement::fromJSON)},
{"IfStatement"_cs, NodeFactoryFn(&IR::IfStatement::fromJSON)},
{"BreakStatement"_cs, NodeFactoryFn(&IR::BreakStatement::fromJSON)},
{"ContinueStatement"_cs, NodeFactoryFn(&IR::ContinueStatement::fromJSON)},
{"BlockStatement"_cs, NodeFactoryFn(&IR::BlockStatement::fromJSON)},
{"MethodCallStatement"_cs, NodeFactoryFn(&IR::MethodCallStatement::fromJSON)},
{"SwitchCase"_cs, NodeFactoryFn(&IR::SwitchCase::fromJSON)},
{"SwitchStatement"_cs, NodeFactoryFn(&IR::SwitchStatement::fromJSON)},
{"ForStatement"_cs, NodeFactoryFn(&IR::ForStatement::fromJSON)},
{"ForInStatement"_cs, NodeFactoryFn(&IR::ForInStatement::fromJSON)},
{"Function"_cs, NodeFactoryFn(&IR::Function::fromJSON)},
{"TableBlock"_cs, NodeFactoryFn(&IR::TableBlock::fromJSON)},
{"ParserBlock"_cs, NodeFactoryFn(&IR::ParserBlock::fromJSON)},
{"ControlBlock"_cs, NodeFactoryFn(&IR::ControlBlock::fromJSON)},
{"PackageBlock"_cs, NodeFactoryFn(&IR::PackageBlock::fromJSON)},
{"ExternBlock"_cs, NodeFactoryFn(&IR::ExternBlock::fromJSON)},
{"ToplevelBlock"_cs, NodeFactoryFn(&IR::ToplevelBlock::fromJSON)},
{"Type_Block"_cs, NodeFactoryFn(&IR::Type_Block::fromJSON)},
{"Type_Counter"_cs, NodeFactoryFn(&IR::Type_Counter::fromJSON)},
{"Type_Expression"_cs, NodeFactoryFn(&IR::Type_Expression::fromJSON)},
{"Type_FieldListCalculation"_cs, NodeFactoryFn(&IR::Type_FieldListCalculation::fromJSON)},
{"Type_Meter"_cs, NodeFactoryFn(&IR::Type_Meter::fromJSON)},
{"Type_Register"_cs, NodeFactoryFn(&IR::Type_Register::fromJSON)},
{"Type_AnyTable"_cs, NodeFactoryFn(&IR::Type_AnyTable::fromJSON)},
{"Header"_cs, NodeFactoryFn(&IR::Header::fromJSON)},
{"HeaderStack"_cs, NodeFactoryFn(&IR::HeaderStack::fromJSON)},
{"v1HeaderType"_cs, NodeFactoryFn(&IR::v1HeaderType::fromJSON)},
{"Metadata"_cs, NodeFactoryFn(&IR::Metadata::fromJSON)},
{"ConcreteHeaderRef"_cs, NodeFactoryFn(&IR::ConcreteHeaderRef::fromJSON)},
{"HeaderStackItemRef"_cs, NodeFactoryFn(&IR::HeaderStackItemRef::fromJSON)},
{"If"_cs, NodeFactoryFn(&IR::If::fromJSON)},
{"NamedCond"_cs, NodeFactoryFn(&IR::NamedCond::fromJSON)},
{"Apply"_cs, NodeFactoryFn(&IR::Apply::fromJSON)},
{"Primitive"_cs, NodeFactoryFn(&IR::Primitive::fromJSON)},
{"FieldList"_cs, NodeFactoryFn(&IR::FieldList::fromJSON)},
{"FieldListCalculation"_cs, NodeFactoryFn(&IR::FieldListCalculation::fromJSON)},
{"CalculatedField"_cs, NodeFactoryFn(&IR::CalculatedField::fromJSON)},
{"ParserValueSet"_cs, NodeFactoryFn(&IR::ParserValueSet::fromJSON)},
{"CaseEntry"_cs, NodeFactoryFn(&IR::CaseEntry::fromJSON)},
{"V1Parser"_cs, NodeFactoryFn(&IR::V1Parser::fromJSON)},
{"ParserException"_cs, NodeFactoryFn(&IR::ParserException::fromJSON)},
{"Counter"_cs, NodeFactoryFn(&IR::Counter::fromJSON)},
{"Meter"_cs, NodeFactoryFn(&IR::Meter::fromJSON)},
{"Register"_cs, NodeFactoryFn(&IR::Register::fromJSON)},
{"PrimitiveAction"_cs, NodeFactoryFn(&IR::PrimitiveAction::fromJSON)},
{"NameList"_cs, NodeFactoryFn(&IR::NameList::fromJSON)},
{"ActionArg"_cs, NodeFactoryFn(&IR::ActionArg::fromJSON)},
{"ActionFunction"_cs, NodeFactoryFn(&IR::ActionFunction::fromJSON)},
{"ActionProfile"_cs, NodeFactoryFn(&IR::ActionProfile::fromJSON)},
{"ActionSelector"_cs, NodeFactoryFn(&IR::ActionSelector::fromJSON)},
{"V1Table"_cs, NodeFactoryFn(&IR::V1Table::fromJSON)},
{"V1Control"_cs, NodeFactoryFn(&IR::V1Control::fromJSON)},
{"AttribLocal"_cs, NodeFactoryFn(&IR::AttribLocal::fromJSON)},
{"AttribLocals"_cs, NodeFactoryFn(&IR::AttribLocals::fromJSON)},
{"Attribute"_cs, NodeFactoryFn(&IR::Attribute::fromJSON)},
{"GlobalRef"_cs, NodeFactoryFn(&IR::GlobalRef::fromJSON)},
{"AttributeRef"_cs, NodeFactoryFn(&IR::AttributeRef::fromJSON)},
{"V1Program"_cs, NodeFactoryFn(&IR::V1Program::fromJSON)},
{"IntMod"_cs, NodeFactoryFn(&IR::IntMod::fromJSON)},
{"DpdkDeclaration"_cs, NodeFactoryFn(&IR::DpdkDeclaration::fromJSON)},
{"DpdkExternDeclaration"_cs, NodeFactoryFn(&IR::DpdkExternDeclaration::fromJSON)},
{"DpdkHeaderType"_cs, NodeFactoryFn(&IR::DpdkHeaderType::fromJSON)},
{"DpdkHeaderInstance"_cs, NodeFactoryFn(&IR::DpdkHeaderInstance::fromJSON)},
{"DpdkStructType"_cs, NodeFactoryFn(&IR::DpdkStructType::fromJSON)},
{"DpdkAction"_cs, NodeFactoryFn(&IR::DpdkAction::fromJSON)},
{"DpdkTable"_cs, NodeFactoryFn(&IR::DpdkTable::fromJSON)},
{"DpdkSelector"_cs, NodeFactoryFn(&IR::DpdkSelector::fromJSON)},
{"DpdkLearner"_cs, NodeFactoryFn(&IR::DpdkLearner::fromJSON)},
{"DpdkAsmProgram"_cs, NodeFactoryFn(&IR::DpdkAsmProgram::fromJSON)},
{"DpdkListStatement"_cs, NodeFactoryFn(&IR::DpdkListStatement::fromJSON)},
{"DpdkApplyStatement"_cs, NodeFactoryFn(&IR::DpdkApplyStatement::fromJSON)},
{"DpdkLearnStatement"_cs, NodeFactoryFn(&IR::DpdkLearnStatement::fromJSON)},
{"DpdkMirrorStatement"_cs, NodeFactoryFn(&IR::DpdkMirrorStatement::fromJSON)},
{"DpdkEmitStatement"_cs, NodeFactoryFn(&IR::DpdkEmitStatement::fromJSON)},
{"DpdkExtractStatement"_cs, NodeFactoryFn(&IR::DpdkExtractStatement::fromJSON)},
{"DpdkLookaheadStatement"_cs, NodeFactoryFn(&IR::DpdkLookaheadStatement::fromJSON)},
{"DpdkJmpLabelStatement"_cs, NodeFactoryFn(&IR::DpdkJmpLabelStatement::fromJSON)},
{"DpdkJmpHitStatement"_cs, NodeFactoryFn(&IR::DpdkJmpHitStatement::fromJSON)},
{"DpdkJmpMissStatement"_cs, NodeFactoryFn(&IR::DpdkJmpMissStatement::fromJSON)},
{"DpdkJmpIfActionRunStatement"_cs, NodeFactoryFn(&IR::DpdkJmpIfActionRunStatement::fromJSON)},
{"DpdkJmpIfActionNotRunStatement"_cs, NodeFactoryFn(&IR::DpdkJmpIfActionNotRunStatement::fromJSON)},
{"DpdkJmpIfInvalidStatement"_cs, NodeFactoryFn(&IR::DpdkJmpIfInvalidStatement::fromJSON)},
{"DpdkJmpIfValidStatement"_cs, NodeFactoryFn(&IR::DpdkJmpIfValidStatement::fromJSON)},
{"DpdkJmpEqualStatement"_cs, NodeFactoryFn(&IR::DpdkJmpEqualStatement::fromJSON)},
{"DpdkJmpNotEqualStatement"_cs, NodeFactoryFn(&IR::DpdkJmpNotEqualStatement::fromJSON)},
{"DpdkJmpGreaterEqualStatement"_cs, NodeFactoryFn(&IR::DpdkJmpGreaterEqualStatement::fromJSON)},
{"DpdkJmpGreaterStatement"_cs, NodeFactoryFn(&IR::DpdkJmpGreaterStatement::fromJSON)},
{"DpdkJmpLessOrEqualStatement"_cs, NodeFactoryFn(&IR::DpdkJmpLessOrEqualStatement::fromJSON)},
{"DpdkJmpLessStatement"_cs, NodeFactoryFn(&IR::DpdkJmpLessStatement::fromJSON)},
{"DpdkRxStatement"_cs, NodeFactoryFn(&IR::DpdkRxStatement::fromJSON)},
{"DpdkTxStatement"_cs, NodeFactoryFn(&IR::DpdkTxStatement::fromJSON)},
{"DpdkMovStatement"_cs, NodeFactoryFn(&IR::DpdkMovStatement::fromJSON)},
{"DpdkMovhStatement"_cs, NodeFactoryFn(&IR::DpdkMovhStatement::fromJSON)},
{"DpdkAddStatement"_cs, NodeFactoryFn(&IR::DpdkAddStatement::fromJSON)},
{"DpdkAndStatement"_cs, NodeFactoryFn(&IR::DpdkAndStatement::fromJSON)},
{"DpdkShlStatement"_cs, NodeFactoryFn(&IR::DpdkShlStatement::fromJSON)},
{"DpdkShrStatement"_cs, NodeFactoryFn(&IR::DpdkShrStatement::fromJSON)},
{"DpdkSubStatement"_cs, NodeFactoryFn(&IR::DpdkSubStatement::fromJSON)},
{"DpdkOrStatement"_cs, NodeFactoryFn(&IR::DpdkOrStatement::fromJSON)},
{"DpdkXorStatement"_cs, NodeFactoryFn(&IR::DpdkXorStatement::fromJSON)},
{"DpdkRecircidStatement"_cs, NodeFactoryFn(&IR::DpdkRecircidStatement::fromJSON)},
{"DpdkReturnStatement"_cs, NodeFactoryFn(&IR::DpdkReturnStatement::fromJSON)},
{"DpdkRearmStatement"_cs, NodeFactoryFn(&IR::DpdkRearmStatement::fromJSON)},
{"DpdkRecirculateStatement"_cs, NodeFactoryFn(&IR::DpdkRecirculateStatement::fromJSON)},
{"DpdkLabelStatement"_cs, NodeFactoryFn(&IR::DpdkLabelStatement::fromJSON)},
{"DpdkChecksumAddStatement"_cs, NodeFactoryFn(&IR::DpdkChecksumAddStatement::fromJSON)},
{"DpdkChecksumSubStatement"_cs, NodeFactoryFn(&IR::DpdkChecksumSubStatement::fromJSON)},
{"DpdkChecksumClearStatement"_cs, NodeFactoryFn(&IR::DpdkChecksumClearStatement::fromJSON)},
{"DpdkHashDeclStatement"_cs, NodeFactoryFn(&IR::DpdkHashDeclStatement::fromJSON)},
{"DpdkGetHashStatement"_cs, NodeFactoryFn(&IR::DpdkGetHashStatement::fromJSON)},
{"DpdkGetChecksumStatement"_cs, NodeFactoryFn(&IR::DpdkGetChecksumStatement::fromJSON)},
{"DpdkCastStatement"_cs, NodeFactoryFn(&IR::DpdkCastStatement::fromJSON)},
{"DpdkVerifyStatement"_cs, NodeFactoryFn(&IR::DpdkVerifyStatement::fromJSON)},
{"DpdkMeterDeclStatement"_cs, NodeFactoryFn(&IR::DpdkMeterDeclStatement::fromJSON)},
{"DpdkMeterExecuteStatement"_cs, NodeFactoryFn(&IR::DpdkMeterExecuteStatement::fromJSON)},
{"DpdkGetTableEntryIndex"_cs, NodeFactoryFn(&IR::DpdkGetTableEntryIndex::fromJSON)},
{"DpdkCounterCountStatement"_cs, NodeFactoryFn(&IR::DpdkCounterCountStatement::fromJSON)},
{"DpdkRegisterDeclStatement"_cs, NodeFactoryFn(&IR::DpdkRegisterDeclStatement::fromJSON)},
{"DpdkRegisterReadStatement"_cs, NodeFactoryFn(&IR::DpdkRegisterReadStatement::fromJSON)},
{"DpdkRegisterWriteStatement"_cs, NodeFactoryFn(&IR::DpdkRegisterWriteStatement::fromJSON)},
{"DpdkValidateStatement"_cs, NodeFactoryFn(&IR::DpdkValidateStatement::fromJSON)},
{"DpdkInvalidateStatement"_cs, NodeFactoryFn(&IR::DpdkInvalidateStatement::fromJSON)},
{"DpdkDropStatement"_cs, NodeFactoryFn(&IR::DpdkDropStatement::fromJSON)},
{"TCKernelMetadata"_cs, NodeFactoryFn(&IR::TCKernelMetadata::fromJSON)},
{"TCActionParam"_cs, NodeFactoryFn(&IR::TCActionParam::fromJSON)},
{"TCDefaultActionParam"_cs, NodeFactoryFn(&IR::TCDefaultActionParam::fromJSON)},
{"TCAction"_cs, NodeFactoryFn(&IR::TCAction::fromJSON)},
{"TCEntry"_cs, NodeFactoryFn(&IR::TCEntry::fromJSON)},
{"TCTable"_cs, NodeFactoryFn(&IR::TCTable::fromJSON)},
{"TCKey"_cs, NodeFactoryFn(&IR::TCKey::fromJSON)},
{"TCExternInstance"_cs, NodeFactoryFn(&IR::TCExternInstance::fromJSON)},
{"TCExtern"_cs, NodeFactoryFn(&IR::TCExtern::fromJSON)},
{"TCPipeline"_cs, NodeFactoryFn(&IR::TCPipeline::fromJSON)},
{"StateVariable"_cs, NodeFactoryFn(&IR::StateVariable::fromJSON)},
{"TaintExpression"_cs, NodeFactoryFn(&IR::TaintExpression::fromJSON)},
{"Extracted_Varbits"_cs, NodeFactoryFn(&IR::Extracted_Varbits::fromJSON)},
{"ConcolicVariable"_cs, NodeFactoryFn(&IR::ConcolicVariable::fromJSON)},
{"HeaderExpression"_cs, NodeFactoryFn(&IR::HeaderExpression::fromJSON)},
{"InOutReference"_cs, NodeFactoryFn(&IR::InOutReference::fromJSON)},
{"UninitializedTaintExpression"_cs, NodeFactoryFn(&IR::UninitializedTaintExpression::fromJSON)},
{"HardwareConstrainedField"_cs, NodeFactoryFn(&IR::BFN::HardwareConstrainedField::fromJSON)},
{"Pipe"_cs, NodeFactoryFn(&IR::BFN::Pipe::fromJSON)},
{"BridgePath"_cs, NodeFactoryFn(&IR::BFN::BridgePath::fromJSON)},
{"Toplevel"_cs, NodeFactoryFn(&IR::BFN::Toplevel::fromJSON)},
{"InstanceRef"_cs, NodeFactoryFn(&IR::InstanceRef::fromJSON)},
{"V1InstanceRef"_cs, NodeFactoryFn(&IR::V1InstanceRef::fromJSON)},
{"MetadataInfo"_cs, NodeFactoryFn(&IR::MetadataInfo::fromJSON)},
{"TempVar"_cs, NodeFactoryFn(&IR::TempVar::fromJSON)},
{"Padding"_cs, NodeFactoryFn(&IR::Padding::fromJSON)},
{"Type_FixedSizeHeader"_cs, NodeFactoryFn(&IR::BFN::Type_FixedSizeHeader::fromJSON)},
{"ContainerRef"_cs, NodeFactoryFn(&IR::BFN::ContainerRef::fromJSON)},
{"ContainerBitRef"_cs, NodeFactoryFn(&IR::BFN::ContainerBitRef::fromJSON)},
{"AliasMember"_cs, NodeFactoryFn(&IR::BFN::AliasMember::fromJSON)},
{"AliasSlice"_cs, NodeFactoryFn(&IR::BFN::AliasSlice::fromJSON)},
{"HashListExpression"_cs, NodeFactoryFn(&IR::HashListExpression::fromJSON)},
{"HashStructExpression"_cs, NodeFactoryFn(&IR::HashStructExpression::fromJSON)},
{"ReinterpretCast"_cs, NodeFactoryFn(&IR::BFN::ReinterpretCast::fromJSON)},
{"SignExtend"_cs, NodeFactoryFn(&IR::BFN::SignExtend::fromJSON)},
{"TnaControl"_cs, NodeFactoryFn(&IR::BFN::TnaControl::fromJSON)},
{"TnaDeparser"_cs, NodeFactoryFn(&IR::BFN::TnaDeparser::fromJSON)},
{"TnaParser"_cs, NodeFactoryFn(&IR::BFN::TnaParser::fromJSON)},
{"P4Thread"_cs, NodeFactoryFn(&IR::BFN::P4Thread::fromJSON)},
{"Table"_cs, NodeFactoryFn(&IR::MAU::Table::fromJSON)},
{"BackendAttached"_cs, NodeFactoryFn(&IR::MAU::BackendAttached::fromJSON)},
{"Counter"_cs, NodeFactoryFn(&IR::MAU::Counter::fromJSON)},
{"Meter"_cs, NodeFactoryFn(&IR::MAU::Meter::fromJSON)},
{"StatefulAlu"_cs, NodeFactoryFn(&IR::MAU::StatefulAlu::fromJSON)},
{"Selector"_cs, NodeFactoryFn(&IR::MAU::Selector::fromJSON)},
{"TernaryIndirect"_cs, NodeFactoryFn(&IR::MAU::TernaryIndirect::fromJSON)},
{"ActionData"_cs, NodeFactoryFn(&IR::MAU::ActionData::fromJSON)},
{"IdleTime"_cs, NodeFactoryFn(&IR::MAU::IdleTime::fromJSON)},
{"TableSeq"_cs, NodeFactoryFn(&IR::MAU::TableSeq::fromJSON)},
{"Primitive"_cs, NodeFactoryFn(&IR::MAU::Primitive::fromJSON)},
{"TypedPrimitive"_cs, NodeFactoryFn(&IR::MAU::TypedPrimitive::fromJSON)},
{"Instruction"_cs, NodeFactoryFn(&IR::MAU::Instruction::fromJSON)},
{"SaluInstruction"_cs, NodeFactoryFn(&IR::MAU::SaluInstruction::fromJSON)},
{"ActionArg"_cs, NodeFactoryFn(&IR::MAU::ActionArg::fromJSON)},
{"ConditionalArg"_cs, NodeFactoryFn(&IR::MAU::ConditionalArg::fromJSON)},
{"StatefulCounter"_cs, NodeFactoryFn(&IR::MAU::StatefulCounter::fromJSON)},
{"StatefulCall"_cs, NodeFactoryFn(&IR::MAU::StatefulCall::fromJSON)},
{"Action"_cs, NodeFactoryFn(&IR::MAU::Action::fromJSON)},
{"SaluAction"_cs, NodeFactoryFn(&IR::MAU::SaluAction::fromJSON)},
{"FieldListExpression"_cs, NodeFactoryFn(&IR::MAU::FieldListExpression::fromJSON)},
{"HashGenExpression"_cs, NodeFactoryFn(&IR::MAU::HashGenExpression::fromJSON)},
{"HashDist"_cs, NodeFactoryFn(&IR::MAU::HashDist::fromJSON)},
{"IXBarExpression"_cs, NodeFactoryFn(&IR::MAU::IXBarExpression::fromJSON)},
{"ActionDataConstant"_cs, NodeFactoryFn(&IR::MAU::ActionDataConstant::fromJSON)},
{"RandomNumber"_cs, NodeFactoryFn(&IR::MAU::RandomNumber::fromJSON)},
{"AttachedOutput"_cs, NodeFactoryFn(&IR::MAU::AttachedOutput::fromJSON)},
{"SaluReg"_cs, NodeFactoryFn(&IR::MAU::SaluReg::fromJSON)},
{"SaluCmpReg"_cs, NodeFactoryFn(&IR::MAU::SaluCmpReg::fromJSON)},
{"SaluFunction"_cs, NodeFactoryFn(&IR::MAU::SaluFunction::fromJSON)},
{"SaluRegfileRow"_cs, NodeFactoryFn(&IR::MAU::SaluRegfileRow::fromJSON)},
{"MultiOperand"_cs, NodeFactoryFn(&IR::MAU::MultiOperand::fromJSON)},
{"WrappedSlice"_cs, NodeFactoryFn(&IR::MAU::WrappedSlice::fromJSON)},
{"TableKey"_cs, NodeFactoryFn(&IR::MAU::TableKey::fromJSON)},
{"RangeMatch"_cs, NodeFactoryFn(&IR::RangeMatch::fromJSON)},
{"UnresolvedHeaderStackIndex"_cs, NodeFactoryFn(&IR::BFN::UnresolvedHeaderStackIndex::fromJSON)},
{"PacketRVal"_cs, NodeFactoryFn(&IR::BFN::PacketRVal::fromJSON)},
{"MetadataRVal"_cs, NodeFactoryFn(&IR::BFN::MetadataRVal::fromJSON)},
{"ConstantRVal"_cs, NodeFactoryFn(&IR::BFN::ConstantRVal::fromJSON)},
{"SavedRVal"_cs, NodeFactoryFn(&IR::BFN::SavedRVal::fromJSON)},
{"FieldLVal"_cs, NodeFactoryFn(&IR::BFN::FieldLVal::fromJSON)},
{"MatchLVal"_cs, NodeFactoryFn(&IR::BFN::MatchLVal::fromJSON)},
{"ChecksumLVal"_cs, NodeFactoryFn(&IR::BFN::ChecksumLVal::fromJSON)},
{"TotalContainerSize"_cs, NodeFactoryFn(&IR::BFN::TotalContainerSize::fromJSON)},
{"Extract"_cs, NodeFactoryFn(&IR::BFN::Extract::fromJSON)},
{"ExtractPhv"_cs, NodeFactoryFn(&IR::BFN::ExtractPhv::fromJSON)},
{"ExtractClot"_cs, NodeFactoryFn(&IR::BFN::ExtractClot::fromJSON)},
{"SaveToRegister"_cs, NodeFactoryFn(&IR::BFN::SaveToRegister::fromJSON)},
{"ParserZeroInit"_cs, NodeFactoryFn(&IR::BFN::ParserZeroInit::fromJSON)},
{"ParserCounterLoadImm"_cs, NodeFactoryFn(&IR::BFN::ParserCounterLoadImm::fromJSON)},
{"ParserCounterLoadPkt"_cs, NodeFactoryFn(&IR::BFN::ParserCounterLoadPkt::fromJSON)},
{"ParserCounterIncrement"_cs, NodeFactoryFn(&IR::BFN::ParserCounterIncrement::fromJSON)},
{"ParserCounterDecrement"_cs, NodeFactoryFn(&IR::BFN::ParserCounterDecrement::fromJSON)},
{"ParserCounterPop"_cs, NodeFactoryFn(&IR::BFN::ParserCounterPop::fromJSON)},
{"ParserCounterIsZero"_cs, NodeFactoryFn(&IR::BFN::ParserCounterIsZero::fromJSON)},
{"ParserCounterIsNegative"_cs, NodeFactoryFn(&IR::BFN::ParserCounterIsNegative::fromJSON)},
{"ParserPrioritySet"_cs, NodeFactoryFn(&IR::BFN::ParserPrioritySet::fromJSON)},
{"ChecksumVerify"_cs, NodeFactoryFn(&IR::BFN::ChecksumVerify::fromJSON)},
{"ChecksumResidualDeposit"_cs, NodeFactoryFn(&IR::BFN::ChecksumResidualDeposit::fromJSON)},
{"ChecksumDepositToClot"_cs, NodeFactoryFn(&IR::BFN::ChecksumDepositToClot::fromJSON)},
{"ChecksumAdd"_cs, NodeFactoryFn(&IR::BFN::ChecksumAdd::fromJSON)},
{"ChecksumSubtract"_cs, NodeFactoryFn(&IR::BFN::ChecksumSubtract::fromJSON)},
{"HdrLenIncStop"_cs, NodeFactoryFn(&IR::BFN::HdrLenIncStop::fromJSON)},
{"ParserConstMatchValue"_cs, NodeFactoryFn(&IR::BFN::ParserConstMatchValue::fromJSON)},
{"ParserPvsMatchValue"_cs, NodeFactoryFn(&IR::BFN::ParserPvsMatchValue::fromJSON)},
{"Phase0"_cs, NodeFactoryFn(&IR::BFN::Phase0::fromJSON)},
{"Transition"_cs, NodeFactoryFn(&IR::BFN::Transition::fromJSON)},
{"Select"_cs, NodeFactoryFn(&IR::BFN::Select::fromJSON)},
{"ParserState"_cs, NodeFactoryFn(&IR::BFN::ParserState::fromJSON)},
{"Parser"_cs, NodeFactoryFn(&IR::BFN::Parser::fromJSON)},
{"GhostParser"_cs, NodeFactoryFn(&IR::BFN::GhostParser::fromJSON)},
{"DigestFieldList"_cs, NodeFactoryFn(&IR::BFN::DigestFieldList::fromJSON)},
{"Digest"_cs, NodeFactoryFn(&IR::BFN::Digest::fromJSON)},
{"EmitField"_cs, NodeFactoryFn(&IR::BFN::EmitField::fromJSON)},
{"EmitConstant"_cs, NodeFactoryFn(&IR::BFN::EmitConstant::fromJSON)},
{"EmitClot"_cs, NodeFactoryFn(&IR::BFN::EmitClot::fromJSON)},
{"ChecksumEntry"_cs, NodeFactoryFn(&IR::BFN::ChecksumEntry::fromJSON)},
{"EmitChecksum"_cs, NodeFactoryFn(&IR::BFN::EmitChecksum::fromJSON)},
{"DeparserParameter"_cs, NodeFactoryFn(&IR::BFN::DeparserParameter::fromJSON)},
{"Deparser"_cs, NodeFactoryFn(&IR::BFN::Deparser::fromJSON)},
{"LoweredPacketRVal"_cs, NodeFactoryFn(&IR::BFN::LoweredPacketRVal::fromJSON)},
{"LoweredMetadataRVal"_cs, NodeFactoryFn(&IR::BFN::LoweredMetadataRVal::fromJSON)},
{"LoweredConstantRVal"_cs, NodeFactoryFn(&IR::BFN::LoweredConstantRVal::fromJSON)},
{"LoweredExtractPhv"_cs, NodeFactoryFn(&IR::BFN::LoweredExtractPhv::fromJSON)},
{"LoweredExtractClot"_cs, NodeFactoryFn(&IR::BFN::LoweredExtractClot::fromJSON)},
{"LoweredParserChecksum"_cs, NodeFactoryFn(&IR::BFN::LoweredParserChecksum::fromJSON)},
{"LoweredSave"_cs, NodeFactoryFn(&IR::BFN::LoweredSave::fromJSON)},
{"LoweredSelect"_cs, NodeFactoryFn(&IR::BFN::LoweredSelect::fromJSON)},
{"LoweredParserMatch"_cs, NodeFactoryFn(&IR::BFN::LoweredParserMatch::fromJSON)},
{"LoweredParserState"_cs, NodeFactoryFn(&IR::BFN::LoweredParserState::fromJSON)},
{"BaseLoweredParser"_cs, NodeFactoryFn(&IR::BFN::BaseLoweredParser::fromJSON)},
{"LoweredParser"_cs, NodeFactoryFn(&IR::BFN::LoweredParser::fromJSON)},
{"LoweredEmitPhv"_cs, NodeFactoryFn(&IR::BFN::LoweredEmitPhv::fromJSON)},
{"LoweredEmitChecksum"_cs, NodeFactoryFn(&IR::BFN::LoweredEmitChecksum::fromJSON)},
{"LoweredEmitClot"_cs, NodeFactoryFn(&IR::BFN::LoweredEmitClot::fromJSON)},
{"LoweredEmitConstant"_cs, NodeFactoryFn(&IR::BFN::LoweredEmitConstant::fromJSON)},
{"LoweredDeparserParameter"_cs, NodeFactoryFn(&IR::BFN::LoweredDeparserParameter::fromJSON)},
{"DigestTableEntry"_cs, NodeFactoryFn(&IR::BFN::DigestTableEntry::fromJSON)},
{"DigestField"_cs, NodeFactoryFn(&IR::BFN::DigestField::fromJSON)},
{"LearningTableEntry"_cs, NodeFactoryFn(&IR::BFN::LearningTableEntry::fromJSON)},
{"LoweredDigest"_cs, NodeFactoryFn(&IR::BFN::LoweredDigest::fromJSON)},
{"ChecksumPhvInput"_cs, NodeFactoryFn(&IR::BFN::ChecksumPhvInput::fromJSON)},
{"ChecksumClotInput"_cs, NodeFactoryFn(&IR::BFN::ChecksumClotInput::fromJSON)},
{"PartialChecksumUnitConfig"_cs, NodeFactoryFn(&IR::BFN::PartialChecksumUnitConfig::fromJSON)},
{"FullChecksumUnitConfig"_cs, NodeFactoryFn(&IR::BFN::FullChecksumUnitConfig::fromJSON)},
{"LoweredDeparser"_cs, NodeFactoryFn(&IR::BFN::LoweredDeparser::fromJSON)} };

template class IR::Vector<IR::Node>;
template class IR::IndexedVector<IR::Node>;
template class IR::Vector<IR::Type>;
template class IR::Vector<IR::StatOrDecl>;
template class IR::IndexedVector<IR::StatOrDecl>;
template class IR::Vector<IR::Declaration>;
template class IR::IndexedVector<IR::Declaration>;
template class IR::Vector<IR::Expression>;
template class IR::Vector<IR::NamedExpression>;
template class IR::IndexedVector<IR::NamedExpression>;
template class IR::Vector<IR::AnnotationToken>;
template class IR::Vector<IR::Annotation>;
template class IR::Vector<IR::Argument>;
template class IR::Vector<IR::Parameter>;
template class IR::IndexedVector<IR::Parameter>;
template class IR::Vector<IR::Type_Var>;
template class IR::IndexedVector<IR::Type_Var>;
template class IR::Vector<IR::StructField>;
template class IR::IndexedVector<IR::StructField>;
template class IR::Vector<IR::Declaration_ID>;
template class IR::IndexedVector<IR::Declaration_ID>;
template class IR::Vector<IR::SerEnumMember>;
template class IR::IndexedVector<IR::SerEnumMember>;
template class IR::Vector<IR::ArgumentInfo>;
template class IR::Vector<IR::Method>;
template class IR::Vector<IR::SelectCase>;
template class IR::Vector<IR::ParserState>;
template class IR::IndexedVector<IR::ParserState>;
template class IR::Vector<IR::P4Parser>;
template class IR::Vector<IR::ActionListElement>;
template class IR::IndexedVector<IR::ActionListElement>;
template class IR::Vector<IR::KeyElement>;
template class IR::Vector<IR::Entry>;
template class IR::Vector<IR::Property>;
template class IR::IndexedVector<IR::Property>;
template class IR::Vector<IR::SwitchCase>;
template class IR::Vector<IR::Primitive>;
template class IR::Vector<IR::CaseEntry>;
template class IR::Vector<IR::DpdkDeclaration>;
template class IR::IndexedVector<IR::DpdkDeclaration>;
template class IR::Vector<IR::DpdkExternDeclaration>;
template class IR::IndexedVector<IR::DpdkExternDeclaration>;
template class IR::Vector<IR::DpdkHeaderType>;
template class IR::IndexedVector<IR::DpdkHeaderType>;
template class IR::Vector<IR::DpdkHeaderInstance>;
template class IR::IndexedVector<IR::DpdkHeaderInstance>;
template class IR::Vector<IR::DpdkStructType>;
template class IR::IndexedVector<IR::DpdkStructType>;
template class IR::Vector<IR::DpdkAsmStatement>;
template class IR::IndexedVector<IR::DpdkAsmStatement>;
template class IR::Vector<IR::DpdkAction>;
template class IR::IndexedVector<IR::DpdkAction>;
template class IR::Vector<IR::DpdkTable>;
template class IR::IndexedVector<IR::DpdkTable>;
template class IR::Vector<IR::DpdkSelector>;
template class IR::IndexedVector<IR::DpdkSelector>;
template class IR::Vector<IR::DpdkLearner>;
template class IR::IndexedVector<IR::DpdkLearner>;
template class IR::Vector<IR::BFN::HardwareConstrainedField>;
template class IR::Vector<IR::BFN::Pipe>;
template class IR::Vector<IR::BFN::ContainerRef>;
template class IR::Vector<IR::MAU::Table>;
template class IR::Vector<IR::MAU::BackendAttached>;
template class IR::Vector<IR::MAU::Primitive>;
template class IR::Vector<IR::MAU::SaluInstruction>;
template class IR::Vector<IR::MAU::StatefulCall>;
template class IR::Vector<IR::MAU::TableKey>;
template class IR::Vector<IR::BFN::FieldLVal>;
template class IR::Vector<IR::BFN::ParserPrimitive>;
template class IR::Vector<IR::BFN::SaveToRegister>;
template class IR::Vector<IR::BFN::ParserCounterPrimitive>;
template class IR::Vector<IR::BFN::Transition>;
template class IR::Vector<IR::BFN::Select>;
template class IR::Vector<IR::BFN::AbstractParser>;
template class IR::Vector<IR::BFN::DigestFieldList>;
template class IR::Vector<IR::BFN::Emit>;
template class IR::Vector<IR::BFN::ChecksumEntry>;
template class IR::Vector<IR::BFN::EmitChecksum>;
template class IR::Vector<IR::BFN::DeparserParameter>;
template class IR::Vector<IR::BFN::LoweredParserPrimitive>;
template class IR::Vector<IR::BFN::LoweredParserChecksum>;
template class IR::Vector<IR::BFN::LoweredSave>;
template class IR::Vector<IR::BFN::LoweredParserMatch>;
template class IR::Vector<IR::BFN::LoweredEmit>;
template class IR::Vector<IR::BFN::LoweredDeparserParameter>;
template class IR::Vector<IR::BFN::DigestTableEntry>;
template class IR::Vector<IR::BFN::LoweredDigest>;
template class IR::Vector<IR::BFN::ChecksumPhvInput>;
template class IR::Vector<IR::BFN::ChecksumClotInput>;
template class IR::Vector<IR::BFN::PartialChecksumUnitConfig>;
template class IR::Vector<IR::BFN::FullChecksumUnitConfig>;

#line 22 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
bool IR::CompileTimeValue::equiv(IR::CompileTimeValue const & other) const {
        return this->getNode()->equiv(*other.getNode());
    }
#line 527 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// Well-defined only for types with fixed width
#line 42 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
int IR::Type::width_bits() const { BUG("width_bits() on type with unknown size: %1%", this); }
#line 531 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// When possible returns the corresponding type that can be inserted
/// in a P4 program; may return a Type_Name
bool IR::Type::operator==(IR::Type const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::Type::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::Type::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::Type::Type(JSONLoader & json) : Node(json) {
}
bool IR::Type::operator==(IR::Node const & a) const { return a == *this; }
/// Allows the retrieval of type parameters
/// @returns the type signature of the apply method
/// prints an error if it finds duplicate names
#line 80 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
void IR::IGeneralNamespace::validate() const { checkDuplicateDeclarations(); }
#line 553 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The parameters of the functional object
/// Returns true if the parameters can be matched with the
/// supplied arguments.
/// The type of the constructor as a method

#line 114 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
const IR::ParameterList *IR::IContainer::getParameters() const { return getConstructorParameters(); }
#line 561 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 120 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
IR::Type const * IR::Type_Base::getP4Type() const { return this; }
#line 564 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Base::operator==(IR::Type_Base const & a) const {
        return Type::operator==(static_cast<const Type &>(a));
    }
bool IR::Type_Base::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        return true;
    }
void IR::Type_Base::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
}
IR::Type_Base::Type_Base(JSONLoader & json) : Type(json) {
}
bool IR::Type_Base::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Base::operator==(IR::Node const & a) const { return a == *this; }
#line 128 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
cstring IR::Type_Unknown::toString() const { return "Unknown type"_cs; }
#line 582 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Unknown::operator==(IR::Type_Unknown const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_Unknown::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_Unknown::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_Unknown::Type_Unknown(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_Unknown::fromJSON(JSONLoader & json) { return new Type_Unknown(json); }
bool IR::Type_Unknown::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_Unknown::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Unknown::operator==(IR::Node const & a) const { return a == *this; }
bool IR::StatOrDecl::operator==(IR::StatOrDecl const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::StatOrDecl::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::StatOrDecl::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::StatOrDecl::StatOrDecl(JSONLoader & json) : Node(json) {
}
bool IR::StatOrDecl::operator==(IR::Node const & a) const { return a == *this; }
#line 139 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
IR::ID IR::Declaration::getName() const { return name; }
#line 616 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 140 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
bool IR::Declaration::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!StatOrDecl::equiv(a_)) return false;
        auto &a = static_cast<const Declaration &>(a_);

#line 140 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
{ return name == a.name; /* ignore declid */ }    }
#line 625 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 144 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
cstring IR::Declaration::toString() const { return externalName(); }
#line 628 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Declaration::operator==(IR::Declaration const & a) const {
        return StatOrDecl::operator==(static_cast<const StatOrDecl &>(a))
        && name == a.name
        && declid == a.declid;
    }
void IR::Declaration::dump_fields(std::ostream & out) const {
    StatOrDecl::dump_fields(out);
        out << " name=" << name;
        out << " declid=" << declid;
}
void IR::Declaration::toJSON(JSONGenerator & json) const {
    StatOrDecl::toJSON(json);
    json.emit("name", name);
    json.emit("declid", declid);
}
IR::Declaration::Declaration(JSONLoader & json) : StatOrDecl(json) {
    json.load("name", name);
    json.load("declid", declid);
}
bool IR::Declaration::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::Declaration::operator==(IR::Node const & a) const { return a == *this; }
#line 155 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
IR::ID IR::Type_Declaration::getName() const { return name; }
#line 652 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 156 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
bool IR::Type_Declaration::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_Declaration &>(a_);

#line 156 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
{ return name == a.name; /* ignore declid */ }    }
#line 661 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 160 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
cstring IR::Type_Declaration::toString() const { return externalName(); }
#line 664 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 161 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
IR::Type const * IR::Type_Declaration::getP4Type() const { return new Type_Name(name); }
#line 667 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Declaration::operator==(IR::Type_Declaration const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && name == a.name
        && declid == a.declid;
    }
void IR::Type_Declaration::dump_fields(std::ostream & out) const {
    Type::dump_fields(out);
        out << " name=" << name;
        out << " declid=" << declid;
}
void IR::Type_Declaration::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("name", name);
    json.emit("declid", declid);
}
IR::Type_Declaration::Type_Declaration(JSONLoader & json) : Type(json) {
    json.load("name", name);
    json.load("declid", declid);
}
bool IR::Type_Declaration::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Declaration::operator==(IR::Node const & a) const { return a == *this; }
/// Note that the type field is not visited.
/// Most P4_16 passes don't use this field.
/// It is a used to hold the result of TypeInferencing for the expression.
/// It is used by the P4_14 front-end and by some back-ends.
/// It is not visited by the visitors by default (can be visited explicitly in preorder)
#line 172 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
void IR::Expression::visit_children(Visitor & v, char const * n) { (void)v; (void)n; }
#line 172 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
void IR::Expression::visit_children(Visitor & v, char const * n) const { (void)v; (void)n; }
#line 698 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(Expression, , )
bool IR::Expression::operator==(IR::Expression const & a) const {
        return this->typeId() == a.typeId()
        && type == a.type;
    }
bool IR::Expression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Expression &>(a_);
        return (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr);
    }
void IR::Expression::validate() const {
#line 711 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(type); }
void IR::Expression::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("type", type);
}
IR::Expression::Expression(JSONLoader & json) : Node(json) {
    json.load("type", type);
}
bool IR::Expression::operator==(IR::Node const & a) const { return a == *this; }
#line 184 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
cstring IR::Operation::toString() const { return getStringOp(); }
#line 724 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Operation::operator==(IR::Operation const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a));
    }
bool IR::Operation::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        return true;
    }
void IR::Operation::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
}
IR::Operation::Operation(JSONLoader & json) : Expression(json) {
}
bool IR::Operation::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Operation::operator==(IR::Node const & a) const { return a == *this; }
#line 195 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
cstring IR::Path::toString() const {
        // This is the ORIGINAL name the user used
        return absl::StrCat(absolute ? "." : "", name.toString());
    }
#line 745 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 203 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
void IR::Path::dbprint(std::ostream & out) const { out << name; }
#line 748 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 204 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
void IR::Path::validate() const {
#line 751 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 204 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
{ BUG_CHECK(!name.name.isNullOrEmpty(), "Empty path"); } }
#line 755 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Path::operator==(IR::Path const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && absolute == a.absolute;
    }
bool IR::Path::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Path &>(a_);
        return name == a.name
        && absolute == a.absolute;
    }
void IR::Path::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
        out << " absolute=" << absolute;
}
void IR::Path::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("absolute", absolute);
}
IR::Path::Path(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("absolute", absolute);
}
IR::Node * IR::Path::fromJSON(JSONLoader & json) { return new Path(json); }
bool IR::Path::operator==(IR::Node const & a) const { return a == *this; }
bool IR::NamedExpression::operator==(IR::NamedExpression const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && expression == a.expression;
    }
bool IR::NamedExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const NamedExpression &>(a_);
        return (expression ? a.expression ? expression->equiv(*a.expression) : false : a.expression == nullptr);
    }
void IR::NamedExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    v.visit(expression, "expression");
}
void IR::NamedExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    v.visit(expression, "expression");
}
void IR::NamedExpression::validate() const {
#line 805 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(expression); }
void IR::NamedExpression::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("expression", expression);
}
IR::NamedExpression::NamedExpression(JSONLoader & json) : Declaration(json) {
    json.load("expression", expression);
}
IR::Node * IR::NamedExpression::fromJSON(JSONLoader & json) { return new NamedExpression(json); }
bool IR::NamedExpression::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::NamedExpression::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::NamedExpression::operator==(IR::Node const & a) const { return a == *this; }

#line 219 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
void IR::AnnotationToken::dbprint(std::ostream & out) const { out << text; }
#line 822 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::AnnotationToken::operator==(IR::AnnotationToken const & a) const {
        return this->typeId() == a.typeId()
        && token_type == a.token_type
        && text == a.text
        && constInfo == a.constInfo;
    }
bool IR::AnnotationToken::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const AnnotationToken &>(a_);
        return token_type == a.token_type
        && text == a.text
        && constInfo == a.constInfo;
    }
void IR::AnnotationToken::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " token_type=" << token_type;
        out << " text=" << text;
        out << " constInfo=" << constInfo;
}
void IR::AnnotationToken::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("token_type", token_type);
    json.emit("text", text);
    if (constInfo != nullptr) json.emit("constInfo", constInfo);
}
IR::AnnotationToken::AnnotationToken(JSONLoader & json) : Node(json) {
    json.load("token_type", token_type);
    json.load("text", text);
    json.load("constInfo", constInfo);
}
IR::Node * IR::AnnotationToken::fromJSON(JSONLoader & json) { return new AnnotationToken(json); }
bool IR::AnnotationToken::operator==(IR::Node const & a) const { return a == *this; }
/// For annotations parsed from P4-16 source.


#line 263 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
IR::Annotation::Annotation(IR::ID n, intmax_t v, bool structured) : name(n), structured(structured) {
        body.emplace<ExpressionAnnotation>(new Constant(v));
    }
#line 863 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"


#line 269 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
IR::Annotation::Annotation(IR::ID n, cstring v, bool structured) : name(n), structured(structured) {
        body.emplace<ExpressionAnnotation>(new StringLiteral(v));
    }
#line 870 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

/// Indicates the control-plane name.
/// Action cannot be a default_action.
/// action can only be a default_action.
/// Code should be executed atomically.
/// Object should not be exposed to the control-plane.
/// P4-14 annotation for varbit fields.
/// P4-14 annotation for varbit fields.
/// Optional parameter annotation
/// Package documentation annotation.
/// Deprecation annotation.
/// Synchronous annotation.
/// extern function/method annotation.
/// extern function/method annotation.
/// noWarn annotation.
/// Match annotation (for value sets).
/// Used for recirculate, etc.
/// Used by compiler implementer to limit debug log to the annotated IR context.
/// annotation to disable certain optimization
/// annotation to mark block of inlined function
/// annotation for likely taken blocks/branchs
/// annotation for likely not taken blocks/branchs
#line 296 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
cstring IR::Annotation::toString() const { return absl::StrCat("@", name); }
#line 895 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 297 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
void IR::Annotation::validate() const {
#line 898 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
    std::visit([&](auto &&variant) {
        using T = std::decay_t<decltype(variant)>;
        if constexpr (std::is_same_v<T, IR::Vector<IR::AnnotationToken>>) { variant.validate(); }
        else if constexpr (std::is_same_v<T,IR::Vector<IR::Expression>>) { variant.validate(); }
        else if constexpr (std::is_same_v<T,IR::IndexedVector<IR::NamedExpression>>) { variant.validate(); }
        else { BUG("Unexpected variant field"); } }, body);

#line 297 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
{
        BUG_CHECK(!name.name.isNullOrEmpty(), "empty annotation name");
    } }
#line 910 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// Extracts name value from a name annotation
/// Extracts a single string argument; error if the argument is not a string
/// Whether the annotation body needs to be parsed.



/// If this is true this is a structured annotation, and there are some
/// constraints on its contents.
bool IR::Annotation::operator==(IR::Annotation const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && body == a.body
        && structured == a.structured;
    }
bool IR::Annotation::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Annotation &>(a_);
        auto body_equivVisitor = [&](auto &&variant) -> bool {
            using T = std::decay_t<decltype(variant)>;
            if constexpr (std::is_same_v<T, Vector<AnnotationToken>>) {return variant.equiv(std::get<IR::Vector<IR::AnnotationToken>>(a.body)); }
            else if constexpr (std::is_same_v<T,Vector<Expression>>) {return variant.equiv(std::get<IR::Vector<IR::Expression>>(a.body)); }
            else if constexpr (std::is_same_v<T,IndexedVector<NamedExpression>>) {return variant.equiv(std::get<IR::IndexedVector<IR::NamedExpression>>(a.body)); }
            else { BUG("Unexpected variant field"); } };
        return name == a.name
        && body.index() == a.body.index() && std::visit(body_equivVisitor, body)
        && structured == a.structured;
    }
void IR::Annotation::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    std::visit([&](auto &&variant) {
        using T = std::decay_t<decltype(variant)>;
        if constexpr (std::is_same_v<T, IR::Vector<IR::AnnotationToken>>) { v.visit(variant, "body::IR::Vector<IR::AnnotationToken>"); }
        else if constexpr (std::is_same_v<T,IR::Vector<IR::Expression>>) { v.visit(variant, "body::IR::Vector<IR::Expression>"); }
        else if constexpr (std::is_same_v<T,IR::IndexedVector<IR::NamedExpression>>) { v.visit(variant, "body::IR::IndexedVector<IR::NamedExpression>"); }
        else { BUG("Unexpected variant field"); } }, body);
}
void IR::Annotation::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    std::visit([&](auto &&variant) {
        using T = std::decay_t<decltype(variant)>;
        if constexpr (std::is_same_v<T, IR::Vector<IR::AnnotationToken>>) { v.visit(variant, "body::IR::Vector<IR::AnnotationToken>"); }
        else if constexpr (std::is_same_v<T,IR::Vector<IR::Expression>>) { v.visit(variant, "body::IR::Vector<IR::Expression>"); }
        else if constexpr (std::is_same_v<T,IR::IndexedVector<IR::NamedExpression>>) { v.visit(variant, "body::IR::IndexedVector<IR::NamedExpression>"); }
        else { BUG("Unexpected variant field"); } }, body);
}
void IR::Annotation::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
    std::visit([&](auto &&variant) {
        using T = std::decay_t<decltype(variant)>;
        if constexpr (std::is_same_v<T, IR::Vector<IR::AnnotationToken>>) {
}         else if constexpr (std::is_same_v<T,IR::Vector<IR::Expression>>) {
}         else if constexpr (std::is_same_v<T,IR::IndexedVector<IR::NamedExpression>>) {
}         else { BUG("Unexpected variant field"); } }, body);
        out << " structured=" << structured;
}
void IR::Annotation::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("body", body);
    json.emit("structured", structured);
}
IR::Annotation::Annotation(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("body", body);
    json.load("structured", structured);
}
IR::Node * IR::Annotation::fromJSON(JSONLoader & json) { return new Annotation(json); }
bool IR::Annotation::operator==(IR::Node const & a) const { return a == *this; }
/// Checks if there is annotation @name
/// Checks if there is annotation @name and it is the only annotation on the
/// node
/// Check if there are any annotations
/// Add annotation if another annotation with the same name is not
/// already present.
/// If annotations with the same name are already present, remove them;
/// add this annotation.
/// If an argument has no name the name.name is nullptr.
#line 462 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
void IR::Argument::dbprint(std::ostream & out) const { out << (name.name.isNullOrEmpty() ? "" : name.name + " = ") << expression; }
#line 994 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 463 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
void IR::Argument::validate() const {
#line 997 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(expression);
#line 463 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
{ CHECK_NULL(expression); } }
#line 1002 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 464 "/builddir/build/BUILD/p4c-1.2.5.8/ir/base.def"
cstring IR::Argument::toString() const {
        std::string result = "";
        if (!name.name.isNullOrEmpty())
            absl::StrAppend(&result, name, " = ");
        return absl::StrCat(result, expression);
    }
#line 1010 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Argument::operator==(IR::Argument const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && expression == a.expression;
    }
bool IR::Argument::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Argument &>(a_);
        return name == a.name
        && (expression ? a.expression ? expression->equiv(*a.expression) : false : a.expression == nullptr);
    }
void IR::Argument::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(expression, "expression");
}
void IR::Argument::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(expression, "expression");
}
void IR::Argument::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
}
void IR::Argument::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("expression", expression);
}
IR::Argument::Argument(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("expression", expression);
}
IR::Node * IR::Argument::fromJSON(JSONLoader & json) { return new Argument(json); }
bool IR::Argument::operator==(IR::Node const & a) const { return a == *this; }
/** @} */
/* end group irdefs */
/* -*-C++-*- */
/** \addtogroup irdefs
  * @{
  */
/*
   This file contains IR related to representating the type hierarchy.
   Some of these classes never appear in the IR tree, they are just
   synthesized by the type-checker.
*/
#line 88 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Any::getVarName() const { return absl::StrCat("int_", declid); }
#line 1061 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 89 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_Any::getDeclId() const { return declid; }
#line 1064 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 90 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Any::dbprint(std::ostream & out) const { out << "ANYTYPE/" << declid; }
#line 1067 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 91 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Any::toString() const { return "ANYTYPE"_cs; }
#line 1070 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 92 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
bool IR::Type_Any::operator==(IR::Type_Any const & a) const { return declid == a.declid; }
#line 1073 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 95 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type const * IR::Type_Any::getP4Type() const { return nullptr; }
#line 1076 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 96 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
bool IR::Type_Any::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_Any &>(a_);

#line 96 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{
        (void)a;  // silence unused warning
        return true; /* ignore declid */
    }    }
#line 1088 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
void IR::Type_Any::dump_fields(std::ostream & out) const {
    Type::dump_fields(out);
        out << " declid=" << declid;
}
void IR::Type_Any::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("declid", declid);
}
IR::Type_Any::Type_Any(JSONLoader & json) : Type(json) {
    json.load("declid", declid);
}
IR::Node * IR::Type_Any::fromJSON(JSONLoader & json) { return new Type_Any(json); }
bool IR::Type_Any::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Any::operator==(IR::Node const & a) const { return a == *this; }
#line 115 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Fragment::dbprint(std::ostream & out) const { out << "FRAGMENT(" << type << ")"; }
#line 1105 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 116 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Fragment::toString() const { return absl::StrCat("FRAGMENT(", type, ")"); }
#line 1108 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 117 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type const * IR::Type_Fragment::getP4Type() const { return nullptr; }
#line 1111 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Fragment::operator==(IR::Type_Fragment const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && type == a.type;
    }
bool IR::Type_Fragment::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_Fragment &>(a_);
        return (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr);
    }
void IR::Type_Fragment::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    v.visit(type, "type");
}
void IR::Type_Fragment::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    v.visit(type, "type");
}
void IR::Type_Fragment::validate() const {
#line 1133 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(type); }
void IR::Type_Fragment::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("type", type);
}
IR::Type_Fragment::Type_Fragment(JSONLoader & json) : Type(json) {
    json.load("type", type);
}
IR::Node * IR::Type_Fragment::fromJSON(JSONLoader & json) { return new Type_Fragment(json); }
bool IR::Type_Fragment::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Fragment::operator==(IR::Node const & a) const { return a == *this; }
#line 129 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Type::toString() const { return absl::StrCat("Type(", type, ")"); }
#line 1148 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 130 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Type::dbprint(std::ostream & out) const { out << "Type(" << type << ")"; }
#line 1151 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 131 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type const * IR::Type_Type::getP4Type() const { return type; }
#line 1154 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 132 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Type::validate() const {
#line 1157 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(type);
#line 132 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{ BUG_CHECK(!type->is<IR::Type_Type>(), "%1%: nested Type_Type", type); } }
#line 1162 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Type::operator==(IR::Type_Type const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && type == a.type;
    }
bool IR::Type_Type::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_Type &>(a_);
        return (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr);
    }
void IR::Type_Type::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    v.visit(type, "type");
}
void IR::Type_Type::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    v.visit(type, "type");
}
void IR::Type_Type::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("type", type);
}
IR::Type_Type::Type_Type(JSONLoader & json) : Type(json) {
    json.load("type", type);
}
IR::Node * IR::Type_Type::fromJSON(JSONLoader & json) { return new Type_Type(json); }
bool IR::Type_Type::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Type::operator==(IR::Node const & a) const { return a == *this; }
#line 149 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_Boolean::width_bits() const { return 1; }
#line 1195 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 150 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Boolean::toString() const { return "bool"_cs; }
#line 1198 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 151 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Boolean::dbprint(std::ostream & out) const { out << "bool"; }
#line 1201 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Boolean::operator==(IR::Type_Boolean const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_Boolean::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_Boolean::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_Boolean::Type_Boolean(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_Boolean::fromJSON(JSONLoader & json) { return new Type_Boolean(json); }
bool IR::Type_Boolean::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_Boolean::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Boolean::operator==(IR::Node const & a) const { return a == *this; }
#line 169 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_State::toString() const { return "state"_cs; }
#line 1221 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 170 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_State::dbprint(std::ostream & out) const { out << "state"; }
#line 1224 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_State::operator==(IR::Type_State const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_State::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_State::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_State::Type_State(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_State::fromJSON(JSONLoader & json) { return new Type_State(json); }
bool IR::Type_State::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_State::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_State::operator==(IR::Node const & a) const { return a == *this; }


#line 193 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_Bits::width_bits() const { return size; }
#line 1246 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 195 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Bits::toString() const { return absl::StrCat(baseName(), "<", size, ">"); }
#line 1249 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 196 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Bits::dbprint(std::ostream & out) const { out << toString(); }
#line 1252 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Bits::operator==(IR::Type_Bits const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a))
        && size == a.size
        && expression == a.expression
        && isSigned == a.isSigned;
    }
bool IR::Type_Bits::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        auto &a = static_cast<const Type_Bits &>(a_);
        return size == a.size
        && (expression ? a.expression ? expression->equiv(*a.expression) : false : a.expression == nullptr)
        && isSigned == a.isSigned;
    }
void IR::Type_Bits::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_Base::visit_children(v, n);
    v.visit(expression, "expression");
}
void IR::Type_Bits::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_Base::visit_children(v, n);
    v.visit(expression, "expression");
}
void IR::Type_Bits::dump_fields(std::ostream & out) const {
    Type_Base::dump_fields(out);
        out << " size=" << size;
        out << " isSigned=" << isSigned;
}
void IR::Type_Bits::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
    json.emit("size", size);
    if (expression != nullptr) json.emit("expression", expression);
    json.emit("isSigned", isSigned);
}
IR::Type_Bits::Type_Bits(JSONLoader & json) : Type_Base(json) {
    json.load("size", size);
    json.load("expression", expression);
    json.load("isSigned", isSigned);
}
IR::Node * IR::Type_Bits::fromJSON(JSONLoader & json) { return new Type_Bits(json); }
bool IR::Type_Bits::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_Bits::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Bits::operator==(IR::Node const & a) const { return a == *this; }


#line 217 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Varbits::toString() const { return absl::StrCat("varbit<", size, ">"); }
#line 1301 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 218 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Varbits::dbprint(std::ostream & out) const { out << "varbit<" << size << ">"; }
#line 1304 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 219 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_Varbits::width_bits() const { return size; }
#line 1307 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Varbits::operator==(IR::Type_Varbits const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a))
        && size == a.size
        && expression == a.expression;
    }
bool IR::Type_Varbits::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        auto &a = static_cast<const Type_Varbits &>(a_);
        return size == a.size
        && (expression ? a.expression ? expression->equiv(*a.expression) : false : a.expression == nullptr);
    }
void IR::Type_Varbits::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_Base::visit_children(v, n);
    v.visit(expression, "expression");
}
void IR::Type_Varbits::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_Base::visit_children(v, n);
    v.visit(expression, "expression");
}
void IR::Type_Varbits::dump_fields(std::ostream & out) const {
    Type_Base::dump_fields(out);
        out << " size=" << size;
}
void IR::Type_Varbits::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
    json.emit("size", size);
    if (expression != nullptr) json.emit("expression", expression);
}
IR::Type_Varbits::Type_Varbits(JSONLoader & json) : Type_Base(json) {
    json.load("size", size);
    json.load("expression", expression);
}
IR::Node * IR::Type_Varbits::fromJSON(JSONLoader & json) { return new Type_Varbits(json); }
bool IR::Type_Varbits::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_Varbits::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Varbits::operator==(IR::Node const & a) const { return a == *this; }
#line 227 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> const & IR::Parameter::getAnnotations() const { return annotations; }
#line 1349 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 228 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> & IR::Parameter::getAnnotations() { return annotations; }
#line 1352 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 233 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Parameter::dbprint(std::ostream & out) const { out << annotations << direction << (direction != IR::Direction::None ? " " : "")
                  << type << ' ' << name; }
#line 1356 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Parameter::operator==(IR::Parameter const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && annotations == a.annotations
        && direction == a.direction
        && type == a.type
        && defaultValue == a.defaultValue;
    }
bool IR::Parameter::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Parameter &>(a_);
        return annotations.equiv(a.annotations)
        && direction == a.direction
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && (defaultValue ? a.defaultValue ? defaultValue->equiv(*a.defaultValue) : false : a.defaultValue == nullptr);
    }
void IR::Parameter::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
    v.visit(defaultValue, "defaultValue");
}
void IR::Parameter::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
    v.visit(defaultValue, "defaultValue");
}
void IR::Parameter::validate() const {
#line 1388 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(type); }
void IR::Parameter::dump_fields(std::ostream & out) const {
    Declaration::dump_fields(out);
        out << " direction=" << direction;
}
void IR::Parameter::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("direction", direction);
    json.emit("type", type);
    if (defaultValue != nullptr) json.emit("defaultValue", defaultValue);
}
IR::Parameter::Parameter(JSONLoader & json) : Declaration(json) {
    json.load("annotations", annotations);
    json.load("direction", direction);
    json.load("type", type);
    json.load("defaultValue", defaultValue);
}
IR::Node * IR::Parameter::fromJSON(JSONLoader & json) { return new Parameter(json); }
bool IR::Parameter::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::Parameter::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::Parameter::operator==(IR::Node const & a) const { return a == *this; }
#line 239 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::ParameterList::validate() const {
#line 1415 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        parameters.validate();
#line 239 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{ parameters.check_null(); } }
#line 1420 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 242 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
Util::Enumerator<const IR::IDeclaration *> * IR::ParameterList::getDeclarations() const {
        return parameters.getDeclarations(); }
#line 1424 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 254 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::ParameterList::getDeclByName(cstring name) const { return getParameter(name); }
#line 1427 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 255 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::ParameterList::getDeclByName(std::string_view name) const { return getParameter(name); }
#line 1430 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 257 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::ParameterList::toString() const {
        return absl::StrJoin(parameters, ", ",
                             [](std::string *out, const auto *p) {
                                 absl::StrAppend(out, p);
                             });
    }
#line 1438 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ParameterList::operator==(IR::ParameterList const & a) const {
        return this->typeId() == a.typeId()
        && parameters == a.parameters;
    }
bool IR::ParameterList::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const ParameterList &>(a_);
        return parameters.equiv(a.parameters);
    }
void IR::ParameterList::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    parameters.visit_children(v, "parameters");
}
void IR::ParameterList::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    parameters.visit_children(v, "parameters");
}
void IR::ParameterList::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("parameters", parameters);
}
IR::ParameterList::ParameterList(JSONLoader & json) : Node(json) {
    json.load("parameters", parameters);
}
IR::Node * IR::ParameterList::fromJSON(JSONLoader & json) { return new ParameterList(json); }
bool IR::ParameterList::operator==(IR::Node const & a) const { return a == *this; }
#line 271 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Var::getVarName() const { return getName(); }
#line 1470 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 272 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_Var::getDeclId() const { return declid; }
#line 1473 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 273 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Var::dbprint(std::ostream & out) const { out << name << "/" << getDeclId(); }
#line 1476 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 274 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Var::toString() const { return getName().toString(); }
#line 1479 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Var::operator==(IR::Type_Var const & a) const {
        return Type_Declaration::operator==(static_cast<const Type_Declaration &>(a));
    }
bool IR::Type_Var::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Declaration::equiv(a_)) return false;
        return true;
    }
void IR::Type_Var::toJSON(JSONGenerator & json) const {
    Type_Declaration::toJSON(json);
}
IR::Type_Var::Type_Var(JSONLoader & json) : Type_Declaration(json) {
}
IR::Node * IR::Type_Var::fromJSON(JSONLoader & json) { return new Type_Var(json); }
bool IR::Type_Var::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_Var::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Var::operator==(IR::Node const & a) const { return a == *this; }
#line 296 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_InfInt::getVarName() const { return absl::StrCat("int_", declid); }
#line 1499 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 297 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_InfInt::getDeclId() const { return declid; }
#line 1502 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 298 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_InfInt::dbprint(std::ostream & out) const { out << "int/" << declid; }
#line 1505 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 299 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_InfInt::toString() const { return "int"_cs; }
#line 1508 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 300 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
bool IR::Type_InfInt::operator==(IR::Type_InfInt const & a) const { return declid == a.declid; }
#line 1511 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 303 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
bool IR::Type_InfInt::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_InfInt &>(a_);

#line 303 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{
        (void)a;  // silence unused warning
        return true; /* ignore declid */
    }    }
#line 1523 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 307 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type const * IR::Type_InfInt::getP4Type() const { return this; }
#line 1526 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 308 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_InfInt::width_bits() const { return 0; }
#line 1529 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
void IR::Type_InfInt::dump_fields(std::ostream & out) const {
    Type::dump_fields(out);
        out << " declid=" << declid;
}
void IR::Type_InfInt::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("declid", declid);
}
IR::Type_InfInt::Type_InfInt(JSONLoader & json) : Type(json) {
    json.load("declid", declid);
}
IR::Node * IR::Type_InfInt::fromJSON(JSONLoader & json) { return new Type_InfInt(json); }
bool IR::Type_InfInt::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_InfInt::operator==(IR::Node const & a) const { return a == *this; }
#line 323 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Dontcare::toString() const { return "_"_cs; }
#line 1546 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 326 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Dontcare::dbprint(std::ostream & out) const { out << "_"; }
#line 1549 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Dontcare::operator==(IR::Type_Dontcare const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_Dontcare::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_Dontcare::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_Dontcare::Type_Dontcare(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_Dontcare::fromJSON(JSONLoader & json) { return new Type_Dontcare(json); }
bool IR::Type_Dontcare::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_Dontcare::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Dontcare::operator==(IR::Node const & a) const { return a == *this; }
#line 341 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Void::toString() const { return "void"_cs; }
#line 1569 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 344 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Void::dbprint(std::ostream & out) const { out << "void"; }
#line 1572 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Void::operator==(IR::Type_Void const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_Void::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_Void::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_Void::Type_Void(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_Void::fromJSON(JSONLoader & json) { return new Type_Void(json); }
bool IR::Type_Void::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_Void::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Void::operator==(IR::Node const & a) const { return a == *this; }
#line 359 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_MatchKind::toString() const { return "match_kind"_cs; }
#line 1592 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 362 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_MatchKind::dbprint(std::ostream & out) const { out << "match_kind"; }
#line 1595 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_MatchKind::operator==(IR::Type_MatchKind const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_MatchKind::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_MatchKind::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_MatchKind::Type_MatchKind(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_MatchKind::fromJSON(JSONLoader & json) { return new Type_MatchKind(json); }
bool IR::Type_MatchKind::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_MatchKind::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_MatchKind::operator==(IR::Node const & a) const { return a == *this; }
#line 367 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
Util::Enumerator<const IR::IDeclaration *> * IR::TypeParameters::getDeclarations() const {
        return parameters.getDeclarations(); }
#line 1616 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 371 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::TypeParameters::getDeclByName(cstring name) const {
        return parameters.getDeclaration(name); }
#line 1620 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 373 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::TypeParameters::getDeclByName(std::string_view name) const {
        return parameters.getDeclaration(name); }
#line 1624 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 376 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::TypeParameters::validate() const {
#line 1627 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        parameters.validate();
#line 376 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{ parameters.check_null(); } }
#line 1632 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 377 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::TypeParameters::toString() const {
        if (parameters.size() == 0)
            return cstring::empty;
        return absl::StrCat("<",
                            absl::StrJoin(parameters, ", ",
                                          [](std::string *out, const auto *p) {
                                              absl::StrAppend(out, p);
                                          }),
                            ">");
    }
#line 1644 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TypeParameters::operator==(IR::TypeParameters const & a) const {
        return this->typeId() == a.typeId()
        && parameters == a.parameters;
    }
bool IR::TypeParameters::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TypeParameters &>(a_);
        return parameters.equiv(a.parameters);
    }
void IR::TypeParameters::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    parameters.visit_children(v, "parameters");
}
void IR::TypeParameters::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    parameters.visit_children(v, "parameters");
}
void IR::TypeParameters::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("parameters", parameters);
}
IR::TypeParameters::TypeParameters(JSONLoader & json) : Node(json) {
    json.load("parameters", parameters);
}
IR::Node * IR::TypeParameters::fromJSON(JSONLoader & json) { return new TypeParameters(json); }
bool IR::TypeParameters::operator==(IR::Node const & a) const { return a == *this; }
#line 392 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> const & IR::StructField::getAnnotations() const { return annotations; }
#line 1676 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 393 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> & IR::StructField::getAnnotations() { return annotations; }
#line 1679 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::StructField::operator==(IR::StructField const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && annotations == a.annotations
        && type == a.type;
    }
bool IR::StructField::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const StructField &>(a_);
        return annotations.equiv(a.annotations)
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr);
    }
void IR::StructField::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
}
void IR::StructField::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
}
void IR::StructField::validate() const {
#line 1705 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(type); }
void IR::StructField::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("type", type);
}
IR::StructField::StructField(JSONLoader & json) : Declaration(json) {
    json.load("annotations", annotations);
    json.load("type", type);
}
IR::Node * IR::StructField::fromJSON(JSONLoader & json) { return new StructField(json); }
bool IR::StructField::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::StructField::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::StructField::operator==(IR::Node const & a) const { return a == *this; }
#line 400 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::TypeParameters *IR::Type_StructLike::getTypeParameters() const { return typeParameters; }
#line 1724 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 401 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
std::vector<const IR::INamespace *> IR::Type_StructLike::getNestedNamespaces() const { return { typeParameters }; }
#line 1727 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 402 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> const & IR::Type_StructLike::getAnnotations() const { return annotations; }
#line 1730 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 403 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> & IR::Type_StructLike::getAnnotations() { return annotations; }
#line 1733 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 404 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
Util::Enumerator<const IR::IDeclaration *> * IR::Type_StructLike::getDeclarations() const {
        return fields.getDeclarations(); }
#line 1737 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// This function returns start offset of the given field name in bits.
/// If the given name is not a valid field name, -1 is returned.
/// The given offset may not be correct if varbit field(s) present in between.
/// Offset for all fields will be correct if:
///  - the type has only fixed width fields
///  - the type has fixed width fields with only one varbit field as a last member.
#line 433 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_StructLike::width_bits() const {
        int rv = 0;
        for (auto f : fields) {
            rv += f->type->width_bits();
        }
        return rv; }
#line 1751 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 439 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Type_StructLike::getDeclByName(cstring name) const {
        return fields.getDeclaration(name); }
#line 1755 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 441 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Type_StructLike::getDeclByName(std::string_view name) const {
        return fields.getDeclaration(name); }
#line 1759 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 443 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_StructLike::validate() const {
#line 1762 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(typeParameters);
        fields.validate();
#line 443 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{ fields.check_null(); } }
#line 1769 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(Type_StructLike, , )
bool IR::Type_StructLike::operator==(IR::Type_StructLike const & a) const {
        return Type_Declaration::operator==(static_cast<const Type_Declaration &>(a))
        && annotations == a.annotations
        && typeParameters == a.typeParameters
        && fields == a.fields;
    }
bool IR::Type_StructLike::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Type_StructLike &>(a_);
        return annotations.equiv(a.annotations)
        && (typeParameters ? a.typeParameters ? typeParameters->equiv(*a.typeParameters) : false : a.typeParameters == nullptr)
        && fields.equiv(a.fields);
    }
void IR::Type_StructLike::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(typeParameters, "typeParameters");
    fields.visit_children(v, "fields");
}
void IR::Type_StructLike::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(typeParameters, "typeParameters");
    fields.visit_children(v, "fields");
}
void IR::Type_StructLike::toJSON(JSONGenerator & json) const {
    Type_Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("typeParameters", typeParameters);
    json.emit("fields", fields);
}
IR::Type_StructLike::Type_StructLike(JSONLoader & json) : Type_Declaration(json) {
    json.load("annotations", annotations);
    json.load("typeParameters", typeParameters);
    json.load("fields", fields);
}
bool IR::Type_StructLike::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_StructLike::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_StructLike::operator==(IR::Node const & a) const { return a == *this; }
#line 450 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Struct::toString() const { return "struct "_cs + externalName(); }
#line 1815 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Struct::operator==(IR::Type_Struct const & a) const {
        return Type_StructLike::operator==(static_cast<const Type_StructLike &>(a));
    }
bool IR::Type_Struct::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_StructLike::equiv(a_)) return false;
        return true;
    }
void IR::Type_Struct::toJSON(JSONGenerator & json) const {
    Type_StructLike::toJSON(json);
}
IR::Type_Struct::Type_Struct(JSONLoader & json) : Type_StructLike(json) {
}
IR::Node * IR::Type_Struct::fromJSON(JSONLoader & json) { return new Type_Struct(json); }
bool IR::Type_Struct::operator==(IR::Type_StructLike const & a) const { return a == *this; }
bool IR::Type_Struct::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_Struct::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Struct::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Type_UnknownStruct::operator==(IR::Type_UnknownStruct const & a) const {
        return Type_StructLike::operator==(static_cast<const Type_StructLike &>(a));
    }
bool IR::Type_UnknownStruct::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_StructLike::equiv(a_)) return false;
        return true;
    }
void IR::Type_UnknownStruct::toJSON(JSONGenerator & json) const {
    Type_StructLike::toJSON(json);
}
IR::Type_UnknownStruct::Type_UnknownStruct(JSONLoader & json) : Type_StructLike(json) {
}
IR::Node * IR::Type_UnknownStruct::fromJSON(JSONLoader & json) { return new Type_UnknownStruct(json); }
bool IR::Type_UnknownStruct::operator==(IR::Type_StructLike const & a) const { return a == *this; }
bool IR::Type_UnknownStruct::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_UnknownStruct::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_UnknownStruct::operator==(IR::Node const & a) const { return a == *this; }
#line 462 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_HeaderUnion::toString() const { return "header_union "_cs + externalName(); }
#line 1854 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 464 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_HeaderUnion::width_bits() const {
        int rv = 0;
        for (auto f : fields)
            rv = std::max(rv, f->type->width_bits());
        return rv; }
#line 1862 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// start offset of any field in a union is 0
bool IR::Type_HeaderUnion::operator==(IR::Type_HeaderUnion const & a) const {
        return Type_StructLike::operator==(static_cast<const Type_StructLike &>(a));
    }
bool IR::Type_HeaderUnion::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_StructLike::equiv(a_)) return false;
        return true;
    }
void IR::Type_HeaderUnion::toJSON(JSONGenerator & json) const {
    Type_StructLike::toJSON(json);
}
IR::Type_HeaderUnion::Type_HeaderUnion(JSONLoader & json) : Type_StructLike(json) {
}
IR::Node * IR::Type_HeaderUnion::fromJSON(JSONLoader & json) { return new Type_HeaderUnion(json); }
bool IR::Type_HeaderUnion::operator==(IR::Type_StructLike const & a) const { return a == *this; }
bool IR::Type_HeaderUnion::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_HeaderUnion::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_HeaderUnion::operator==(IR::Node const & a) const { return a == *this; }
#line 484 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Header::toString() const { return "header "_cs + externalName(); }
#line 1884 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Header::operator==(IR::Type_Header const & a) const {
        return Type_StructLike::operator==(static_cast<const Type_StructLike &>(a));
    }
bool IR::Type_Header::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_StructLike::equiv(a_)) return false;
        return true;
    }
void IR::Type_Header::toJSON(JSONGenerator & json) const {
    Type_StructLike::toJSON(json);
}
IR::Type_Header::Type_Header(JSONLoader & json) : Type_StructLike(json) {
}
IR::Node * IR::Type_Header::fromJSON(JSONLoader & json) { return new Type_Header(json); }
bool IR::Type_Header::operator==(IR::Type_StructLike const & a) const { return a == *this; }
bool IR::Type_Header::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_Header::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Header::operator==(IR::Node const & a) const { return a == *this; }
#line 489 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Set::dbprint(std::ostream & out) const { Node::dbprint(out); out << "<" << elementType << ">"; }
#line 1905 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 490 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Set::toString() const { return absl::StrCat("set<", elementType, ">"); }
#line 1908 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 491 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type const * IR::Type_Set::getP4Type() const { return nullptr; }
#line 1911 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 492 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_Set::width_bits() const {
        /// returning the width of the set elements, not the set itself, which doesn't
        /// really have a sensible size
        return elementType->width_bits(); }
#line 1917 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Set::operator==(IR::Type_Set const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && elementType == a.elementType;
    }
bool IR::Type_Set::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_Set &>(a_);
        return (elementType ? a.elementType ? elementType->equiv(*a.elementType) : false : a.elementType == nullptr);
    }
void IR::Type_Set::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    v.visit(elementType, "elementType");
}
void IR::Type_Set::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    v.visit(elementType, "elementType");
}
void IR::Type_Set::validate() const {
#line 1939 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(elementType); }
void IR::Type_Set::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("elementType", elementType);
}
IR::Type_Set::Type_Set(JSONLoader & json) : Type(json) {
    json.load("elementType", elementType);
}
IR::Node * IR::Type_Set::fromJSON(JSONLoader & json) { return new Type_Set(json); }
bool IR::Type_Set::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Set::operator==(IR::Node const & a) const { return a == *this; }


#line 508 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_BaseList::validate() const {
#line 1956 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        components.validate();
#line 508 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{ components.check_null(); } }
#line 1961 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 509 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
size_t IR::Type_BaseList::getSize() const { return components.size(); }
#line 1964 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 510 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::Type *IR::Type_BaseList::at(size_t index) const { return components.at(index); }
#line 1967 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 511 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_BaseList::width_bits() const {
        /// returning sum of the width of the elements
        int rv = 0;
        for (auto f : components) {
            rv += f->width_bits();
        }
        return rv; }
#line 1976 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 518 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_BaseList::asString(char const * name) const {
        return
            absl::StrCat(name,
                         "<",
                         absl::StrJoin(components, ", ",
                                       [](std::string *out, const auto *t) {
                                           absl::StrAppend(out, t);
                                       }),
                         ">");
    }
#line 1988 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 528 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_BaseList::toString() const {
        return asString("baselist");
    }
#line 1993 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_BaseList::operator==(IR::Type_BaseList const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && components == a.components;
    }
bool IR::Type_BaseList::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_BaseList &>(a_);
        return components.equiv(a.components);
    }
void IR::Type_BaseList::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    components.visit_children(v, "components");
}
void IR::Type_BaseList::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    components.visit_children(v, "components");
}
void IR::Type_BaseList::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("components", components);
}
IR::Type_BaseList::Type_BaseList(JSONLoader & json) : Type(json) {
    json.load("components", components);
}
bool IR::Type_BaseList::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_BaseList::operator==(IR::Node const & a) const { return a == *this; }

#line 537 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_List::toString() const {
        return asString("tuple");
    }
#line 2028 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_List::operator==(IR::Type_List const & a) const {
        return Type_BaseList::operator==(static_cast<const Type_BaseList &>(a));
    }
bool IR::Type_List::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_BaseList::equiv(a_)) return false;
        return true;
    }
void IR::Type_List::toJSON(JSONGenerator & json) const {
    Type_BaseList::toJSON(json);
}
IR::Type_List::Type_List(JSONLoader & json) : Type_BaseList(json) {
}
IR::Node * IR::Type_List::fromJSON(JSONLoader & json) { return new Type_List(json); }
bool IR::Type_List::operator==(IR::Type_BaseList const & a) const { return a == *this; }
bool IR::Type_List::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_List::operator==(IR::Node const & a) const { return a == *this; }
#line 545 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Tuple::toString() const {
        return asString("tuple");
    }
#line 2050 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Tuple::operator==(IR::Type_Tuple const & a) const {
        return Type_BaseList::operator==(static_cast<const Type_BaseList &>(a));
    }
bool IR::Type_Tuple::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_BaseList::equiv(a_)) return false;
        return true;
    }
void IR::Type_Tuple::toJSON(JSONGenerator & json) const {
    Type_BaseList::toJSON(json);
}
IR::Type_Tuple::Type_Tuple(JSONLoader & json) : Type_BaseList(json) {
}
IR::Node * IR::Type_Tuple::fromJSON(JSONLoader & json) { return new Type_Tuple(json); }
bool IR::Type_Tuple::operator==(IR::Type_BaseList const & a) const { return a == *this; }
bool IR::Type_Tuple::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Tuple::operator==(IR::Node const & a) const { return a == *this; }
#line 554 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_P4List::toString() const {
        return absl::StrCat("list<", elementType, ">");
    }
#line 2072 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_P4List::operator==(IR::Type_P4List const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && elementType == a.elementType;
    }
bool IR::Type_P4List::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_P4List &>(a_);
        return (elementType ? a.elementType ? elementType->equiv(*a.elementType) : false : a.elementType == nullptr);
    }
void IR::Type_P4List::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    v.visit(elementType, "elementType");
}
void IR::Type_P4List::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    v.visit(elementType, "elementType");
}
void IR::Type_P4List::validate() const {
#line 2094 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(elementType); }
void IR::Type_P4List::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("elementType", elementType);
}
IR::Type_P4List::Type_P4List(JSONLoader & json) : Type(json) {
    json.load("elementType", elementType);
}
IR::Node * IR::Type_P4List::fromJSON(JSONLoader & json) { return new Type_P4List(json); }
bool IR::Type_P4List::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_P4List::operator==(IR::Node const & a) const { return a == *this; }
#line 564 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> const & IR::Type_ArchBlock::getAnnotations() const { return annotations; }
#line 2109 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 565 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> & IR::Type_ArchBlock::getAnnotations() { return annotations; }
#line 2112 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 566 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::TypeParameters *IR::Type_ArchBlock::getTypeParameters() const { return typeParameters; }
#line 2115 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 567 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
Util::Enumerator<const IR::IDeclaration *> * IR::Type_ArchBlock::getDeclarations() const {
        return typeParameters->getDeclarations(); }
#line 2119 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 569 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Type_ArchBlock::getDeclByName(cstring name) const {
        return typeParameters->getDeclByName(name); }
#line 2123 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 571 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Type_ArchBlock::getDeclByName(std::string_view name) const {
        return typeParameters->getDeclByName(name); }
#line 2127 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_ArchBlock::operator==(IR::Type_ArchBlock const & a) const {
        return Type_Declaration::operator==(static_cast<const Type_Declaration &>(a))
        && annotations == a.annotations
        && typeParameters == a.typeParameters;
    }
bool IR::Type_ArchBlock::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Type_ArchBlock &>(a_);
        return annotations.equiv(a.annotations)
        && (typeParameters ? a.typeParameters ? typeParameters->equiv(*a.typeParameters) : false : a.typeParameters == nullptr);
    }
void IR::Type_ArchBlock::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(typeParameters, "typeParameters");
}
void IR::Type_ArchBlock::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(typeParameters, "typeParameters");
}
void IR::Type_ArchBlock::validate() const {
#line 2153 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(typeParameters); }
void IR::Type_ArchBlock::toJSON(JSONGenerator & json) const {
    Type_Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("typeParameters", typeParameters);
}
IR::Type_ArchBlock::Type_ArchBlock(JSONLoader & json) : Type_Declaration(json) {
    json.load("annotations", annotations);
    json.load("typeParameters", typeParameters);
}
bool IR::Type_ArchBlock::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_ArchBlock::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_ArchBlock::operator==(IR::Node const & a) const { return a == *this; }
#line 576 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::Type *IR::Type_Package::getType() const { return this; }
#line 2171 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 579 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::ParameterList *IR::Type_Package::getConstructorParameters() const { return constructorParams; }
#line 2174 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 580 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Package::toString() const { return "package "_cs + externalName(); }
#line 2177 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 581 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
Util::Enumerator<const IR::IDeclaration *> * IR::Type_Package::getDeclarations() const {
        return typeParameters->getDeclarations()->concat(constructorParams->getDeclarations()); }
#line 2181 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 583 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Type_Package::getDeclByName(cstring name) const {
        auto decl = constructorParams->getDeclByName(name);
        if (!decl) decl = typeParameters->getDeclByName(name);
        return decl; }
#line 2187 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 587 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Type_Package::getDeclByName(std::string_view name) const {
        auto decl = constructorParams->getDeclByName(name);
        if (!decl) decl = typeParameters->getDeclByName(name);
        return decl; }
#line 2193 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Package::operator==(IR::Type_Package const & a) const {
        return Type_ArchBlock::operator==(static_cast<const Type_ArchBlock &>(a))
        && constructorParams == a.constructorParams;
    }
bool IR::Type_Package::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_ArchBlock::equiv(a_)) return false;
        auto &a = static_cast<const Type_Package &>(a_);
        return (constructorParams ? a.constructorParams ? constructorParams->equiv(*a.constructorParams) : false : a.constructorParams == nullptr);
    }
void IR::Type_Package::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_ArchBlock::visit_children(v, n);
    v.visit(constructorParams, "constructorParams");
}
void IR::Type_Package::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_ArchBlock::visit_children(v, n);
    v.visit(constructorParams, "constructorParams");
}
void IR::Type_Package::validate() const {
#line 2215 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(constructorParams); }
void IR::Type_Package::toJSON(JSONGenerator & json) const {
    Type_ArchBlock::toJSON(json);
    json.emit("constructorParams", constructorParams);
}
IR::Type_Package::Type_Package(JSONLoader & json) : Type_ArchBlock(json) {
    json.load("constructorParams", constructorParams);
}
IR::Node * IR::Type_Package::fromJSON(JSONLoader & json) { return new Type_Package(json); }
bool IR::Type_Package::operator==(IR::Type_ArchBlock const & a) const { return a == *this; }
bool IR::Type_Package::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_Package::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Package::operator==(IR::Node const & a) const { return a == *this; }
#line 596 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::ParameterList *IR::Type_Parser::getApplyParameters() const { return applyParams; }
#line 2232 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 597 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Parser::toString() const { return "parser "_cs + externalName(); }
#line 2235 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Parser::operator==(IR::Type_Parser const & a) const {
        return Type_ArchBlock::operator==(static_cast<const Type_ArchBlock &>(a))
        && applyParams == a.applyParams;
    }
bool IR::Type_Parser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_ArchBlock::equiv(a_)) return false;
        auto &a = static_cast<const Type_Parser &>(a_);
        return (applyParams ? a.applyParams ? applyParams->equiv(*a.applyParams) : false : a.applyParams == nullptr);
    }
void IR::Type_Parser::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_ArchBlock::visit_children(v, n);
    v.visit(applyParams, "applyParams");
}
void IR::Type_Parser::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_ArchBlock::visit_children(v, n);
    v.visit(applyParams, "applyParams");
}
void IR::Type_Parser::validate() const {
#line 2257 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(applyParams); }
void IR::Type_Parser::toJSON(JSONGenerator & json) const {
    Type_ArchBlock::toJSON(json);
    json.emit("applyParams", applyParams);
}
IR::Type_Parser::Type_Parser(JSONLoader & json) : Type_ArchBlock(json) {
    json.load("applyParams", applyParams);
}
IR::Node * IR::Type_Parser::fromJSON(JSONLoader & json) { return new Type_Parser(json); }
bool IR::Type_Parser::operator==(IR::Type_ArchBlock const & a) const { return a == *this; }
bool IR::Type_Parser::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_Parser::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Parser::operator==(IR::Node const & a) const { return a == *this; }
#line 603 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::ParameterList *IR::Type_Control::getApplyParameters() const { return applyParams; }
#line 2274 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 604 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Control::toString() const { return "control "_cs + externalName(); }
#line 2277 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Control::operator==(IR::Type_Control const & a) const {
        return Type_ArchBlock::operator==(static_cast<const Type_ArchBlock &>(a))
        && applyParams == a.applyParams;
    }
bool IR::Type_Control::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_ArchBlock::equiv(a_)) return false;
        auto &a = static_cast<const Type_Control &>(a_);
        return (applyParams ? a.applyParams ? applyParams->equiv(*a.applyParams) : false : a.applyParams == nullptr);
    }
void IR::Type_Control::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_ArchBlock::visit_children(v, n);
    v.visit(applyParams, "applyParams");
}
void IR::Type_Control::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_ArchBlock::visit_children(v, n);
    v.visit(applyParams, "applyParams");
}
void IR::Type_Control::validate() const {
#line 2299 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(applyParams); }
void IR::Type_Control::toJSON(JSONGenerator & json) const {
    Type_ArchBlock::toJSON(json);
    json.emit("applyParams", applyParams);
}
IR::Type_Control::Type_Control(JSONLoader & json) : Type_ArchBlock(json) {
    json.load("applyParams", applyParams);
}
IR::Node * IR::Type_Control::fromJSON(JSONLoader & json) { return new Type_Control(json); }
bool IR::Type_Control::operator==(IR::Type_ArchBlock const & a) const { return a == *this; }
bool IR::Type_Control::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_Control::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Control::operator==(IR::Node const & a) const { return a == *this; }
#line 610 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type_Name::Type_Name(IR::ID id) : Type(id.srcInfo), path(new IR::Path(id)) {}
#line 2316 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 611 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Name::toString() const { return path->name; }
#line 2319 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 612 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Name::dbprint(std::ostream & out) const { out << path->toString(); }
#line 2322 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 613 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type const * IR::Type_Name::getP4Type() const { return this; }
#line 2325 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 614 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_Name::width_bits() const {
        BUG("Type_Name is not a canonical type, use getTypeType()?");
        return 0;
    }
#line 2331 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Name::operator==(IR::Type_Name const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && path == a.path;
    }
bool IR::Type_Name::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_Name &>(a_);
        return (path ? a.path ? path->equiv(*a.path) : false : a.path == nullptr);
    }
void IR::Type_Name::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    v.visit(path, "path");
}
void IR::Type_Name::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    v.visit(path, "path");
}
void IR::Type_Name::validate() const {
#line 2353 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(path); }
void IR::Type_Name::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("path", path);
}
IR::Type_Name::Type_Name(JSONLoader & json) : Type(json) {
    json.load("path", path);
}
IR::Node * IR::Type_Name::fromJSON(JSONLoader & json) { return new Type_Name(json); }
bool IR::Type_Name::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Name::operator==(IR::Node const & a) const { return a == *this; }
#line 623 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Array::toString() const {
        return
            absl::StrCat(elementType,
                         "[",
                         sizeKnown() ? size->toString() : "?"_cs,
                         "]"); }
#line 2373 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 629 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Array::dbprint(std::ostream & out) const { out << elementType << "[" << size << "]"; }
#line 2376 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 639 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type const * IR::Type_Array::getP4Type() const { return new IR::Type_Array(srcInfo, elementType->getP4Type(), size); }
#line 2379 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 641 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_Array::width_bits() const { return getSize() * elementType->width_bits(); }
#line 2382 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Array::operator==(IR::Type_Array const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && elementType == a.elementType
        && size == a.size;
    }
bool IR::Type_Array::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_Array &>(a_);
        return (elementType ? a.elementType ? elementType->equiv(*a.elementType) : false : a.elementType == nullptr)
        && (size ? a.size ? size->equiv(*a.size) : false : a.size == nullptr);
    }
void IR::Type_Array::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    v.visit(elementType, "elementType");
    v.visit(size, "size");
}
void IR::Type_Array::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    v.visit(elementType, "elementType");
    v.visit(size, "size");
}
void IR::Type_Array::validate() const {
#line 2408 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(elementType);
        CHECK_NULL(size); }
void IR::Type_Array::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("elementType", elementType);
    json.emit("size", size);
}
IR::Type_Array::Type_Array(JSONLoader & json) : Type(json) {
    json.load("elementType", elementType);
    json.load("size", size);
}
IR::Node * IR::Type_Array::fromJSON(JSONLoader & json) { return new Type_Array(json); }
bool IR::Type_Array::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Array::operator==(IR::Node const & a) const { return a == *this; }
#line 652 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Specialized::validate() const {
#line 2426 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(baseType);
        CHECK_NULL(arguments);
#line 652 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{ arguments->check_null(); } }
#line 2432 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 654 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Specialized::toString() const {
        return
            absl::StrCat(baseType,
                         "<",
                         absl::StrJoin(*arguments, ", ",
                                       [](std::string *out, const auto *t) {
                                           absl::StrAppend(out, t);
                                       }),
                         ">");
    }
#line 2444 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 664 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type_Specialized::Type_Specialized(cstring bt, std::initializer_list<const IR::Type *> args) : baseType(new Type_Name(bt)), arguments(new Vector<Type>(args)) {}
#line 2447 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Specialized::operator==(IR::Type_Specialized const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && baseType == a.baseType
        && arguments == a.arguments;
    }
bool IR::Type_Specialized::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_Specialized &>(a_);
        return (baseType ? a.baseType ? baseType->equiv(*a.baseType) : false : a.baseType == nullptr)
        && (arguments ? a.arguments ? arguments->equiv(*a.arguments) : false : a.arguments == nullptr);
    }
void IR::Type_Specialized::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    v.visit(baseType, "baseType");
    v.visit(arguments, "arguments");
}
void IR::Type_Specialized::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    v.visit(baseType, "baseType");
    v.visit(arguments, "arguments");
}
void IR::Type_Specialized::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("baseType", baseType);
    json.emit("arguments", arguments);
}
IR::Type_Specialized::Type_Specialized(JSONLoader & json) : Type(json) {
    json.load("baseType", baseType);
    json.load("arguments", arguments);
}
IR::Node * IR::Type_Specialized::fromJSON(JSONLoader & json) { return new Type_Specialized(json); }
bool IR::Type_Specialized::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Specialized::operator==(IR::Node const & a) const { return a == *this; }
/// canonical baseType; always IMayBeGenericType
/// canonical type arguments
/// 'substituted' is baseType with all type
/// variables substituted with the arguments.

#line 677 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_SpecializedCanonical::validate() const {
#line 2491 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(baseType);
        CHECK_NULL(arguments);
        CHECK_NULL(substituted);
#line 677 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{
        arguments->check_null();
        BUG_CHECK(baseType->is<IMayBeGenericType>(), "base type %1% is not generic", baseType);
        BUG_CHECK(substituted->is<IMayBeGenericType>(), "substituted %1% is not generic", substituted);
    } }
#line 2502 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 683 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_SpecializedCanonical::toString() const { return baseType->toString() + "<...>"_cs; }
#line 2505 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_SpecializedCanonical::operator==(IR::Type_SpecializedCanonical const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && baseType == a.baseType
        && arguments == a.arguments
        && substituted == a.substituted;
    }
bool IR::Type_SpecializedCanonical::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_SpecializedCanonical &>(a_);
        return (baseType ? a.baseType ? baseType->equiv(*a.baseType) : false : a.baseType == nullptr)
        && (arguments ? a.arguments ? arguments->equiv(*a.arguments) : false : a.arguments == nullptr)
        && (substituted ? a.substituted ? substituted->equiv(*a.substituted) : false : a.substituted == nullptr);
    }
void IR::Type_SpecializedCanonical::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    v.visit(baseType, "baseType");
    v.visit(arguments, "arguments");
    v.visit(substituted, "substituted");
}
void IR::Type_SpecializedCanonical::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    v.visit(baseType, "baseType");
    v.visit(arguments, "arguments");
    v.visit(substituted, "substituted");
}
void IR::Type_SpecializedCanonical::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("baseType", baseType);
    json.emit("arguments", arguments);
    json.emit("substituted", substituted);
}
IR::Type_SpecializedCanonical::Type_SpecializedCanonical(JSONLoader & json) : Type(json) {
    json.load("baseType", baseType);
    json.load("arguments", arguments);
    json.load("substituted", substituted);
}
IR::Node * IR::Type_SpecializedCanonical::fromJSON(JSONLoader & json) { return new Type_SpecializedCanonical(json); }
bool IR::Type_SpecializedCanonical::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_SpecializedCanonical::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Declaration_ID::operator==(IR::Declaration_ID const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a));
    }
bool IR::Declaration_ID::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        return true;
    }
void IR::Declaration_ID::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
}
IR::Declaration_ID::Declaration_ID(JSONLoader & json) : Declaration(json) {
}
IR::Node * IR::Declaration_ID::fromJSON(JSONLoader & json) { return new Declaration_ID(json); }
bool IR::Declaration_ID::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::Declaration_ID::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::Declaration_ID::operator==(IR::Node const & a) const { return a == *this; }
#line 707 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_String::toString() const { return "string"_cs; }
#line 2567 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_String::operator==(IR::Type_String const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_String::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_String::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_String::Type_String(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_String::fromJSON(JSONLoader & json) { return new Type_String(json); }
bool IR::Type_String::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_String::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_String::operator==(IR::Node const & a) const { return a == *this; }
#line 713 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> const & IR::Type_Enum::getAnnotations() const { return annotations; }
#line 2587 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 714 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> & IR::Type_Enum::getAnnotations() { return annotations; }
#line 2590 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 715 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
Util::Enumerator<const IR::IDeclaration *> * IR::Type_Enum::getDeclarations() const {
        return members.getDeclarations(); }
#line 2594 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 717 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Type_Enum::getDeclByName(cstring name) const {
        return members.getDeclaration(name); }
#line 2598 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 719 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Type_Enum::getDeclByName(std::string_view name) const {
        return members.getDeclaration(name); }
#line 2602 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 722 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Enum::validate() const {
#line 2605 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        members.validate();
#line 722 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{ members.check_null(); } }
#line 2611 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Enum::operator==(IR::Type_Enum const & a) const {
        return Type_Declaration::operator==(static_cast<const Type_Declaration &>(a))
        && annotations == a.annotations
        && members == a.members;
    }
bool IR::Type_Enum::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Type_Enum &>(a_);
        return annotations.equiv(a.annotations)
        && members.equiv(a.members);
    }
void IR::Type_Enum::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    members.visit_children(v, "members");
}
void IR::Type_Enum::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    members.visit_children(v, "members");
}
void IR::Type_Enum::toJSON(JSONGenerator & json) const {
    Type_Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("members", members);
}
IR::Type_Enum::Type_Enum(JSONLoader & json) : Type_Declaration(json) {
    json.load("annotations", annotations);
    json.load("members", members);
}
IR::Node * IR::Type_Enum::fromJSON(JSONLoader & json) { return new Type_Enum(json); }
bool IR::Type_Enum::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_Enum::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Enum::operator==(IR::Node const & a) const { return a == *this; }
#line 728 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::SerEnumMember::validate() const {
#line 2651 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(value);
#line 728 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{ CHECK_NULL(value); } }
#line 2656 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::SerEnumMember::operator==(IR::SerEnumMember const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && value == a.value;
    }
bool IR::SerEnumMember::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const SerEnumMember &>(a_);
        return (value ? a.value ? value->equiv(*a.value) : false : a.value == nullptr);
    }
void IR::SerEnumMember::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    v.visit(value, "value");
}
void IR::SerEnumMember::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    v.visit(value, "value");
}
void IR::SerEnumMember::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("value", value);
}
IR::SerEnumMember::SerEnumMember(JSONLoader & json) : Declaration(json) {
    json.load("value", value);
}
IR::Node * IR::SerEnumMember::fromJSON(JSONLoader & json) { return new SerEnumMember(json); }
bool IR::SerEnumMember::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::SerEnumMember::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::SerEnumMember::operator==(IR::Node const & a) const { return a == *this; }
#line 737 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> const & IR::Type_SerEnum::getAnnotations() const { return annotations; }
#line 2690 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 738 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> & IR::Type_SerEnum::getAnnotations() { return annotations; }
#line 2693 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 739 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
Util::Enumerator<const IR::IDeclaration *> * IR::Type_SerEnum::getDeclarations() const {
        return members.getDeclarations(); }
#line 2697 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 741 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Type_SerEnum::getDeclByName(cstring name) const {
        return members.getDeclaration(name); }
#line 2701 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 743 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Type_SerEnum::getDeclByName(std::string_view name) const {
        return members.getDeclaration(name); }
#line 2705 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 746 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_SerEnum::validate() const {
#line 2708 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(type);
        members.validate();
#line 746 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{ members.check_null(); } }
#line 2715 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 747 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_SerEnum::width_bits() const { return type->width_bits(); }
#line 2718 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_SerEnum::operator==(IR::Type_SerEnum const & a) const {
        return Type_Declaration::operator==(static_cast<const Type_Declaration &>(a))
        && annotations == a.annotations
        && type == a.type
        && members == a.members;
    }
bool IR::Type_SerEnum::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Type_SerEnum &>(a_);
        return annotations.equiv(a.annotations)
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && members.equiv(a.members);
    }
void IR::Type_SerEnum::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
    members.visit_children(v, "members");
}
void IR::Type_SerEnum::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
    members.visit_children(v, "members");
}
void IR::Type_SerEnum::toJSON(JSONGenerator & json) const {
    Type_Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("type", type);
    json.emit("members", members);
}
IR::Type_SerEnum::Type_SerEnum(JSONLoader & json) : Type_Declaration(json) {
    json.load("annotations", annotations);
    json.load("type", type);
    json.load("members", members);
}
IR::Node * IR::Type_SerEnum::fromJSON(JSONLoader & json) { return new Type_SerEnum(json); }
bool IR::Type_SerEnum::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_SerEnum::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_SerEnum::operator==(IR::Node const & a) const { return a == *this; }
#line 753 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::ParameterList *IR::Type_Table::getApplyParameters() const { return new ParameterList(); }
#line 2764 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// names for the fields of the struct returned
/// by applying a table
#line 759 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type const * IR::Type_Table::getP4Type() const { return nullptr; }
#line 2769 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 760 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Table::dbprint(std::ostream & out) const { out << table->name; }
#line 2772 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Table::operator==(IR::Type_Table const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && table == a.table;
    }
bool IR::Type_Table::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_Table &>(a_);
        return (table ? a.table ? table->equiv(*a.table) : false : a.table == nullptr);
    }
void IR::Type_Table::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    v.visit(table, "table");
}
void IR::Type_Table::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    v.visit(table, "table");
}
void IR::Type_Table::validate() const {
#line 2794 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(table); }
void IR::Type_Table::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("table", table);
}
IR::Type_Table::Type_Table(JSONLoader & json) : Type(json) {
    json.load("table", table);
}
IR::Node * IR::Type_Table::fromJSON(JSONLoader & json) { return new Type_Table(json); }
bool IR::Type_Table::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Table::operator==(IR::Node const & a) const { return a == *this; }
#line 769 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type const * IR::Type_ActionEnum::getP4Type() const { return nullptr; }
#line 2809 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_ActionEnum::operator==(IR::Type_ActionEnum const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && actionList == a.actionList;
    }
bool IR::Type_ActionEnum::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_ActionEnum &>(a_);
        return (actionList ? a.actionList ? actionList->equiv(*a.actionList) : false : a.actionList == nullptr);
    }
void IR::Type_ActionEnum::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    v.visit(actionList, "actionList");
}
void IR::Type_ActionEnum::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    v.visit(actionList, "actionList");
}
void IR::Type_ActionEnum::validate() const {
#line 2831 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(actionList); }
void IR::Type_ActionEnum::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("actionList", actionList);
}
IR::Type_ActionEnum::Type_ActionEnum(JSONLoader & json) : Type(json) {
    json.load("actionList", actionList);
}
IR::Node * IR::Type_ActionEnum::fromJSON(JSONLoader & json) { return new Type_ActionEnum(json); }
bool IR::Type_ActionEnum::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_ActionEnum::operator==(IR::Node const & a) const { return a == *this; }


#line 786 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::TypeParameters *IR::Type_MethodBase::getTypeParameters() const { return typeParameters; }
#line 2848 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 788 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_MethodBase::toString() const { return "<Method>"_cs; }
#line 2851 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 789 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type const * IR::Type_MethodBase::getP4Type() const { return nullptr; }
#line 2854 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 790 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
Util::Enumerator<const IR::IDeclaration *> * IR::Type_MethodBase::getDeclarations() const {
        return typeParameters->getDeclarations()->concat(parameters->getDeclarations()); }
#line 2858 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 792 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Type_MethodBase::getDeclByName(cstring name) const {
        auto decl = parameters->getDeclByName(name);
        if (!decl) decl = typeParameters->getDeclByName(name);
        return decl; }
#line 2864 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 796 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Type_MethodBase::getDeclByName(std::string_view name) const {
        auto decl = parameters->getDeclByName(name);
        if (!decl) decl = typeParameters->getDeclByName(name);
        return decl; }
#line 2870 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_MethodBase::operator==(IR::Type_MethodBase const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && typeParameters == a.typeParameters
        && returnType == a.returnType
        && parameters == a.parameters;
    }
bool IR::Type_MethodBase::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_MethodBase &>(a_);
        return (typeParameters ? a.typeParameters ? typeParameters->equiv(*a.typeParameters) : false : a.typeParameters == nullptr)
        && (returnType ? a.returnType ? returnType->equiv(*a.returnType) : false : a.returnType == nullptr)
        && (parameters ? a.parameters ? parameters->equiv(*a.parameters) : false : a.parameters == nullptr);
    }
void IR::Type_MethodBase::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    v.visit(typeParameters, "typeParameters");
    v.visit(returnType, "returnType");
    v.visit(parameters, "parameters");
}
void IR::Type_MethodBase::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    v.visit(typeParameters, "typeParameters");
    v.visit(returnType, "returnType");
    v.visit(parameters, "parameters");
}
void IR::Type_MethodBase::validate() const {
#line 2900 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(typeParameters);
        CHECK_NULL(parameters); }
void IR::Type_MethodBase::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("typeParameters", typeParameters);
    if (returnType != nullptr) json.emit("returnType", returnType);
    json.emit("parameters", parameters);
}
IR::Type_MethodBase::Type_MethodBase(JSONLoader & json) : Type(json) {
    json.load("typeParameters", typeParameters);
    json.load("returnType", returnType);
    json.load("parameters", parameters);
}
bool IR::Type_MethodBase::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_MethodBase::operator==(IR::Node const & a) const { return a == *this; }

#line 807 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_Method::toString() const { return name; }
#line 2920 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Method::operator==(IR::Type_Method const & a) const {
        return Type_MethodBase::operator==(static_cast<const Type_MethodBase &>(a))
        && name == a.name;
    }
bool IR::Type_Method::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_MethodBase::equiv(a_)) return false;
        auto &a = static_cast<const Type_Method &>(a_);
        return name == a.name;
    }
void IR::Type_Method::dump_fields(std::ostream & out) const {
    Type_MethodBase::dump_fields(out);
        out << " name=" << name;
}
void IR::Type_Method::toJSON(JSONGenerator & json) const {
    Type_MethodBase::toJSON(json);
    json.emit("name", name);
}
IR::Type_Method::Type_Method(JSONLoader & json) : Type_MethodBase(json) {
    json.load("name", name);
}
IR::Node * IR::Type_Method::fromJSON(JSONLoader & json) { return new Type_Method(json); }
bool IR::Type_Method::operator==(IR::Type_MethodBase const & a) const { return a == *this; }
bool IR::Type_Method::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Method::operator==(IR::Node const & a) const { return a == *this; }
#line 817 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::ArgumentInfo::toString() const { return argument->toString(); }
#line 2948 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ArgumentInfo::operator==(IR::ArgumentInfo const & a) const {
        return this->typeId() == a.typeId()
        && leftValue == a.leftValue
        && compileTimeConstant == a.compileTimeConstant
        && type == a.type
        && argument == a.argument;
    }
bool IR::ArgumentInfo::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const ArgumentInfo &>(a_);
        return leftValue == a.leftValue
        && compileTimeConstant == a.compileTimeConstant
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && (argument ? a.argument ? argument->equiv(*a.argument) : false : a.argument == nullptr);
    }
void IR::ArgumentInfo::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(type, "type");
    v.visit(argument, "argument");
}
void IR::ArgumentInfo::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(type, "type");
    v.visit(argument, "argument");
}
void IR::ArgumentInfo::validate() const {
#line 2978 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(type);
        CHECK_NULL(argument); }
void IR::ArgumentInfo::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " leftValue=" << leftValue;
        out << " compileTimeConstant=" << compileTimeConstant;
}
void IR::ArgumentInfo::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("leftValue", leftValue);
    json.emit("compileTimeConstant", compileTimeConstant);
    json.emit("type", type);
    json.emit("argument", argument);
}
IR::ArgumentInfo::ArgumentInfo(JSONLoader & json) : Node(json) {
    json.load("leftValue", leftValue);
    json.load("compileTimeConstant", compileTimeConstant);
    json.load("type", type);
    json.load("argument", argument);
}
IR::Node * IR::ArgumentInfo::fromJSON(JSONLoader & json) { return new ArgumentInfo(json); }
bool IR::ArgumentInfo::operator==(IR::Node const & a) const { return a == *this; }
#line 828 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_MethodCall::validate() const {
#line 3004 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(typeArguments);
        CHECK_NULL(returnType);
        CHECK_NULL(arguments);
#line 828 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{ typeArguments->check_null(); arguments->check_null(); } }
#line 3011 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 829 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Type const * IR::Type_MethodCall::getP4Type() const { return nullptr; }
#line 3014 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 830 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
cstring IR::Type_MethodCall::toString() const { return "<Method call>"_cs; }
#line 3017 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_MethodCall::operator==(IR::Type_MethodCall const & a) const {
        return Type::operator==(static_cast<const Type &>(a))
        && typeArguments == a.typeArguments
        && returnType == a.returnType
        && arguments == a.arguments;
    }
bool IR::Type_MethodCall::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type::equiv(a_)) return false;
        auto &a = static_cast<const Type_MethodCall &>(a_);
        return (typeArguments ? a.typeArguments ? typeArguments->equiv(*a.typeArguments) : false : a.typeArguments == nullptr)
        && (returnType ? a.returnType ? returnType->equiv(*a.returnType) : false : a.returnType == nullptr)
        && (arguments ? a.arguments ? arguments->equiv(*a.arguments) : false : a.arguments == nullptr);
    }
void IR::Type_MethodCall::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type::visit_children(v, n);
    v.visit(typeArguments, "typeArguments");
    v.visit(returnType, "returnType");
    v.visit(arguments, "arguments");
}
void IR::Type_MethodCall::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type::visit_children(v, n);
    v.visit(typeArguments, "typeArguments");
    v.visit(returnType, "returnType");
    v.visit(arguments, "arguments");
}
void IR::Type_MethodCall::toJSON(JSONGenerator & json) const {
    Type::toJSON(json);
    json.emit("typeArguments", typeArguments);
    json.emit("returnType", returnType);
    json.emit("arguments", arguments);
}
IR::Type_MethodCall::Type_MethodCall(JSONLoader & json) : Type(json) {
    json.load("typeArguments", typeArguments);
    json.load("returnType", returnType);
    json.load("arguments", arguments);
}
IR::Node * IR::Type_MethodCall::fromJSON(JSONLoader & json) { return new Type_MethodCall(json); }
bool IR::Type_MethodCall::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_MethodCall::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Type_Action::operator==(IR::Type_Action const & a) const {
        return Type_MethodBase::operator==(static_cast<const Type_MethodBase &>(a));
    }
bool IR::Type_Action::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_MethodBase::equiv(a_)) return false;
        return true;
    }
void IR::Type_Action::toJSON(JSONGenerator & json) const {
    Type_MethodBase::toJSON(json);
}
IR::Type_Action::Type_Action(JSONLoader & json) : Type_MethodBase(json) {
}
IR::Node * IR::Type_Action::fromJSON(JSONLoader & json) { return new Type_Action(json); }
bool IR::Type_Action::operator==(IR::Type_MethodBase const & a) const { return a == *this; }
bool IR::Type_Action::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Action::operator==(IR::Node const & a) const { return a == *this; }
#line 847 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> const & IR::Method::getAnnotations() const { return annotations; }
#line 3079 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 848 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> & IR::Method::getAnnotations() { return annotations; }
#line 3082 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 849 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::ParameterList *IR::Method::getParameters() const { return type->parameters; }
#line 3085 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 851 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Method::getDeclByName(cstring name) const {
        return type->parameters->getDeclByName(name); }
#line 3090 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 853 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::IDeclaration *IR::Method::getDeclByName(std::string_view name) const {
        return type->parameters->getDeclByName(name); }
#line 3094 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 855 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
Util::Enumerator<const IR::IDeclaration *> * IR::Method::getDeclarations() const {
        return type->parameters->getDeclarations(); }
#line 3098 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Method::operator==(IR::Method const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && type == a.type
        && isAbstract == a.isAbstract
        && annotations == a.annotations;
    }
bool IR::Method::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Method &>(a_);
        return (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && isAbstract == a.isAbstract
        && annotations.equiv(a.annotations);
    }
void IR::Method::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    v.visit(type, "type");
    annotations.visit_children(v, "annotations");
}
void IR::Method::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    v.visit(type, "type");
    annotations.visit_children(v, "annotations");
}
void IR::Method::validate() const {
#line 3126 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(type);
        annotations.validate(); }
void IR::Method::dump_fields(std::ostream & out) const {
    Declaration::dump_fields(out);
        out << " isAbstract=" << isAbstract;
}
void IR::Method::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("type", type);
    json.emit("isAbstract", isAbstract);
    json.emit("annotations", annotations);
}
IR::Method::Method(JSONLoader & json) : Declaration(json) {
    json.load("type", type);
    json.load("isAbstract", isAbstract);
    json.load("annotations", annotations);
}
IR::Node * IR::Method::fromJSON(JSONLoader & json) { return new Method(json); }
bool IR::Method::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::Method::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::Method::operator==(IR::Node const & a) const { return a == *this; }
#line 862 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_Typedef::width_bits() const { return type->width_bits(); }
#line 3151 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 863 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> const & IR::Type_Typedef::getAnnotations() const { return annotations; }
#line 3154 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 864 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> & IR::Type_Typedef::getAnnotations() { return annotations; }
#line 3157 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Typedef::operator==(IR::Type_Typedef const & a) const {
        return Type_Declaration::operator==(static_cast<const Type_Declaration &>(a))
        && annotations == a.annotations
        && type == a.type;
    }
bool IR::Type_Typedef::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Type_Typedef &>(a_);
        return annotations.equiv(a.annotations)
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr);
    }
void IR::Type_Typedef::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
}
void IR::Type_Typedef::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
}
void IR::Type_Typedef::validate() const {
#line 3183 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(type); }
void IR::Type_Typedef::toJSON(JSONGenerator & json) const {
    Type_Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("type", type);
}
IR::Type_Typedef::Type_Typedef(JSONLoader & json) : Type_Declaration(json) {
    json.load("annotations", annotations);
    json.load("type", type);
}
IR::Node * IR::Type_Typedef::fromJSON(JSONLoader & json) { return new Type_Typedef(json); }
bool IR::Type_Typedef::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_Typedef::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Typedef::operator==(IR::Node const & a) const { return a == *this; }
#line 876 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
int IR::Type_Newtype::width_bits() const { return type->width_bits(); }
#line 3202 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 877 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> const & IR::Type_Newtype::getAnnotations() const { return annotations; }
#line 3205 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 878 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> & IR::Type_Newtype::getAnnotations() { return annotations; }
#line 3208 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Newtype::operator==(IR::Type_Newtype const & a) const {
        return Type_Declaration::operator==(static_cast<const Type_Declaration &>(a))
        && annotations == a.annotations
        && type == a.type;
    }
bool IR::Type_Newtype::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Type_Newtype &>(a_);
        return annotations.equiv(a.annotations)
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr);
    }
void IR::Type_Newtype::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
}
void IR::Type_Newtype::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
}
void IR::Type_Newtype::validate() const {
#line 3234 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(type); }
void IR::Type_Newtype::toJSON(JSONGenerator & json) const {
    Type_Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("type", type);
}
IR::Type_Newtype::Type_Newtype(JSONLoader & json) : Type_Declaration(json) {
    json.load("annotations", annotations);
    json.load("type", type);
}
IR::Node * IR::Type_Newtype::fromJSON(JSONLoader & json) { return new Type_Newtype(json); }
bool IR::Type_Newtype::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_Newtype::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Newtype::operator==(IR::Node const & a) const { return a == *this; }


#line 890 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
std::vector<const IR::INamespace *> IR::Type_Extern::getNestedNamespaces() const { return { typeParameters }; }
#line 3255 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 891 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
Util::Enumerator<const IR::IDeclaration *> * IR::Type_Extern::getDeclarations() const {
        return attributes.valueEnumerator()->as<const IDeclaration*>()
            ->concat(methods.getEnumerator()->as<const IDeclaration*>()); }
#line 3260 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 894 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::TypeParameters *IR::Type_Extern::getTypeParameters() const { return typeParameters; }
#line 3263 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 895 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
void IR::Type_Extern::validate() const {
#line 3266 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(typeParameters);
        methods.validate();
        attributes.validate();
        annotations.validate();
#line 895 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
{ methods.check_null(); } }
#line 3274 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 896 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> const & IR::Type_Extern::getAnnotations() const { return annotations; }
#line 3277 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 897 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
IR::Vector<IR::Annotation> & IR::Type_Extern::getAnnotations() { return annotations; }
#line 3280 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// Returns the method that matches the specified arguments.
/// Returns nullptr if no method or more than one method match.
/// In the latter case it also reports an error.
/// Returns the constructor that matches the specified arguments.
/// Returns nullptr if no constructor or more than one constructor matches.
/// In the latter case it also reports an error.
#line 905 "/builddir/build/BUILD/p4c-1.2.5.8/ir/type.def"
const IR::Method *IR::Type_Extern::lookupConstructor(const IR::Vector<IR::Argument>* arguments) const { return lookupMethod(name, arguments); }
#line 3289 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Extern::operator==(IR::Type_Extern const & a) const {
        return Type_Declaration::operator==(static_cast<const Type_Declaration &>(a))
        && typeParameters == a.typeParameters
        && methods == a.methods
        && attributes == a.attributes
        && annotations == a.annotations;
    }
bool IR::Type_Extern::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Type_Extern &>(a_);
        return (typeParameters ? a.typeParameters ? typeParameters->equiv(*a.typeParameters) : false : a.typeParameters == nullptr)
        && methods.equiv(a.methods)
        && attributes.equiv(a.attributes)
        && annotations.equiv(a.annotations);
    }
void IR::Type_Extern::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_Declaration::visit_children(v, n);
    v.visit(typeParameters, "typeParameters");
    methods.visit_children(v, "methods");
    attributes.visit_children(v, "attributes");
    annotations.visit_children(v, "annotations");
}
void IR::Type_Extern::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_Declaration::visit_children(v, n);
    v.visit(typeParameters, "typeParameters");
    methods.visit_children(v, "methods");
    attributes.visit_children(v, "attributes");
    annotations.visit_children(v, "annotations");
}
void IR::Type_Extern::toJSON(JSONGenerator & json) const {
    Type_Declaration::toJSON(json);
    json.emit("typeParameters", typeParameters);
    json.emit("methods", methods);
    json.emit("attributes", attributes);
    json.emit("annotations", annotations);
}
IR::Type_Extern::Type_Extern(JSONLoader & json) : Type_Declaration(json) {
    json.load("typeParameters", typeParameters);
    json.load("methods", methods);
    json.load("attributes", attributes);
    json.load("annotations", annotations);
}
IR::Node * IR::Type_Extern::fromJSON(JSONLoader & json) { return new Type_Extern(json); }
bool IR::Type_Extern::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_Extern::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Extern::operator==(IR::Node const & a) const { return a == *this; }
#line 15 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::Operation_Unary::toString() const { return getStringOp() + expr->toString(); }
#line 3341 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Operation_Unary::operator==(IR::Operation_Unary const & a) const {
        return Operation::operator==(static_cast<const Operation &>(a))
        && expr == a.expr;
    }
bool IR::Operation_Unary::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation::equiv(a_)) return false;
        auto &a = static_cast<const Operation_Unary &>(a_);
        return (expr ? a.expr ? expr->equiv(*a.expr) : false : a.expr == nullptr);
    }
void IR::Operation_Unary::visit_children(Visitor & v, char const * n) {
    (void)n;
    Operation::visit_children(v, n);
    v.visit(expr, "expr");
}
void IR::Operation_Unary::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Operation::visit_children(v, n);
    v.visit(expr, "expr");
}
void IR::Operation_Unary::validate() const {
#line 3363 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(expr); }
void IR::Operation_Unary::toJSON(JSONGenerator & json) const {
    Operation::toJSON(json);
    json.emit("expr", expr);
}
IR::Operation_Unary::Operation_Unary(JSONLoader & json) : Operation(json) {
    json.load("expr", expr);
}
bool IR::Operation_Unary::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Operation_Unary::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Operation_Unary::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Neg::operator==(IR::Neg const & a) const {
        return Operation_Unary::operator==(static_cast<const Operation_Unary &>(a));
    }
bool IR::Neg::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Unary::equiv(a_)) return false;
        return true;
    }
void IR::Neg::toJSON(JSONGenerator & json) const {
    Operation_Unary::toJSON(json);
}
IR::Neg::Neg(JSONLoader & json) : Operation_Unary(json) {
}
IR::Node * IR::Neg::fromJSON(JSONLoader & json) { return new Neg(json); }
bool IR::Neg::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::Neg::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Neg::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Neg::operator==(IR::Node const & a) const { return a == *this; }
bool IR::UPlus::operator==(IR::UPlus const & a) const {
        return Operation_Unary::operator==(static_cast<const Operation_Unary &>(a));
    }
bool IR::UPlus::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Unary::equiv(a_)) return false;
        return true;
    }
void IR::UPlus::toJSON(JSONGenerator & json) const {
    Operation_Unary::toJSON(json);
}
IR::UPlus::UPlus(JSONLoader & json) : Operation_Unary(json) {
}
IR::Node * IR::UPlus::fromJSON(JSONLoader & json) { return new UPlus(json); }
bool IR::UPlus::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::UPlus::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::UPlus::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::UPlus::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Cmpl::operator==(IR::Cmpl const & a) const {
        return Operation_Unary::operator==(static_cast<const Operation_Unary &>(a));
    }
bool IR::Cmpl::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Unary::equiv(a_)) return false;
        return true;
    }
void IR::Cmpl::toJSON(JSONGenerator & json) const {
    Operation_Unary::toJSON(json);
}
IR::Cmpl::Cmpl(JSONLoader & json) : Operation_Unary(json) {
}
IR::Node * IR::Cmpl::fromJSON(JSONLoader & json) { return new Cmpl(json); }
bool IR::Cmpl::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::Cmpl::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Cmpl::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Cmpl::operator==(IR::Node const & a) const { return a == *this; }
bool IR::LNot::operator==(IR::LNot const & a) const {
        return Operation_Unary::operator==(static_cast<const Operation_Unary &>(a));
    }
bool IR::LNot::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Unary::equiv(a_)) return false;
        return true;
    }
void IR::LNot::toJSON(JSONGenerator & json) const {
    Operation_Unary::toJSON(json);
}
IR::LNot::LNot(JSONLoader & json) : Operation_Unary(json) {
}
IR::Node * IR::LNot::fromJSON(JSONLoader & json) { return new LNot(json); }
bool IR::LNot::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::LNot::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::LNot::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::LNot::operator==(IR::Node const & a) const { return a == *this; }
#line 47 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::Operation_Binary::toString() const {
        // FIXME: Do not use debug printing to render user-side strings
        std::stringstream tmp;
        tmp << DBPrint::Prec_Low;
        dbprint(tmp);
        return tmp.str();
    }
#line 3456 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Operation_Binary::operator==(IR::Operation_Binary const & a) const {
        return Operation::operator==(static_cast<const Operation &>(a))
        && left == a.left
        && right == a.right;
    }
bool IR::Operation_Binary::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation::equiv(a_)) return false;
        auto &a = static_cast<const Operation_Binary &>(a_);
        return (left ? a.left ? left->equiv(*a.left) : false : a.left == nullptr)
        && (right ? a.right ? right->equiv(*a.right) : false : a.right == nullptr);
    }
void IR::Operation_Binary::visit_children(Visitor & v, char const * n) {
    (void)n;
    Operation::visit_children(v, n);
    v.visit(left, "left");
    v.visit(right, "right");
}
void IR::Operation_Binary::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Operation::visit_children(v, n);
    v.visit(left, "left");
    v.visit(right, "right");
}
void IR::Operation_Binary::validate() const {
#line 3482 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(left);
        CHECK_NULL(right); }
void IR::Operation_Binary::toJSON(JSONGenerator & json) const {
    Operation::toJSON(json);
    json.emit("left", left);
    json.emit("right", right);
}
IR::Operation_Binary::Operation_Binary(JSONLoader & json) : Operation(json) {
    json.load("left", left);
    json.load("right", right);
}
bool IR::Operation_Binary::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Operation_Binary::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Operation_Binary::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Operation_Ternary::operator==(IR::Operation_Ternary const & a) const {
        return Operation::operator==(static_cast<const Operation &>(a))
        && e0 == a.e0
        && e1 == a.e1
        && e2 == a.e2;
    }
bool IR::Operation_Ternary::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation::equiv(a_)) return false;
        auto &a = static_cast<const Operation_Ternary &>(a_);
        return (e0 ? a.e0 ? e0->equiv(*a.e0) : false : a.e0 == nullptr)
        && (e1 ? a.e1 ? e1->equiv(*a.e1) : false : a.e1 == nullptr)
        && (e2 ? a.e2 ? e2->equiv(*a.e2) : false : a.e2 == nullptr);
    }
void IR::Operation_Ternary::visit_children(Visitor & v, char const * n) {
    (void)n;
    Operation::visit_children(v, n);
    v.visit(e0, "e0");
    v.visit(e1, "e1");
    v.visit(e2, "e2");
}
void IR::Operation_Ternary::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Operation::visit_children(v, n);
    v.visit(e0, "e0");
    v.visit(e1, "e1");
    v.visit(e2, "e2");
}
void IR::Operation_Ternary::validate() const {
#line 3527 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(e0);
        CHECK_NULL(e1);
        CHECK_NULL(e2); }
void IR::Operation_Ternary::toJSON(JSONGenerator & json) const {
    Operation::toJSON(json);
    json.emit("e0", e0);
    json.emit("e1", e1);
    json.emit("e2", e2);
}
IR::Operation_Ternary::Operation_Ternary(JSONLoader & json) : Operation(json) {
    json.load("e0", e0);
    json.load("e1", e1);
    json.load("e2", e2);
}
bool IR::Operation_Ternary::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Operation_Ternary::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Operation_Ternary::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Operation_Relation::operator==(IR::Operation_Relation const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::Operation_Relation::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::Operation_Relation::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::Operation_Relation::Operation_Relation(JSONLoader & json) : Operation_Binary(json) {
}
bool IR::Operation_Relation::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Operation_Relation::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Operation_Relation::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Operation_Relation::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Mul::operator==(IR::Mul const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::Mul::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::Mul::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::Mul::Mul(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::Mul::fromJSON(JSONLoader & json) { return new Mul(json); }
bool IR::Mul::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Mul::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Mul::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Mul::operator==(IR::Node const & a) const { return a == *this; }
IRNODE_DEFINE_APPLY_OVERLOAD(Statement, , )
bool IR::Statement::operator==(IR::Statement const & a) const {
        return StatOrDecl::operator==(static_cast<const StatOrDecl &>(a));
    }
bool IR::Statement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!StatOrDecl::equiv(a_)) return false;
        return true;
    }
void IR::Statement::toJSON(JSONGenerator & json) const {
    StatOrDecl::toJSON(json);
}
IR::Statement::Statement(JSONLoader & json) : StatOrDecl(json) {
}
bool IR::Statement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::Statement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BaseAssignmentStatement::operator==(IR::BaseAssignmentStatement const & a) const {
        return Statement::operator==(static_cast<const Statement &>(a))
        && left == a.left
        && right == a.right;
    }
bool IR::BaseAssignmentStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Statement::equiv(a_)) return false;
        auto &a = static_cast<const BaseAssignmentStatement &>(a_);
        return (left ? a.left ? left->equiv(*a.left) : false : a.left == nullptr)
        && (right ? a.right ? right->equiv(*a.right) : false : a.right == nullptr);
    }
void IR::BaseAssignmentStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    Statement::visit_children(v, n);
    v.visit(left, "left");
    v.visit(right, "right");
}
void IR::BaseAssignmentStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Statement::visit_children(v, n);
    v.visit(left, "left");
    v.visit(right, "right");
}
void IR::BaseAssignmentStatement::validate() const {
#line 3622 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(left);
        CHECK_NULL(right); }
void IR::BaseAssignmentStatement::toJSON(JSONGenerator & json) const {
    Statement::toJSON(json);
    json.emit("left", left);
    json.emit("right", right);
}
IR::BaseAssignmentStatement::BaseAssignmentStatement(JSONLoader & json) : Statement(json) {
    json.load("left", left);
    json.load("right", right);
}
bool IR::BaseAssignmentStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::BaseAssignmentStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::BaseAssignmentStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 482 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::OpAssignmentStatement::toString() const { return absl::StrCat(left, " ", getStringOp(), "= ", right); }
#line 3640 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::OpAssignmentStatement::operator==(IR::OpAssignmentStatement const & a) const {
        return BaseAssignmentStatement::operator==(static_cast<const BaseAssignmentStatement &>(a));
    }
bool IR::OpAssignmentStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!BaseAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::OpAssignmentStatement::toJSON(JSONGenerator & json) const {
    BaseAssignmentStatement::toJSON(json);
}
IR::OpAssignmentStatement::OpAssignmentStatement(JSONLoader & json) : BaseAssignmentStatement(json) {
}
bool IR::OpAssignmentStatement::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::OpAssignmentStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::OpAssignmentStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::OpAssignmentStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::MulAssign::operator==(IR::MulAssign const & a) const {
        return OpAssignmentStatement::operator==(static_cast<const OpAssignmentStatement &>(a));
    }
bool IR::MulAssign::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!OpAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::MulAssign::toJSON(JSONGenerator & json) const {
    OpAssignmentStatement::toJSON(json);
}
IR::MulAssign::MulAssign(JSONLoader & json) : OpAssignmentStatement(json) {
}
IR::Node * IR::MulAssign::fromJSON(JSONLoader & json) { return new MulAssign(json); }
bool IR::MulAssign::operator==(IR::OpAssignmentStatement const & a) const { return a == *this; }
bool IR::MulAssign::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::MulAssign::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::MulAssign::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::MulAssign::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Div::operator==(IR::Div const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::Div::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::Div::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::Div::Div(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::Div::fromJSON(JSONLoader & json) { return new Div(json); }
bool IR::Div::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Div::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Div::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Div::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DivAssign::operator==(IR::DivAssign const & a) const {
        return OpAssignmentStatement::operator==(static_cast<const OpAssignmentStatement &>(a));
    }
bool IR::DivAssign::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!OpAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::DivAssign::toJSON(JSONGenerator & json) const {
    OpAssignmentStatement::toJSON(json);
}
IR::DivAssign::DivAssign(JSONLoader & json) : OpAssignmentStatement(json) {
}
IR::Node * IR::DivAssign::fromJSON(JSONLoader & json) { return new DivAssign(json); }
bool IR::DivAssign::operator==(IR::OpAssignmentStatement const & a) const { return a == *this; }
bool IR::DivAssign::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::DivAssign::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::DivAssign::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::DivAssign::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Mod::operator==(IR::Mod const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::Mod::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::Mod::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::Mod::Mod(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::Mod::fromJSON(JSONLoader & json) { return new Mod(json); }
bool IR::Mod::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Mod::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Mod::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Mod::operator==(IR::Node const & a) const { return a == *this; }
bool IR::ModAssign::operator==(IR::ModAssign const & a) const {
        return OpAssignmentStatement::operator==(static_cast<const OpAssignmentStatement &>(a));
    }
bool IR::ModAssign::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!OpAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::ModAssign::toJSON(JSONGenerator & json) const {
    OpAssignmentStatement::toJSON(json);
}
IR::ModAssign::ModAssign(JSONLoader & json) : OpAssignmentStatement(json) {
}
IR::Node * IR::ModAssign::fromJSON(JSONLoader & json) { return new ModAssign(json); }
bool IR::ModAssign::operator==(IR::OpAssignmentStatement const & a) const { return a == *this; }
bool IR::ModAssign::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::ModAssign::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::ModAssign::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::ModAssign::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Add::operator==(IR::Add const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::Add::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::Add::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::Add::Add(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::Add::fromJSON(JSONLoader & json) { return new Add(json); }
bool IR::Add::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Add::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Add::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Add::operator==(IR::Node const & a) const { return a == *this; }
bool IR::AddAssign::operator==(IR::AddAssign const & a) const {
        return OpAssignmentStatement::operator==(static_cast<const OpAssignmentStatement &>(a));
    }
bool IR::AddAssign::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!OpAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::AddAssign::toJSON(JSONGenerator & json) const {
    OpAssignmentStatement::toJSON(json);
}
IR::AddAssign::AddAssign(JSONLoader & json) : OpAssignmentStatement(json) {
}
IR::Node * IR::AddAssign::fromJSON(JSONLoader & json) { return new AddAssign(json); }
bool IR::AddAssign::operator==(IR::OpAssignmentStatement const & a) const { return a == *this; }
bool IR::AddAssign::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::AddAssign::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::AddAssign::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::AddAssign::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Sub::operator==(IR::Sub const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::Sub::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::Sub::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::Sub::Sub(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::Sub::fromJSON(JSONLoader & json) { return new Sub(json); }
bool IR::Sub::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Sub::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Sub::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Sub::operator==(IR::Node const & a) const { return a == *this; }
bool IR::SubAssign::operator==(IR::SubAssign const & a) const {
        return OpAssignmentStatement::operator==(static_cast<const OpAssignmentStatement &>(a));
    }
bool IR::SubAssign::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!OpAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::SubAssign::toJSON(JSONGenerator & json) const {
    OpAssignmentStatement::toJSON(json);
}
IR::SubAssign::SubAssign(JSONLoader & json) : OpAssignmentStatement(json) {
}
IR::Node * IR::SubAssign::fromJSON(JSONLoader & json) { return new SubAssign(json); }
bool IR::SubAssign::operator==(IR::OpAssignmentStatement const & a) const { return a == *this; }
bool IR::SubAssign::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::SubAssign::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::SubAssign::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::SubAssign::operator==(IR::Node const & a) const { return a == *this; }
bool IR::AddSat::operator==(IR::AddSat const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::AddSat::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::AddSat::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::AddSat::AddSat(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::AddSat::fromJSON(JSONLoader & json) { return new AddSat(json); }
bool IR::AddSat::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::AddSat::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::AddSat::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::AddSat::operator==(IR::Node const & a) const { return a == *this; }
bool IR::AddSatAssign::operator==(IR::AddSatAssign const & a) const {
        return OpAssignmentStatement::operator==(static_cast<const OpAssignmentStatement &>(a));
    }
bool IR::AddSatAssign::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!OpAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::AddSatAssign::toJSON(JSONGenerator & json) const {
    OpAssignmentStatement::toJSON(json);
}
IR::AddSatAssign::AddSatAssign(JSONLoader & json) : OpAssignmentStatement(json) {
}
IR::Node * IR::AddSatAssign::fromJSON(JSONLoader & json) { return new AddSatAssign(json); }
bool IR::AddSatAssign::operator==(IR::OpAssignmentStatement const & a) const { return a == *this; }
bool IR::AddSatAssign::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::AddSatAssign::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::AddSatAssign::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::AddSatAssign::operator==(IR::Node const & a) const { return a == *this; }
bool IR::SubSat::operator==(IR::SubSat const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::SubSat::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::SubSat::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::SubSat::SubSat(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::SubSat::fromJSON(JSONLoader & json) { return new SubSat(json); }
bool IR::SubSat::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::SubSat::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::SubSat::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::SubSat::operator==(IR::Node const & a) const { return a == *this; }
bool IR::SubSatAssign::operator==(IR::SubSatAssign const & a) const {
        return OpAssignmentStatement::operator==(static_cast<const OpAssignmentStatement &>(a));
    }
bool IR::SubSatAssign::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!OpAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::SubSatAssign::toJSON(JSONGenerator & json) const {
    OpAssignmentStatement::toJSON(json);
}
IR::SubSatAssign::SubSatAssign(JSONLoader & json) : OpAssignmentStatement(json) {
}
IR::Node * IR::SubSatAssign::fromJSON(JSONLoader & json) { return new SubSatAssign(json); }
bool IR::SubSatAssign::operator==(IR::OpAssignmentStatement const & a) const { return a == *this; }
bool IR::SubSatAssign::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::SubSatAssign::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::SubSatAssign::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::SubSatAssign::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Shl::operator==(IR::Shl const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::Shl::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::Shl::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::Shl::Shl(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::Shl::fromJSON(JSONLoader & json) { return new Shl(json); }
bool IR::Shl::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Shl::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Shl::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Shl::operator==(IR::Node const & a) const { return a == *this; }
bool IR::ShlAssign::operator==(IR::ShlAssign const & a) const {
        return OpAssignmentStatement::operator==(static_cast<const OpAssignmentStatement &>(a));
    }
bool IR::ShlAssign::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!OpAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::ShlAssign::toJSON(JSONGenerator & json) const {
    OpAssignmentStatement::toJSON(json);
}
IR::ShlAssign::ShlAssign(JSONLoader & json) : OpAssignmentStatement(json) {
}
IR::Node * IR::ShlAssign::fromJSON(JSONLoader & json) { return new ShlAssign(json); }
bool IR::ShlAssign::operator==(IR::OpAssignmentStatement const & a) const { return a == *this; }
bool IR::ShlAssign::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::ShlAssign::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::ShlAssign::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::ShlAssign::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Shr::operator==(IR::Shr const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::Shr::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::Shr::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::Shr::Shr(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::Shr::fromJSON(JSONLoader & json) { return new Shr(json); }
bool IR::Shr::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Shr::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Shr::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Shr::operator==(IR::Node const & a) const { return a == *this; }
bool IR::ShrAssign::operator==(IR::ShrAssign const & a) const {
        return OpAssignmentStatement::operator==(static_cast<const OpAssignmentStatement &>(a));
    }
bool IR::ShrAssign::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!OpAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::ShrAssign::toJSON(JSONGenerator & json) const {
    OpAssignmentStatement::toJSON(json);
}
IR::ShrAssign::ShrAssign(JSONLoader & json) : OpAssignmentStatement(json) {
}
IR::Node * IR::ShrAssign::fromJSON(JSONLoader & json) { return new ShrAssign(json); }
bool IR::ShrAssign::operator==(IR::OpAssignmentStatement const & a) const { return a == *this; }
bool IR::ShrAssign::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::ShrAssign::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::ShrAssign::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::ShrAssign::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Equ::operator==(IR::Equ const & a) const {
        return Operation_Relation::operator==(static_cast<const Operation_Relation &>(a));
    }
bool IR::Equ::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Relation::equiv(a_)) return false;
        return true;
    }
void IR::Equ::toJSON(JSONGenerator & json) const {
    Operation_Relation::toJSON(json);
}
IR::Equ::Equ(JSONLoader & json) : Operation_Relation(json) {
}
IR::Node * IR::Equ::fromJSON(JSONLoader & json) { return new Equ(json); }
bool IR::Equ::operator==(IR::Operation_Relation const & a) const { return a == *this; }
bool IR::Equ::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Equ::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Equ::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Equ::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Neq::operator==(IR::Neq const & a) const {
        return Operation_Relation::operator==(static_cast<const Operation_Relation &>(a));
    }
bool IR::Neq::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Relation::equiv(a_)) return false;
        return true;
    }
void IR::Neq::toJSON(JSONGenerator & json) const {
    Operation_Relation::toJSON(json);
}
IR::Neq::Neq(JSONLoader & json) : Operation_Relation(json) {
}
IR::Node * IR::Neq::fromJSON(JSONLoader & json) { return new Neq(json); }
bool IR::Neq::operator==(IR::Operation_Relation const & a) const { return a == *this; }
bool IR::Neq::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Neq::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Neq::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Neq::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Lss::operator==(IR::Lss const & a) const {
        return Operation_Relation::operator==(static_cast<const Operation_Relation &>(a));
    }
bool IR::Lss::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Relation::equiv(a_)) return false;
        return true;
    }
void IR::Lss::toJSON(JSONGenerator & json) const {
    Operation_Relation::toJSON(json);
}
IR::Lss::Lss(JSONLoader & json) : Operation_Relation(json) {
}
IR::Node * IR::Lss::fromJSON(JSONLoader & json) { return new Lss(json); }
bool IR::Lss::operator==(IR::Operation_Relation const & a) const { return a == *this; }
bool IR::Lss::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Lss::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Lss::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Lss::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Leq::operator==(IR::Leq const & a) const {
        return Operation_Relation::operator==(static_cast<const Operation_Relation &>(a));
    }
bool IR::Leq::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Relation::equiv(a_)) return false;
        return true;
    }
void IR::Leq::toJSON(JSONGenerator & json) const {
    Operation_Relation::toJSON(json);
}
IR::Leq::Leq(JSONLoader & json) : Operation_Relation(json) {
}
IR::Node * IR::Leq::fromJSON(JSONLoader & json) { return new Leq(json); }
bool IR::Leq::operator==(IR::Operation_Relation const & a) const { return a == *this; }
bool IR::Leq::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Leq::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Leq::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Leq::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Grt::operator==(IR::Grt const & a) const {
        return Operation_Relation::operator==(static_cast<const Operation_Relation &>(a));
    }
bool IR::Grt::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Relation::equiv(a_)) return false;
        return true;
    }
void IR::Grt::toJSON(JSONGenerator & json) const {
    Operation_Relation::toJSON(json);
}
IR::Grt::Grt(JSONLoader & json) : Operation_Relation(json) {
}
IR::Node * IR::Grt::fromJSON(JSONLoader & json) { return new Grt(json); }
bool IR::Grt::operator==(IR::Operation_Relation const & a) const { return a == *this; }
bool IR::Grt::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Grt::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Grt::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Grt::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Geq::operator==(IR::Geq const & a) const {
        return Operation_Relation::operator==(static_cast<const Operation_Relation &>(a));
    }
bool IR::Geq::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Relation::equiv(a_)) return false;
        return true;
    }
void IR::Geq::toJSON(JSONGenerator & json) const {
    Operation_Relation::toJSON(json);
}
IR::Geq::Geq(JSONLoader & json) : Operation_Relation(json) {
}
IR::Node * IR::Geq::fromJSON(JSONLoader & json) { return new Geq(json); }
bool IR::Geq::operator==(IR::Operation_Relation const & a) const { return a == *this; }
bool IR::Geq::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Geq::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Geq::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Geq::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BAnd::operator==(IR::BAnd const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::BAnd::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::BAnd::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::BAnd::BAnd(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::BAnd::fromJSON(JSONLoader & json) { return new BAnd(json); }
bool IR::BAnd::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::BAnd::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::BAnd::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BAnd::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BAndAssign::operator==(IR::BAndAssign const & a) const {
        return OpAssignmentStatement::operator==(static_cast<const OpAssignmentStatement &>(a));
    }
bool IR::BAndAssign::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!OpAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::BAndAssign::toJSON(JSONGenerator & json) const {
    OpAssignmentStatement::toJSON(json);
}
IR::BAndAssign::BAndAssign(JSONLoader & json) : OpAssignmentStatement(json) {
}
IR::Node * IR::BAndAssign::fromJSON(JSONLoader & json) { return new BAndAssign(json); }
bool IR::BAndAssign::operator==(IR::OpAssignmentStatement const & a) const { return a == *this; }
bool IR::BAndAssign::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::BAndAssign::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::BAndAssign::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::BAndAssign::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BOr::operator==(IR::BOr const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::BOr::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::BOr::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::BOr::BOr(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::BOr::fromJSON(JSONLoader & json) { return new BOr(json); }
bool IR::BOr::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::BOr::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::BOr::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BOr::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BOrAssign::operator==(IR::BOrAssign const & a) const {
        return OpAssignmentStatement::operator==(static_cast<const OpAssignmentStatement &>(a));
    }
bool IR::BOrAssign::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!OpAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::BOrAssign::toJSON(JSONGenerator & json) const {
    OpAssignmentStatement::toJSON(json);
}
IR::BOrAssign::BOrAssign(JSONLoader & json) : OpAssignmentStatement(json) {
}
IR::Node * IR::BOrAssign::fromJSON(JSONLoader & json) { return new BOrAssign(json); }
bool IR::BOrAssign::operator==(IR::OpAssignmentStatement const & a) const { return a == *this; }
bool IR::BOrAssign::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::BOrAssign::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::BOrAssign::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::BOrAssign::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BXor::operator==(IR::BXor const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::BXor::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::BXor::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::BXor::BXor(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::BXor::fromJSON(JSONLoader & json) { return new BXor(json); }
bool IR::BXor::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::BXor::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::BXor::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BXor::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BXorAssign::operator==(IR::BXorAssign const & a) const {
        return OpAssignmentStatement::operator==(static_cast<const OpAssignmentStatement &>(a));
    }
bool IR::BXorAssign::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!OpAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::BXorAssign::toJSON(JSONGenerator & json) const {
    OpAssignmentStatement::toJSON(json);
}
IR::BXorAssign::BXorAssign(JSONLoader & json) : OpAssignmentStatement(json) {
}
IR::Node * IR::BXorAssign::fromJSON(JSONLoader & json) { return new BXorAssign(json); }
bool IR::BXorAssign::operator==(IR::OpAssignmentStatement const & a) const { return a == *this; }
bool IR::BXorAssign::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::BXorAssign::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::BXorAssign::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::BXorAssign::operator==(IR::Node const & a) const { return a == *this; }
bool IR::LAnd::operator==(IR::LAnd const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::LAnd::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::LAnd::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::LAnd::LAnd(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::LAnd::fromJSON(JSONLoader & json) { return new LAnd(json); }
bool IR::LAnd::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::LAnd::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::LAnd::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::LAnd::operator==(IR::Node const & a) const { return a == *this; }
bool IR::LOr::operator==(IR::LOr const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::LOr::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::LOr::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::LOr::LOr(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::LOr::fromJSON(JSONLoader & json) { return new LOr(json); }
bool IR::LOr::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::LOr::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::LOr::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::LOr::operator==(IR::Node const & a) const { return a == *this; }
#line 253 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::Dots::dbprint(std::ostream & out) const { out << "..."; }
#line 4236 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 254 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::Dots::toString() const { return "..."_cs; }
#line 4239 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Dots::operator==(IR::Dots const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a));
    }
bool IR::Dots::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        return true;
    }
void IR::Dots::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
}
IR::Dots::Dots(JSONLoader & json) : Expression(json) {
}
IR::Node * IR::Dots::fromJSON(JSONLoader & json) { return new Dots(json); }
bool IR::Dots::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Dots::operator==(IR::Node const & a) const { return a == *this; }
#line 263 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::NamedDots::dbprint(std::ostream & out) const { out << "..."; }
#line 4258 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 264 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::NamedDots::toString() const { return "..."_cs; }
#line 4261 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::NamedDots::operator==(IR::NamedDots const & a) const {
        return NamedExpression::operator==(static_cast<const NamedExpression &>(a));
    }
bool IR::NamedDots::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!NamedExpression::equiv(a_)) return false;
        return true;
    }
void IR::NamedDots::toJSON(JSONGenerator & json) const {
    NamedExpression::toJSON(json);
}
IR::NamedDots::NamedDots(JSONLoader & json) : NamedExpression(json) {
}
IR::Node * IR::NamedDots::fromJSON(JSONLoader & json) { return new NamedDots(json); }
bool IR::NamedDots::operator==(IR::NamedExpression const & a) const { return a == *this; }
bool IR::NamedDots::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::NamedDots::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::NamedDots::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Literal::operator==(IR::Literal const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a));
    }
bool IR::Literal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        return true;
    }
void IR::Literal::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
}
IR::Literal::Literal(JSONLoader & json) : Expression(json) {
}
bool IR::Literal::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Literal::operator==(IR::Node const & a) const { return a == *this; }
/// base used when reading/writing
/// if noWarning is true, no warning is emitted



/// @return a constant. Any constant returned here is interned. Base is always 10.



#line 354 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::Constant::toString() const {
        unsigned width;
        bool sign;
        if (const IR::Type_Bits* tb = type->to<IR::Type_Bits>()) {
            width = tb->size;
            sign = tb->isSigned;
        } else {
            width = 0;
            sign = false;
        }
        return Util::toString(value, width, sign, base);
    }
#line 4317 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 366 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::Constant::visit_children(Visitor & v, char const * n) { (void)n; v.visit(type, "type"); }
#line 366 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::Constant::visit_children(Visitor & v, char const * n) const { (void)n; v.visit(type, "type"); }
#line 4322 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Constant::operator==(IR::Constant const & a) const {
        return Literal::operator==(static_cast<const Literal &>(a))
        && value == a.value
        && base == a.base;
    }
bool IR::Constant::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Literal::equiv(a_)) return false;
        auto &a = static_cast<const Constant &>(a_);
        return value == a.value
        && base == a.base;
    }
void IR::Constant::dump_fields(std::ostream & out) const {
    Literal::dump_fields(out);
        out << " value=" << value;
        out << " base=" << base;
}
void IR::Constant::toJSON(JSONGenerator & json) const {
    Literal::toJSON(json);
    json.emit("value", value);
    json.emit("base", base);
}
IR::Constant::Constant(JSONLoader & json) : Literal(json) {
    json.load("value", value);
    json.load("base", base);
}
IR::Node * IR::Constant::fromJSON(JSONLoader & json) { return new Constant(json); }
bool IR::Constant::operator==(IR::Literal const & a) const { return a == *this; }
bool IR::Constant::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Constant::operator==(IR::Node const & a) const { return a == *this; }
#line 371 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::BoolLiteral::toString() const { return value ? "true"_cs : "false"_cs; }
#line 4355 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// @return a bool literal. Both booleans are interned.
bool IR::BoolLiteral::operator==(IR::BoolLiteral const & a) const {
        return Literal::operator==(static_cast<const Literal &>(a))
        && value == a.value;
    }
bool IR::BoolLiteral::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Literal::equiv(a_)) return false;
        auto &a = static_cast<const BoolLiteral &>(a_);
        return value == a.value;
    }
void IR::BoolLiteral::dump_fields(std::ostream & out) const {
    Literal::dump_fields(out);
        out << " value=" << value;
}
void IR::BoolLiteral::toJSON(JSONGenerator & json) const {
    Literal::toJSON(json);
    json.emit("value", value);
}
IR::BoolLiteral::BoolLiteral(JSONLoader & json) : Literal(json) {
    json.load("value", value);
}
IR::Node * IR::BoolLiteral::fromJSON(JSONLoader & json) { return new BoolLiteral(json); }
bool IR::BoolLiteral::operator==(IR::Literal const & a) const { return a == *this; }
bool IR::BoolLiteral::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BoolLiteral::operator==(IR::Node const & a) const { return a == *this; }
#line 379 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::StringLiteral::validate() const {
#line 4384 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 379 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
{ if (value.isNull()) BUG("null StringLiteral"); } }
#line 4388 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 380 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::StringLiteral::toString() const { return absl::StrCat("\"", value.escapeJson(), "\""); }
#line 4391 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// @returns a string literal. The value is cached.
bool IR::StringLiteral::operator==(IR::StringLiteral const & a) const {
        return Literal::operator==(static_cast<const Literal &>(a))
        && value == a.value;
    }
bool IR::StringLiteral::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Literal::equiv(a_)) return false;
        auto &a = static_cast<const StringLiteral &>(a_);
        return value == a.value;
    }
void IR::StringLiteral::dump_fields(std::ostream & out) const {
    Literal::dump_fields(out);
        out << " value=" << value;
}
void IR::StringLiteral::toJSON(JSONGenerator & json) const {
    Literal::toJSON(json);
    json.emit("value", value);
}
IR::StringLiteral::StringLiteral(JSONLoader & json) : Literal(json) {
    json.load("value", value);
}
IR::Node * IR::StringLiteral::fromJSON(JSONLoader & json) { return new StringLiteral(json); }
bool IR::StringLiteral::operator==(IR::Literal const & a) const { return a == *this; }
bool IR::StringLiteral::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::StringLiteral::operator==(IR::Node const & a) const { return a == *this; }
#line 395 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::PathExpression::toString() const { return path->toString(); }
#line 4420 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::PathExpression::operator==(IR::PathExpression const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && path == a.path;
    }
bool IR::PathExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const PathExpression &>(a_);
        return (path ? a.path ? path->equiv(*a.path) : false : a.path == nullptr);
    }
void IR::PathExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(path, "path");
}
void IR::PathExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(path, "path");
}
void IR::PathExpression::validate() const {
#line 4442 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(path); }
void IR::PathExpression::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("path", path);
}
IR::PathExpression::PathExpression(JSONLoader & json) : Expression(json) {
    json.load("path", path);
}
IR::Node * IR::PathExpression::fromJSON(JSONLoader & json) { return new PathExpression(json); }
bool IR::PathExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::PathExpression::operator==(IR::Node const & a) const { return a == *this; }
#line 406 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::TypeNameExpression::dbprint(std::ostream & out) const { out << typeName; }
#line 4457 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 407 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::TypeNameExpression::toString() const { return typeName->toString(); }
#line 4460 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 408 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::TypeNameExpression::validate() const {
#line 4463 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(typeName);
#line 408 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
{ BUG_CHECK(typeName->is<Type_Name>() || typeName->is<Type_Specialized>(),
                         "%1% unexpected type in TypeNameExpression", typeName); } }
#line 4469 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TypeNameExpression::operator==(IR::TypeNameExpression const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && typeName == a.typeName;
    }
bool IR::TypeNameExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const TypeNameExpression &>(a_);
        return (typeName ? a.typeName ? typeName->equiv(*a.typeName) : false : a.typeName == nullptr);
    }
void IR::TypeNameExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(typeName, "typeName");
}
void IR::TypeNameExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(typeName, "typeName");
}
void IR::TypeNameExpression::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("typeName", typeName);
}
IR::TypeNameExpression::TypeNameExpression(JSONLoader & json) : Expression(json) {
    json.load("typeName", typeName);
}
IR::Node * IR::TypeNameExpression::fromJSON(JSONLoader & json) { return new TypeNameExpression(json); }
bool IR::TypeNameExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::TypeNameExpression::operator==(IR::Node const & a) const { return a == *this; }
bool IR::AbstractSlice::operator==(IR::AbstractSlice const & a) const {
        return Operation_Ternary::operator==(static_cast<const Operation_Ternary &>(a));
    }
bool IR::AbstractSlice::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Ternary::equiv(a_)) return false;
        return true;
    }
void IR::AbstractSlice::toJSON(JSONGenerator & json) const {
    Operation_Ternary::toJSON(json);
}
IR::AbstractSlice::AbstractSlice(JSONLoader & json) : Operation_Ternary(json) {
}
bool IR::AbstractSlice::operator==(IR::Operation_Ternary const & a) const { return a == *this; }
bool IR::AbstractSlice::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::AbstractSlice::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::AbstractSlice::operator==(IR::Node const & a) const { return a == *this; }
#line 420 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::Slice::toString() const { return absl::StrCat(e0, "[", e1, ":", e2, "]"); }
#line 4519 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 422 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
unsigned IR::Slice::getH() const { return e1->checkedTo<IR::Constant>()->asUnsigned(); }
#line 4523 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 423 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
unsigned IR::Slice::getL() const { return e2->checkedTo<IR::Constant>()->asUnsigned(); }
#line 4526 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

bool IR::Slice::operator==(IR::Slice const & a) const {
        return AbstractSlice::operator==(static_cast<const AbstractSlice &>(a));
    }
bool IR::Slice::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!AbstractSlice::equiv(a_)) return false;
        return true;
    }
void IR::Slice::toJSON(JSONGenerator & json) const {
    AbstractSlice::toJSON(json);
}
IR::Slice::Slice(JSONLoader & json) : AbstractSlice(json) {
}
IR::Node * IR::Slice::fromJSON(JSONLoader & json) { return new Slice(json); }
bool IR::Slice::operator==(IR::AbstractSlice const & a) const { return a == *this; }
bool IR::Slice::operator==(IR::Operation_Ternary const & a) const { return a == *this; }
bool IR::Slice::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Slice::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Slice::operator==(IR::Node const & a) const { return a == *this; }
#line 438 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::PlusSlice::toString() const { return absl::StrCat(e0, "[", e1, "+:", e2, "]"); }
#line 4549 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 439 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
unsigned IR::PlusSlice::getH() const {
        BUG_CHECK(e1->is<IR::Constant>(), "non-const PlusSlice not handled");
        return e1->to<IR::Constant>()->asUnsigned() + e2->checkedTo<IR::Constant>()->asUnsigned() - 1; }
#line 4554 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 442 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
unsigned IR::PlusSlice::getL() const {
        BUG_CHECK(e1->is<IR::Constant>(), "non-const PlusSlice not handled");
        return e1->to<IR::Constant>()->asUnsigned(); }
#line 4559 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::PlusSlice::operator==(IR::PlusSlice const & a) const {
        return AbstractSlice::operator==(static_cast<const AbstractSlice &>(a));
    }
bool IR::PlusSlice::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!AbstractSlice::equiv(a_)) return false;
        return true;
    }
void IR::PlusSlice::toJSON(JSONGenerator & json) const {
    AbstractSlice::toJSON(json);
}
IR::PlusSlice::PlusSlice(JSONLoader & json) : AbstractSlice(json) {
}
IR::Node * IR::PlusSlice::fromJSON(JSONLoader & json) { return new PlusSlice(json); }
bool IR::PlusSlice::operator==(IR::AbstractSlice const & a) const { return a == *this; }
bool IR::PlusSlice::operator==(IR::Operation_Ternary const & a) const { return a == *this; }
bool IR::PlusSlice::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::PlusSlice::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::PlusSlice::operator==(IR::Node const & a) const { return a == *this; }
#line 459 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::Member::toString() const { return absl::StrCat(expr, ".", member); }
#line 4581 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Member::operator==(IR::Member const & a) const {
        return Operation_Unary::operator==(static_cast<const Operation_Unary &>(a))
        && member == a.member;
    }
bool IR::Member::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Unary::equiv(a_)) return false;
        auto &a = static_cast<const Member &>(a_);
        return member == a.member;
    }
void IR::Member::dump_fields(std::ostream & out) const {
    Operation_Unary::dump_fields(out);
        out << " member=" << member;
}
void IR::Member::toJSON(JSONGenerator & json) const {
    Operation_Unary::toJSON(json);
    json.emit("member", member);
}
IR::Member::Member(JSONLoader & json) : Operation_Unary(json) {
    json.load("member", member);
}
IR::Node * IR::Member::fromJSON(JSONLoader & json) { return new Member(json); }
bool IR::Member::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::Member::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Member::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Member::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Concat::operator==(IR::Concat const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::Concat::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::Concat::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::Concat::Concat(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::Concat::fromJSON(JSONLoader & json) { return new Concat(json); }
bool IR::Concat::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Concat::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Concat::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Concat::operator==(IR::Node const & a) const { return a == *this; }
#line 479 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::ArrayIndex::toString() const { return absl::StrCat(left, "[", right, "]"); }
#line 4628 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ArrayIndex::operator==(IR::ArrayIndex const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::ArrayIndex::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::ArrayIndex::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::ArrayIndex::ArrayIndex(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::ArrayIndex::fromJSON(JSONLoader & json) { return new ArrayIndex(json); }
bool IR::ArrayIndex::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::ArrayIndex::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::ArrayIndex::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::ArrayIndex::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Range::operator==(IR::Range const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::Range::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::Range::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::Range::Range(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::Range::fromJSON(JSONLoader & json) { return new Range(json); }
bool IR::Range::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Range::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Range::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Range::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Mask::operator==(IR::Mask const & a) const {
        return Operation_Binary::operator==(static_cast<const Operation_Binary &>(a));
    }
bool IR::Mask::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Binary::equiv(a_)) return false;
        return true;
    }
void IR::Mask::toJSON(JSONGenerator & json) const {
    Operation_Binary::toJSON(json);
}
IR::Mask::Mask(JSONLoader & json) : Operation_Binary(json) {
}
IR::Node * IR::Mask::fromJSON(JSONLoader & json) { return new Mask(json); }
bool IR::Mask::operator==(IR::Operation_Binary const & a) const { return a == *this; }
bool IR::Mask::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Mask::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Mask::operator==(IR::Node const & a) const { return a == *this; }
#line 499 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::Mux::visit_children(Visitor & v, char const * n) {
        (void)n;
        v.visit(e0, "e0");
        SplitFlowVisit<Expression>(v, e1, e2).run_visit(); }
#line 499 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::Mux::visit_children(Visitor & v, char const * n) const {
        (void)n;
        v.visit(e0, "e0");
        SplitFlowVisit<Expression>(v, e1, e2).run_visit(); }
#line 4693 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Mux::operator==(IR::Mux const & a) const {
        return Operation_Ternary::operator==(static_cast<const Operation_Ternary &>(a));
    }
bool IR::Mux::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Ternary::equiv(a_)) return false;
        return true;
    }
void IR::Mux::toJSON(JSONGenerator & json) const {
    Operation_Ternary::toJSON(json);
}
IR::Mux::Mux(JSONLoader & json) : Operation_Ternary(json) {
}
IR::Node * IR::Mux::fromJSON(JSONLoader & json) { return new Mux(json); }
bool IR::Mux::operator==(IR::Operation_Ternary const & a) const { return a == *this; }
bool IR::Mux::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Mux::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Mux::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DefaultExpression::operator==(IR::DefaultExpression const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a));
    }
bool IR::DefaultExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        return true;
    }
void IR::DefaultExpression::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
}
IR::DefaultExpression::DefaultExpression(JSONLoader & json) : Expression(json) {
}
IR::Node * IR::DefaultExpression::fromJSON(JSONLoader & json) { return new DefaultExpression(json); }
bool IR::DefaultExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::DefaultExpression::operator==(IR::Node const & a) const { return a == *this; }
#line 512 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::This::toString() const { return "this"_cs; }
#line 4730 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::This::operator==(IR::This const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && id == a.id;
    }
bool IR::This::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const This &>(a_);
        return id == a.id;
    }
void IR::This::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " id=" << id;
}
void IR::This::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("id", id);
}
IR::This::This(JSONLoader & json) : Expression(json) {
    json.load("id", id);
}
IR::Node * IR::This::fromJSON(JSONLoader & json) { return new This(json); }
bool IR::This::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::This::operator==(IR::Node const & a) const { return a == *this; }
/// These will generally always be the same, except when a cast to a type argument of
/// a generic occurs.  Then at some point, the 'destType' will be specialized to a concrete
/// type, and 'type' will only be updated later when type inferencing occurs
#line 525 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::Cast::toString() const { return absl::StrCat("(", destType, ")", expr); }
#line 4760 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 526 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::Cast::validate() const {
#line 4763 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(destType);
#line 526 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
{ BUG_CHECK(!destType->is<Type_Unknown>(), "%1%: Cannot cast to unknown type", this); } }
#line 4768 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Cast::operator==(IR::Cast const & a) const {
        return Operation_Unary::operator==(static_cast<const Operation_Unary &>(a))
        && destType == a.destType
        && implicit == a.implicit;
    }
bool IR::Cast::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Unary::equiv(a_)) return false;
        auto &a = static_cast<const Cast &>(a_);
        return (destType ? a.destType ? destType->equiv(*a.destType) : false : a.destType == nullptr)
        && implicit == a.implicit;
    }
void IR::Cast::visit_children(Visitor & v, char const * n) {
    (void)n;
    Operation_Unary::visit_children(v, n);
    v.visit(destType, "destType");
}
void IR::Cast::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Operation_Unary::visit_children(v, n);
    v.visit(destType, "destType");
}
void IR::Cast::dump_fields(std::ostream & out) const {
    Operation_Unary::dump_fields(out);
        out << " implicit=" << implicit;
}
void IR::Cast::toJSON(JSONGenerator & json) const {
    Operation_Unary::toJSON(json);
    json.emit("destType", destType);
    json.emit("implicit", implicit);
}
IR::Cast::Cast(JSONLoader & json) : Operation_Unary(json) {
    json.load("destType", destType);
    json.load("implicit", implicit);
}
IR::Node * IR::Cast::fromJSON(JSONLoader & json) { return new Cast(json); }
bool IR::Cast::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::Cast::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Cast::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Cast::operator==(IR::Node const & a) const { return a == *this; }
#line 532 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::SelectCase::dbprint(std::ostream & out) const { out << keyset << ": " << state; }
#line 4811 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::SelectCase::operator==(IR::SelectCase const & a) const {
        return this->typeId() == a.typeId()
        && keyset == a.keyset
        && state == a.state;
    }
bool IR::SelectCase::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const SelectCase &>(a_);
        return (keyset ? a.keyset ? keyset->equiv(*a.keyset) : false : a.keyset == nullptr)
        && (state ? a.state ? state->equiv(*a.state) : false : a.state == nullptr);
    }
void IR::SelectCase::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(keyset, "keyset");
    v.visit(state, "state");
}
void IR::SelectCase::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(keyset, "keyset");
    v.visit(state, "state");
}
void IR::SelectCase::validate() const {
#line 4837 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(keyset);
        CHECK_NULL(state); }
void IR::SelectCase::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("keyset", keyset);
    json.emit("state", state);
}
IR::SelectCase::SelectCase(JSONLoader & json) : Node(json) {
    json.load("keyset", keyset);
    json.load("state", state);
}
IR::Node * IR::SelectCase::fromJSON(JSONLoader & json) { return new SelectCase(json); }
bool IR::SelectCase::operator==(IR::Node const & a) const { return a == *this; }
#line 538 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::SelectExpression::visit_children(Visitor & v, char const * n) {
        (void)n;
        v.visit(select, "select");
        SplitFlowVisitVector<SelectCase>(v, selectCases).run_visit(); }
#line 538 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::SelectExpression::visit_children(Visitor & v, char const * n) const {
        (void)n;
        v.visit(select, "select");
        SplitFlowVisitVector<SelectCase>(v, selectCases).run_visit(); }
#line 4862 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::SelectExpression::operator==(IR::SelectExpression const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && select == a.select
        && selectCases == a.selectCases;
    }
bool IR::SelectExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const SelectExpression &>(a_);
        return (select ? a.select ? select->equiv(*a.select) : false : a.select == nullptr)
        && selectCases.equiv(a.selectCases);
    }
void IR::SelectExpression::validate() const {
#line 4876 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(select);
        selectCases.validate(); }
void IR::SelectExpression::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("select", select);
    json.emit("selectCases", selectCases);
}
IR::SelectExpression::SelectExpression(JSONLoader & json) : Expression(json) {
    json.load("select", select);
    json.load("selectCases", selectCases);
}
IR::Node * IR::SelectExpression::fromJSON(JSONLoader & json) { return new SelectExpression(json); }
bool IR::SelectExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::SelectExpression::operator==(IR::Node const & a) const { return a == *this; }
#line 548 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::MethodCallExpression::toString() const {
        return
            absl::StrCat(method, "(",
                         absl::StrJoin(*arguments, ", ",
                                       [](std::string *out, const Argument *arg) {
                                           absl::StrAppend(out, arg);
                                       }),
                         ")");
    }
#line 4902 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 557 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::MethodCallExpression::validate() const {
#line 4905 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(method);
        CHECK_NULL(typeArguments);
        CHECK_NULL(arguments);
#line 557 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
{ typeArguments->check_null(); arguments->check_null(); } }
#line 4912 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 563 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
IR::MethodCallExpression::MethodCallExpression(const IR::Expression* m, std::initializer_list<IR::Expression const *> const & a) : method(m), arguments(nullptr)  {
        auto arguments = new Vector<Argument>;
        for (auto arg : a) arguments->push_back(new Argument(arg));
        this->arguments = arguments; }
#line 4918 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MethodCallExpression::operator==(IR::MethodCallExpression const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && method == a.method
        && typeArguments == a.typeArguments
        && arguments == a.arguments;
    }
bool IR::MethodCallExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const MethodCallExpression &>(a_);
        return (method ? a.method ? method->equiv(*a.method) : false : a.method == nullptr)
        && (typeArguments ? a.typeArguments ? typeArguments->equiv(*a.typeArguments) : false : a.typeArguments == nullptr)
        && (arguments ? a.arguments ? arguments->equiv(*a.arguments) : false : a.arguments == nullptr);
    }
void IR::MethodCallExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(method, "method");
    v.visit(typeArguments, "typeArguments");
    v.visit(arguments, "arguments");
}
void IR::MethodCallExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(method, "method");
    v.visit(typeArguments, "typeArguments");
    v.visit(arguments, "arguments");
}
void IR::MethodCallExpression::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("method", method);
    json.emit("typeArguments", typeArguments);
    json.emit("arguments", arguments);
}
IR::MethodCallExpression::MethodCallExpression(JSONLoader & json) : Expression(json) {
    json.load("method", method);
    json.load("typeArguments", typeArguments);
    json.load("arguments", arguments);
}
IR::Node * IR::MethodCallExpression::fromJSON(JSONLoader & json) { return new MethodCallExpression(json); }
bool IR::MethodCallExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MethodCallExpression::operator==(IR::Node const & a) const { return a == *this; }

#line 573 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::ConstructorCallExpression::toString() const { return constructedType->toString(); }
#line 4964 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 574 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::ConstructorCallExpression::validate() const {
#line 4967 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(constructedType);
        CHECK_NULL(arguments);
#line 574 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
{ BUG_CHECK(constructedType->is<Type_Name>() ||
                        constructedType->is<Type_Specialized>(),
                        "%1%: unexpected type", constructedType);
        arguments->check_null(); } }
#line 4976 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ConstructorCallExpression::operator==(IR::ConstructorCallExpression const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && constructedType == a.constructedType
        && arguments == a.arguments;
    }
bool IR::ConstructorCallExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const ConstructorCallExpression &>(a_);
        return (constructedType ? a.constructedType ? constructedType->equiv(*a.constructedType) : false : a.constructedType == nullptr)
        && (arguments ? a.arguments ? arguments->equiv(*a.arguments) : false : a.arguments == nullptr);
    }
void IR::ConstructorCallExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(constructedType, "constructedType");
    v.visit(arguments, "arguments");
}
void IR::ConstructorCallExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(constructedType, "constructedType");
    v.visit(arguments, "arguments");
}
void IR::ConstructorCallExpression::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("constructedType", constructedType);
    json.emit("arguments", arguments);
}
IR::ConstructorCallExpression::ConstructorCallExpression(JSONLoader & json) : Expression(json) {
    json.load("constructedType", constructedType);
    json.load("arguments", arguments);
}
IR::Node * IR::ConstructorCallExpression::fromJSON(JSONLoader & json) { return new ConstructorCallExpression(json); }
bool IR::ConstructorCallExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::ConstructorCallExpression::operator==(IR::Node const & a) const { return a == *this; }
#line 582 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::BaseListExpression::validate() const {
#line 5015 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        components.validate();
#line 582 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
{ components.check_null(); } }
#line 5020 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 591 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::BaseListExpression::toString() const {
        return components.empty() ? "{}" :
            absl::StrCat("{ ",
                         absl::StrJoin(components, ", ",
                                       [](std::string *out, const Expression *comp)  {
                                           absl::StrAppend(out, comp);
                                       }),
                         " }");
    }
#line 5031 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BaseListExpression::operator==(IR::BaseListExpression const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && components == a.components;
    }
bool IR::BaseListExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const BaseListExpression &>(a_);
        return components.equiv(a.components);
    }
void IR::BaseListExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    components.visit_children(v, "components");
}
void IR::BaseListExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    components.visit_children(v, "components");
}
void IR::BaseListExpression::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("components", components);
}
IR::BaseListExpression::BaseListExpression(JSONLoader & json) : Expression(json) {
    json.load("components", components);
}
IR::Node * IR::BaseListExpression::fromJSON(JSONLoader & json) { return new BaseListExpression(json); }
bool IR::BaseListExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BaseListExpression::operator==(IR::Node const & a) const { return a == *this; }
bool IR::ListExpression::operator==(IR::ListExpression const & a) const {
        return BaseListExpression::operator==(static_cast<const BaseListExpression &>(a));
    }
bool IR::ListExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!BaseListExpression::equiv(a_)) return false;
        return true;
    }
void IR::ListExpression::toJSON(JSONGenerator & json) const {
    BaseListExpression::toJSON(json);
}
IR::ListExpression::ListExpression(JSONLoader & json) : BaseListExpression(json) {
}
IR::Node * IR::ListExpression::fromJSON(JSONLoader & json) { return new ListExpression(json); }
bool IR::ListExpression::operator==(IR::BaseListExpression const & a) const { return a == *this; }
bool IR::ListExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::ListExpression::operator==(IR::Node const & a) const { return a == *this; }
bool IR::P4ListExpression::operator==(IR::P4ListExpression const & a) const {
        return BaseListExpression::operator==(static_cast<const BaseListExpression &>(a))
        && elementType == a.elementType;
    }
bool IR::P4ListExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!BaseListExpression::equiv(a_)) return false;
        auto &a = static_cast<const P4ListExpression &>(a_);
        return (elementType ? a.elementType ? elementType->equiv(*a.elementType) : false : a.elementType == nullptr);
    }
void IR::P4ListExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    BaseListExpression::visit_children(v, n);
    v.visit(elementType, "elementType");
}
void IR::P4ListExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    BaseListExpression::visit_children(v, n);
    v.visit(elementType, "elementType");
}
void IR::P4ListExpression::validate() const {
#line 5100 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(elementType); }
void IR::P4ListExpression::toJSON(JSONGenerator & json) const {
    BaseListExpression::toJSON(json);
    json.emit("elementType", elementType);
}
IR::P4ListExpression::P4ListExpression(JSONLoader & json) : BaseListExpression(json) {
    json.load("elementType", elementType);
}
IR::Node * IR::P4ListExpression::fromJSON(JSONLoader & json) { return new P4ListExpression(json); }
bool IR::P4ListExpression::operator==(IR::BaseListExpression const & a) const { return a == *this; }
bool IR::P4ListExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::P4ListExpression::operator==(IR::Node const & a) const { return a == *this; }
/// The struct or header type that is being intialized.
/// May only be known after type checking; so it can be nullptr.
#line 630 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::StructExpression::validate() const {
#line 5118 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        components.validate();
#line 630 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
{
        components.check_null(); components.validate();
        BUG_CHECK(structType == nullptr || structType->is<IR::Type_Name>() ||
                  structType->is<IR::Type_Specialized>(),
                  "%1%: unexpected struct type", this);
    } }
#line 5128 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 645 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::StructExpression::toString() const {
        return components.empty() ? "{}" :
            absl::StrCat("{ ",
                         absl::StrJoin(components, ", ",
                                       [](std::string *out, const NamedExpression *comp)  {
                                           absl::StrAppend(out, comp, " = ", comp->expression);
                                       }),
                         " }");
    }
#line 5139 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::StructExpression::operator==(IR::StructExpression const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && structType == a.structType
        && components == a.components;
    }
bool IR::StructExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const StructExpression &>(a_);
        return (structType ? a.structType ? structType->equiv(*a.structType) : false : a.structType == nullptr)
        && components.equiv(a.components);
    }
void IR::StructExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(structType, "structType");
    components.visit_children(v, "components");
}
void IR::StructExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(structType, "structType");
    components.visit_children(v, "components");
}
void IR::StructExpression::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    if (structType != nullptr) json.emit("structType", structType);
    json.emit("components", components);
}
IR::StructExpression::StructExpression(JSONLoader & json) : Expression(json) {
    json.load("structType", structType);
    json.load("components", components);
}
IR::Node * IR::StructExpression::fromJSON(JSONLoader & json) { return new StructExpression(json); }
bool IR::StructExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::StructExpression::operator==(IR::Node const & a) const { return a == *this; }
bool IR::Invalid::operator==(IR::Invalid const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a));
    }
bool IR::Invalid::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        return true;
    }
void IR::Invalid::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
}
IR::Invalid::Invalid(JSONLoader & json) : Expression(json) {
}
IR::Node * IR::Invalid::fromJSON(JSONLoader & json) { return new Invalid(json); }
bool IR::Invalid::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Invalid::operator==(IR::Node const & a) const { return a == *this; }
bool IR::InvalidHeader::operator==(IR::InvalidHeader const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && headerType == a.headerType;
    }
bool IR::InvalidHeader::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const InvalidHeader &>(a_);
        return (headerType ? a.headerType ? headerType->equiv(*a.headerType) : false : a.headerType == nullptr);
    }
void IR::InvalidHeader::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(headerType, "headerType");
}
void IR::InvalidHeader::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(headerType, "headerType");
}
void IR::InvalidHeader::validate() const {
#line 5213 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(headerType); }
void IR::InvalidHeader::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("headerType", headerType);
}
IR::InvalidHeader::InvalidHeader(JSONLoader & json) : Expression(json) {
    json.load("headerType", headerType);
}
IR::Node * IR::InvalidHeader::fromJSON(JSONLoader & json) { return new InvalidHeader(json); }
bool IR::InvalidHeader::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::InvalidHeader::operator==(IR::Node const & a) const { return a == *this; }
bool IR::InvalidHeaderUnion::operator==(IR::InvalidHeaderUnion const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && headerUnionType == a.headerUnionType;
    }
bool IR::InvalidHeaderUnion::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const InvalidHeaderUnion &>(a_);
        return (headerUnionType ? a.headerUnionType ? headerUnionType->equiv(*a.headerUnionType) : false : a.headerUnionType == nullptr);
    }
void IR::InvalidHeaderUnion::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(headerUnionType, "headerUnionType");
}
void IR::InvalidHeaderUnion::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(headerUnionType, "headerUnionType");
}
void IR::InvalidHeaderUnion::validate() const {
#line 5247 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(headerUnionType); }
void IR::InvalidHeaderUnion::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("headerUnionType", headerUnionType);
}
IR::InvalidHeaderUnion::InvalidHeaderUnion(JSONLoader & json) : Expression(json) {
    json.load("headerUnionType", headerUnionType);
}
IR::Node * IR::InvalidHeaderUnion::fromJSON(JSONLoader & json) { return new InvalidHeaderUnion(json); }
bool IR::InvalidHeaderUnion::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::InvalidHeaderUnion::operator==(IR::Node const & a) const { return a == *this; }
/// May only be known after type checking; so it can be nullptr.
#line 674 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::HeaderStackExpression::validate() const {
#line 5263 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 674 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
{
        components.check_null();
    } }
#line 5269 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::HeaderStackExpression::operator==(IR::HeaderStackExpression const & a) const {
        return BaseListExpression::operator==(static_cast<const BaseListExpression &>(a))
        && headerStackType == a.headerStackType;
    }
bool IR::HeaderStackExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!BaseListExpression::equiv(a_)) return false;
        auto &a = static_cast<const HeaderStackExpression &>(a_);
        return (headerStackType ? a.headerStackType ? headerStackType->equiv(*a.headerStackType) : false : a.headerStackType == nullptr);
    }
void IR::HeaderStackExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    BaseListExpression::visit_children(v, n);
    v.visit(headerStackType, "headerStackType");
}
void IR::HeaderStackExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    BaseListExpression::visit_children(v, n);
    v.visit(headerStackType, "headerStackType");
}
void IR::HeaderStackExpression::toJSON(JSONGenerator & json) const {
    BaseListExpression::toJSON(json);
    if (headerStackType != nullptr) json.emit("headerStackType", headerStackType);
}
IR::HeaderStackExpression::HeaderStackExpression(JSONLoader & json) : BaseListExpression(json) {
    json.load("headerStackType", headerStackType);
}
IR::Node * IR::HeaderStackExpression::fromJSON(JSONLoader & json) { return new HeaderStackExpression(json); }
bool IR::HeaderStackExpression::operator==(IR::BaseListExpression const & a) const { return a == *this; }
bool IR::HeaderStackExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::HeaderStackExpression::operator==(IR::Node const & a) const { return a == *this; }
#line 683 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::ListCompileTimeValue::validate() const {
#line 5303 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        components.validate();
#line 683 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
{
        for (auto v : components)
            BUG_CHECK(v->is<CompileTimeValue>(), "%1%: not a compile-time value", v); } }
#line 5310 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ListCompileTimeValue::operator==(IR::ListCompileTimeValue const & a) const {
        return this->typeId() == a.typeId()
        && components == a.components;
    }
bool IR::ListCompileTimeValue::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const ListCompileTimeValue &>(a_);
        return components.equiv(a.components);
    }
void IR::ListCompileTimeValue::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    components.visit_children(v, "components");
}
void IR::ListCompileTimeValue::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    components.visit_children(v, "components");
}
void IR::ListCompileTimeValue::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("components", components);
}
IR::ListCompileTimeValue::ListCompileTimeValue(JSONLoader & json) : Node(json) {
    json.load("components", components);
}
IR::Node * IR::ListCompileTimeValue::fromJSON(JSONLoader & json) { return new ListCompileTimeValue(json); }
bool IR::ListCompileTimeValue::operator==(IR::Node const & a) const { return a == *this; }
#line 693 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::P4ListCompileTimeValue::validate() const {
#line 5342 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        components.validate();
#line 693 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
{
        for (auto v : components)
            BUG_CHECK(v->is<CompileTimeValue>(), "%1%: not a compile-time value", v); } }
#line 5349 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::P4ListCompileTimeValue::operator==(IR::P4ListCompileTimeValue const & a) const {
        return this->typeId() == a.typeId()
        && components == a.components;
    }
bool IR::P4ListCompileTimeValue::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const P4ListCompileTimeValue &>(a_);
        return components.equiv(a.components);
    }
void IR::P4ListCompileTimeValue::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    components.visit_children(v, "components");
}
void IR::P4ListCompileTimeValue::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    components.visit_children(v, "components");
}
void IR::P4ListCompileTimeValue::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("components", components);
}
IR::P4ListCompileTimeValue::P4ListCompileTimeValue(JSONLoader & json) : Node(json) {
    json.load("components", components);
}
IR::Node * IR::P4ListCompileTimeValue::fromJSON(JSONLoader & json) { return new P4ListCompileTimeValue(json); }
bool IR::P4ListCompileTimeValue::operator==(IR::Node const & a) const { return a == *this; }
#line 703 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::StructCompileTimeValue::validate() const {
#line 5381 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        components.validate();
#line 703 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
{
        for (auto v : components)
            BUG_CHECK(v->is<CompileTimeValue>(), "%1%: not a compile-time value", v); } }
#line 5388 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::StructCompileTimeValue::operator==(IR::StructCompileTimeValue const & a) const {
        return this->typeId() == a.typeId()
        && components == a.components;
    }
bool IR::StructCompileTimeValue::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const StructCompileTimeValue &>(a_);
        return components.equiv(a.components);
    }
void IR::StructCompileTimeValue::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    components.visit_children(v, "components");
}
void IR::StructCompileTimeValue::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    components.visit_children(v, "components");
}
void IR::StructCompileTimeValue::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("components", components);
}
IR::StructCompileTimeValue::StructCompileTimeValue(JSONLoader & json) : Node(json) {
    json.load("components", components);
}
IR::Node * IR::StructCompileTimeValue::fromJSON(JSONLoader & json) { return new StructCompileTimeValue(json); }
bool IR::StructCompileTimeValue::operator==(IR::Node const & a) const { return a == *this; }
#line 713 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::CompileTimeMethodCall::validate() const {
#line 5420 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 713 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
{
        for (auto v : *arguments)
            BUG_CHECK(v->is<CompileTimeValue>(), "%1%: not a compile-time value", v); } }
#line 5426 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::CompileTimeMethodCall::operator==(IR::CompileTimeMethodCall const & a) const {
        return MethodCallExpression::operator==(static_cast<const MethodCallExpression &>(a));
    }
bool IR::CompileTimeMethodCall::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!MethodCallExpression::equiv(a_)) return false;
        return true;
    }
void IR::CompileTimeMethodCall::toJSON(JSONGenerator & json) const {
    MethodCallExpression::toJSON(json);
}
IR::CompileTimeMethodCall::CompileTimeMethodCall(JSONLoader & json) : MethodCallExpression(json) {
}
IR::Node * IR::CompileTimeMethodCall::fromJSON(JSONLoader & json) { return new CompileTimeMethodCall(json); }
bool IR::CompileTimeMethodCall::operator==(IR::MethodCallExpression const & a) const { return a == *this; }
bool IR::CompileTimeMethodCall::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::CompileTimeMethodCall::operator==(IR::Node const & a) const { return a == *this; }
/// The label of the symbolic variable.
/// A symbolic variable always has a type and no source info.
/// Implements comparisons so that SymbolicVariables can be used as map keys.
#line 735 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
cstring IR::SymbolicVariable::toString() const { return absl::StrCat("|", label, "(", type, ")|"); }
#line 5449 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 737 "/builddir/build/BUILD/p4c-1.2.5.8/ir/expression.def"
void IR::SymbolicVariable::dbprint(std::ostream & out) const { out << "|" + label +"(" << type << ")|"; }
#line 5452 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::SymbolicVariable::operator==(IR::SymbolicVariable const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && label == a.label;
    }
bool IR::SymbolicVariable::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const SymbolicVariable &>(a_);
        return label == a.label;
    }
void IR::SymbolicVariable::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " label=" << label;
}
void IR::SymbolicVariable::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("label", label);
}
IR::SymbolicVariable::SymbolicVariable(JSONLoader & json) : Expression(json) {
    json.load("label", label);
}
IR::Node * IR::SymbolicVariable::fromJSON(JSONLoader & json) { return new SymbolicVariable(json); }
bool IR::SymbolicVariable::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::SymbolicVariable::operator==(IR::Node const & a) const { return a == *this; }

#line 63 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::ParserState::getAnnotations() const { return annotations; }
#line 5480 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 64 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::ParserState::getAnnotations() { return annotations; }
#line 5483 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 65 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
Util::Enumerator<const IR::IDeclaration *> * IR::ParserState::getDeclarations() const {
        return components.getDeclarations(); }
#line 5487 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 67 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::IDeclaration const * IR::ParserState::getDeclByName(cstring name) const {
        return components.getDeclaration(name); }
#line 5491 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 69 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::IDeclaration const * IR::ParserState::getDeclByName(std::string_view name) const {
        return components.getDeclaration(name); }
#line 5495 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 77 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::ParserState::validate() const {
#line 5498 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        components.validate();
#line 77 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{
        if (selectExpression != nullptr)
            BUG_CHECK(selectExpression->is<IR::PathExpression>() ||
                      selectExpression->is<IR::SelectExpression>(),
                      "%1%: unexpected select expression", selectExpression);
    } }
#line 5509 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ParserState::operator==(IR::ParserState const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && annotations == a.annotations
        && components == a.components
        && selectExpression == a.selectExpression;
    }
bool IR::ParserState::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const ParserState &>(a_);
        return annotations.equiv(a.annotations)
        && components.equiv(a.components)
        && (selectExpression ? a.selectExpression ? selectExpression->equiv(*a.selectExpression) : false : a.selectExpression == nullptr);
    }
void IR::ParserState::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    components.visit_children(v, "components");
    v.visit(selectExpression, "selectExpression");
}
void IR::ParserState::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    components.visit_children(v, "components");
    v.visit(selectExpression, "selectExpression");
}
void IR::ParserState::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("components", components);
    if (selectExpression != nullptr) json.emit("selectExpression", selectExpression);
}
IR::ParserState::ParserState(JSONLoader & json) : Declaration(json) {
    json.load("annotations", annotations);
    json.load("components", components);
    json.load("selectExpression", selectExpression);
}
IR::Node * IR::ParserState::fromJSON(JSONLoader & json) { return new ParserState(json); }
bool IR::ParserState::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::ParserState::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::ParserState::operator==(IR::Node const & a) const { return a == *this; }
#line 92 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::P4Parser::getAnnotations() const { return type->getAnnotations(); }
#line 5555 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 93 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::P4Parser::getAnnotations() { BUG("cannot modify annotations"); return *new Vector<Annotation>(); }
#line 5558 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 94 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::TypeParameters *IR::P4Parser::getTypeParameters() const { return type->getTypeParameters(); }
#line 5561 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 95 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
std::vector<const IR::INamespace *> IR::P4Parser::getNestedNamespaces() const {
        return { type->typeParameters, type->applyParams, constructorParams }; }
#line 5565 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 97 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
Util::Enumerator<const IR::IDeclaration *> * IR::P4Parser::getDeclarations() const {
        return parserLocals.getDeclarations()->concat(states.getDeclarations()); }
#line 5569 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 99 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::P4Parser::getDeclByName(cstring name) const {
        auto decl = parserLocals.getDeclaration(name);
        if (!decl) decl = states.getDeclaration(name);
        return decl; }
#line 5575 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 103 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::P4Parser::getDeclByName(std::string_view name) const {
        auto decl = parserLocals.getDeclaration(name);
        if (!decl) decl = states.getDeclaration(name);
        return decl; }
#line 5581 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 108 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::Type_Method *IR::P4Parser::getApplyMethodType() const { return type->getApplyMethodType(); }
#line 5584 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 109 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::ParameterList *IR::P4Parser::getApplyParameters() const { return type->getApplyParameters(); }
#line 5587 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 111 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::ParameterList *IR::P4Parser::getConstructorParameters() const { return constructorParams; }
#line 5590 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 113 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::Type *IR::P4Parser::getType() const { return this->type; }
#line 5593 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(P4Parser, , )
#line 115 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::P4Parser::validate() const {
#line 5597 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(type);
        CHECK_NULL(constructorParams);
        parserLocals.validate();
        states.validate();
#line 115 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{
        if (!(name == type->name))
            BUG("Name mismatch for %1%: %2% != %3%", this, name, type->name);
        parserLocals.check_null();
        states.check_null();
        checkDuplicates();
        for (auto d : parserLocals)
            BUG_CHECK(!d->is<ParserState>(), "%1%: state in locals", d);
    } }
#line 5613 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 124 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::P4Parser::toString() const { return "parser "_cs + externalName(); }
#line 5616 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::P4Parser::operator==(IR::P4Parser const & a) const {
        return Type_Declaration::operator==(static_cast<const Type_Declaration &>(a))
        && type == a.type
        && constructorParams == a.constructorParams
        && parserLocals == a.parserLocals
        && states == a.states;
    }
bool IR::P4Parser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Declaration::equiv(a_)) return false;
        auto &a = static_cast<const P4Parser &>(a_);
        return (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && (constructorParams ? a.constructorParams ? constructorParams->equiv(*a.constructorParams) : false : a.constructorParams == nullptr)
        && parserLocals.equiv(a.parserLocals)
        && states.equiv(a.states);
    }
void IR::P4Parser::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_Declaration::visit_children(v, n);
    v.visit(type, "type");
    v.visit(constructorParams, "constructorParams");
    parserLocals.visit_children(v, "parserLocals");
    states.visit_children(v, "states");
}
void IR::P4Parser::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_Declaration::visit_children(v, n);
    v.visit(type, "type");
    v.visit(constructorParams, "constructorParams");
    parserLocals.visit_children(v, "parserLocals");
    states.visit_children(v, "states");
}
void IR::P4Parser::toJSON(JSONGenerator & json) const {
    Type_Declaration::toJSON(json);
    json.emit("type", type);
    json.emit("constructorParams", constructorParams);
    json.emit("parserLocals", parserLocals);
    json.emit("states", states);
}
IR::P4Parser::P4Parser(JSONLoader & json) : Type_Declaration(json) {
    json.load("type", type);
    json.load("constructorParams", constructorParams);
    json.load("parserLocals", parserLocals);
    json.load("states", states);
}
IR::Node * IR::P4Parser::fromJSON(JSONLoader & json) { return new P4Parser(json); }
bool IR::P4Parser::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::P4Parser::operator==(IR::Type const & a) const { return a == *this; }
bool IR::P4Parser::operator==(IR::Node const & a) const { return a == *this; }
#line 133 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::P4Control::getAnnotations() const { return type->getAnnotations(); }
#line 5668 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 134 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::P4Control::getAnnotations() { BUG("cannot modify annotations"); return *new Vector<Annotation>(); }
#line 5671 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 135 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::TypeParameters *IR::P4Control::getTypeParameters() const { return type->getTypeParameters(); }
#line 5674 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 136 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
std::vector<const IR::INamespace *> IR::P4Control::getNestedNamespaces() const {
        return { type->typeParameters, type->applyParams, constructorParams }; }
#line 5678 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 138 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
Util::Enumerator<const IR::IDeclaration *> * IR::P4Control::getDeclarations() const {
        return controlLocals.getDeclarations(); }
#line 5682 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 140 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::Type_Method *IR::P4Control::getApplyMethodType() const { return type->getApplyMethodType(); }
#line 5685 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 141 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::ParameterList *IR::P4Control::getApplyParameters() const { return type->getApplyParameters(); }
#line 5688 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 143 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::P4Control::getDeclByName(cstring name) const {
        return controlLocals.getDeclaration(name); }
#line 5692 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 145 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::P4Control::getDeclByName(std::string_view name) const {
        return controlLocals.getDeclaration(name); }
#line 5696 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 147 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::ParameterList *IR::P4Control::getConstructorParameters() const { return constructorParams; }
#line 5699 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 148 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::Type *IR::P4Control::getType() const { return this->type; }
#line 5702 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(P4Control, , )
#line 150 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::P4Control::validate() const {
#line 5706 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(type);
        CHECK_NULL(constructorParams);
        controlLocals.validate();
        CHECK_NULL(body);
#line 150 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{
        if (!(name == type->name))
            BUG("Name mismatch for %1%: %2% != %3%", this, name, type->name);
        controlLocals.check_null();
    } }
#line 5718 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 155 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::P4Control::toString() const { return "control "_cs + externalName(); }
#line 5721 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::P4Control::operator==(IR::P4Control const & a) const {
        return Type_Declaration::operator==(static_cast<const Type_Declaration &>(a))
        && type == a.type
        && constructorParams == a.constructorParams
        && controlLocals == a.controlLocals
        && body == a.body;
    }
bool IR::P4Control::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Declaration::equiv(a_)) return false;
        auto &a = static_cast<const P4Control &>(a_);
        return (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && (constructorParams ? a.constructorParams ? constructorParams->equiv(*a.constructorParams) : false : a.constructorParams == nullptr)
        && controlLocals.equiv(a.controlLocals)
        && (body ? a.body ? body->equiv(*a.body) : false : a.body == nullptr);
    }
void IR::P4Control::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_Declaration::visit_children(v, n);
    v.visit(type, "type");
    v.visit(constructorParams, "constructorParams");
    controlLocals.visit_children(v, "controlLocals");
    v.visit(body, "body");
}
void IR::P4Control::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_Declaration::visit_children(v, n);
    v.visit(type, "type");
    v.visit(constructorParams, "constructorParams");
    controlLocals.visit_children(v, "controlLocals");
    v.visit(body, "body");
}
void IR::P4Control::toJSON(JSONGenerator & json) const {
    Type_Declaration::toJSON(json);
    json.emit("type", type);
    json.emit("constructorParams", constructorParams);
    json.emit("controlLocals", controlLocals);
    json.emit("body", body);
}
IR::P4Control::P4Control(JSONLoader & json) : Type_Declaration(json) {
    json.load("type", type);
    json.load("constructorParams", constructorParams);
    json.load("controlLocals", controlLocals);
    json.load("body", body);
}
IR::Node * IR::P4Control::fromJSON(JSONLoader & json) { return new P4Control(json); }
bool IR::P4Control::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::P4Control::operator==(IR::Type const & a) const { return a == *this; }
bool IR::P4Control::operator==(IR::Node const & a) const { return a == *this; }
#line 163 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
Util::Enumerator<const IR::IDeclaration *> * IR::P4Action::getDeclarations() const {
        return parameters->getDeclarations(); }
#line 5774 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 165 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::P4Action::getDeclByName(cstring name) const {
        return parameters->getDeclByName(name); }
#line 5778 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 167 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::P4Action::getDeclByName(std::string_view name) const {
        return parameters->getDeclByName(name); }
#line 5782 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 169 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::P4Action::getAnnotations() const { return annotations; }
#line 5785 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 170 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::P4Action::getAnnotations() { return annotations; }
#line 5788 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 171 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::ParameterList *IR::P4Action::getParameters() const { return parameters; }
#line 5791 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::P4Action::operator==(IR::P4Action const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && annotations == a.annotations
        && parameters == a.parameters
        && body == a.body;
    }
bool IR::P4Action::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const P4Action &>(a_);
        return annotations.equiv(a.annotations)
        && (parameters ? a.parameters ? parameters->equiv(*a.parameters) : false : a.parameters == nullptr)
        && (body ? a.body ? body->equiv(*a.body) : false : a.body == nullptr);
    }
void IR::P4Action::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(parameters, "parameters");
    v.visit(body, "body");
}
void IR::P4Action::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(parameters, "parameters");
    v.visit(body, "body");
}
void IR::P4Action::validate() const {
#line 5821 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(parameters);
        CHECK_NULL(body); }
void IR::P4Action::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("parameters", parameters);
    json.emit("body", body);
}
IR::P4Action::P4Action(JSONLoader & json) : Declaration(json) {
    json.load("annotations", annotations);
    json.load("parameters", parameters);
    json.load("body", body);
}
IR::Node * IR::P4Action::fromJSON(JSONLoader & json) { return new P4Action(json); }
bool IR::P4Action::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::P4Action::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::P4Action::operator==(IR::Node const & a) const { return a == *this; }
#line 177 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
Util::Enumerator<const IR::IDeclaration *> * IR::Type_Error::getDeclarations() const {
        return members.getDeclarations(); }
#line 5844 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 179 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::Type_Error::getDeclByName(cstring name) const {
        return members.getDeclaration(name); }
#line 5848 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 181 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::Type_Error::getDeclByName(std::string_view name) const {
        return members.getDeclaration(name); }
#line 5852 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 183 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::Type_Error::validate() const {
#line 5855 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        members.validate();
#line 183 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{ members.check_null(); } }
#line 5860 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Error::operator==(IR::Type_Error const & a) const {
        return Type_Declaration::operator==(static_cast<const Type_Declaration &>(a))
        && members == a.members;
    }
bool IR::Type_Error::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Type_Error &>(a_);
        return members.equiv(a.members);
    }
void IR::Type_Error::visit_children(Visitor & v, char const * n) {
    (void)n;
    Type_Declaration::visit_children(v, n);
    members.visit_children(v, "members");
}
void IR::Type_Error::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Type_Declaration::visit_children(v, n);
    members.visit_children(v, "members");
}
void IR::Type_Error::toJSON(JSONGenerator & json) const {
    Type_Declaration::toJSON(json);
    json.emit("members", members);
}
IR::Type_Error::Type_Error(JSONLoader & json) : Type_Declaration(json) {
    json.load("members", members);
}
IR::Node * IR::Type_Error::fromJSON(JSONLoader & json) { return new Type_Error(json); }
bool IR::Type_Error::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::Type_Error::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Error::operator==(IR::Node const & a) const { return a == *this; }
#line 189 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
Util::Enumerator<const IR::IDeclaration *> * IR::Declaration_MatchKind::getDeclarations() const {
        return members.getDeclarations(); }
#line 5895 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 191 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::Declaration_MatchKind::getDeclByName(cstring name) const {
        return members.getDeclaration(name); }
#line 5899 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 193 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::Declaration_MatchKind::getDeclByName(std::string_view name) const {
        return members.getDeclaration(name); }
#line 5903 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 195 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::Declaration_MatchKind::validate() const {
#line 5906 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        members.validate();
#line 195 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{ members.check_null(); } }
#line 5911 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Declaration_MatchKind::operator==(IR::Declaration_MatchKind const & a) const {
        return this->typeId() == a.typeId()
        && members == a.members;
    }
bool IR::Declaration_MatchKind::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Declaration_MatchKind &>(a_);
        return members.equiv(a.members);
    }
void IR::Declaration_MatchKind::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    members.visit_children(v, "members");
}
void IR::Declaration_MatchKind::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    members.visit_children(v, "members");
}
void IR::Declaration_MatchKind::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("members", members);
}
IR::Declaration_MatchKind::Declaration_MatchKind(JSONLoader & json) : Node(json) {
    json.load("members", members);
}
IR::Node * IR::Declaration_MatchKind::fromJSON(JSONLoader & json) { return new Declaration_MatchKind(json); }
bool IR::Declaration_MatchKind::operator==(IR::Node const & a) const { return a == *this; }
bool IR::PropertyValue::operator==(IR::PropertyValue const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::PropertyValue::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::PropertyValue::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::PropertyValue::PropertyValue(JSONLoader & json) : Node(json) {
}
bool IR::PropertyValue::operator==(IR::Node const & a) const { return a == *this; }
#line 204 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::ExpressionValue::dbprint(std::ostream & out) const { out << expression; }
#line 5957 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ExpressionValue::operator==(IR::ExpressionValue const & a) const {
        return PropertyValue::operator==(static_cast<const PropertyValue &>(a))
        && expression == a.expression;
    }
bool IR::ExpressionValue::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!PropertyValue::equiv(a_)) return false;
        auto &a = static_cast<const ExpressionValue &>(a_);
        return (expression ? a.expression ? expression->equiv(*a.expression) : false : a.expression == nullptr);
    }
void IR::ExpressionValue::visit_children(Visitor & v, char const * n) {
    (void)n;
    PropertyValue::visit_children(v, n);
    v.visit(expression, "expression");
}
void IR::ExpressionValue::visit_children(Visitor & v, char const * n) const {
    (void)n;
    PropertyValue::visit_children(v, n);
    v.visit(expression, "expression");
}
void IR::ExpressionValue::validate() const {
#line 5979 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(expression); }
void IR::ExpressionValue::toJSON(JSONGenerator & json) const {
    PropertyValue::toJSON(json);
    json.emit("expression", expression);
}
IR::ExpressionValue::ExpressionValue(JSONLoader & json) : PropertyValue(json) {
    json.load("expression", expression);
}
IR::Node * IR::ExpressionValue::fromJSON(JSONLoader & json) { return new ExpressionValue(json); }
bool IR::ExpressionValue::operator==(IR::PropertyValue const & a) const { return a == *this; }
bool IR::ExpressionValue::operator==(IR::Node const & a) const { return a == *this; }
#line 209 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::ExpressionListValue::dbprint(std::ostream & out) const { out << expressions; }
#line 5994 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ExpressionListValue::operator==(IR::ExpressionListValue const & a) const {
        return PropertyValue::operator==(static_cast<const PropertyValue &>(a))
        && expressions == a.expressions;
    }
bool IR::ExpressionListValue::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!PropertyValue::equiv(a_)) return false;
        auto &a = static_cast<const ExpressionListValue &>(a_);
        return expressions.equiv(a.expressions);
    }
void IR::ExpressionListValue::visit_children(Visitor & v, char const * n) {
    (void)n;
    PropertyValue::visit_children(v, n);
    expressions.visit_children(v, "expressions");
}
void IR::ExpressionListValue::visit_children(Visitor & v, char const * n) const {
    (void)n;
    PropertyValue::visit_children(v, n);
    expressions.visit_children(v, "expressions");
}
void IR::ExpressionListValue::validate() const {
#line 6016 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        expressions.validate(); }
void IR::ExpressionListValue::toJSON(JSONGenerator & json) const {
    PropertyValue::toJSON(json);
    json.emit("expressions", expressions);
}
IR::ExpressionListValue::ExpressionListValue(JSONLoader & json) : PropertyValue(json) {
    json.load("expressions", expressions);
}
IR::Node * IR::ExpressionListValue::fromJSON(JSONLoader & json) { return new ExpressionListValue(json); }
bool IR::ExpressionListValue::operator==(IR::PropertyValue const & a) const { return a == *this; }
bool IR::ExpressionListValue::operator==(IR::Node const & a) const { return a == *this; }

#line 216 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::ActionListElement::dbprint(std::ostream & out) const { out << annotations << expression; }
#line 6032 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 217 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::ID IR::ActionListElement::getName() const { return getPath()->name; }
#line 6035 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 219 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::ActionListElement::getAnnotations() const { return annotations; }
#line 6038 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 220 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::ActionListElement::getAnnotations() { return annotations; }
#line 6041 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 221 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::ActionListElement::validate() const {
#line 6044 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(expression);
#line 221 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{
        BUG_CHECK(expression->is<IR::PathExpression>() ||
                  expression->is<IR::MethodCallExpression>(),
                  "%1%: unexpected expression", expression);
    } }
#line 6054 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 226 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::ActionListElement::toString() const { return getName().toString(); }
#line 6057 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ActionListElement::operator==(IR::ActionListElement const & a) const {
        return this->typeId() == a.typeId()
        && annotations == a.annotations
        && expression == a.expression;
    }
bool IR::ActionListElement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const ActionListElement &>(a_);
        return annotations.equiv(a.annotations)
        && (expression ? a.expression ? expression->equiv(*a.expression) : false : a.expression == nullptr);
    }
void IR::ActionListElement::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(expression, "expression");
}
void IR::ActionListElement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(expression, "expression");
}
void IR::ActionListElement::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("expression", expression);
}
IR::ActionListElement::ActionListElement(JSONLoader & json) : Node(json) {
    json.load("annotations", annotations);
    json.load("expression", expression);
}
IR::Node * IR::ActionListElement::fromJSON(JSONLoader & json) { return new ActionListElement(json); }
bool IR::ActionListElement::operator==(IR::Node const & a) const { return a == *this; }
#line 232 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::ActionList::validate() const {
#line 6095 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        actionList.validate();
#line 232 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{ actionList.check_null(); } }
#line 6100 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ActionList::operator==(IR::ActionList const & a) const {
        return PropertyValue::operator==(static_cast<const PropertyValue &>(a))
        && actionList == a.actionList;
    }
bool IR::ActionList::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!PropertyValue::equiv(a_)) return false;
        auto &a = static_cast<const ActionList &>(a_);
        return actionList.equiv(a.actionList);
    }
void IR::ActionList::visit_children(Visitor & v, char const * n) {
    (void)n;
    PropertyValue::visit_children(v, n);
    actionList.visit_children(v, "actionList");
}
void IR::ActionList::visit_children(Visitor & v, char const * n) const {
    (void)n;
    PropertyValue::visit_children(v, n);
    actionList.visit_children(v, "actionList");
}
void IR::ActionList::toJSON(JSONGenerator & json) const {
    PropertyValue::toJSON(json);
    json.emit("actionList", actionList);
}
IR::ActionList::ActionList(JSONLoader & json) : PropertyValue(json) {
    json.load("actionList", actionList);
}
IR::Node * IR::ActionList::fromJSON(JSONLoader & json) { return new ActionList(json); }
bool IR::ActionList::operator==(IR::PropertyValue const & a) const { return a == *this; }
bool IR::ActionList::operator==(IR::Node const & a) const { return a == *this; }
#line 244 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::KeyElement::getAnnotations() const { return annotations; }
#line 6133 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 245 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::KeyElement::getAnnotations() { return annotations; }
#line 6136 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 246 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Node const * IR::KeyElement::transform_visit(Transform & v) {
        // call this from Transform::preorder(KeyElement) if the transform might split
        // the expression into a Vector<Expression>
        v.visit(annotations, "annotations");
        auto exp = v.apply_visitor(expression, "expression");
        v.visit(matchType, "matchType");
        v.prune();
        if (exp == expression) {
        } else if (auto vec = exp->to<Vector<Expression>>()) {
            auto *rv = new Vector<KeyElement>();
            for (auto el : *vec) {
                auto *kel = clone();
                kel->expression = el;
                rv->push_back(kel); }
            return rv;
        } else {
            expression = exp->to<IR::Expression>(); }
        return this; }
#line 6156 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::KeyElement::operator==(IR::KeyElement const & a) const {
        return this->typeId() == a.typeId()
        && expression == a.expression
        && matchType == a.matchType
        && annotations == a.annotations;
    }
bool IR::KeyElement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const KeyElement &>(a_);
        return (expression ? a.expression ? expression->equiv(*a.expression) : false : a.expression == nullptr)
        && (matchType ? a.matchType ? matchType->equiv(*a.matchType) : false : a.matchType == nullptr)
        && annotations.equiv(a.annotations);
    }
void IR::KeyElement::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(expression, "expression");
    v.visit(matchType, "matchType");
    annotations.visit_children(v, "annotations");
}
void IR::KeyElement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(expression, "expression");
    v.visit(matchType, "matchType");
    annotations.visit_children(v, "annotations");
}
void IR::KeyElement::validate() const {
#line 6186 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(expression);
        CHECK_NULL(matchType);
        annotations.validate(); }
void IR::KeyElement::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("expression", expression);
    json.emit("matchType", matchType);
    json.emit("annotations", annotations);
}
IR::KeyElement::KeyElement(JSONLoader & json) : Node(json) {
    json.load("expression", expression);
    json.load("matchType", matchType);
    json.load("annotations", annotations);
}
IR::Node * IR::KeyElement::fromJSON(JSONLoader & json) { return new KeyElement(json); }
bool IR::KeyElement::operator==(IR::Node const & a) const { return a == *this; }
#line 269 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::Key::validate() const {
#line 6206 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        keyElements.validate();
#line 269 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{ keyElements.check_null(); } }
#line 6211 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Key::operator==(IR::Key const & a) const {
        return PropertyValue::operator==(static_cast<const PropertyValue &>(a))
        && keyElements == a.keyElements;
    }
bool IR::Key::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!PropertyValue::equiv(a_)) return false;
        auto &a = static_cast<const Key &>(a_);
        return keyElements.equiv(a.keyElements);
    }
void IR::Key::visit_children(Visitor & v, char const * n) {
    (void)n;
    PropertyValue::visit_children(v, n);
    keyElements.visit_children(v, "keyElements");
}
void IR::Key::visit_children(Visitor & v, char const * n) const {
    (void)n;
    PropertyValue::visit_children(v, n);
    keyElements.visit_children(v, "keyElements");
}
void IR::Key::toJSON(JSONGenerator & json) const {
    PropertyValue::toJSON(json);
    json.emit("keyElements", keyElements);
}
IR::Key::Key(JSONLoader & json) : PropertyValue(json) {
    json.load("keyElements", keyElements);
}
IR::Node * IR::Key::fromJSON(JSONLoader & json) { return new Key(json); }
bool IR::Key::operator==(IR::PropertyValue const & a) const { return a == *this; }
bool IR::Key::operator==(IR::Node const & a) const { return a == *this; }
/// annotations are optional (supported annotations: @priority(value))
/// optional const annotation
/// optional entry priority
/// must be a tuple expression
/// typically a MethodCallExpression.
/// The action must be defined in action list
/// True if the entry is not a list.
#line 284 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::Entry::getAnnotations() const { return annotations; }
#line 6251 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 285 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::Entry::getAnnotations() { return annotations; }
#line 6254 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 288 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::Entry::dbprint(std::ostream & out) const { out << annotations << keys << action; }
#line 6257 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Entry::operator==(IR::Entry const & a) const {
        return this->typeId() == a.typeId()
        && annotations == a.annotations
        && isConst == a.isConst
        && priority == a.priority
        && keys == a.keys
        && action == a.action
        && singleton == a.singleton;
    }
bool IR::Entry::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Entry &>(a_);
        return annotations.equiv(a.annotations)
        && isConst == a.isConst
        && (priority ? a.priority ? priority->equiv(*a.priority) : false : a.priority == nullptr)
        && (keys ? a.keys ? keys->equiv(*a.keys) : false : a.keys == nullptr)
        && (action ? a.action ? action->equiv(*a.action) : false : a.action == nullptr)
        && singleton == a.singleton;
    }
void IR::Entry::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(priority, "priority");
    v.visit(keys, "keys");
    v.visit(action, "action");
}
void IR::Entry::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(priority, "priority");
    v.visit(keys, "keys");
    v.visit(action, "action");
}
void IR::Entry::validate() const {
#line 6295 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(keys);
        CHECK_NULL(action); }
void IR::Entry::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " isConst=" << isConst;
        out << " singleton=" << singleton;
}
void IR::Entry::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("isConst", isConst);
    if (priority != nullptr) json.emit("priority", priority);
    json.emit("keys", keys);
    json.emit("action", action);
    json.emit("singleton", singleton);
}
IR::Entry::Entry(JSONLoader & json) : Node(json) {
    json.load("annotations", annotations);
    json.load("isConst", isConst);
    json.load("priority", priority);
    json.load("keys", keys);
    json.load("action", action);
    json.load("singleton", singleton);
}
IR::Node * IR::Entry::fromJSON(JSONLoader & json) { return new Entry(json); }
bool IR::Entry::operator==(IR::Node const & a) const { return a == *this; }
#line 295 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::EntriesList::dbprint(std::ostream & out) const { out << "{ " << entries << "}"; }
#line 6326 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::EntriesList::operator==(IR::EntriesList const & a) const {
        return PropertyValue::operator==(static_cast<const PropertyValue &>(a))
        && entries == a.entries;
    }
bool IR::EntriesList::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!PropertyValue::equiv(a_)) return false;
        auto &a = static_cast<const EntriesList &>(a_);
        return entries.equiv(a.entries);
    }
void IR::EntriesList::visit_children(Visitor & v, char const * n) {
    (void)n;
    PropertyValue::visit_children(v, n);
    entries.visit_children(v, "entries");
}
void IR::EntriesList::visit_children(Visitor & v, char const * n) const {
    (void)n;
    PropertyValue::visit_children(v, n);
    entries.visit_children(v, "entries");
}
void IR::EntriesList::validate() const {
#line 6348 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        entries.validate(); }
void IR::EntriesList::toJSON(JSONGenerator & json) const {
    PropertyValue::toJSON(json);
    json.emit("entries", entries);
}
IR::EntriesList::EntriesList(JSONLoader & json) : PropertyValue(json) {
    json.load("entries", entries);
}
IR::Node * IR::EntriesList::fromJSON(JSONLoader & json) { return new EntriesList(json); }
bool IR::EntriesList::operator==(IR::PropertyValue const & a) const { return a == *this; }
bool IR::EntriesList::operator==(IR::Node const & a) const { return a == *this; }
#line 302 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::Property::getAnnotations() const { return annotations; }
#line 6363 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 303 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::Property::getAnnotations() { return annotations; }
#line 6366 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 304 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::Property::dbprint(std::ostream & out) const { out << annotations << (isConstant ? "const " : "") << name << " = " << value; }
#line 6369 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Property::operator==(IR::Property const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && annotations == a.annotations
        && value == a.value
        && isConstant == a.isConstant;
    }
bool IR::Property::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Property &>(a_);
        return annotations.equiv(a.annotations)
        && (value ? a.value ? value->equiv(*a.value) : false : a.value == nullptr)
        && isConstant == a.isConstant;
    }
void IR::Property::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(value, "value");
}
void IR::Property::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(value, "value");
}
void IR::Property::validate() const {
#line 6397 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(value); }
void IR::Property::dump_fields(std::ostream & out) const {
    Declaration::dump_fields(out);
        out << " isConstant=" << isConstant;
}
void IR::Property::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("value", value);
    json.emit("isConstant", isConstant);
}
IR::Property::Property(JSONLoader & json) : Declaration(json) {
    json.load("annotations", annotations);
    json.load("value", value);
    json.load("isConstant", isConstant);
}
IR::Node * IR::Property::fromJSON(JSONLoader & json) { return new Property(json); }
bool IR::Property::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::Property::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::Property::operator==(IR::Node const & a) const { return a == *this; }
#line 309 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::TableProperties::toString() const { return absl::StrCat("TableProperties(", properties.size(), ")"); }
#line 6422 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 310 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
Util::Enumerator<const IR::IDeclaration *> * IR::TableProperties::getDeclarations() const {
        return properties.getDeclarations(); }
#line 6426 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 314 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::TableProperties::getDeclByName(cstring name) const {
        return properties.getDeclaration(name); }
#line 6430 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 318 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::TableProperties::getDeclByName(std::string_view name) const {
        return properties.getDeclaration(name); }
#line 6434 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 328 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::TableProperties::validate() const {
#line 6437 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        properties.validate();
#line 328 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{ properties.check_null(); properties.validate(); } }
#line 6442 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TableProperties::operator==(IR::TableProperties const & a) const {
        return this->typeId() == a.typeId()
        && properties == a.properties;
    }
bool IR::TableProperties::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TableProperties &>(a_);
        return properties.equiv(a.properties);
    }
void IR::TableProperties::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    properties.visit_children(v, "properties");
}
void IR::TableProperties::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    properties.visit_children(v, "properties");
}
void IR::TableProperties::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("properties", properties);
}
IR::TableProperties::TableProperties(JSONLoader & json) : Node(json) {
    json.load("properties", properties);
}
IR::Node * IR::TableProperties::fromJSON(JSONLoader & json) { return new TableProperties(json); }
bool IR::TableProperties::operator==(IR::Node const & a) const { return a == *this; }
#line 335 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::P4Table::getAnnotations() const { return annotations; }
#line 6474 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 336 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::P4Table::getAnnotations() { return annotations; }
#line 6477 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 338 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::ParameterList *IR::P4Table::getApplyParameters() const { return new ParameterList(); }
#line 6480 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::P4Table::operator==(IR::P4Table const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && annotations == a.annotations
        && properties == a.properties;
    }
bool IR::P4Table::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const P4Table &>(a_);
        return annotations.equiv(a.annotations)
        && (properties ? a.properties ? properties->equiv(*a.properties) : false : a.properties == nullptr);
    }
void IR::P4Table::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(properties, "properties");
}
void IR::P4Table::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(properties, "properties");
}
void IR::P4Table::validate() const {
#line 6506 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(properties); }
void IR::P4Table::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("properties", properties);
}
IR::P4Table::P4Table(JSONLoader & json) : Declaration(json) {
    json.load("annotations", annotations);
    json.load("properties", properties);
}
IR::Node * IR::P4Table::fromJSON(JSONLoader & json) { return new P4Table(json); }
bool IR::P4Table::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::P4Table::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::P4Table::operator==(IR::Node const & a) const { return a == *this; }

#line 390 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::P4ValueSet::getAnnotations() const { return annotations; }
#line 6526 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 391 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::P4ValueSet::getAnnotations() { return annotations; }
#line 6529 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::P4ValueSet::operator==(IR::P4ValueSet const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && annotations == a.annotations
        && elementType == a.elementType
        && size == a.size;
    }
bool IR::P4ValueSet::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const P4ValueSet &>(a_);
        return annotations.equiv(a.annotations)
        && (elementType ? a.elementType ? elementType->equiv(*a.elementType) : false : a.elementType == nullptr)
        && (size ? a.size ? size->equiv(*a.size) : false : a.size == nullptr);
    }
void IR::P4ValueSet::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(elementType, "elementType");
    v.visit(size, "size");
}
void IR::P4ValueSet::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(elementType, "elementType");
    v.visit(size, "size");
}
void IR::P4ValueSet::validate() const {
#line 6559 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(elementType);
        CHECK_NULL(size); }
void IR::P4ValueSet::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("elementType", elementType);
    json.emit("size", size);
}
IR::P4ValueSet::P4ValueSet(JSONLoader & json) : Declaration(json) {
    json.load("annotations", annotations);
    json.load("elementType", elementType);
    json.load("size", size);
}
IR::Node * IR::P4ValueSet::fromJSON(JSONLoader & json) { return new P4ValueSet(json); }
bool IR::P4ValueSet::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::P4ValueSet::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::P4ValueSet::operator==(IR::Node const & a) const { return a == *this; }
#line 399 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::Declaration_Variable::getAnnotations() const { return annotations; }
#line 6581 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 400 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::Declaration_Variable::getAnnotations() { return annotations; }
#line 6584 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 401 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::Declaration_Variable::dbprint(std::ostream & out) const {
        out << annotations << type << ' ' << name;
        if (initializer) out << " = " << *initializer; }
#line 6589 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Declaration_Variable::operator==(IR::Declaration_Variable const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && annotations == a.annotations
        && type == a.type
        && initializer == a.initializer;
    }
bool IR::Declaration_Variable::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Declaration_Variable &>(a_);
        return annotations.equiv(a.annotations)
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && (initializer ? a.initializer ? initializer->equiv(*a.initializer) : false : a.initializer == nullptr);
    }
void IR::Declaration_Variable::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
    v.visit(initializer, "initializer");
}
void IR::Declaration_Variable::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
    v.visit(initializer, "initializer");
}
void IR::Declaration_Variable::validate() const {
#line 6619 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(type); }
void IR::Declaration_Variable::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("type", type);
    if (initializer != nullptr) json.emit("initializer", initializer);
}
IR::Declaration_Variable::Declaration_Variable(JSONLoader & json) : Declaration(json) {
    json.load("annotations", annotations);
    json.load("type", type);
    json.load("initializer", initializer);
}
IR::Node * IR::Declaration_Variable::fromJSON(JSONLoader & json) { return new Declaration_Variable(json); }
bool IR::Declaration_Variable::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::Declaration_Variable::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::Declaration_Variable::operator==(IR::Node const & a) const { return a == *this; }
#line 411 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::Declaration_Constant::getAnnotations() const { return annotations; }
#line 6640 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 412 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::Declaration_Constant::getAnnotations() { return annotations; }
#line 6643 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 413 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::Declaration_Constant::toString() const { return Declaration::toString(); }
#line 6646 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 414 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::Declaration_Constant::dbprint(std::ostream & out) const { out << annotations << type << ' ' << name << " = " << *initializer; }
#line 6649 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Declaration_Constant::operator==(IR::Declaration_Constant const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && annotations == a.annotations
        && type == a.type
        && initializer == a.initializer;
    }
bool IR::Declaration_Constant::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Declaration_Constant &>(a_);
        return annotations.equiv(a.annotations)
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && (initializer ? a.initializer ? initializer->equiv(*a.initializer) : false : a.initializer == nullptr);
    }
void IR::Declaration_Constant::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
    v.visit(initializer, "initializer");
}
void IR::Declaration_Constant::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
    v.visit(initializer, "initializer");
}
void IR::Declaration_Constant::validate() const {
#line 6679 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(type);
        CHECK_NULL(initializer); }
void IR::Declaration_Constant::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("type", type);
    json.emit("initializer", initializer);
}
IR::Declaration_Constant::Declaration_Constant(JSONLoader & json) : Declaration(json) {
    json.load("annotations", annotations);
    json.load("type", type);
    json.load("initializer", initializer);
}
IR::Node * IR::Declaration_Constant::fromJSON(JSONLoader & json) { return new Declaration_Constant(json); }
bool IR::Declaration_Constant::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::Declaration_Constant::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::Declaration_Constant::operator==(IR::Node const & a) const { return a == *this; }



#line 429 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::Declaration_Instance::getAnnotations() const { return annotations; }
#line 6704 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 430 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::Declaration_Instance::getAnnotations() { return annotations; }
#line 6707 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 431 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::Type *IR::Declaration_Instance::getType() const { return type; }
#line 6710 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 432 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::ID IR::Declaration_Instance::Name() const { return name; }
#line 6713 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 433 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::Declaration_Instance::validate() const {
#line 6716 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(type);
        CHECK_NULL(arguments);
        properties.validate();
#line 433 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{ arguments->check_null(); } }
#line 6724 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Declaration_Instance::operator==(IR::Declaration_Instance const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && annotations == a.annotations
        && type == a.type
        && arguments == a.arguments
        && properties == a.properties
        && initializer == a.initializer;
    }
bool IR::Declaration_Instance::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Declaration_Instance &>(a_);
        return annotations.equiv(a.annotations)
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && (arguments ? a.arguments ? arguments->equiv(*a.arguments) : false : a.arguments == nullptr)
        && properties.equiv(a.properties)
        && (initializer ? a.initializer ? initializer->equiv(*a.initializer) : false : a.initializer == nullptr);
    }
void IR::Declaration_Instance::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
    v.visit(arguments, "arguments");
    properties.visit_children(v, "properties");
    v.visit(initializer, "initializer");
}
void IR::Declaration_Instance::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
    v.visit(arguments, "arguments");
    properties.visit_children(v, "properties");
    v.visit(initializer, "initializer");
}
void IR::Declaration_Instance::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("type", type);
    json.emit("arguments", arguments);
    json.emit("properties", properties);
    if (initializer != nullptr) json.emit("initializer", initializer);
}
IR::Declaration_Instance::Declaration_Instance(JSONLoader & json) : Declaration(json) {
    json.load("annotations", annotations);
    json.load("type", type);
    json.load("arguments", arguments);
    json.load("properties", properties);
    json.load("initializer", initializer);
}
IR::Node * IR::Declaration_Instance::fromJSON(JSONLoader & json) { return new Declaration_Instance(json); }
bool IR::Declaration_Instance::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::Declaration_Instance::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::Declaration_Instance::operator==(IR::Node const & a) const { return a == *this; }
/// Top-level program objects.
/// This is not an IndexedVector because:
/// - we allow overloaded function-like objects.
/// - not all objects in a P4Program are declarations (e.g., match_kind is not).
#line 444 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::P4Program::validate() const {
#line 6786 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        objects.validate();
#line 444 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{ objects.check_null(); } }
#line 6791 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(P4Program, , )
bool IR::P4Program::operator==(IR::P4Program const & a) const {
        return this->typeId() == a.typeId()
        && objects == a.objects;
    }
bool IR::P4Program::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const P4Program &>(a_);
        return objects.equiv(a.objects);
    }
void IR::P4Program::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    objects.visit_children(v, "objects");
}
void IR::P4Program::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    objects.visit_children(v, "objects");
}
void IR::P4Program::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("objects", objects);
}
IR::P4Program::P4Program(JSONLoader & json) : Node(json) {
    json.load("objects", objects);
}
IR::Node * IR::P4Program::fromJSON(JSONLoader & json) { return new P4Program(json); }
bool IR::P4Program::operator==(IR::Node const & a) const { return a == *this; }
#line 456 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::ExitStatement::toString() const { return "exit"_cs; }
#line 6824 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 457 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::ExitStatement::dbprint(std::ostream & out) const { out << "exit"; }
#line 6827 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ExitStatement::operator==(IR::ExitStatement const & a) const {
        return Statement::operator==(static_cast<const Statement &>(a));
    }
bool IR::ExitStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Statement::equiv(a_)) return false;
        return true;
    }
void IR::ExitStatement::toJSON(JSONGenerator & json) const {
    Statement::toJSON(json);
}
IR::ExitStatement::ExitStatement(JSONLoader & json) : Statement(json) {
}
IR::Node * IR::ExitStatement::fromJSON(JSONLoader & json) { return new ExitStatement(json); }
bool IR::ExitStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::ExitStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::ExitStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 462 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::ReturnStatement::toString() const { return absl::StrCat("return ",
                                  (expression ?
                                   expression->toString() : ""_cs)); }
#line 6849 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ReturnStatement::operator==(IR::ReturnStatement const & a) const {
        return Statement::operator==(static_cast<const Statement &>(a))
        && expression == a.expression;
    }
bool IR::ReturnStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Statement::equiv(a_)) return false;
        auto &a = static_cast<const ReturnStatement &>(a_);
        return (expression ? a.expression ? expression->equiv(*a.expression) : false : a.expression == nullptr);
    }
void IR::ReturnStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    Statement::visit_children(v, n);
    v.visit(expression, "expression");
}
void IR::ReturnStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Statement::visit_children(v, n);
    v.visit(expression, "expression");
}
void IR::ReturnStatement::toJSON(JSONGenerator & json) const {
    Statement::toJSON(json);
    if (expression != nullptr) json.emit("expression", expression);
}
IR::ReturnStatement::ReturnStatement(JSONLoader & json) : Statement(json) {
    json.load("expression", expression);
}
IR::Node * IR::ReturnStatement::fromJSON(JSONLoader & json) { return new ReturnStatement(json); }
bool IR::ReturnStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::ReturnStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::ReturnStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 468 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::EmptyStatement::dbprint(std::ostream & out) const { out << ""; }
#line 6883 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::EmptyStatement::operator==(IR::EmptyStatement const & a) const {
        return Statement::operator==(static_cast<const Statement &>(a));
    }
bool IR::EmptyStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Statement::equiv(a_)) return false;
        return true;
    }
void IR::EmptyStatement::toJSON(JSONGenerator & json) const {
    Statement::toJSON(json);
}
IR::EmptyStatement::EmptyStatement(JSONLoader & json) : Statement(json) {
}
IR::Node * IR::EmptyStatement::fromJSON(JSONLoader & json) { return new EmptyStatement(json); }
bool IR::EmptyStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::EmptyStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::EmptyStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 477 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::AssignmentStatement::toString() const { return absl::StrCat(left, " = ", right); }
#line 6903 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::AssignmentStatement::operator==(IR::AssignmentStatement const & a) const {
        return BaseAssignmentStatement::operator==(static_cast<const BaseAssignmentStatement &>(a));
    }
bool IR::AssignmentStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!BaseAssignmentStatement::equiv(a_)) return false;
        return true;
    }
void IR::AssignmentStatement::toJSON(JSONGenerator & json) const {
    BaseAssignmentStatement::toJSON(json);
}
IR::AssignmentStatement::AssignmentStatement(JSONLoader & json) : BaseAssignmentStatement(json) {
}
IR::Node * IR::AssignmentStatement::fromJSON(JSONLoader & json) { return new AssignmentStatement(json); }
bool IR::AssignmentStatement::operator==(IR::BaseAssignmentStatement const & a) const { return a == *this; }
bool IR::AssignmentStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::AssignmentStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::AssignmentStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 490 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::IfStatement::visit_children(Visitor & v, char const * n) {
        (void)n;
        v.visit(condition, "condition");
        SplitFlowVisit<Statement>(v, ifTrue, ifFalse).run_visit(); }
#line 490 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::IfStatement::visit_children(Visitor & v, char const * n) const {
        (void)n;
        v.visit(condition, "condition");
        SplitFlowVisit<Statement>(v, ifTrue, ifFalse).run_visit(); }
#line 6932 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::IfStatement::operator==(IR::IfStatement const & a) const {
        return Statement::operator==(static_cast<const Statement &>(a))
        && condition == a.condition
        && ifTrue == a.ifTrue
        && ifFalse == a.ifFalse;
    }
bool IR::IfStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Statement::equiv(a_)) return false;
        auto &a = static_cast<const IfStatement &>(a_);
        return (condition ? a.condition ? condition->equiv(*a.condition) : false : a.condition == nullptr)
        && (ifTrue ? a.ifTrue ? ifTrue->equiv(*a.ifTrue) : false : a.ifTrue == nullptr)
        && (ifFalse ? a.ifFalse ? ifFalse->equiv(*a.ifFalse) : false : a.ifFalse == nullptr);
    }
void IR::IfStatement::validate() const {
#line 6948 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(condition);
        CHECK_NULL(ifTrue); }
void IR::IfStatement::toJSON(JSONGenerator & json) const {
    Statement::toJSON(json);
    json.emit("condition", condition);
    json.emit("ifTrue", ifTrue);
    if (ifFalse != nullptr) json.emit("ifFalse", ifFalse);
}
IR::IfStatement::IfStatement(JSONLoader & json) : Statement(json) {
    json.load("condition", condition);
    json.load("ifTrue", ifTrue);
    json.load("ifFalse", ifFalse);
}
IR::Node * IR::IfStatement::fromJSON(JSONLoader & json) { return new IfStatement(json); }
bool IR::IfStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::IfStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::IfStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 497 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::BreakStatement::toString() const { return "break"_cs; }
#line 6969 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 498 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::BreakStatement::dbprint(std::ostream & out) const { out << "break"; }
#line 6972 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

bool IR::BreakStatement::operator==(IR::BreakStatement const & a) const {
        return Statement::operator==(static_cast<const Statement &>(a));
    }
bool IR::BreakStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Statement::equiv(a_)) return false;
        return true;
    }
void IR::BreakStatement::toJSON(JSONGenerator & json) const {
    Statement::toJSON(json);
}
IR::BreakStatement::BreakStatement(JSONLoader & json) : Statement(json) {
}
IR::Node * IR::BreakStatement::fromJSON(JSONLoader & json) { return new BreakStatement(json); }
bool IR::BreakStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::BreakStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::BreakStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 503 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::ContinueStatement::toString() const { return "continue"_cs; }
#line 6993 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 504 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::ContinueStatement::dbprint(std::ostream & out) const { out << "continue"; }
#line 6996 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

bool IR::ContinueStatement::operator==(IR::ContinueStatement const & a) const {
        return Statement::operator==(static_cast<const Statement &>(a));
    }
bool IR::ContinueStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Statement::equiv(a_)) return false;
        return true;
    }
void IR::ContinueStatement::toJSON(JSONGenerator & json) const {
    Statement::toJSON(json);
}
IR::ContinueStatement::ContinueStatement(JSONLoader & json) : Statement(json) {
}
IR::Node * IR::ContinueStatement::fromJSON(JSONLoader & json) { return new ContinueStatement(json); }
bool IR::ContinueStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::ContinueStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::ContinueStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 511 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::BlockStatement::getDeclByName(cstring name) const {
        return components.getDeclaration(name); }
#line 7018 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 513 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::BlockStatement::getDeclByName(std::string_view name) const {
        return components.getDeclaration(name); }
#line 7022 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 515 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
Util::Enumerator<const IR::IDeclaration *> * IR::BlockStatement::getDeclarations() const {
        return components.getDeclarations(); }
#line 7026 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 517 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::BlockStatement::push_back(const IR::StatOrDecl* st) { components.push_back(st); }
#line 7029 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 518 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::BlockStatement::getAnnotations() const { return annotations; }
#line 7032 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 519 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::BlockStatement::getAnnotations() { return annotations; }
#line 7035 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BlockStatement::operator==(IR::BlockStatement const & a) const {
        return Statement::operator==(static_cast<const Statement &>(a))
        && annotations == a.annotations
        && components == a.components;
    }
bool IR::BlockStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Statement::equiv(a_)) return false;
        auto &a = static_cast<const BlockStatement &>(a_);
        return annotations.equiv(a.annotations)
        && components.equiv(a.components);
    }
void IR::BlockStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    Statement::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    components.visit_children(v, "components");
}
void IR::BlockStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Statement::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    components.visit_children(v, "components");
}
void IR::BlockStatement::validate() const {
#line 7061 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        components.validate(); }
void IR::BlockStatement::toJSON(JSONGenerator & json) const {
    Statement::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("components", components);
}
IR::BlockStatement::BlockStatement(JSONLoader & json) : Statement(json) {
    json.load("annotations", annotations);
    json.load("components", components);
}
IR::Node * IR::BlockStatement::fromJSON(JSONLoader & json) { return new BlockStatement(json); }
bool IR::BlockStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::BlockStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::BlockStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 533 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::MethodCallStatement::toString() const { return methodCall->toString(); }
#line 7080 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MethodCallStatement::operator==(IR::MethodCallStatement const & a) const {
        return Statement::operator==(static_cast<const Statement &>(a))
        && methodCall == a.methodCall;
    }
bool IR::MethodCallStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Statement::equiv(a_)) return false;
        auto &a = static_cast<const MethodCallStatement &>(a_);
        return (methodCall ? a.methodCall ? methodCall->equiv(*a.methodCall) : false : a.methodCall == nullptr);
    }
void IR::MethodCallStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    Statement::visit_children(v, n);
    v.visit(methodCall, "methodCall");
}
void IR::MethodCallStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Statement::visit_children(v, n);
    v.visit(methodCall, "methodCall");
}
void IR::MethodCallStatement::validate() const {
#line 7102 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(methodCall); }
void IR::MethodCallStatement::toJSON(JSONGenerator & json) const {
    Statement::toJSON(json);
    json.emit("methodCall", methodCall);
}
IR::MethodCallStatement::MethodCallStatement(JSONLoader & json) : Statement(json) {
    json.load("methodCall", methodCall);
}
IR::Node * IR::MethodCallStatement::fromJSON(JSONLoader & json) { return new MethodCallStatement(json); }
bool IR::MethodCallStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::MethodCallStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::MethodCallStatement::operator==(IR::Node const & a) const { return a == *this; }

#line 540 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::SwitchCase::validate() const {
#line 7119 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(label);
#line 540 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{
        BUG_CHECK(statement == nullptr || statement->is<IR::BlockStatement>(),
                  "%1%: Expected a block statement",
                  statement);
    } }
#line 7128 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::SwitchCase::operator==(IR::SwitchCase const & a) const {
        return this->typeId() == a.typeId()
        && label == a.label
        && statement == a.statement;
    }
bool IR::SwitchCase::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const SwitchCase &>(a_);
        return (label ? a.label ? label->equiv(*a.label) : false : a.label == nullptr)
        && (statement ? a.statement ? statement->equiv(*a.statement) : false : a.statement == nullptr);
    }
void IR::SwitchCase::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(label, "label");
    v.visit(statement, "statement");
}
void IR::SwitchCase::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(label, "label");
    v.visit(statement, "statement");
}
void IR::SwitchCase::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("label", label);
    if (statement != nullptr) json.emit("statement", statement);
}
IR::SwitchCase::SwitchCase(JSONLoader & json) : Node(json) {
    json.load("label", label);
    json.load("statement", statement);
}
IR::Node * IR::SwitchCase::fromJSON(JSONLoader & json) { return new SwitchCase(json); }
bool IR::SwitchCase::operator==(IR::Node const & a) const { return a == *this; }
#line 552 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::SwitchStatement::visit_children(Visitor & v, char const * n) {
        (void)n;
        v.visit(expression, "expression");
        SplitFlowVisit<SwitchCase> split(v);
        for (auto &c : cases) split.addNode(c);
        split.run_visit(); }
#line 552 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::SwitchStatement::visit_children(Visitor & v, char const * n) const {
        (void)n;
        v.visit(expression, "expression");
        SplitFlowVisit<SwitchCase> split(v);
        for (auto &c : cases) split.addNode(c);
        split.run_visit(); }
#line 7178 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::SwitchStatement::operator==(IR::SwitchStatement const & a) const {
        return Statement::operator==(static_cast<const Statement &>(a))
        && expression == a.expression
        && cases == a.cases;
    }
bool IR::SwitchStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Statement::equiv(a_)) return false;
        auto &a = static_cast<const SwitchStatement &>(a_);
        return (expression ? a.expression ? expression->equiv(*a.expression) : false : a.expression == nullptr)
        && cases.equiv(a.cases);
    }
void IR::SwitchStatement::validate() const {
#line 7192 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(expression);
        cases.validate(); }
void IR::SwitchStatement::toJSON(JSONGenerator & json) const {
    Statement::toJSON(json);
    json.emit("expression", expression);
    json.emit("cases", cases);
}
IR::SwitchStatement::SwitchStatement(JSONLoader & json) : Statement(json) {
    json.load("expression", expression);
    json.load("cases", cases);
}
IR::Node * IR::SwitchStatement::fromJSON(JSONLoader & json) { return new SwitchStatement(json); }
bool IR::SwitchStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::SwitchStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::SwitchStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 563 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::LoopStatement::getAnnotations() const { return annotations; }
#line 7211 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 564 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::LoopStatement::getAnnotations() { return annotations; }
#line 7214 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::LoopStatement::operator==(IR::LoopStatement const & a) const {
        return Statement::operator==(static_cast<const Statement &>(a))
        && annotations == a.annotations;
    }
bool IR::LoopStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Statement::equiv(a_)) return false;
        auto &a = static_cast<const LoopStatement &>(a_);
        return annotations.equiv(a.annotations);
    }
void IR::LoopStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    Statement::visit_children(v, n);
    annotations.visit_children(v, "annotations");
}
void IR::LoopStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Statement::visit_children(v, n);
    annotations.visit_children(v, "annotations");
}
void IR::LoopStatement::validate() const {
#line 7236 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate(); }
void IR::LoopStatement::toJSON(JSONGenerator & json) const {
    Statement::toJSON(json);
    json.emit("annotations", annotations);
}
IR::LoopStatement::LoopStatement(JSONLoader & json) : Statement(json) {
    json.load("annotations", annotations);
}
bool IR::LoopStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::LoopStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::LoopStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 572 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::ForStatement::getDeclByName(cstring name) const {
        return init.getDeclaration(name); }
#line 7252 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 574 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::ForStatement::getDeclByName(std::string_view name) const {
        return init.getDeclaration(name); }
#line 7256 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 576 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
Util::Enumerator<const IR::IDeclaration *> * IR::ForStatement::getDeclarations() const {
        return init.getDeclarations(); }
#line 7260 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ForStatement::operator==(IR::ForStatement const & a) const {
        return LoopStatement::operator==(static_cast<const LoopStatement &>(a))
        && init == a.init
        && condition == a.condition
        && updates == a.updates
        && body == a.body;
    }
bool IR::ForStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoopStatement::equiv(a_)) return false;
        auto &a = static_cast<const ForStatement &>(a_);
        return init.equiv(a.init)
        && (condition ? a.condition ? condition->equiv(*a.condition) : false : a.condition == nullptr)
        && updates.equiv(a.updates)
        && (body ? a.body ? body->equiv(*a.body) : false : a.body == nullptr);
    }
void IR::ForStatement::validate() const {
#line 7278 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        init.validate();
        CHECK_NULL(condition);
        updates.validate();
        CHECK_NULL(body); }
void IR::ForStatement::toJSON(JSONGenerator & json) const {
    LoopStatement::toJSON(json);
    json.emit("init", init);
    json.emit("condition", condition);
    json.emit("updates", updates);
    json.emit("body", body);
}
IR::ForStatement::ForStatement(JSONLoader & json) : LoopStatement(json) {
    json.load("init", init);
    json.load("condition", condition);
    json.load("updates", updates);
    json.load("body", body);
}
IR::Node * IR::ForStatement::fromJSON(JSONLoader & json) { return new ForStatement(json); }
bool IR::ForStatement::operator==(IR::LoopStatement const & a) const { return a == *this; }
bool IR::ForStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::ForStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::ForStatement::operator==(IR::Node const & a) const { return a == *this; }


#line 594 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::ForInStatement::ForInStatement(Util::SourceInfo si, const IR::Vector<IR::Annotation>* a, const IR::PathExpression* pe, const IR::Expression* c, const IR::Statement* b) : LoopStatement(si, *a), decl(nullptr), ref(pe), collection(c), body(b) {}
#line 7306 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 596 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::ForInStatement::getDeclByName(cstring name) const {
        return decl && decl->name == name ? decl : nullptr; }
#line 7310 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 598 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::ForInStatement::getDeclByName(std::string_view name) const {
        return decl && decl->name.name == name ? decl : nullptr; }
#line 7314 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 600 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
Util::Enumerator<const IR::IDeclaration *> * IR::ForInStatement::getDeclarations() const {
        if (decl) return new Util::SingleEnumerator<const IDeclaration *>(decl);
        return new Util::EmptyEnumerator<const IDeclaration *>(); }
#line 7319 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ForInStatement::operator==(IR::ForInStatement const & a) const {
        return LoopStatement::operator==(static_cast<const LoopStatement &>(a))
        && decl == a.decl
        && ref == a.ref
        && collection == a.collection
        && body == a.body;
    }
bool IR::ForInStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoopStatement::equiv(a_)) return false;
        auto &a = static_cast<const ForInStatement &>(a_);
        return (decl ? a.decl ? decl->equiv(*a.decl) : false : a.decl == nullptr)
        && (ref ? a.ref ? ref->equiv(*a.ref) : false : a.ref == nullptr)
        && (collection ? a.collection ? collection->equiv(*a.collection) : false : a.collection == nullptr)
        && (body ? a.body ? body->equiv(*a.body) : false : a.body == nullptr);
    }
void IR::ForInStatement::validate() const {
#line 7337 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(ref);
        CHECK_NULL(collection);
        CHECK_NULL(body); }
void IR::ForInStatement::toJSON(JSONGenerator & json) const {
    LoopStatement::toJSON(json);
    if (decl != nullptr) json.emit("decl", decl);
    json.emit("ref", ref);
    json.emit("collection", collection);
    json.emit("body", body);
}
IR::ForInStatement::ForInStatement(JSONLoader & json) : LoopStatement(json) {
    json.load("decl", decl);
    json.load("ref", ref);
    json.load("collection", collection);
    json.load("body", body);
}
IR::Node * IR::ForInStatement::fromJSON(JSONLoader & json) { return new ForInStatement(json); }
bool IR::ForInStatement::operator==(IR::LoopStatement const & a) const { return a == *this; }
bool IR::ForInStatement::operator==(IR::Statement const & a) const { return a == *this; }
bool IR::ForInStatement::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::ForInStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 616 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::ParameterList *IR::Function::getParameters() const {
        return type->parameters;
    }
#line 7364 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 619 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
Util::Enumerator<const IR::IDeclaration *> * IR::Function::getDeclarations() const {
        return type->parameters->getDeclarations(); }
#line 7368 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 621 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::Function::getDeclByName(cstring name) const {
        return type->parameters->getDeclByName(name); }
#line 7372 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 623 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::IDeclaration *IR::Function::getDeclByName(std::string_view name) const {
        return type->parameters->getDeclByName(name); }
#line 7376 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 625 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
std::vector<const IR::INamespace *> IR::Function::getNestedNamespaces() const {
        return { type->typeParameters }; }
#line 7380 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 627 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> const & IR::Function::getAnnotations() const { return annotations; }
#line 7383 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 628 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::Vector<IR::Annotation> & IR::Function::getAnnotations() { return annotations; }
#line 7386 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Function::operator==(IR::Function const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && annotations == a.annotations
        && type == a.type
        && body == a.body;
    }
bool IR::Function::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Function &>(a_);
        return annotations.equiv(a.annotations)
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && (body ? a.body ? body->equiv(*a.body) : false : a.body == nullptr);
    }
void IR::Function::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
    v.visit(body, "body");
}
void IR::Function::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
    v.visit(body, "body");
}
void IR::Function::validate() const {
#line 7416 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        CHECK_NULL(type);
        CHECK_NULL(body); }
void IR::Function::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("type", type);
    json.emit("body", body);
}
IR::Function::Function(JSONLoader & json) : Declaration(json) {
    json.load("annotations", annotations);
    json.load("type", type);
    json.load("body", body);
}
IR::Node * IR::Function::fromJSON(JSONLoader & json) { return new Function(json); }
bool IR::Function::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::Function::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::Function::operator==(IR::Node const & a) const { return a == *this; }
/// Node that evaluates to this block.
/// This is either a Declaration_Instance or a ConstructorCallExpression.
/// One value for each Node inside that evaluates to a compile-time constant.
/// This includes all constructor parameters, and all inner nested blocks.
/// value can be null for parameters which are optional
#line 660 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::Block::visit_children(Visitor & v, char const * n) { (void)v; (void)n; }
#line 660 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::Block::visit_children(Visitor & v, char const * n) const { (void)v; (void)n; }
#line 7445 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 661 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::IDeclaration const * IR::Block::getContainer() const { return nullptr; }
#line 7448 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Block::operator==(IR::Block const & a) const {
        return this->typeId() == a.typeId()
        && node == a.node
        && constantValue == a.constantValue;
    }
bool IR::Block::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Block &>(a_);
        return (node ? a.node ? node->equiv(*a.node) : false : a.node == nullptr)
        && constantValue == a.constantValue;
    }
void IR::Block::validate() const {
#line 7462 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(node); }
void IR::Block::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("node", node);
    json.emit("constantValue", constantValue);
}
IR::Block::Block(JSONLoader & json) : Node(json) {
    json.load("node", node);
    json.load("constantValue", constantValue);
}
bool IR::Block::operator==(IR::Node const & a) const { return a == *this; }
#line 666 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::IDeclaration const * IR::TableBlock::getContainer() const { return container; }
#line 7477 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TableBlock::operator==(IR::TableBlock const & a) const {
        return Block::operator==(static_cast<const Block &>(a))
        && container == a.container;
    }
bool IR::TableBlock::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Block::equiv(a_)) return false;
        auto &a = static_cast<const TableBlock &>(a_);
        return (container ? a.container ? container->equiv(*a.container) : false : a.container == nullptr);
    }
void IR::TableBlock::visit_children(Visitor & v, char const * n) {
    (void)n;
    Block::visit_children(v, n);
    v.visit(container, "container");
}
void IR::TableBlock::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Block::visit_children(v, n);
    v.visit(container, "container");
}
void IR::TableBlock::validate() const {
#line 7499 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(container); }
void IR::TableBlock::toJSON(JSONGenerator & json) const {
    Block::toJSON(json);
    json.emit("container", container);
}
IR::TableBlock::TableBlock(JSONLoader & json) : Block(json) {
    json.load("container", container);
}
IR::Node * IR::TableBlock::fromJSON(JSONLoader & json) { return new TableBlock(json); }
bool IR::TableBlock::operator==(IR::Block const & a) const { return a == *this; }
bool IR::TableBlock::operator==(IR::Node const & a) const { return a == *this; }

/// @return the argument that the given parameter was instantiated with.
/// It's a fatal error if no such parameter exists.
/// @return the argument that the given parameter was instantiated with, or
/// null if no such parameter exists.
bool IR::InstantiatedBlock::operator==(IR::InstantiatedBlock const & a) const {
        return Block::operator==(static_cast<const Block &>(a))
        && instanceType == a.instanceType;
    }
bool IR::InstantiatedBlock::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Block::equiv(a_)) return false;
        auto &a = static_cast<const InstantiatedBlock &>(a_);
        return (instanceType ? a.instanceType ? instanceType->equiv(*a.instanceType) : false : a.instanceType == nullptr);
    }
void IR::InstantiatedBlock::visit_children(Visitor & v, char const * n) {
    (void)n;
    Block::visit_children(v, n);
    v.visit(instanceType, "instanceType");
}
void IR::InstantiatedBlock::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Block::visit_children(v, n);
    v.visit(instanceType, "instanceType");
}
void IR::InstantiatedBlock::validate() const {
#line 7538 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(instanceType); }
void IR::InstantiatedBlock::toJSON(JSONGenerator & json) const {
    Block::toJSON(json);
    json.emit("instanceType", instanceType);
}
IR::InstantiatedBlock::InstantiatedBlock(JSONLoader & json) : Block(json) {
    json.load("instanceType", instanceType);
}
bool IR::InstantiatedBlock::operator==(IR::Block const & a) const { return a == *this; }
bool IR::InstantiatedBlock::operator==(IR::Node const & a) const { return a == *this; }
#line 690 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::ParameterList *IR::ParserBlock::getConstructorParameters() const {
        return container->constructorParams; }
#line 7553 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 692 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::ParserBlock::toString() const { return container->toString(); }
#line 7556 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 693 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::ID IR::ParserBlock::getName() const { return container->getName(); }
#line 7559 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 694 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::IDeclaration const * IR::ParserBlock::getContainer() const { return container; }
#line 7562 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ParserBlock::operator==(IR::ParserBlock const & a) const {
        return InstantiatedBlock::operator==(static_cast<const InstantiatedBlock &>(a))
        && container == a.container;
    }
bool IR::ParserBlock::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!InstantiatedBlock::equiv(a_)) return false;
        auto &a = static_cast<const ParserBlock &>(a_);
        return (container ? a.container ? container->equiv(*a.container) : false : a.container == nullptr);
    }
void IR::ParserBlock::visit_children(Visitor & v, char const * n) {
    (void)n;
    InstantiatedBlock::visit_children(v, n);
    v.visit(container, "container");
}
void IR::ParserBlock::visit_children(Visitor & v, char const * n) const {
    (void)n;
    InstantiatedBlock::visit_children(v, n);
    v.visit(container, "container");
}
void IR::ParserBlock::validate() const {
#line 7584 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(container); }
void IR::ParserBlock::toJSON(JSONGenerator & json) const {
    InstantiatedBlock::toJSON(json);
    json.emit("container", container);
}
IR::ParserBlock::ParserBlock(JSONLoader & json) : InstantiatedBlock(json) {
    json.load("container", container);
}
IR::Node * IR::ParserBlock::fromJSON(JSONLoader & json) { return new ParserBlock(json); }
bool IR::ParserBlock::operator==(IR::InstantiatedBlock const & a) const { return a == *this; }
bool IR::ParserBlock::operator==(IR::Block const & a) const { return a == *this; }
bool IR::ParserBlock::operator==(IR::Node const & a) const { return a == *this; }
#line 700 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::ParameterList *IR::ControlBlock::getConstructorParameters() const {
        return container->constructorParams; }
#line 7601 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 702 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::ControlBlock::toString() const { return container->toString(); }
#line 7604 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 703 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::ID IR::ControlBlock::getName() const { return container->getName(); }
#line 7607 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 704 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::IDeclaration const * IR::ControlBlock::getContainer() const { return container; }
#line 7610 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ControlBlock::operator==(IR::ControlBlock const & a) const {
        return InstantiatedBlock::operator==(static_cast<const InstantiatedBlock &>(a))
        && container == a.container;
    }
bool IR::ControlBlock::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!InstantiatedBlock::equiv(a_)) return false;
        auto &a = static_cast<const ControlBlock &>(a_);
        return (container ? a.container ? container->equiv(*a.container) : false : a.container == nullptr);
    }
void IR::ControlBlock::visit_children(Visitor & v, char const * n) {
    (void)n;
    InstantiatedBlock::visit_children(v, n);
    v.visit(container, "container");
}
void IR::ControlBlock::visit_children(Visitor & v, char const * n) const {
    (void)n;
    InstantiatedBlock::visit_children(v, n);
    v.visit(container, "container");
}
void IR::ControlBlock::validate() const {
#line 7632 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(container); }
void IR::ControlBlock::toJSON(JSONGenerator & json) const {
    InstantiatedBlock::toJSON(json);
    json.emit("container", container);
}
IR::ControlBlock::ControlBlock(JSONLoader & json) : InstantiatedBlock(json) {
    json.load("container", container);
}
IR::Node * IR::ControlBlock::fromJSON(JSONLoader & json) { return new ControlBlock(json); }
bool IR::ControlBlock::operator==(IR::InstantiatedBlock const & a) const { return a == *this; }
bool IR::ControlBlock::operator==(IR::Block const & a) const { return a == *this; }
bool IR::ControlBlock::operator==(IR::Node const & a) const { return a == *this; }
#line 710 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::ParameterList *IR::PackageBlock::getConstructorParameters() const { return type->constructorParams; }
#line 7648 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 711 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::PackageBlock::toString() const { return type->toString(); }
#line 7651 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 712 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::ID IR::PackageBlock::getName() const { return type->getName(); }
#line 7654 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::PackageBlock::operator==(IR::PackageBlock const & a) const {
        return InstantiatedBlock::operator==(static_cast<const InstantiatedBlock &>(a))
        && type == a.type;
    }
bool IR::PackageBlock::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!InstantiatedBlock::equiv(a_)) return false;
        auto &a = static_cast<const PackageBlock &>(a_);
        return (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr);
    }
void IR::PackageBlock::visit_children(Visitor & v, char const * n) {
    (void)n;
    InstantiatedBlock::visit_children(v, n);
    v.visit(type, "type");
}
void IR::PackageBlock::visit_children(Visitor & v, char const * n) const {
    (void)n;
    InstantiatedBlock::visit_children(v, n);
    v.visit(type, "type");
}
void IR::PackageBlock::validate() const {
#line 7676 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(type); }
void IR::PackageBlock::toJSON(JSONGenerator & json) const {
    InstantiatedBlock::toJSON(json);
    json.emit("type", type);
}
IR::PackageBlock::PackageBlock(JSONLoader & json) : InstantiatedBlock(json) {
    json.load("type", type);
}
IR::Node * IR::PackageBlock::fromJSON(JSONLoader & json) { return new PackageBlock(json); }
bool IR::PackageBlock::operator==(IR::InstantiatedBlock const & a) const { return a == *this; }
bool IR::PackageBlock::operator==(IR::Block const & a) const { return a == *this; }
bool IR::PackageBlock::operator==(IR::Node const & a) const { return a == *this; }

#line 719 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::ParameterList *IR::ExternBlock::getConstructorParameters() const {
        return constructor->type->parameters; }
#line 7694 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 721 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
cstring IR::ExternBlock::toString() const { return type->toString(); }
#line 7697 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 722 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::ID IR::ExternBlock::getName() const { return type->getName(); }
#line 7700 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ExternBlock::operator==(IR::ExternBlock const & a) const {
        return InstantiatedBlock::operator==(static_cast<const InstantiatedBlock &>(a))
        && type == a.type
        && constructor == a.constructor;
    }
bool IR::ExternBlock::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!InstantiatedBlock::equiv(a_)) return false;
        auto &a = static_cast<const ExternBlock &>(a_);
        return (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && (constructor ? a.constructor ? constructor->equiv(*a.constructor) : false : a.constructor == nullptr);
    }
void IR::ExternBlock::visit_children(Visitor & v, char const * n) {
    (void)n;
    InstantiatedBlock::visit_children(v, n);
    v.visit(type, "type");
    v.visit(constructor, "constructor");
}
void IR::ExternBlock::visit_children(Visitor & v, char const * n) const {
    (void)n;
    InstantiatedBlock::visit_children(v, n);
    v.visit(type, "type");
    v.visit(constructor, "constructor");
}
void IR::ExternBlock::validate() const {
#line 7726 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(type);
        CHECK_NULL(constructor); }
void IR::ExternBlock::toJSON(JSONGenerator & json) const {
    InstantiatedBlock::toJSON(json);
    json.emit("type", type);
    json.emit("constructor", constructor);
}
IR::ExternBlock::ExternBlock(JSONLoader & json) : InstantiatedBlock(json) {
    json.load("type", type);
    json.load("constructor", constructor);
}
IR::Node * IR::ExternBlock::fromJSON(JSONLoader & json) { return new ExternBlock(json); }
bool IR::ExternBlock::operator==(IR::InstantiatedBlock const & a) const { return a == *this; }
bool IR::ExternBlock::operator==(IR::Block const & a) const { return a == *this; }
bool IR::ExternBlock::operator==(IR::Node const & a) const { return a == *this; }
#line 728 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
const IR::P4Program *IR::ToplevelBlock::getProgram() const { return node->to<IR::P4Program>(); }
#line 7745 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 730 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
IR::ID IR::ToplevelBlock::getName() const { return "main"; }
#line 7748 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 732 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
void IR::ToplevelBlock::validate() const {
#line 7751 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 732 "/builddir/build/BUILD/p4c-1.2.5.8/ir/ir.def"
{ BUG_CHECK(node->is<IR::P4Program>(), "%1%: expected a P4Program", node); } }
#line 7755 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ToplevelBlock::operator==(IR::ToplevelBlock const & a) const {
        return Block::operator==(static_cast<const Block &>(a));
    }
bool IR::ToplevelBlock::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Block::equiv(a_)) return false;
        return true;
    }
void IR::ToplevelBlock::toJSON(JSONGenerator & json) const {
    Block::toJSON(json);
}
IR::ToplevelBlock::ToplevelBlock(JSONLoader & json) : Block(json) {
}
IR::Node * IR::ToplevelBlock::fromJSON(JSONLoader & json) { return new ToplevelBlock(json); }
bool IR::ToplevelBlock::operator==(IR::Block const & a) const { return a == *this; }
bool IR::ToplevelBlock::operator==(IR::Node const & a) const { return a == *this; }
/** @} */
/* end group irdefs */
/* -*-C++-*- */
/*
  This file contains IR classes needed just for the P4 v1.0/v1.1 front-end.
*/
/** \addtogroup irdefs
  * @{
  */
#line 51 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::Type_Block::toString() const { return "block"_cs; }
#line 7783 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 53 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::Type_Block::dbprint(std::ostream & out) const { out << "block"; }
#line 7786 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Block::operator==(IR::Type_Block const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_Block::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_Block::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_Block::Type_Block(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_Block::fromJSON(JSONLoader & json) { return new Type_Block(json); }
bool IR::Type_Block::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_Block::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Block::operator==(IR::Node const & a) const { return a == *this; }
#line 56 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::Type_Counter::toString() const { return "counter"_cs; }
#line 7806 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 58 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::Type_Counter::dbprint(std::ostream & out) const { out << "counter"; }
#line 7809 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Counter::operator==(IR::Type_Counter const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_Counter::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_Counter::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_Counter::Type_Counter(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_Counter::fromJSON(JSONLoader & json) { return new Type_Counter(json); }
bool IR::Type_Counter::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_Counter::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Counter::operator==(IR::Node const & a) const { return a == *this; }
#line 61 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::Type_Expression::toString() const { return "expression"_cs; }
#line 7829 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 63 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::Type_Expression::dbprint(std::ostream & out) const { out << "expression"; }
#line 7832 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Expression::operator==(IR::Type_Expression const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_Expression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_Expression::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_Expression::Type_Expression(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_Expression::fromJSON(JSONLoader & json) { return new Type_Expression(json); }
bool IR::Type_Expression::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_Expression::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Expression::operator==(IR::Node const & a) const { return a == *this; }
#line 66 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::Type_FieldListCalculation::toString() const { return "field_list_calculation"_cs; }
#line 7852 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 68 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::Type_FieldListCalculation::dbprint(std::ostream & out) const { out << "field_list_calculation"; }
#line 7855 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_FieldListCalculation::operator==(IR::Type_FieldListCalculation const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_FieldListCalculation::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_FieldListCalculation::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_FieldListCalculation::Type_FieldListCalculation(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_FieldListCalculation::fromJSON(JSONLoader & json) { return new Type_FieldListCalculation(json); }
bool IR::Type_FieldListCalculation::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_FieldListCalculation::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_FieldListCalculation::operator==(IR::Node const & a) const { return a == *this; }
#line 71 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::Type_Meter::toString() const { return "meter"_cs; }
#line 7875 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 73 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::Type_Meter::dbprint(std::ostream & out) const { out << "meter"; }
#line 7878 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Meter::operator==(IR::Type_Meter const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_Meter::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_Meter::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_Meter::Type_Meter(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_Meter::fromJSON(JSONLoader & json) { return new Type_Meter(json); }
bool IR::Type_Meter::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_Meter::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Meter::operator==(IR::Node const & a) const { return a == *this; }
#line 76 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::Type_Register::toString() const { return "register"_cs; }
#line 7898 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 78 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::Type_Register::dbprint(std::ostream & out) const { out << "register"; }
#line 7901 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_Register::operator==(IR::Type_Register const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_Register::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_Register::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_Register::Type_Register(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_Register::fromJSON(JSONLoader & json) { return new Type_Register(json); }
bool IR::Type_Register::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_Register::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_Register::operator==(IR::Node const & a) const { return a == *this; }
#line 81 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::Type_AnyTable::toString() const { return "table"_cs; }
#line 7921 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 83 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::Type_AnyTable::dbprint(std::ostream & out) const { out << "table"; }
#line 7924 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Type_AnyTable::operator==(IR::Type_AnyTable const & a) const {
        return Type_Base::operator==(static_cast<const Type_Base &>(a));
    }
bool IR::Type_AnyTable::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Base::equiv(a_)) return false;
        return true;
    }
void IR::Type_AnyTable::toJSON(JSONGenerator & json) const {
    Type_Base::toJSON(json);
}
IR::Type_AnyTable::Type_AnyTable(JSONLoader & json) : Type_Base(json) {
}
IR::Node * IR::Type_AnyTable::fromJSON(JSONLoader & json) { return new Type_AnyTable(json); }
bool IR::Type_AnyTable::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Type_AnyTable::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Type_AnyTable::operator==(IR::Node const & a) const { return a == *this; }
#line 92 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> const & IR::HeaderOrMetadata::getAnnotations() const { return annotations; }
#line 7944 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 93 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> & IR::HeaderOrMetadata::getAnnotations() { return annotations; }
#line 7947 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 94 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::HeaderOrMetadata::HeaderOrMetadata(IR::ID n, const IR::Type_StructLike* t) : type_name(t->name), name(n), type(t) {}
#line 7950 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 96 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::HeaderOrMetadata::dbprint(std::ostream & out) const { out << type_name << ' ' << name; }
#line 7953 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::HeaderOrMetadata::operator==(IR::HeaderOrMetadata const & a) const {
        return this->typeId() == a.typeId()
        && type_name == a.type_name
        && name == a.name
        && annotations == a.annotations
        && type == a.type;
    }
bool IR::HeaderOrMetadata::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const HeaderOrMetadata &>(a_);
        return type_name == a.type_name
        && name == a.name
        && annotations.equiv(a.annotations)
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr);
    }
void IR::HeaderOrMetadata::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
}
void IR::HeaderOrMetadata::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    v.visit(type, "type");
}
void IR::HeaderOrMetadata::validate() const {
#line 7983 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate(); }
void IR::HeaderOrMetadata::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " type_name=" << type_name;
        out << " name=" << name;
}
void IR::HeaderOrMetadata::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("type_name", type_name);
    json.emit("name", name);
    json.emit("annotations", annotations);
    if (type != nullptr) json.emit("type", type);
}
IR::HeaderOrMetadata::HeaderOrMetadata(JSONLoader & json) : Node(json) {
    json.load("type_name", type_name);
    json.load("name", name);
    json.load("annotations", annotations);
    json.load("type", type);
}
bool IR::HeaderOrMetadata::operator==(IR::Node const & a) const { return a == *this; }
#line 100 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Header::Header(IR::ID n, const IR::Type_Header* t) : HeaderOrMetadata(n, t) {}
#line 8007 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Header::operator==(IR::Header const & a) const {
        return HeaderOrMetadata::operator==(static_cast<const HeaderOrMetadata &>(a));
    }
bool IR::Header::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!HeaderOrMetadata::equiv(a_)) return false;
        return true;
    }
void IR::Header::toJSON(JSONGenerator & json) const {
    HeaderOrMetadata::toJSON(json);
}
IR::Header::Header(JSONLoader & json) : HeaderOrMetadata(json) {
}
IR::Node * IR::Header::fromJSON(JSONLoader & json) { return new Header(json); }
bool IR::Header::operator==(IR::HeaderOrMetadata const & a) const { return a == *this; }
bool IR::Header::operator==(IR::Node const & a) const { return a == *this; }
#line 106 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::HeaderStack::HeaderStack(IR::ID n, const IR::Type_Header* t, int sz) : HeaderOrMetadata(n, t), size(sz) {}
#line 8026 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::HeaderStack::operator==(IR::HeaderStack const & a) const {
        return HeaderOrMetadata::operator==(static_cast<const HeaderOrMetadata &>(a))
        && size == a.size;
    }
bool IR::HeaderStack::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!HeaderOrMetadata::equiv(a_)) return false;
        auto &a = static_cast<const HeaderStack &>(a_);
        return size == a.size;
    }
void IR::HeaderStack::dump_fields(std::ostream & out) const {
    HeaderOrMetadata::dump_fields(out);
        out << " size=" << size;
}
void IR::HeaderStack::toJSON(JSONGenerator & json) const {
    HeaderOrMetadata::toJSON(json);
    json.emit("size", size);
}
IR::HeaderStack::HeaderStack(JSONLoader & json) : HeaderOrMetadata(json) {
    json.load("size", size);
}
IR::Node * IR::HeaderStack::fromJSON(JSONLoader & json) { return new HeaderStack(json); }
bool IR::HeaderStack::operator==(IR::HeaderOrMetadata const & a) const { return a == *this; }
bool IR::HeaderStack::operator==(IR::Node const & a) const { return a == *this; }
#line 115 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::v1HeaderType::v1HeaderType(IR::Type_Struct const * m, IR::Type_Header const * h) : v1HeaderType(Util::SourceInfo(), m->name, m, h) {}
#line 8053 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 117 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::v1HeaderType::dbprint(std::ostream & out) const { out << "header " << name; }
#line 8056 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::v1HeaderType::operator==(IR::v1HeaderType const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && as_metadata == a.as_metadata
        && as_header == a.as_header;
    }
bool IR::v1HeaderType::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const v1HeaderType &>(a_);
        return name == a.name
        && (as_metadata ? a.as_metadata ? as_metadata->equiv(*a.as_metadata) : false : a.as_metadata == nullptr)
        && (as_header ? a.as_header ? as_header->equiv(*a.as_header) : false : a.as_header == nullptr);
    }
void IR::v1HeaderType::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(as_metadata, "as_metadata");
    v.visit(as_header, "as_header");
}
void IR::v1HeaderType::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(as_metadata, "as_metadata");
    v.visit(as_header, "as_header");
}
void IR::v1HeaderType::validate() const {
#line 8084 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(as_metadata); }
void IR::v1HeaderType::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
}
void IR::v1HeaderType::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("as_metadata", as_metadata);
    if (as_header != nullptr) json.emit("as_header", as_header);
}
IR::v1HeaderType::v1HeaderType(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("as_metadata", as_metadata);
    json.load("as_header", as_header);
}
IR::Node * IR::v1HeaderType::fromJSON(JSONLoader & json) { return new v1HeaderType(json); }
bool IR::v1HeaderType::operator==(IR::Node const & a) const { return a == *this; }
#line 121 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Metadata::Metadata(IR::ID n, const IR::Type_StructLike* t) : HeaderOrMetadata(n, t) {}
#line 8106 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Metadata::operator==(IR::Metadata const & a) const {
        return HeaderOrMetadata::operator==(static_cast<const HeaderOrMetadata &>(a));
    }
bool IR::Metadata::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!HeaderOrMetadata::equiv(a_)) return false;
        return true;
    }
void IR::Metadata::toJSON(JSONGenerator & json) const {
    HeaderOrMetadata::toJSON(json);
}
IR::Metadata::Metadata(JSONLoader & json) : HeaderOrMetadata(json) {
}
IR::Node * IR::Metadata::fromJSON(JSONLoader & json) { return new Metadata(json); }
bool IR::Metadata::operator==(IR::HeaderOrMetadata const & a) const { return a == *this; }
bool IR::Metadata::operator==(IR::Node const & a) const { return a == *this; }
bool IR::HeaderRef::operator==(IR::HeaderRef const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a));
    }
bool IR::HeaderRef::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        return true;
    }
void IR::HeaderRef::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
}
IR::HeaderRef::HeaderRef(JSONLoader & json) : Expression(json) {
}
bool IR::HeaderRef::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::HeaderRef::operator==(IR::Node const & a) const { return a == *this; }
#line 132 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
const IR::HeaderOrMetadata *IR::ConcreteHeaderRef::baseRef() const { return ref; }
#line 8140 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 133 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::ConcreteHeaderRef::toString() const { return ref->name; }
#line 8143 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 134 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::ConcreteHeaderRef::dbprint(std::ostream & out) const { out << ref->name; }
#line 8146 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ConcreteHeaderRef::operator==(IR::ConcreteHeaderRef const & a) const {
        return HeaderRef::operator==(static_cast<const HeaderRef &>(a))
        && ref == a.ref;
    }
bool IR::ConcreteHeaderRef::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!HeaderRef::equiv(a_)) return false;
        auto &a = static_cast<const ConcreteHeaderRef &>(a_);
        return (ref ? a.ref ? ref->equiv(*a.ref) : false : a.ref == nullptr);
    }
void IR::ConcreteHeaderRef::visit_children(Visitor & v, char const * n) {
    (void)n;
    HeaderRef::visit_children(v, n);
    v.visit(ref, "ref");
}
void IR::ConcreteHeaderRef::visit_children(Visitor & v, char const * n) const {
    (void)n;
    HeaderRef::visit_children(v, n);
    v.visit(ref, "ref");
}
void IR::ConcreteHeaderRef::validate() const {
#line 8168 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(ref); }
void IR::ConcreteHeaderRef::toJSON(JSONGenerator & json) const {
    HeaderRef::toJSON(json);
    json.emit("ref", ref);
}
IR::ConcreteHeaderRef::ConcreteHeaderRef(JSONLoader & json) : HeaderRef(json) {
    json.load("ref", ref);
}
IR::Node * IR::ConcreteHeaderRef::fromJSON(JSONLoader & json) { return new ConcreteHeaderRef(json); }
bool IR::ConcreteHeaderRef::operator==(IR::HeaderRef const & a) const { return a == *this; }
bool IR::ConcreteHeaderRef::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::ConcreteHeaderRef::operator==(IR::Node const & a) const { return a == *this; }
#line 144 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
const IR::Expression *IR::HeaderStackItemRef::base() const { return base_; }
#line 8184 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// Returns `nullptr` if the base is not `HeaderOrMetadata` (e.g. when this
/// is stack ref of an expression such as `lookahead`).
#line 147 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
const IR::HeaderOrMetadata *IR::HeaderStackItemRef::baseRef() const {
        auto hdrRef = base_->to<HeaderRef>();
        return hdrRef ? hdrRef->baseRef() : nullptr;
    }
#line 8192 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 151 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
const IR::Expression *IR::HeaderStackItemRef::index() const { return index_; }
#line 8195 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 152 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::HeaderStackItemRef::set_base(const IR::Expression* b) { base_ = b; }
#line 8198 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 153 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::HeaderStackItemRef::toString() const { return base_->toString() + "[" + index_->toString() + "]"; }
#line 8201 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::HeaderStackItemRef::operator==(IR::HeaderStackItemRef const & a) const {
        return HeaderRef::operator==(static_cast<const HeaderRef &>(a))
        && base_ == a.base_
        && index_ == a.index_;
    }
bool IR::HeaderStackItemRef::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!HeaderRef::equiv(a_)) return false;
        auto &a = static_cast<const HeaderStackItemRef &>(a_);
        return (base_ ? a.base_ ? base_->equiv(*a.base_) : false : a.base_ == nullptr)
        && (index_ ? a.index_ ? index_->equiv(*a.index_) : false : a.index_ == nullptr);
    }
void IR::HeaderStackItemRef::visit_children(Visitor & v, char const * n) {
    (void)n;
    HeaderRef::visit_children(v, n);
    v.visit(base_, "base_");
    v.visit(index_, "index_");
}
void IR::HeaderStackItemRef::visit_children(Visitor & v, char const * n) const {
    (void)n;
    HeaderRef::visit_children(v, n);
    v.visit(base_, "base_");
    v.visit(index_, "index_");
}
void IR::HeaderStackItemRef::validate() const {
#line 8227 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(base_);
        CHECK_NULL(index_); }
void IR::HeaderStackItemRef::toJSON(JSONGenerator & json) const {
    HeaderRef::toJSON(json);
    json.emit("base_", base_);
    json.emit("index_", index_);
}
IR::HeaderStackItemRef::HeaderStackItemRef(JSONLoader & json) : HeaderRef(json) {
    json.load("base_", base_);
    json.load("index_", index_);
}
IR::Node * IR::HeaderStackItemRef::fromJSON(JSONLoader & json) { return new HeaderStackItemRef(json); }
bool IR::HeaderStackItemRef::operator==(IR::HeaderRef const & a) const { return a == *this; }
bool IR::HeaderStackItemRef::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::HeaderStackItemRef::operator==(IR::Node const & a) const { return a == *this; }
#line 160 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::If::visit_children(Visitor & v, char const * n) {
        v.visit(pred, "pred");
        SplitFlowVisit<Vector<Expression>>(v, ifTrue, ifFalse).run_visit();
        Expression::visit_children(v, n);
    }
#line 160 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::If::visit_children(Visitor & v, char const * n) const {
        v.visit(pred, "pred");
        SplitFlowVisit<Vector<Expression>>(v, ifTrue, ifFalse).run_visit();
        Expression::visit_children(v, n);
    }
#line 8256 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::If::operator==(IR::If const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && pred == a.pred
        && ifTrue == a.ifTrue
        && ifFalse == a.ifFalse;
    }
bool IR::If::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const If &>(a_);
        return (pred ? a.pred ? pred->equiv(*a.pred) : false : a.pred == nullptr)
        && (ifTrue ? a.ifTrue ? ifTrue->equiv(*a.ifTrue) : false : a.ifTrue == nullptr)
        && (ifFalse ? a.ifFalse ? ifFalse->equiv(*a.ifFalse) : false : a.ifFalse == nullptr);
    }
void IR::If::validate() const {
#line 8272 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(pred); }
void IR::If::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("pred", pred);
    if (ifTrue != nullptr) json.emit("ifTrue", ifTrue);
    if (ifFalse != nullptr) json.emit("ifFalse", ifFalse);
}
IR::If::If(JSONLoader & json) : Expression(json) {
    json.load("pred", pred);
    json.load("ifTrue", ifTrue);
    json.load("ifFalse", ifFalse);
}
IR::Node * IR::If::fromJSON(JSONLoader & json) { return new If(json); }
bool IR::If::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::If::operator==(IR::Node const & a) const { return a == *this; }
#line 172 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::NamedCond::NamedCond(IR::If const & i) : If(i), name(unique_name()) {}
#line 8291 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 173 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
bool IR::NamedCond::operator==(IR::NamedCond const & a) const { return If::operator==(static_cast<const If &>(a)); }
#line 8294 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::NamedCond::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!If::equiv(a_)) return false;
        auto &a = static_cast<const NamedCond &>(a_);
        return name == a.name;
    }
void IR::NamedCond::dump_fields(std::ostream & out) const {
    If::dump_fields(out);
        out << " name=" << name;
}
void IR::NamedCond::toJSON(JSONGenerator & json) const {
    If::toJSON(json);
    json.emit("name", name);
}
IR::NamedCond::NamedCond(JSONLoader & json) : If(json) {
    json.load("name", name);
}
IR::Node * IR::NamedCond::fromJSON(JSONLoader & json) { return new NamedCond(json); }
bool IR::NamedCond::operator==(IR::If const & a) const { return a == *this; }
bool IR::NamedCond::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::NamedCond::operator==(IR::Node const & a) const { return a == *this; }


bool IR::Apply::operator==(IR::Apply const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && name == a.name
        && actions == a.actions
        && position == a.position;
    }
bool IR::Apply::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const Apply &>(a_);
        return name == a.name
        && actions.equiv(a.actions)
        && position.equiv(a.position);
    }
void IR::Apply::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    actions.visit_children(v, "actions");
    position.visit_children(v, "position");
}
void IR::Apply::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    actions.visit_children(v, "actions");
    position.visit_children(v, "position");
}
void IR::Apply::validate() const {
#line 8345 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        actions.validate();
        position.validate(); }
void IR::Apply::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " name=" << name;
}
void IR::Apply::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("name", name);
    json.emit("actions", actions);
    json.emit("position", position);
}
IR::Apply::Apply(JSONLoader & json) : Expression(json) {
    json.load("name", name);
    json.load("actions", actions);
    json.load("position", position);
}
IR::Node * IR::Apply::fromJSON(JSONLoader & json) { return new Apply(json); }
bool IR::Apply::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Apply::operator==(IR::Node const & a) const { return a == *this; }
#line 190 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Primitive::Primitive(cstring n, const IR::Vector<IR::Expression>* l) : name(n) {
        if (l) for (auto e : *l) operands.push_back(e); }
#line 8370 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 192 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Primitive::Primitive(Util::SourceInfo si, cstring n, const IR::Vector<IR::Expression>* l) : Operation(si), name(n) {
        if (l) for (auto e : *l) operands.push_back(e); }
#line 8374 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 194 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Primitive::Primitive(cstring n, const IR::Expression* a1) : name(n) {
        operands.push_back(a1); }
#line 8378 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 196 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Primitive::Primitive(Util::SourceInfo si, cstring n, const IR::Expression* a1) : Operation(si), name(n) {
        operands.push_back(a1); }
#line 8382 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 198 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Primitive::Primitive(cstring n, const IR::Expression* a1, const IR::Expression* a2) : name(n) {
        operands.push_back(a1); operands.push_back(a2); }
#line 8386 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 200 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Primitive::Primitive(Util::SourceInfo si, cstring n, const IR::Expression* a1, const IR::Expression* a2) : Operation(si), name(n) {
        operands.push_back(a1); operands.push_back(a2); }
#line 8390 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 203 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Primitive::Primitive(cstring n, const IR::Expression* a1, const IR::Vector<IR::Expression>* a2) : name(n) {
        operands.push_back(a1);
        if (a2) for (auto e : *a2) operands.push_back(e); }
#line 8395 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 206 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Primitive::Primitive(Util::SourceInfo si, cstring n, const IR::Expression* a1, const IR::Vector<IR::Expression>* a2) : Operation(si), name(n) {
        operands.push_back(a1);
        if (a2) for (auto e : *a2) operands.push_back(e); }
#line 8400 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 210 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Primitive::Primitive(cstring n, const IR::Expression* a1, const IR::Expression* a2, const IR::Expression* a3) : name(n) {
        operands.push_back(a1); operands.push_back(a2); operands.push_back(a3); }
#line 8404 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 212 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Primitive::Primitive(Util::SourceInfo si, cstring n, const IR::Expression* a1, const IR::Expression* a2, const IR::Expression* a3) : Operation(si), name(n) {
        operands.push_back(a1); operands.push_back(a2); operands.push_back(a3); }
#line 8408 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(Primitive, , )
bool IR::Primitive::operator==(IR::Primitive const & a) const {
        return Operation::operator==(static_cast<const Operation &>(a))
        && name == a.name
        && operands == a.operands;
    }
bool IR::Primitive::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation::equiv(a_)) return false;
        auto &a = static_cast<const Primitive &>(a_);
        return name == a.name
        && operands.equiv(a.operands);
    }
void IR::Primitive::visit_children(Visitor & v, char const * n) {
    (void)n;
    Operation::visit_children(v, n);
    operands.visit_children(v, "operands");
}
void IR::Primitive::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Operation::visit_children(v, n);
    operands.visit_children(v, "operands");
}
void IR::Primitive::validate() const {
#line 8433 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        operands.validate(); }
void IR::Primitive::dump_fields(std::ostream & out) const {
    Operation::dump_fields(out);
        out << " name=" << name;
}
void IR::Primitive::toJSON(JSONGenerator & json) const {
    Operation::toJSON(json);
    json.emit("name", name);
    json.emit("operands", operands);
}
IR::Primitive::Primitive(JSONLoader & json) : Operation(json) {
    json.load("name", name);
    json.load("operands", operands);
}
IR::Node * IR::Primitive::fromJSON(JSONLoader & json) { return new Primitive(json); }
bool IR::Primitive::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::Primitive::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Primitive::operator==(IR::Node const & a) const { return a == *this; }
#line 229 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> const & IR::FieldList::getAnnotations() const { return annotations; }
#line 8455 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 230 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> & IR::FieldList::getAnnotations() { return annotations; }
#line 8458 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::FieldList::operator==(IR::FieldList const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && payload == a.payload
        && annotations == a.annotations
        && fields == a.fields;
    }
bool IR::FieldList::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const FieldList &>(a_);
        return name == a.name
        && payload == a.payload
        && annotations.equiv(a.annotations)
        && fields.equiv(a.fields);
    }
void IR::FieldList::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    fields.visit_children(v, "fields");
}
void IR::FieldList::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    fields.visit_children(v, "fields");
}
void IR::FieldList::validate() const {
#line 8488 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        fields.validate(); }
void IR::FieldList::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
        out << " payload=" << payload;
}
void IR::FieldList::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("payload", payload);
    json.emit("annotations", annotations);
    json.emit("fields", fields);
}
IR::FieldList::FieldList(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("payload", payload);
    json.load("annotations", annotations);
    json.load("fields", fields);
}
IR::Node * IR::FieldList::fromJSON(JSONLoader & json) { return new FieldList(json); }
bool IR::FieldList::operator==(IR::Node const & a) const { return a == *this; }
#line 240 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> const & IR::FieldListCalculation::getAnnotations() const { return annotations; }
#line 8514 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 241 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> & IR::FieldListCalculation::getAnnotations() { return annotations; }
#line 8517 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::FieldListCalculation::operator==(IR::FieldListCalculation const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && input == a.input
        && input_fields == a.input_fields
        && algorithm == a.algorithm
        && output_width == a.output_width
        && annotations == a.annotations;
    }
bool IR::FieldListCalculation::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const FieldListCalculation &>(a_);
        return name == a.name
        && (input ? a.input ? input->equiv(*a.input) : false : a.input == nullptr)
        && (input_fields ? a.input_fields ? input_fields->equiv(*a.input_fields) : false : a.input_fields == nullptr)
        && (algorithm ? a.algorithm ? algorithm->equiv(*a.algorithm) : false : a.algorithm == nullptr)
        && output_width == a.output_width
        && annotations.equiv(a.annotations);
    }
void IR::FieldListCalculation::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(input, "input");
    v.visit(input_fields, "input_fields");
    v.visit(algorithm, "algorithm");
    annotations.visit_children(v, "annotations");
}
void IR::FieldListCalculation::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(input, "input");
    v.visit(input_fields, "input_fields");
    v.visit(algorithm, "algorithm");
    annotations.visit_children(v, "annotations");
}
void IR::FieldListCalculation::validate() const {
#line 8555 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate(); }
void IR::FieldListCalculation::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
        out << " output_width=" << output_width;
}
void IR::FieldListCalculation::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    if (input != nullptr) json.emit("input", input);
    if (input_fields != nullptr) json.emit("input_fields", input_fields);
    if (algorithm != nullptr) json.emit("algorithm", algorithm);
    json.emit("output_width", output_width);
    json.emit("annotations", annotations);
}
IR::FieldListCalculation::FieldListCalculation(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("input", input);
    json.load("input_fields", input_fields);
    json.load("algorithm", algorithm);
    json.load("output_width", output_width);
    json.load("annotations", annotations);
}
IR::Node * IR::FieldListCalculation::fromJSON(JSONLoader & json) { return new FieldListCalculation(json); }
bool IR::FieldListCalculation::operator==(IR::Node const & a) const { return a == *this; }
#line 251 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::CalculatedField::update_or_verify::update_or_verify() { }
#line 8584 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

bool IR::CalculatedField::update_or_verify::operator==(IR::CalculatedField::update_or_verify const & a) const {
        return update == a.update
        && name == a.name
        && cond == a.cond;
    }
void IR::CalculatedField::update_or_verify::toJSON(JSONGenerator & json) const {
    json.emit("update", update);
    json.emit("name", name);
    json.emit("cond", cond);
}
IR::CalculatedField::update_or_verify::update_or_verify(JSONLoader & json)  {
    json.load("update", update);
    json.load("name", name);
    json.load("cond", cond);
}
IR::CalculatedField::update_or_verify * IR::CalculatedField::update_or_verify::fromJSON(JSONLoader & json) { return new update_or_verify(json); }
#line 255 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> const & IR::CalculatedField::getAnnotations() const { return annotations; }
#line 8604 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 256 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> & IR::CalculatedField::getAnnotations() { return annotations; }
#line 8607 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 257 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::CalculatedField::visit_children(Visitor & v, char const * n) {
        (void)n;
        v.visit(field, "field");
        for (auto &s : specs) v.visit(s.cond, s.name.name.c_str());
        v.visit(annotations, "annotations"); }
#line 257 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::CalculatedField::visit_children(Visitor & v, char const * n) const {
        (void)n;
        v.visit(field, "field");
        for (auto &s : specs) v.visit(s.cond, s.name.name.c_str());
        v.visit(annotations, "annotations"); }
#line 8620 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::CalculatedField::operator==(IR::CalculatedField const & a) const {
        return this->typeId() == a.typeId()
        && field == a.field
        && specs == a.specs
        && annotations == a.annotations;
    }
bool IR::CalculatedField::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const CalculatedField &>(a_);
        return (field ? a.field ? field->equiv(*a.field) : false : a.field == nullptr)
        && specs == a.specs
        && annotations.equiv(a.annotations);
    }
void IR::CalculatedField::validate() const {
#line 8636 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate(); }
void IR::CalculatedField::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    if (field != nullptr) json.emit("field", field);
    json.emit("specs", specs);
    json.emit("annotations", annotations);
}
IR::CalculatedField::CalculatedField(JSONLoader & json) : Node(json) {
    json.load("field", field);
    json.load("specs", specs);
    json.load("annotations", annotations);
}
IR::Node * IR::CalculatedField::fromJSON(JSONLoader & json) { return new CalculatedField(json); }
bool IR::CalculatedField::operator==(IR::Node const & a) const { return a == *this; }
#line 267 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> const & IR::ParserValueSet::getAnnotations() const { return annotations; }
#line 8654 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 268 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> & IR::ParserValueSet::getAnnotations() { return annotations; }
#line 8657 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 269 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::ParserValueSet::dbprint(std::ostream & out) const { out << node_type_name() << " " << name; }
#line 8660 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 270 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::ParserValueSet::toString() const { return node_type_name() + " " + name; }
#line 8663 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ParserValueSet::operator==(IR::ParserValueSet const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && annotations == a.annotations;
    }
bool IR::ParserValueSet::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const ParserValueSet &>(a_);
        return name == a.name
        && annotations.equiv(a.annotations);
    }
void IR::ParserValueSet::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
}
void IR::ParserValueSet::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
}
void IR::ParserValueSet::validate() const {
#line 8687 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate(); }
void IR::ParserValueSet::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
}
void IR::ParserValueSet::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("annotations", annotations);
}
IR::ParserValueSet::ParserValueSet(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("annotations", annotations);
}
IR::Node * IR::ParserValueSet::fromJSON(JSONLoader & json) { return new ParserValueSet(json); }
bool IR::ParserValueSet::operator==(IR::Node const & a) const { return a == *this; }
bool IR::CaseEntry::operator==(IR::CaseEntry const & a) const {
        return this->typeId() == a.typeId()
        && values == a.values
        && action == a.action;
    }
bool IR::CaseEntry::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const CaseEntry &>(a_);
        return values == a.values
        && action == a.action;
    }
void IR::CaseEntry::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " action=" << action;
}
void IR::CaseEntry::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("values", values);
    json.emit("action", action);
}
IR::CaseEntry::CaseEntry(JSONLoader & json) : Node(json) {
    json.load("values", values);
    json.load("action", action);
}
IR::Node * IR::CaseEntry::fromJSON(JSONLoader & json) { return new CaseEntry(json); }
bool IR::CaseEntry::operator==(IR::Node const & a) const { return a == *this; }
#line 287 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> const & IR::V1Parser::getAnnotations() const { return annotations; }
#line 8734 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 288 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> & IR::V1Parser::getAnnotations() { return annotations; }
#line 8737 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 289 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::V1Parser::toString() const { return node_type_name() + " " + name; }
#line 8740 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::V1Parser::operator==(IR::V1Parser const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && stmts == a.stmts
        && select == a.select
        && cases == a.cases
        && default_return == a.default_return
        && parse_error == a.parse_error
        && drop == a.drop
        && annotations == a.annotations;
    }
bool IR::V1Parser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const V1Parser &>(a_);
        return name == a.name
        && stmts.equiv(a.stmts)
        && (select ? a.select ? select->equiv(*a.select) : false : a.select == nullptr)
        && (cases ? a.cases ? cases->equiv(*a.cases) : false : a.cases == nullptr)
        && default_return == a.default_return
        && parse_error == a.parse_error
        && drop == a.drop
        && annotations.equiv(a.annotations);
    }
void IR::V1Parser::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    stmts.visit_children(v, "stmts");
    v.visit(select, "select");
    v.visit(cases, "cases");
    annotations.visit_children(v, "annotations");
}
void IR::V1Parser::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    stmts.visit_children(v, "stmts");
    v.visit(select, "select");
    v.visit(cases, "cases");
    annotations.visit_children(v, "annotations");
}
void IR::V1Parser::validate() const {
#line 8782 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        stmts.validate();
        annotations.validate(); }
void IR::V1Parser::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
        out << " default_return=" << default_return;
        out << " parse_error=" << parse_error;
        out << " drop=" << drop;
}
void IR::V1Parser::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("stmts", stmts);
    if (select != nullptr) json.emit("select", select);
    if (cases != nullptr) json.emit("cases", cases);
    json.emit("default_return", default_return);
    json.emit("parse_error", parse_error);
    json.emit("drop", drop);
    json.emit("annotations", annotations);
}
IR::V1Parser::V1Parser(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("stmts", stmts);
    json.load("select", select);
    json.load("cases", cases);
    json.load("default_return", default_return);
    json.load("parse_error", parse_error);
    json.load("drop", drop);
    json.load("annotations", annotations);
}
IR::Node * IR::V1Parser::fromJSON(JSONLoader & json) { return new V1Parser(json); }
bool IR::V1Parser::operator==(IR::Node const & a) const { return a == *this; }
bool IR::ParserException::operator==(IR::ParserException const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::ParserException::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::ParserException::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::ParserException::ParserException(JSONLoader & json) : Node(json) {
}
IR::Node * IR::ParserException::fromJSON(JSONLoader & json) { return new ParserException(json); }
bool IR::ParserException::operator==(IR::Node const & a) const { return a == *this; }
#line 297 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::ID IR::Attached::Name() const { return name; }
#line 8833 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 299 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
const IR::Type *IR::Attached::getType() const { return Type_Unknown::get(); }
#line 8836 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 300 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> const & IR::Attached::getAnnotations() const { return annotations; }
#line 8839 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 301 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> & IR::Attached::getAnnotations() { return annotations; }
#line 8842 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 302 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
bool IR::Attached::indexed() const { return false; }
#line 8845 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 303 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Attached * IR::Attached::clone_rename(char const * ext) const {
        Attached *rv = clone();
        rv->name = ID(Util::SourceInfo(), rv->name.name + ext);
        return rv; }
#line 8851 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 307 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::Attached::dbprint(std::ostream & out) const { out << node_type_name() << " " << name; }
#line 8854 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 308 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::Attached::toString() const { return node_type_name() + " " + name; }
#line 8857 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Attached::operator==(IR::Attached const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && annotations == a.annotations;
    }
bool IR::Attached::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Attached &>(a_);
        return name == a.name
        && annotations.equiv(a.annotations);
    }
void IR::Attached::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
}
void IR::Attached::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
}
void IR::Attached::validate() const {
#line 8881 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate(); }
void IR::Attached::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
}
void IR::Attached::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("annotations", annotations);
}
IR::Attached::Attached(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("annotations", annotations);
}
bool IR::Attached::operator==(IR::Node const & a) const { return a == *this; }
#line 316 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
bool IR::Stateful::indexed() const { return !direct; }
#line 8900 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

bool IR::Stateful::operator==(IR::Stateful const & a) const {
        return Attached::operator==(static_cast<const Attached &>(a))
        && table == a.table
        && direct == a.direct
        && saturating == a.saturating
        && instance_count == a.instance_count;
    }
bool IR::Stateful::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Attached::equiv(a_)) return false;
        auto &a = static_cast<const Stateful &>(a_);
        return table == a.table
        && direct == a.direct
        && saturating == a.saturating
        && instance_count == a.instance_count;
    }
void IR::Stateful::dump_fields(std::ostream & out) const {
    Attached::dump_fields(out);
        out << " table=" << table;
        out << " direct=" << direct;
        out << " saturating=" << saturating;
        out << " instance_count=" << instance_count;
}
void IR::Stateful::toJSON(JSONGenerator & json) const {
    Attached::toJSON(json);
    json.emit("table", table);
    json.emit("direct", direct);
    json.emit("saturating", saturating);
    json.emit("instance_count", instance_count);
}
IR::Stateful::Stateful(JSONLoader & json) : Attached(json) {
    json.load("table", table);
    json.load("direct", direct);
    json.load("saturating", saturating);
    json.load("instance_count", instance_count);
}
bool IR::Stateful::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::Stateful::operator==(IR::Node const & a) const { return a == *this; }
#line 322 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::CounterOrMeter::settype(cstring t) {
        if (strcasecmp(t.c_str(), "packets") == 0) type = CounterType::PACKETS;
        else if (strcasecmp(t.c_str(), "bytes") == 0) type = CounterType::BYTES;
        else if (strcasecmp(t.c_str(), "packets_and_bytes") == 0 ||
                 strcasecmp(t.c_str(), "PacketAndBytes") == 0) type = CounterType::BOTH;
        else error(ErrorType::ERR_UNKNOWN, "%s: Unknown type %s", srcInfo, t); }
#line 8947 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

bool IR::CounterOrMeter::operator==(IR::CounterOrMeter const & a) const {
        return Stateful::operator==(static_cast<const Stateful &>(a))
        && type == a.type;
    }
bool IR::CounterOrMeter::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Stateful::equiv(a_)) return false;
        auto &a = static_cast<const CounterOrMeter &>(a_);
        return type == a.type;
    }
void IR::CounterOrMeter::dump_fields(std::ostream & out) const {
    Stateful::dump_fields(out);
        out << " type=" << type;
}
void IR::CounterOrMeter::toJSON(JSONGenerator & json) const {
    Stateful::toJSON(json);
    json.emit("type", type);
}
IR::CounterOrMeter::CounterOrMeter(JSONLoader & json) : Stateful(json) {
    json.load("type", type);
}
bool IR::CounterOrMeter::operator==(IR::Stateful const & a) const { return a == *this; }
bool IR::CounterOrMeter::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::CounterOrMeter::operator==(IR::Node const & a) const { return a == *this; }
#line 333 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
char const * IR::Counter::kind() const { return "stats"; }
#line 8975 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 334 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Type const * IR::Counter::getType() const { return Type_Counter::get(); }
#line 8978 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Counter::operator==(IR::Counter const & a) const {
        return CounterOrMeter::operator==(static_cast<const CounterOrMeter &>(a))
        && max_width == a.max_width
        && min_width == a.min_width;
    }
bool IR::Counter::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!CounterOrMeter::equiv(a_)) return false;
        auto &a = static_cast<const Counter &>(a_);
        return max_width == a.max_width
        && min_width == a.min_width;
    }
void IR::Counter::dump_fields(std::ostream & out) const {
    CounterOrMeter::dump_fields(out);
        out << " max_width=" << max_width;
        out << " min_width=" << min_width;
}
void IR::Counter::toJSON(JSONGenerator & json) const {
    CounterOrMeter::toJSON(json);
    json.emit("max_width", max_width);
    json.emit("min_width", min_width);
}
IR::Counter::Counter(JSONLoader & json) : CounterOrMeter(json) {
    json.load("max_width", max_width);
    json.load("min_width", min_width);
}
IR::Node * IR::Counter::fromJSON(JSONLoader & json) { return new Counter(json); }
bool IR::Counter::operator==(IR::CounterOrMeter const & a) const { return a == *this; }
bool IR::Counter::operator==(IR::Stateful const & a) const { return a == *this; }
bool IR::Counter::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::Counter::operator==(IR::Node const & a) const { return a == *this; }
#line 341 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
char const * IR::Meter::kind() const { return "meter"; }
#line 9012 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 342 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
const IR::Type *IR::Meter::getType() const { return Type_Meter::get(); }
#line 9015 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Meter::operator==(IR::Meter const & a) const {
        return CounterOrMeter::operator==(static_cast<const CounterOrMeter &>(a))
        && result == a.result
        && pre_color == a.pre_color
        && implementation == a.implementation;
    }
bool IR::Meter::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!CounterOrMeter::equiv(a_)) return false;
        auto &a = static_cast<const Meter &>(a_);
        return (result ? a.result ? result->equiv(*a.result) : false : a.result == nullptr)
        && (pre_color ? a.pre_color ? pre_color->equiv(*a.pre_color) : false : a.pre_color == nullptr)
        && implementation == a.implementation;
    }
void IR::Meter::visit_children(Visitor & v, char const * n) {
    (void)n;
    CounterOrMeter::visit_children(v, n);
    v.visit(result, "result");
    v.visit(pre_color, "pre_color");
}
void IR::Meter::visit_children(Visitor & v, char const * n) const {
    (void)n;
    CounterOrMeter::visit_children(v, n);
    v.visit(result, "result");
    v.visit(pre_color, "pre_color");
}
void IR::Meter::dump_fields(std::ostream & out) const {
    CounterOrMeter::dump_fields(out);
        out << " implementation=" << implementation;
}
void IR::Meter::toJSON(JSONGenerator & json) const {
    CounterOrMeter::toJSON(json);
    if (result != nullptr) json.emit("result", result);
    if (pre_color != nullptr) json.emit("pre_color", pre_color);
    json.emit("implementation", implementation);
}
IR::Meter::Meter(JSONLoader & json) : CounterOrMeter(json) {
    json.load("result", result);
    json.load("pre_color", pre_color);
    json.load("implementation", implementation);
}
IR::Node * IR::Meter::fromJSON(JSONLoader & json) { return new Meter(json); }
bool IR::Meter::operator==(IR::CounterOrMeter const & a) const { return a == *this; }
bool IR::Meter::operator==(IR::Stateful const & a) const { return a == *this; }
bool IR::Meter::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::Meter::operator==(IR::Node const & a) const { return a == *this; }

#line 350 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
char const * IR::Register::kind() const { return "register"; }
#line 9065 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 351 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
const IR::Type *IR::Register::getType() const { return Type_Register::get(); }
#line 9068 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Register::operator==(IR::Register const & a) const {
        return Stateful::operator==(static_cast<const Stateful &>(a))
        && layout == a.layout
        && width == a.width
        && signed_ == a.signed_;
    }
bool IR::Register::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Stateful::equiv(a_)) return false;
        auto &a = static_cast<const Register &>(a_);
        return layout == a.layout
        && width == a.width
        && signed_ == a.signed_;
    }
void IR::Register::dump_fields(std::ostream & out) const {
    Stateful::dump_fields(out);
        out << " layout=" << layout;
        out << " width=" << width;
        out << " signed_=" << signed_;
}
void IR::Register::toJSON(JSONGenerator & json) const {
    Stateful::toJSON(json);
    json.emit("layout", layout);
    json.emit("width", width);
    json.emit("signed_", signed_);
}
IR::Register::Register(JSONLoader & json) : Stateful(json) {
    json.load("layout", layout);
    json.load("width", width);
    json.load("signed_", signed_);
}
IR::Node * IR::Register::fromJSON(JSONLoader & json) { return new Register(json); }
bool IR::Register::operator==(IR::Stateful const & a) const { return a == *this; }
bool IR::Register::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::Register::operator==(IR::Node const & a) const { return a == *this; }
bool IR::PrimitiveAction::operator==(IR::PrimitiveAction const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::PrimitiveAction::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::PrimitiveAction::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::PrimitiveAction::PrimitiveAction(JSONLoader & json) : Node(json) {
}
IR::Node * IR::PrimitiveAction::fromJSON(JSONLoader & json) { return new PrimitiveAction(json); }
bool IR::PrimitiveAction::operator==(IR::Node const & a) const { return a == *this; }
#line 358 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::NameList::NameList(Util::SourceInfo si, cstring n) { names.emplace_back(si, n); }
#line 9121 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 359 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::NameList::NameList(Util::SourceInfo si, IR::ID n) { names.emplace_back(si, n); }
#line 9124 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 360 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::NameList::dump_fields(std::ostream & out) const { out << "names=" << names; }
#line 9127 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::NameList::operator==(IR::NameList const & a) const {
        return this->typeId() == a.typeId()
        && names == a.names;
    }
bool IR::NameList::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const NameList &>(a_);
        return names == a.names;
    }
void IR::NameList::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("names", names);
}
IR::NameList::NameList(JSONLoader & json) : Node(json) {
    json.load("names", names);
}
IR::Node * IR::NameList::fromJSON(JSONLoader & json) { return new NameList(json); }
bool IR::NameList::operator==(IR::Node const & a) const { return a == *this; }
#line 369 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::ActionArg::dbprint(std::ostream & out) const { out << action_name << ':' << name; }
#line 9149 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 370 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::ActionArg::toString() const { return name.name; }
#line 9152 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ActionArg::operator==(IR::ActionArg const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && action_name == a.action_name
        && name == a.name
        && read == a.read
        && write == a.write;
    }
bool IR::ActionArg::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const ActionArg &>(a_);
        return action_name == a.action_name
        && name == a.name
        && read == a.read
        && write == a.write;
    }
void IR::ActionArg::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " action_name=" << action_name;
        out << " name=" << name;
        out << " read=" << read;
        out << " write=" << write;
}
void IR::ActionArg::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("action_name", action_name);
    json.emit("name", name);
    json.emit("read", read);
    json.emit("write", write);
}
IR::ActionArg::ActionArg(JSONLoader & json) : Expression(json) {
    json.load("action_name", action_name);
    json.load("name", name);
    json.load("read", read);
    json.load("write", write);
}
IR::Node * IR::ActionArg::fromJSON(JSONLoader & json) { return new ActionArg(json); }
bool IR::ActionArg::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::ActionArg::operator==(IR::Node const & a) const { return a == *this; }
#line 380 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> const & IR::ActionFunction::getAnnotations() const { return annotations; }
#line 9194 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 381 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> & IR::ActionFunction::getAnnotations() { return annotations; }
#line 9197 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 382 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
const IR::ActionArg *IR::ActionFunction::arg(cstring n) const {
        for (auto a : args)
            if (a->name == n)
                return a;
        return nullptr; }
#line 9204 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 387 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::ActionFunction::visit_children(Visitor & v, char const * n) {
        (void)n;
        v.visit(action, "action");
        // DANGER -- visiting action first so type inferencing will push types to
        // DANGER -- action args based on use.  This is immoral.
        for (auto &a : args) v.visit(a, "arg");
        v.visit(annotations, "annotations");
    }
#line 387 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::ActionFunction::visit_children(Visitor & v, char const * n) const {
        (void)n;
        v.visit(action, "action");
        // DANGER -- visiting action first so type inferencing will push types to
        // DANGER -- action args based on use.  This is immoral.
        for (auto &a : args) v.visit(a, "arg");
        v.visit(annotations, "annotations");
    }
#line 9223 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 395 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::ActionFunction::toString() const {
        return "action "_cs + name + " {\n"_cs +
                 cstring::join(action.begin(), action.end(), ";\n") +
                 " }"_cs; }
#line 9229 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ActionFunction::operator==(IR::ActionFunction const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && action == a.action
        && args == a.args
        && annotations == a.annotations;
    }
bool IR::ActionFunction::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const ActionFunction &>(a_);
        return name == a.name
        && action.equiv(a.action)
        && args == a.args
        && annotations.equiv(a.annotations);
    }
void IR::ActionFunction::validate() const {
#line 9247 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        action.validate();
        annotations.validate(); }
void IR::ActionFunction::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
}
void IR::ActionFunction::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("action", action);
    json.emit("args", args);
    json.emit("annotations", annotations);
}
IR::ActionFunction::ActionFunction(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("action", action);
    json.load("args", args);
    json.load("annotations", annotations);
}
IR::Node * IR::ActionFunction::fromJSON(JSONLoader & json) { return new ActionFunction(json); }
bool IR::ActionFunction::operator==(IR::Node const & a) const { return a == *this; }
#line 405 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
char const * IR::ActionProfile::kind() const { return "action_profile"; }
#line 9272 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 406 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
bool IR::ActionProfile::indexed() const { return true; }
#line 9275 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ActionProfile::operator==(IR::ActionProfile const & a) const {
        return Attached::operator==(static_cast<const Attached &>(a))
        && selector == a.selector
        && actions == a.actions
        && size == a.size;
    }
bool IR::ActionProfile::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Attached::equiv(a_)) return false;
        auto &a = static_cast<const ActionProfile &>(a_);
        return selector == a.selector
        && actions == a.actions
        && size == a.size;
    }
void IR::ActionProfile::dump_fields(std::ostream & out) const {
    Attached::dump_fields(out);
        out << " selector=" << selector;
        out << " size=" << size;
}
void IR::ActionProfile::toJSON(JSONGenerator & json) const {
    Attached::toJSON(json);
    json.emit("selector", selector);
    json.emit("actions", actions);
    json.emit("size", size);
}
IR::ActionProfile::ActionProfile(JSONLoader & json) : Attached(json) {
    json.load("selector", selector);
    json.load("actions", actions);
    json.load("size", size);
}
IR::Node * IR::ActionProfile::fromJSON(JSONLoader & json) { return new ActionProfile(json); }
bool IR::ActionProfile::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::ActionProfile::operator==(IR::Node const & a) const { return a == *this; }
#line 414 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
char const * IR::ActionSelector::kind() const { return "action_selector"; }
#line 9311 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ActionSelector::operator==(IR::ActionSelector const & a) const {
        return Attached::operator==(static_cast<const Attached &>(a))
        && key == a.key
        && key_fields == a.key_fields
        && mode == a.mode
        && type == a.type;
    }
bool IR::ActionSelector::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Attached::equiv(a_)) return false;
        auto &a = static_cast<const ActionSelector &>(a_);
        return key == a.key
        && (key_fields ? a.key_fields ? key_fields->equiv(*a.key_fields) : false : a.key_fields == nullptr)
        && mode == a.mode
        && type == a.type;
    }
void IR::ActionSelector::visit_children(Visitor & v, char const * n) {
    (void)n;
    Attached::visit_children(v, n);
    v.visit(key_fields, "key_fields");
}
void IR::ActionSelector::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Attached::visit_children(v, n);
    v.visit(key_fields, "key_fields");
}
void IR::ActionSelector::dump_fields(std::ostream & out) const {
    Attached::dump_fields(out);
        out << " key=" << key;
        out << " mode=" << mode;
        out << " type=" << type;
}
void IR::ActionSelector::toJSON(JSONGenerator & json) const {
    Attached::toJSON(json);
    json.emit("key", key);
    if (key_fields != nullptr) json.emit("key_fields", key_fields);
    json.emit("mode", mode);
    json.emit("type", type);
}
IR::ActionSelector::ActionSelector(JSONLoader & json) : Attached(json) {
    json.load("key", key);
    json.load("key_fields", key_fields);
    json.load("mode", mode);
    json.load("type", type);
}
IR::Node * IR::ActionSelector::fromJSON(JSONLoader & json) { return new ActionSelector(json); }
bool IR::ActionSelector::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::ActionSelector::operator==(IR::Node const & a) const { return a == *this; }

#line 432 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::V1Table::addProperty(const IR::Property* prop) { properties.push_back(prop); }
#line 9363 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 433 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> const & IR::V1Table::getAnnotations() const { return annotations; }
#line 9366 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 434 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> & IR::V1Table::getAnnotations() { return annotations; }
#line 9369 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 435 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::V1Table::toString() const { return node_type_name() + " " + name; }
#line 9372 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 436 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::ID IR::V1Table::Name() const { return name; }
#line 9375 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 437 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
const IR::Type *IR::V1Table::getType() const { return Type_AnyTable::get(); }
#line 9378 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::V1Table::operator==(IR::V1Table const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && reads == a.reads
        && reads_types == a.reads_types
        && min_size == a.min_size
        && max_size == a.max_size
        && size == a.size
        && action_profile == a.action_profile
        && actions == a.actions
        && default_action == a.default_action
        && default_action_is_const == a.default_action_is_const
        && default_action_args == a.default_action_args
        && properties == a.properties
        && annotations == a.annotations;
    }
bool IR::V1Table::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const V1Table &>(a_);
        return name == a.name
        && (reads ? a.reads ? reads->equiv(*a.reads) : false : a.reads == nullptr)
        && reads_types == a.reads_types
        && min_size == a.min_size
        && max_size == a.max_size
        && size == a.size
        && action_profile == a.action_profile
        && actions == a.actions
        && default_action == a.default_action
        && default_action_is_const == a.default_action_is_const
        && (default_action_args ? a.default_action_args ? default_action_args->equiv(*a.default_action_args) : false : a.default_action_args == nullptr)
        && properties.equiv(a.properties)
        && annotations.equiv(a.annotations);
    }
void IR::V1Table::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(reads, "reads");
    v.visit(default_action_args, "default_action_args");
    properties.visit_children(v, "properties");
    annotations.visit_children(v, "annotations");
}
void IR::V1Table::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(reads, "reads");
    v.visit(default_action_args, "default_action_args");
    properties.visit_children(v, "properties");
    annotations.visit_children(v, "annotations");
}
void IR::V1Table::validate() const {
#line 9430 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        properties.validate();
        annotations.validate(); }
void IR::V1Table::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
        out << " min_size=" << min_size;
        out << " max_size=" << max_size;
        out << " size=" << size;
        out << " action_profile=" << action_profile;
        out << " default_action=" << default_action;
        out << " default_action_is_const=" << default_action_is_const;
}
void IR::V1Table::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    if (reads != nullptr) json.emit("reads", reads);
    json.emit("reads_types", reads_types);
    json.emit("min_size", min_size);
    json.emit("max_size", max_size);
    json.emit("size", size);
    json.emit("action_profile", action_profile);
    json.emit("actions", actions);
    json.emit("default_action", default_action);
    json.emit("default_action_is_const", default_action_is_const);
    if (default_action_args != nullptr) json.emit("default_action_args", default_action_args);
    json.emit("properties", properties);
    json.emit("annotations", annotations);
}
IR::V1Table::V1Table(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("reads", reads);
    json.load("reads_types", reads_types);
    json.load("min_size", min_size);
    json.load("max_size", max_size);
    json.load("size", size);
    json.load("action_profile", action_profile);
    json.load("actions", actions);
    json.load("default_action", default_action);
    json.load("default_action_is_const", default_action_is_const);
    json.load("default_action_args", default_action_args);
    json.load("properties", properties);
    json.load("annotations", annotations);
}
IR::Node * IR::V1Table::fromJSON(JSONLoader & json) { return new V1Table(json); }
bool IR::V1Table::operator==(IR::Node const & a) const { return a == *this; }
#line 445 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::V1Control::V1Control(IR::ID n) : name(n), code(new Vector<Expression>()) {}
#line 9479 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 446 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::V1Control::V1Control(Util::SourceInfo si, IR::ID n) : Node(si), name(n), code(new Vector<Expression>()) {}
#line 9482 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(V1Control, , )
#line 448 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> const & IR::V1Control::getAnnotations() const { return annotations; }
#line 9486 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 449 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::Vector<IR::Annotation> & IR::V1Control::getAnnotations() { return annotations; }
#line 9489 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 450 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::V1Control::toString() const { return node_type_name() + " " + name; }
#line 9492 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::V1Control::operator==(IR::V1Control const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && code == a.code
        && annotations == a.annotations;
    }
bool IR::V1Control::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const V1Control &>(a_);
        return name == a.name
        && (code ? a.code ? code->equiv(*a.code) : false : a.code == nullptr)
        && annotations.equiv(a.annotations);
    }
void IR::V1Control::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(code, "code");
    annotations.visit_children(v, "annotations");
}
void IR::V1Control::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(code, "code");
    annotations.visit_children(v, "annotations");
}
void IR::V1Control::validate() const {
#line 9520 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(code);
        annotations.validate(); }
void IR::V1Control::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
}
void IR::V1Control::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("code", code);
    json.emit("annotations", annotations);
}
IR::V1Control::V1Control(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("code", code);
    json.load("annotations", annotations);
}
IR::Node * IR::V1Control::fromJSON(JSONLoader & json) { return new V1Control(json); }
bool IR::V1Control::operator==(IR::Node const & a) const { return a == *this; }
#line 455 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::ID IR::AttribLocal::getName() const { return name; }
#line 9543 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 456 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::AttribLocal::dbprint(std::ostream & out) const { out << name; }
#line 9546 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::AttribLocal::operator==(IR::AttribLocal const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && name == a.name;
    }
bool IR::AttribLocal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const AttribLocal &>(a_);
        return name == a.name;
    }
void IR::AttribLocal::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " name=" << name;
}
void IR::AttribLocal::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("name", name);
}
IR::AttribLocal::AttribLocal(JSONLoader & json) : Expression(json) {
    json.load("name", name);
}
IR::Node * IR::AttribLocal::fromJSON(JSONLoader & json) { return new AttribLocal(json); }
bool IR::AttribLocal::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::AttribLocal::operator==(IR::Node const & a) const { return a == *this; }
#line 462 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
Util::Enumerator<const IR::IDeclaration *> * IR::AttribLocals::getDeclarations() const {
        return locals.valueEnumerator()->as<const IDeclaration *>(); }
#line 9574 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 464 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
const IR::IDeclaration *IR::AttribLocals::getDeclByName(cstring name) const { return locals[name]; }
#line 9577 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 465 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
const IR::IDeclaration *IR::AttribLocals::getDeclByName(std::string_view name) const { return locals[cstring(name)]; }
#line 9580 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::AttribLocals::operator==(IR::AttribLocals const & a) const {
        return this->typeId() == a.typeId()
        && locals == a.locals;
    }
bool IR::AttribLocals::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const AttribLocals &>(a_);
        return locals.equiv(a.locals);
    }
void IR::AttribLocals::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    locals.visit_children(v, "locals");
}
void IR::AttribLocals::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    locals.visit_children(v, "locals");
}
void IR::AttribLocals::validate() const {
#line 9602 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        locals.validate(); }
void IR::AttribLocals::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("locals", locals);
}
IR::AttribLocals::AttribLocals(JSONLoader & json) : Node(json) {
    json.load("locals", locals);
}
IR::Node * IR::AttribLocals::fromJSON(JSONLoader & json) { return new AttribLocals(json); }
bool IR::AttribLocals::operator==(IR::Node const & a) const { return a == *this; }
#line 472 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::Attribute::dbprint(std::ostream & out) const { if (type) out << type << ' '; out << name; }
#line 9616 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Attribute::operator==(IR::Attribute const & a) const {
        return Declaration::operator==(static_cast<const Declaration &>(a))
        && type == a.type
        && locals == a.locals
        && optional == a.optional;
    }
bool IR::Attribute::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration::equiv(a_)) return false;
        auto &a = static_cast<const Attribute &>(a_);
        return (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && (locals ? a.locals ? locals->equiv(*a.locals) : false : a.locals == nullptr)
        && optional == a.optional;
    }
void IR::Attribute::visit_children(Visitor & v, char const * n) {
    (void)n;
    Declaration::visit_children(v, n);
    v.visit(type, "type");
    v.visit(locals, "locals");
}
void IR::Attribute::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Declaration::visit_children(v, n);
    v.visit(type, "type");
    v.visit(locals, "locals");
}
void IR::Attribute::dump_fields(std::ostream & out) const {
    Declaration::dump_fields(out);
        out << " optional=" << optional;
}
void IR::Attribute::toJSON(JSONGenerator & json) const {
    Declaration::toJSON(json);
    if (type != nullptr) json.emit("type", type);
    if (locals != nullptr) json.emit("locals", locals);
    json.emit("optional", optional);
}
IR::Attribute::Attribute(JSONLoader & json) : Declaration(json) {
    json.load("type", type);
    json.load("locals", locals);
    json.load("optional", optional);
}
IR::Node * IR::Attribute::fromJSON(JSONLoader & json) { return new Attribute(json); }
bool IR::Attribute::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::Attribute::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::Attribute::operator==(IR::Node const & a) const { return a == *this; }


#line 479 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::GlobalRef::validate() const {
#line 9666 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(obj);
#line 479 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
{ BUG_CHECK(obj->is<IInstance>(), "Invalid object %1%", obj); } }
#line 9671 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 480 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::GlobalRef::toString() const { return obj->to<IInstance>()->toString(); }
#line 9674 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 481 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
IR::ID IR::GlobalRef::Name() const { return obj->to<IInstance>()->Name(); }
#line 9677 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 482 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::GlobalRef::dbprint(std::ostream & out) const { out << obj->to<IInstance>()->Name(); }
#line 9680 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::GlobalRef::operator==(IR::GlobalRef const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && obj == a.obj;
    }
bool IR::GlobalRef::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const GlobalRef &>(a_);
        return (obj ? a.obj ? obj->equiv(*a.obj) : false : a.obj == nullptr);
    }
void IR::GlobalRef::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(obj, "obj");
}
void IR::GlobalRef::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(obj, "obj");
}
void IR::GlobalRef::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("obj", obj);
}
IR::GlobalRef::GlobalRef(JSONLoader & json) : Expression(json) {
    json.load("obj", obj);
}
IR::Node * IR::GlobalRef::fromJSON(JSONLoader & json) { return new GlobalRef(json); }
bool IR::GlobalRef::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::GlobalRef::operator==(IR::Node const & a) const { return a == *this; }
#line 490 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
cstring IR::AttributeRef::toString() const { return attrib->name; }
#line 9713 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 491 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::AttributeRef::dbprint(std::ostream & out) const { out << attrib->name; }
#line 9716 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::AttributeRef::operator==(IR::AttributeRef const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && extern_name == a.extern_name
        && extern_type == a.extern_type
        && attrib == a.attrib;
    }
bool IR::AttributeRef::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const AttributeRef &>(a_);
        return extern_name == a.extern_name
        && (extern_type ? a.extern_type ? extern_type->equiv(*a.extern_type) : false : a.extern_type == nullptr)
        && (attrib ? a.attrib ? attrib->equiv(*a.attrib) : false : a.attrib == nullptr);
    }
void IR::AttributeRef::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(extern_type, "extern_type");
    v.visit(attrib, "attrib");
}
void IR::AttributeRef::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(extern_type, "extern_type");
    v.visit(attrib, "attrib");
}
void IR::AttributeRef::validate() const {
#line 9744 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(extern_type);
        CHECK_NULL(attrib); }
void IR::AttributeRef::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " extern_name=" << extern_name;
}
void IR::AttributeRef::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("extern_name", extern_name);
    json.emit("extern_type", extern_type);
    json.emit("attrib", attrib);
}
IR::AttributeRef::AttributeRef(JSONLoader & json) : Expression(json) {
    json.load("extern_name", extern_name);
    json.load("extern_type", extern_type);
    json.load("attrib", attrib);
}
IR::Node * IR::AttributeRef::fromJSON(JSONLoader & json) { return new AttributeRef(json); }
bool IR::AttributeRef::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::AttributeRef::operator==(IR::Node const & a) const { return a == *this; }
#line 502 "/builddir/build/BUILD/p4c-1.2.5.8/frontends/p4-14/ir-v1.def"
void IR::V1Program::add(cstring name, IR::Node const * n) { scope.add(name, n); }
#line 9768 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(V1Program, , )
bool IR::V1Program::operator==(IR::V1Program const & a) const {
        return this->typeId() == a.typeId()
        && scope == a.scope;
    }
bool IR::V1Program::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const V1Program &>(a_);
        return scope.equiv(a.scope);
    }
void IR::V1Program::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    scope.visit_children(v, "scope");
}
void IR::V1Program::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    scope.visit_children(v, "scope");
}
void IR::V1Program::validate() const {
#line 9791 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        scope.validate(); }
void IR::V1Program::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("scope", scope);
}
IR::V1Program::V1Program(JSONLoader & json) : Node(json) {
    json.load("scope", scope);
}
IR::Node * IR::V1Program::fromJSON(JSONLoader & json) { return new V1Program(json); }
bool IR::V1Program::operator==(IR::Node const & a) const { return a == *this; }
bool IR::IntMod::operator==(IR::IntMod const & a) const {
        return Operation_Unary::operator==(static_cast<const Operation_Unary &>(a))
        && width == a.width;
    }
bool IR::IntMod::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Unary::equiv(a_)) return false;
        auto &a = static_cast<const IntMod &>(a_);
        return width == a.width;
    }
void IR::IntMod::dump_fields(std::ostream & out) const {
    Operation_Unary::dump_fields(out);
        out << " width=" << width;
}
void IR::IntMod::toJSON(JSONGenerator & json) const {
    Operation_Unary::toJSON(json);
    json.emit("width", width);
}
IR::IntMod::IntMod(JSONLoader & json) : Operation_Unary(json) {
    json.load("width", width);
}
IR::Node * IR::IntMod::fromJSON(JSONLoader & json) { return new IntMod(json); }
bool IR::IntMod::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::IntMod::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::IntMod::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::IntMod::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkDeclaration::operator==(IR::DpdkDeclaration const & a) const {
        return this->typeId() == a.typeId()
        && global == a.global;
    }
bool IR::DpdkDeclaration::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const DpdkDeclaration &>(a_);
        return (global ? a.global ? global->equiv(*a.global) : false : a.global == nullptr);
    }
void IR::DpdkDeclaration::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(global, "global");
}
void IR::DpdkDeclaration::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(global, "global");
}
void IR::DpdkDeclaration::validate() const {
#line 9850 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(global); }
void IR::DpdkDeclaration::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("global", global);
}
IR::DpdkDeclaration::DpdkDeclaration(JSONLoader & json) : Node(json) {
    json.load("global", global);
}
IR::Node * IR::DpdkDeclaration::fromJSON(JSONLoader & json) { return new DpdkDeclaration(json); }
bool IR::DpdkDeclaration::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkExternDeclaration::operator==(IR::DpdkExternDeclaration const & a) const {
        return Declaration_Instance::operator==(static_cast<const Declaration_Instance &>(a));
    }
bool IR::DpdkExternDeclaration::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Declaration_Instance::equiv(a_)) return false;
        return true;
    }
void IR::DpdkExternDeclaration::toJSON(JSONGenerator & json) const {
    Declaration_Instance::toJSON(json);
}
IR::DpdkExternDeclaration::DpdkExternDeclaration(JSONLoader & json) : Declaration_Instance(json) {
}
IR::Node * IR::DpdkExternDeclaration::fromJSON(JSONLoader & json) { return new DpdkExternDeclaration(json); }
bool IR::DpdkExternDeclaration::operator==(IR::Declaration_Instance const & a) const { return a == *this; }
bool IR::DpdkExternDeclaration::operator==(IR::Declaration const & a) const { return a == *this; }
bool IR::DpdkExternDeclaration::operator==(IR::StatOrDecl const & a) const { return a == *this; }
bool IR::DpdkExternDeclaration::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkHeaderType::operator==(IR::DpdkHeaderType const & a) const {
        return Type_Header::operator==(static_cast<const Type_Header &>(a));
    }
bool IR::DpdkHeaderType::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Header::equiv(a_)) return false;
        return true;
    }
void IR::DpdkHeaderType::toJSON(JSONGenerator & json) const {
    Type_Header::toJSON(json);
}
IR::DpdkHeaderType::DpdkHeaderType(JSONLoader & json) : Type_Header(json) {
}
IR::Node * IR::DpdkHeaderType::fromJSON(JSONLoader & json) { return new DpdkHeaderType(json); }
bool IR::DpdkHeaderType::operator==(IR::Type_Header const & a) const { return a == *this; }
bool IR::DpdkHeaderType::operator==(IR::Type_StructLike const & a) const { return a == *this; }
bool IR::DpdkHeaderType::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::DpdkHeaderType::operator==(IR::Type const & a) const { return a == *this; }
bool IR::DpdkHeaderType::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkHeaderInstance::operator==(IR::DpdkHeaderInstance const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && headerType == a.headerType;
    }
bool IR::DpdkHeaderInstance::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const DpdkHeaderInstance &>(a_);
        return (name ? a.name ? name->equiv(*a.name) : false : a.name == nullptr)
        && (headerType ? a.headerType ? headerType->equiv(*a.headerType) : false : a.headerType == nullptr);
    }
void IR::DpdkHeaderInstance::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(name, "name");
    v.visit(headerType, "headerType");
}
void IR::DpdkHeaderInstance::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(name, "name");
    v.visit(headerType, "headerType");
}
void IR::DpdkHeaderInstance::validate() const {
#line 9924 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(name);
        CHECK_NULL(headerType); }
void IR::DpdkHeaderInstance::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("headerType", headerType);
}
IR::DpdkHeaderInstance::DpdkHeaderInstance(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("headerType", headerType);
}
IR::Node * IR::DpdkHeaderInstance::fromJSON(JSONLoader & json) { return new DpdkHeaderInstance(json); }
bool IR::DpdkHeaderInstance::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkStructType::operator==(IR::DpdkStructType const & a) const {
        return Type_Struct::operator==(static_cast<const Type_Struct &>(a));
    }
bool IR::DpdkStructType::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Struct::equiv(a_)) return false;
        return true;
    }
void IR::DpdkStructType::toJSON(JSONGenerator & json) const {
    Type_Struct::toJSON(json);
}
IR::DpdkStructType::DpdkStructType(JSONLoader & json) : Type_Struct(json) {
}
IR::Node * IR::DpdkStructType::fromJSON(JSONLoader & json) { return new DpdkStructType(json); }
bool IR::DpdkStructType::operator==(IR::Type_Struct const & a) const { return a == *this; }
bool IR::DpdkStructType::operator==(IR::Type_StructLike const & a) const { return a == *this; }
bool IR::DpdkStructType::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::DpdkStructType::operator==(IR::Type const & a) const { return a == *this; }
bool IR::DpdkStructType::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkAsmStatement::operator==(IR::DpdkAsmStatement const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::DpdkAsmStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::DpdkAsmStatement::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::DpdkAsmStatement::DpdkAsmStatement(JSONLoader & json) : Node(json) {
}
bool IR::DpdkAsmStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 43 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::Vector<IR::Annotation> const & IR::DpdkAction::getAnnotations() const { return annotations; }
#line 9974 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 44 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::Vector<IR::Annotation> & IR::DpdkAction::getAnnotations() { return annotations; }
#line 9977 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkAction::operator==(IR::DpdkAction const & a) const {
        return this->typeId() == a.typeId()
        && annotations == a.annotations
        && statements == a.statements
        && name == a.name
        && para == a.para;
    }
bool IR::DpdkAction::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const DpdkAction &>(a_);
        return annotations.equiv(a.annotations)
        && statements.equiv(a.statements)
        && name == a.name
        && para.equiv(a.para);
    }
void IR::DpdkAction::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    statements.visit_children(v, "statements");
    para.visit_children(v, "para");
}
void IR::DpdkAction::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    annotations.visit_children(v, "annotations");
    statements.visit_children(v, "statements");
    para.visit_children(v, "para");
}
void IR::DpdkAction::validate() const {
#line 10009 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate();
        statements.validate();
        para.validate(); }
void IR::DpdkAction::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
}
void IR::DpdkAction::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("annotations", annotations);
    json.emit("statements", statements);
    json.emit("name", name);
    json.emit("para", para);
}
IR::DpdkAction::DpdkAction(JSONLoader & json) : Node(json) {
    json.load("annotations", annotations);
    json.load("statements", statements);
    json.load("name", name);
    json.load("para", para);
}
IR::Node * IR::DpdkAction::fromJSON(JSONLoader & json) { return new DpdkAction(json); }
bool IR::DpdkAction::operator==(IR::Node const & a) const { return a == *this; }
#line 55 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
const IR::Key *IR::DpdkTable::getKey() const {
        auto kp = properties->getProperty(TableProperties::keyPropertyName);
        if (kp == nullptr)
            return nullptr;
        if (!kp->value->is<IR::Key>()) {
            ::P4::error(ErrorType::ERR_INVALID, "%1%: must be a key", kp);
            return nullptr; }
        return kp->value->to<IR::Key>(); }
#line 10042 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 63 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
const IR::EntriesList *IR::DpdkTable::getEntries() const {
        auto ep = properties->getProperty(TableProperties::entriesPropertyName);
        if (ep == nullptr)
            return nullptr;
        if (!ep->value->is<IR::EntriesList>()) {
            ::P4::error(ErrorType::ERR_INVALID, "%1%: must be a list of entries", ep);
            return nullptr;
        }
        return ep->value->to<IR::EntriesList>();
    }
#line 10054 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkTable::operator==(IR::DpdkTable const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && match_keys == a.match_keys
        && actions == a.actions
        && default_action == a.default_action
        && properties == a.properties
        && default_action_paraList == a.default_action_paraList;
    }
bool IR::DpdkTable::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const DpdkTable &>(a_);
        return name == a.name
        && (match_keys ? a.match_keys ? match_keys->equiv(*a.match_keys) : false : a.match_keys == nullptr)
        && (actions ? a.actions ? actions->equiv(*a.actions) : false : a.actions == nullptr)
        && (default_action ? a.default_action ? default_action->equiv(*a.default_action) : false : a.default_action == nullptr)
        && (properties ? a.properties ? properties->equiv(*a.properties) : false : a.properties == nullptr)
        && default_action_paraList.equiv(a.default_action_paraList);
    }
void IR::DpdkTable::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(match_keys, "match_keys");
    v.visit(actions, "actions");
    v.visit(default_action, "default_action");
    v.visit(properties, "properties");
    default_action_paraList.visit_children(v, "default_action_paraList");
}
void IR::DpdkTable::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(match_keys, "match_keys");
    v.visit(actions, "actions");
    v.visit(default_action, "default_action");
    v.visit(properties, "properties");
    default_action_paraList.visit_children(v, "default_action_paraList");
}
void IR::DpdkTable::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
}
void IR::DpdkTable::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("match_keys", match_keys);
    json.emit("actions", actions);
    json.emit("default_action", default_action);
    json.emit("properties", properties);
    json.emit("default_action_paraList", default_action_paraList);
}
IR::DpdkTable::DpdkTable(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("match_keys", match_keys);
    json.load("actions", actions);
    json.load("default_action", default_action);
    json.load("properties", properties);
    json.load("default_action_paraList", default_action_paraList);
}
IR::Node * IR::DpdkTable::fromJSON(JSONLoader & json) { return new DpdkTable(json); }
bool IR::DpdkTable::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkSelector::operator==(IR::DpdkSelector const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && group_id == a.group_id
        && member_id == a.member_id
        && selectors == a.selectors
        && n_groups_max == a.n_groups_max
        && n_members_per_group_max == a.n_members_per_group_max;
    }
bool IR::DpdkSelector::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const DpdkSelector &>(a_);
        return name == a.name
        && (group_id ? a.group_id ? group_id->equiv(*a.group_id) : false : a.group_id == nullptr)
        && (member_id ? a.member_id ? member_id->equiv(*a.member_id) : false : a.member_id == nullptr)
        && (selectors ? a.selectors ? selectors->equiv(*a.selectors) : false : a.selectors == nullptr)
        && n_groups_max == a.n_groups_max
        && n_members_per_group_max == a.n_members_per_group_max;
    }
void IR::DpdkSelector::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(group_id, "group_id");
    v.visit(member_id, "member_id");
    v.visit(selectors, "selectors");
}
void IR::DpdkSelector::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(group_id, "group_id");
    v.visit(member_id, "member_id");
    v.visit(selectors, "selectors");
}
void IR::DpdkSelector::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
        out << " n_groups_max=" << n_groups_max;
        out << " n_members_per_group_max=" << n_members_per_group_max;
}
void IR::DpdkSelector::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("group_id", group_id);
    json.emit("member_id", member_id);
    json.emit("selectors", selectors);
    json.emit("n_groups_max", n_groups_max);
    json.emit("n_members_per_group_max", n_members_per_group_max);
}
IR::DpdkSelector::DpdkSelector(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("group_id", group_id);
    json.load("member_id", member_id);
    json.load("selectors", selectors);
    json.load("n_groups_max", n_groups_max);
    json.load("n_members_per_group_max", n_members_per_group_max);
}
IR::Node * IR::DpdkSelector::fromJSON(JSONLoader & json) { return new DpdkSelector(json); }
bool IR::DpdkSelector::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkLearner::operator==(IR::DpdkLearner const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && match_keys == a.match_keys
        && actions == a.actions
        && default_action == a.default_action
        && properties == a.properties;
    }
bool IR::DpdkLearner::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const DpdkLearner &>(a_);
        return name == a.name
        && (match_keys ? a.match_keys ? match_keys->equiv(*a.match_keys) : false : a.match_keys == nullptr)
        && (actions ? a.actions ? actions->equiv(*a.actions) : false : a.actions == nullptr)
        && (default_action ? a.default_action ? default_action->equiv(*a.default_action) : false : a.default_action == nullptr)
        && (properties ? a.properties ? properties->equiv(*a.properties) : false : a.properties == nullptr);
    }
void IR::DpdkLearner::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(match_keys, "match_keys");
    v.visit(actions, "actions");
    v.visit(default_action, "default_action");
    v.visit(properties, "properties");
}
void IR::DpdkLearner::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(match_keys, "match_keys");
    v.visit(actions, "actions");
    v.visit(default_action, "default_action");
    v.visit(properties, "properties");
}
void IR::DpdkLearner::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
}
void IR::DpdkLearner::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("match_keys", match_keys);
    json.emit("actions", actions);
    json.emit("default_action", default_action);
    json.emit("properties", properties);
}
IR::DpdkLearner::DpdkLearner(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("match_keys", match_keys);
    json.load("actions", actions);
    json.load("default_action", default_action);
    json.load("properties", properties);
}
IR::Node * IR::DpdkLearner::fromJSON(JSONLoader & json) { return new DpdkLearner(json); }
bool IR::DpdkLearner::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkAsmProgram::operator==(IR::DpdkAsmProgram const & a) const {
        return this->typeId() == a.typeId()
        && headerType == a.headerType
        && structType == a.structType
        && headerInstance == a.headerInstance
        && externDeclarations == a.externDeclarations
        && actions == a.actions
        && tables == a.tables
        && selectors == a.selectors
        && learners == a.learners
        && statements == a.statements
        && globals == a.globals;
    }
bool IR::DpdkAsmProgram::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const DpdkAsmProgram &>(a_);
        return headerType.equiv(a.headerType)
        && structType.equiv(a.structType)
        && headerInstance.equiv(a.headerInstance)
        && externDeclarations.equiv(a.externDeclarations)
        && actions.equiv(a.actions)
        && tables.equiv(a.tables)
        && selectors.equiv(a.selectors)
        && learners.equiv(a.learners)
        && statements.equiv(a.statements)
        && globals.equiv(a.globals);
    }
void IR::DpdkAsmProgram::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    headerType.visit_children(v, "headerType");
    structType.visit_children(v, "structType");
    headerInstance.visit_children(v, "headerInstance");
    externDeclarations.visit_children(v, "externDeclarations");
    actions.visit_children(v, "actions");
    tables.visit_children(v, "tables");
    selectors.visit_children(v, "selectors");
    learners.visit_children(v, "learners");
    statements.visit_children(v, "statements");
    globals.visit_children(v, "globals");
}
void IR::DpdkAsmProgram::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    headerType.visit_children(v, "headerType");
    structType.visit_children(v, "structType");
    headerInstance.visit_children(v, "headerInstance");
    externDeclarations.visit_children(v, "externDeclarations");
    actions.visit_children(v, "actions");
    tables.visit_children(v, "tables");
    selectors.visit_children(v, "selectors");
    learners.visit_children(v, "learners");
    statements.visit_children(v, "statements");
    globals.visit_children(v, "globals");
}
void IR::DpdkAsmProgram::validate() const {
#line 10287 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        headerType.validate();
        structType.validate();
        headerInstance.validate();
        externDeclarations.validate();
        actions.validate();
        tables.validate();
        selectors.validate();
        learners.validate();
        statements.validate();
        globals.validate(); }
void IR::DpdkAsmProgram::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("headerType", headerType);
    json.emit("structType", structType);
    json.emit("headerInstance", headerInstance);
    json.emit("externDeclarations", externDeclarations);
    json.emit("actions", actions);
    json.emit("tables", tables);
    json.emit("selectors", selectors);
    json.emit("learners", learners);
    json.emit("statements", statements);
    json.emit("globals", globals);
}
IR::DpdkAsmProgram::DpdkAsmProgram(JSONLoader & json) : Node(json) {
    json.load("headerType", headerType);
    json.load("structType", structType);
    json.load("headerInstance", headerInstance);
    json.load("externDeclarations", externDeclarations);
    json.load("actions", actions);
    json.load("tables", tables);
    json.load("selectors", selectors);
    json.load("learners", learners);
    json.load("statements", statements);
    json.load("globals", globals);
}
IR::Node * IR::DpdkAsmProgram::fromJSON(JSONLoader & json) { return new DpdkAsmProgram(json); }
bool IR::DpdkAsmProgram::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkListStatement::operator==(IR::DpdkListStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && statements == a.statements;
    }
bool IR::DpdkListStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkListStatement &>(a_);
        return statements.equiv(a.statements);
    }
void IR::DpdkListStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    statements.visit_children(v, "statements");
}
void IR::DpdkListStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    statements.visit_children(v, "statements");
}
void IR::DpdkListStatement::validate() const {
#line 10347 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        statements.validate(); }
void IR::DpdkListStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("statements", statements);
}
IR::DpdkListStatement::DpdkListStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("statements", statements);
}
IR::Node * IR::DpdkListStatement::fromJSON(JSONLoader & json) { return new DpdkListStatement(json); }
bool IR::DpdkListStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkListStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkApplyStatement::operator==(IR::DpdkApplyStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && table == a.table;
    }
bool IR::DpdkApplyStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkApplyStatement &>(a_);
        return table == a.table;
    }
void IR::DpdkApplyStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " table=" << table;
}
void IR::DpdkApplyStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("table", table);
}
IR::DpdkApplyStatement::DpdkApplyStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("table", table);
}
IR::Node * IR::DpdkApplyStatement::fromJSON(JSONLoader & json) { return new DpdkApplyStatement(json); }
bool IR::DpdkApplyStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkApplyStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkLearnStatement::operator==(IR::DpdkLearnStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && action == a.action
        && timeout == a.timeout
        && argument == a.argument;
    }
bool IR::DpdkLearnStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkLearnStatement &>(a_);
        return action == a.action
        && (timeout ? a.timeout ? timeout->equiv(*a.timeout) : false : a.timeout == nullptr)
        && (argument ? a.argument ? argument->equiv(*a.argument) : false : a.argument == nullptr);
    }
void IR::DpdkLearnStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(timeout, "timeout");
    v.visit(argument, "argument");
}
void IR::DpdkLearnStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(timeout, "timeout");
    v.visit(argument, "argument");
}
void IR::DpdkLearnStatement::validate() const {
#line 10411 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(timeout); }
void IR::DpdkLearnStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " action=" << action;
}
void IR::DpdkLearnStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("action", action);
    json.emit("timeout", timeout);
    if (argument != nullptr) json.emit("argument", argument);
}
IR::DpdkLearnStatement::DpdkLearnStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("action", action);
    json.load("timeout", timeout);
    json.load("argument", argument);
}
IR::Node * IR::DpdkLearnStatement::fromJSON(JSONLoader & json) { return new DpdkLearnStatement(json); }
bool IR::DpdkLearnStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkLearnStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkMirrorStatement::operator==(IR::DpdkMirrorStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && slotId == a.slotId
        && sessionId == a.sessionId;
    }
bool IR::DpdkMirrorStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkMirrorStatement &>(a_);
        return (slotId ? a.slotId ? slotId->equiv(*a.slotId) : false : a.slotId == nullptr)
        && (sessionId ? a.sessionId ? sessionId->equiv(*a.sessionId) : false : a.sessionId == nullptr);
    }
void IR::DpdkMirrorStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(slotId, "slotId");
    v.visit(sessionId, "sessionId");
}
void IR::DpdkMirrorStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(slotId, "slotId");
    v.visit(sessionId, "sessionId");
}
void IR::DpdkMirrorStatement::validate() const {
#line 10457 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(slotId);
        CHECK_NULL(sessionId); }
void IR::DpdkMirrorStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("slotId", slotId);
    json.emit("sessionId", sessionId);
}
IR::DpdkMirrorStatement::DpdkMirrorStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("slotId", slotId);
    json.load("sessionId", sessionId);
}
IR::Node * IR::DpdkMirrorStatement::fromJSON(JSONLoader & json) { return new DpdkMirrorStatement(json); }
bool IR::DpdkMirrorStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkMirrorStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkEmitStatement::operator==(IR::DpdkEmitStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && header == a.header;
    }
bool IR::DpdkEmitStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkEmitStatement &>(a_);
        return (header ? a.header ? header->equiv(*a.header) : false : a.header == nullptr);
    }
void IR::DpdkEmitStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(header, "header");
}
void IR::DpdkEmitStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(header, "header");
}
void IR::DpdkEmitStatement::validate() const {
#line 10494 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(header); }
void IR::DpdkEmitStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("header", header);
}
IR::DpdkEmitStatement::DpdkEmitStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("header", header);
}
IR::Node * IR::DpdkEmitStatement::fromJSON(JSONLoader & json) { return new DpdkEmitStatement(json); }
bool IR::DpdkEmitStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkEmitStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkExtractStatement::operator==(IR::DpdkExtractStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && header == a.header
        && length == a.length;
    }
bool IR::DpdkExtractStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkExtractStatement &>(a_);
        return (header ? a.header ? header->equiv(*a.header) : false : a.header == nullptr)
        && (length ? a.length ? length->equiv(*a.length) : false : a.length == nullptr);
    }
void IR::DpdkExtractStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(header, "header");
    v.visit(length, "length");
}
void IR::DpdkExtractStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(header, "header");
    v.visit(length, "length");
}
void IR::DpdkExtractStatement::validate() const {
#line 10532 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(header); }
void IR::DpdkExtractStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("header", header);
    if (length != nullptr) json.emit("length", length);
}
IR::DpdkExtractStatement::DpdkExtractStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("header", header);
    json.load("length", length);
}
IR::Node * IR::DpdkExtractStatement::fromJSON(JSONLoader & json) { return new DpdkExtractStatement(json); }
bool IR::DpdkExtractStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkExtractStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkLookaheadStatement::operator==(IR::DpdkLookaheadStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && header == a.header;
    }
bool IR::DpdkLookaheadStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkLookaheadStatement &>(a_);
        return (header ? a.header ? header->equiv(*a.header) : false : a.header == nullptr);
    }
void IR::DpdkLookaheadStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(header, "header");
}
void IR::DpdkLookaheadStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(header, "header");
}
void IR::DpdkLookaheadStatement::validate() const {
#line 10568 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(header); }
void IR::DpdkLookaheadStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("header", header);
}
IR::DpdkLookaheadStatement::DpdkLookaheadStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("header", header);
}
IR::Node * IR::DpdkLookaheadStatement::fromJSON(JSONLoader & json) { return new DpdkLookaheadStatement(json); }
bool IR::DpdkLookaheadStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkLookaheadStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 166 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpStatement::DpdkJmpStatement(cstring instruction, cstring l) :
        instruction(instruction), label(l.toUpper()) { }
#line 10584 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpStatement::operator==(IR::DpdkJmpStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && instruction == a.instruction
        && label == a.label;
    }
bool IR::DpdkJmpStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkJmpStatement &>(a_);
        return instruction == a.instruction
        && label == a.label;
    }
void IR::DpdkJmpStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " instruction=" << instruction;
        out << " label=" << label;
}
void IR::DpdkJmpStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("instruction", instruction);
    json.emit("label", label);
}
IR::DpdkJmpStatement::DpdkJmpStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("instruction", instruction);
    json.load("label", label);
}
bool IR::DpdkJmpStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 172 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpLabelStatement::DpdkJmpLabelStatement(cstring label) :
        DpdkJmpStatement("jmp"_cs, label) { }
#line 10616 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpLabelStatement::operator==(IR::DpdkJmpLabelStatement const & a) const {
        return DpdkJmpStatement::operator==(static_cast<const DpdkJmpStatement &>(a));
    }
bool IR::DpdkJmpLabelStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpLabelStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpStatement::toJSON(json);
}
IR::DpdkJmpLabelStatement::DpdkJmpLabelStatement(JSONLoader & json) : DpdkJmpStatement(json) {
}
IR::Node * IR::DpdkJmpLabelStatement::fromJSON(JSONLoader & json) { return new DpdkJmpLabelStatement(json); }
bool IR::DpdkJmpLabelStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpLabelStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpLabelStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 179 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpHitStatement::DpdkJmpHitStatement(cstring label) :
        DpdkJmpStatement("jmph"_cs, label) { }
#line 10637 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpHitStatement::operator==(IR::DpdkJmpHitStatement const & a) const {
        return DpdkJmpStatement::operator==(static_cast<const DpdkJmpStatement &>(a));
    }
bool IR::DpdkJmpHitStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpHitStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpStatement::toJSON(json);
}
IR::DpdkJmpHitStatement::DpdkJmpHitStatement(JSONLoader & json) : DpdkJmpStatement(json) {
}
IR::Node * IR::DpdkJmpHitStatement::fromJSON(JSONLoader & json) { return new DpdkJmpHitStatement(json); }
bool IR::DpdkJmpHitStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpHitStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpHitStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 186 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpMissStatement::DpdkJmpMissStatement(cstring label) :
        DpdkJmpStatement("jmpnh"_cs, label) { }
#line 10658 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpMissStatement::operator==(IR::DpdkJmpMissStatement const & a) const {
        return DpdkJmpStatement::operator==(static_cast<const DpdkJmpStatement &>(a));
    }
bool IR::DpdkJmpMissStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpMissStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpStatement::toJSON(json);
}
IR::DpdkJmpMissStatement::DpdkJmpMissStatement(JSONLoader & json) : DpdkJmpStatement(json) {
}
IR::Node * IR::DpdkJmpMissStatement::fromJSON(JSONLoader & json) { return new DpdkJmpMissStatement(json); }
bool IR::DpdkJmpMissStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpMissStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpMissStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 195 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpActionStatement::DpdkJmpActionStatement(cstring instruction, cstring label, IR::ID action) :
        DpdkJmpStatement(instruction, label), action(action) { }
#line 10679 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpActionStatement::operator==(IR::DpdkJmpActionStatement const & a) const {
        return DpdkJmpStatement::operator==(static_cast<const DpdkJmpStatement &>(a))
        && action == a.action;
    }
bool IR::DpdkJmpActionStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkJmpActionStatement &>(a_);
        return action == a.action;
    }
void IR::DpdkJmpActionStatement::dump_fields(std::ostream & out) const {
    DpdkJmpStatement::dump_fields(out);
        out << " action=" << action;
}
void IR::DpdkJmpActionStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpStatement::toJSON(json);
    json.emit("action", action);
}
IR::DpdkJmpActionStatement::DpdkJmpActionStatement(JSONLoader & json) : DpdkJmpStatement(json) {
    json.load("action", action);
}
bool IR::DpdkJmpActionStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpActionStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpActionStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 202 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpIfActionRunStatement::DpdkJmpIfActionRunStatement(cstring label, cstring act) :
        DpdkJmpActionStatement("jmpa"_cs, label, act) { }
#line 10707 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpIfActionRunStatement::operator==(IR::DpdkJmpIfActionRunStatement const & a) const {
        return DpdkJmpActionStatement::operator==(static_cast<const DpdkJmpActionStatement &>(a));
    }
bool IR::DpdkJmpIfActionRunStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpActionStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpIfActionRunStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpActionStatement::toJSON(json);
}
IR::DpdkJmpIfActionRunStatement::DpdkJmpIfActionRunStatement(JSONLoader & json) : DpdkJmpActionStatement(json) {
}
IR::Node * IR::DpdkJmpIfActionRunStatement::fromJSON(JSONLoader & json) { return new DpdkJmpIfActionRunStatement(json); }
bool IR::DpdkJmpIfActionRunStatement::operator==(IR::DpdkJmpActionStatement const & a) const { return a == *this; }
bool IR::DpdkJmpIfActionRunStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpIfActionRunStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpIfActionRunStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 209 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpIfActionNotRunStatement::DpdkJmpIfActionNotRunStatement(cstring label, cstring act) :
        DpdkJmpActionStatement("jmpna"_cs, label, act) { }
#line 10729 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpIfActionNotRunStatement::operator==(IR::DpdkJmpIfActionNotRunStatement const & a) const {
        return DpdkJmpActionStatement::operator==(static_cast<const DpdkJmpActionStatement &>(a));
    }
bool IR::DpdkJmpIfActionNotRunStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpActionStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpIfActionNotRunStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpActionStatement::toJSON(json);
}
IR::DpdkJmpIfActionNotRunStatement::DpdkJmpIfActionNotRunStatement(JSONLoader & json) : DpdkJmpActionStatement(json) {
}
IR::Node * IR::DpdkJmpIfActionNotRunStatement::fromJSON(JSONLoader & json) { return new DpdkJmpIfActionNotRunStatement(json); }
bool IR::DpdkJmpIfActionNotRunStatement::operator==(IR::DpdkJmpActionStatement const & a) const { return a == *this; }
bool IR::DpdkJmpIfActionNotRunStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpIfActionNotRunStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpIfActionNotRunStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 217 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpHeaderStatement::DpdkJmpHeaderStatement(cstring instruction, cstring label, const IR::Expression* hdr) :
        DpdkJmpStatement(instruction, label), header(hdr) { }
#line 10751 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpHeaderStatement::operator==(IR::DpdkJmpHeaderStatement const & a) const {
        return DpdkJmpStatement::operator==(static_cast<const DpdkJmpStatement &>(a))
        && header == a.header;
    }
bool IR::DpdkJmpHeaderStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkJmpHeaderStatement &>(a_);
        return (header ? a.header ? header->equiv(*a.header) : false : a.header == nullptr);
    }
void IR::DpdkJmpHeaderStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkJmpStatement::visit_children(v, n);
    v.visit(header, "header");
}
void IR::DpdkJmpHeaderStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkJmpStatement::visit_children(v, n);
    v.visit(header, "header");
}
void IR::DpdkJmpHeaderStatement::validate() const {
#line 10773 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(header); }
void IR::DpdkJmpHeaderStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpStatement::toJSON(json);
    json.emit("header", header);
}
IR::DpdkJmpHeaderStatement::DpdkJmpHeaderStatement(JSONLoader & json) : DpdkJmpStatement(json) {
    json.load("header", header);
}
bool IR::DpdkJmpHeaderStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpHeaderStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpHeaderStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 223 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpIfInvalidStatement::DpdkJmpIfInvalidStatement(cstring label, const IR::Expression* hdr) :
        DpdkJmpHeaderStatement("jmpnv"_cs, label, hdr) { }
#line 10789 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpIfInvalidStatement::operator==(IR::DpdkJmpIfInvalidStatement const & a) const {
        return DpdkJmpHeaderStatement::operator==(static_cast<const DpdkJmpHeaderStatement &>(a));
    }
bool IR::DpdkJmpIfInvalidStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpHeaderStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpIfInvalidStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpHeaderStatement::toJSON(json);
}
IR::DpdkJmpIfInvalidStatement::DpdkJmpIfInvalidStatement(JSONLoader & json) : DpdkJmpHeaderStatement(json) {
}
IR::Node * IR::DpdkJmpIfInvalidStatement::fromJSON(JSONLoader & json) { return new DpdkJmpIfInvalidStatement(json); }
bool IR::DpdkJmpIfInvalidStatement::operator==(IR::DpdkJmpHeaderStatement const & a) const { return a == *this; }
bool IR::DpdkJmpIfInvalidStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpIfInvalidStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpIfInvalidStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 230 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpIfValidStatement::DpdkJmpIfValidStatement(cstring label, const IR::Expression* hdr) :
        DpdkJmpHeaderStatement("jmpv"_cs, label, hdr) { }
#line 10811 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpIfValidStatement::operator==(IR::DpdkJmpIfValidStatement const & a) const {
        return DpdkJmpHeaderStatement::operator==(static_cast<const DpdkJmpHeaderStatement &>(a));
    }
bool IR::DpdkJmpIfValidStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpHeaderStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpIfValidStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpHeaderStatement::toJSON(json);
}
IR::DpdkJmpIfValidStatement::DpdkJmpIfValidStatement(JSONLoader & json) : DpdkJmpHeaderStatement(json) {
}
IR::Node * IR::DpdkJmpIfValidStatement::fromJSON(JSONLoader & json) { return new DpdkJmpIfValidStatement(json); }
bool IR::DpdkJmpIfValidStatement::operator==(IR::DpdkJmpHeaderStatement const & a) const { return a == *this; }
bool IR::DpdkJmpIfValidStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpIfValidStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpIfValidStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 239 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpCondStatement::DpdkJmpCondStatement(cstring instruction, cstring label, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkJmpStatement(instruction, label), src1(src1), src2(src2) { }
#line 10833 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpCondStatement::operator==(IR::DpdkJmpCondStatement const & a) const {
        return DpdkJmpStatement::operator==(static_cast<const DpdkJmpStatement &>(a))
        && src1 == a.src1
        && src2 == a.src2;
    }
bool IR::DpdkJmpCondStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkJmpCondStatement &>(a_);
        return (src1 ? a.src1 ? src1->equiv(*a.src1) : false : a.src1 == nullptr)
        && (src2 ? a.src2 ? src2->equiv(*a.src2) : false : a.src2 == nullptr);
    }
void IR::DpdkJmpCondStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkJmpStatement::visit_children(v, n);
    v.visit(src1, "src1");
    v.visit(src2, "src2");
}
void IR::DpdkJmpCondStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkJmpStatement::visit_children(v, n);
    v.visit(src1, "src1");
    v.visit(src2, "src2");
}
void IR::DpdkJmpCondStatement::validate() const {
#line 10859 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(src1);
        CHECK_NULL(src2); }
void IR::DpdkJmpCondStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpStatement::toJSON(json);
    json.emit("src1", src1);
    json.emit("src2", src2);
}
IR::DpdkJmpCondStatement::DpdkJmpCondStatement(JSONLoader & json) : DpdkJmpStatement(json) {
    json.load("src1", src1);
    json.load("src2", src2);
}
bool IR::DpdkJmpCondStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpCondStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpCondStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 244 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpEqualStatement::DpdkJmpEqualStatement(cstring label, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkJmpCondStatement("jmpeq"_cs, label, src1, src2) { }
#line 10878 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpEqualStatement::operator==(IR::DpdkJmpEqualStatement const & a) const {
        return DpdkJmpCondStatement::operator==(static_cast<const DpdkJmpCondStatement &>(a));
    }
bool IR::DpdkJmpEqualStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpCondStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpEqualStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpCondStatement::toJSON(json);
}
IR::DpdkJmpEqualStatement::DpdkJmpEqualStatement(JSONLoader & json) : DpdkJmpCondStatement(json) {
}
IR::Node * IR::DpdkJmpEqualStatement::fromJSON(JSONLoader & json) { return new DpdkJmpEqualStatement(json); }
bool IR::DpdkJmpEqualStatement::operator==(IR::DpdkJmpCondStatement const & a) const { return a == *this; }
bool IR::DpdkJmpEqualStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpEqualStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpEqualStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 251 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpNotEqualStatement::DpdkJmpNotEqualStatement(cstring label, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkJmpCondStatement("jmpneq"_cs, label, src1, src2) { }
#line 10900 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpNotEqualStatement::operator==(IR::DpdkJmpNotEqualStatement const & a) const {
        return DpdkJmpCondStatement::operator==(static_cast<const DpdkJmpCondStatement &>(a));
    }
bool IR::DpdkJmpNotEqualStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpCondStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpNotEqualStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpCondStatement::toJSON(json);
}
IR::DpdkJmpNotEqualStatement::DpdkJmpNotEqualStatement(JSONLoader & json) : DpdkJmpCondStatement(json) {
}
IR::Node * IR::DpdkJmpNotEqualStatement::fromJSON(JSONLoader & json) { return new DpdkJmpNotEqualStatement(json); }
bool IR::DpdkJmpNotEqualStatement::operator==(IR::DpdkJmpCondStatement const & a) const { return a == *this; }
bool IR::DpdkJmpNotEqualStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpNotEqualStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpNotEqualStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 258 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpGreaterEqualStatement::DpdkJmpGreaterEqualStatement(cstring label, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkJmpCondStatement("jmpge"_cs, label, src1, src2) { }
#line 10922 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpGreaterEqualStatement::operator==(IR::DpdkJmpGreaterEqualStatement const & a) const {
        return DpdkJmpCondStatement::operator==(static_cast<const DpdkJmpCondStatement &>(a));
    }
bool IR::DpdkJmpGreaterEqualStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpCondStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpGreaterEqualStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpCondStatement::toJSON(json);
}
IR::DpdkJmpGreaterEqualStatement::DpdkJmpGreaterEqualStatement(JSONLoader & json) : DpdkJmpCondStatement(json) {
}
IR::Node * IR::DpdkJmpGreaterEqualStatement::fromJSON(JSONLoader & json) { return new DpdkJmpGreaterEqualStatement(json); }
bool IR::DpdkJmpGreaterEqualStatement::operator==(IR::DpdkJmpCondStatement const & a) const { return a == *this; }
bool IR::DpdkJmpGreaterEqualStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpGreaterEqualStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpGreaterEqualStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 265 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpGreaterStatement::DpdkJmpGreaterStatement(cstring label, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkJmpCondStatement("jmpgt"_cs, label, src1, src2) { }
#line 10944 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpGreaterStatement::operator==(IR::DpdkJmpGreaterStatement const & a) const {
        return DpdkJmpCondStatement::operator==(static_cast<const DpdkJmpCondStatement &>(a));
    }
bool IR::DpdkJmpGreaterStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpCondStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpGreaterStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpCondStatement::toJSON(json);
}
IR::DpdkJmpGreaterStatement::DpdkJmpGreaterStatement(JSONLoader & json) : DpdkJmpCondStatement(json) {
}
IR::Node * IR::DpdkJmpGreaterStatement::fromJSON(JSONLoader & json) { return new DpdkJmpGreaterStatement(json); }
bool IR::DpdkJmpGreaterStatement::operator==(IR::DpdkJmpCondStatement const & a) const { return a == *this; }
bool IR::DpdkJmpGreaterStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpGreaterStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpGreaterStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 272 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpLessOrEqualStatement::DpdkJmpLessOrEqualStatement(cstring label, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkJmpCondStatement("jmple"_cs, label, src1, src2) { }
#line 10966 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpLessOrEqualStatement::operator==(IR::DpdkJmpLessOrEqualStatement const & a) const {
        return DpdkJmpCondStatement::operator==(static_cast<const DpdkJmpCondStatement &>(a));
    }
bool IR::DpdkJmpLessOrEqualStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpCondStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpLessOrEqualStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpCondStatement::toJSON(json);
}
IR::DpdkJmpLessOrEqualStatement::DpdkJmpLessOrEqualStatement(JSONLoader & json) : DpdkJmpCondStatement(json) {
}
IR::Node * IR::DpdkJmpLessOrEqualStatement::fromJSON(JSONLoader & json) { return new DpdkJmpLessOrEqualStatement(json); }
bool IR::DpdkJmpLessOrEqualStatement::operator==(IR::DpdkJmpCondStatement const & a) const { return a == *this; }
bool IR::DpdkJmpLessOrEqualStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpLessOrEqualStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpLessOrEqualStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 279 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkJmpLessStatement::DpdkJmpLessStatement(cstring label, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkJmpCondStatement("jmplt"_cs, label, src1, src2) { }
#line 10988 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkJmpLessStatement::operator==(IR::DpdkJmpLessStatement const & a) const {
        return DpdkJmpCondStatement::operator==(static_cast<const DpdkJmpCondStatement &>(a));
    }
bool IR::DpdkJmpLessStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkJmpCondStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkJmpLessStatement::toJSON(JSONGenerator & json) const {
    DpdkJmpCondStatement::toJSON(json);
}
IR::DpdkJmpLessStatement::DpdkJmpLessStatement(JSONLoader & json) : DpdkJmpCondStatement(json) {
}
IR::Node * IR::DpdkJmpLessStatement::fromJSON(JSONLoader & json) { return new DpdkJmpLessStatement(json); }
bool IR::DpdkJmpLessStatement::operator==(IR::DpdkJmpCondStatement const & a) const { return a == *this; }
bool IR::DpdkJmpLessStatement::operator==(IR::DpdkJmpStatement const & a) const { return a == *this; }
bool IR::DpdkJmpLessStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkJmpLessStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkRxStatement::operator==(IR::DpdkRxStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && port == a.port;
    }
bool IR::DpdkRxStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkRxStatement &>(a_);
        return (port ? a.port ? port->equiv(*a.port) : false : a.port == nullptr);
    }
void IR::DpdkRxStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(port, "port");
}
void IR::DpdkRxStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(port, "port");
}
void IR::DpdkRxStatement::validate() const {
#line 11028 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(port); }
void IR::DpdkRxStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("port", port);
}
IR::DpdkRxStatement::DpdkRxStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("port", port);
}
IR::Node * IR::DpdkRxStatement::fromJSON(JSONLoader & json) { return new DpdkRxStatement(json); }
bool IR::DpdkRxStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkRxStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkTxStatement::operator==(IR::DpdkTxStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && port == a.port;
    }
bool IR::DpdkTxStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkTxStatement &>(a_);
        return (port ? a.port ? port->equiv(*a.port) : false : a.port == nullptr);
    }
void IR::DpdkTxStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(port, "port");
}
void IR::DpdkTxStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(port, "port");
}
void IR::DpdkTxStatement::validate() const {
#line 11062 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(port); }
void IR::DpdkTxStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("port", port);
}
IR::DpdkTxStatement::DpdkTxStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("port", port);
}
IR::Node * IR::DpdkTxStatement::fromJSON(JSONLoader & json) { return new DpdkTxStatement(json); }
bool IR::DpdkTxStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkTxStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 299 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkAssignmentStatement::DpdkAssignmentStatement(cstring instruction, const IR::Expression* dst) :
        instruction(instruction), dst(dst) { }
#line 11078 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkAssignmentStatement::operator==(IR::DpdkAssignmentStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && instruction == a.instruction
        && dst == a.dst;
    }
bool IR::DpdkAssignmentStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkAssignmentStatement &>(a_);
        return instruction == a.instruction
        && (dst ? a.dst ? dst->equiv(*a.dst) : false : a.dst == nullptr);
    }
void IR::DpdkAssignmentStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(dst, "dst");
}
void IR::DpdkAssignmentStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(dst, "dst");
}
void IR::DpdkAssignmentStatement::validate() const {
#line 11102 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(dst); }
void IR::DpdkAssignmentStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " instruction=" << instruction;
}
void IR::DpdkAssignmentStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("instruction", instruction);
    json.emit("dst", dst);
}
IR::DpdkAssignmentStatement::DpdkAssignmentStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("instruction", instruction);
    json.load("dst", dst);
}
bool IR::DpdkAssignmentStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkAssignmentStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 307 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkUnaryStatement::DpdkUnaryStatement(cstring instruction, const IR::Expression* dst, const IR::Expression* src) :
        DpdkAssignmentStatement(instruction, dst), src(src) { }
#line 11123 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkUnaryStatement::operator==(IR::DpdkUnaryStatement const & a) const {
        return DpdkAssignmentStatement::operator==(static_cast<const DpdkAssignmentStatement &>(a))
        && src == a.src;
    }
bool IR::DpdkUnaryStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAssignmentStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkUnaryStatement &>(a_);
        return (src ? a.src ? src->equiv(*a.src) : false : a.src == nullptr);
    }
void IR::DpdkUnaryStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAssignmentStatement::visit_children(v, n);
    v.visit(src, "src");
}
void IR::DpdkUnaryStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAssignmentStatement::visit_children(v, n);
    v.visit(src, "src");
}
void IR::DpdkUnaryStatement::validate() const {
#line 11145 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(src); }
void IR::DpdkUnaryStatement::toJSON(JSONGenerator & json) const {
    DpdkAssignmentStatement::toJSON(json);
    json.emit("src", src);
}
IR::DpdkUnaryStatement::DpdkUnaryStatement(JSONLoader & json) : DpdkAssignmentStatement(json) {
    json.load("src", src);
}
bool IR::DpdkUnaryStatement::operator==(IR::DpdkAssignmentStatement const & a) const { return a == *this; }
bool IR::DpdkUnaryStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkUnaryStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 312 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkMovStatement::DpdkMovStatement(const IR::Expression* dst, const IR::Expression* src) :
        DpdkUnaryStatement("mov"_cs, dst, src) { }
#line 11161 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkMovStatement::operator==(IR::DpdkMovStatement const & a) const {
        return DpdkUnaryStatement::operator==(static_cast<const DpdkUnaryStatement &>(a));
    }
bool IR::DpdkMovStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkUnaryStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkMovStatement::toJSON(JSONGenerator & json) const {
    DpdkUnaryStatement::toJSON(json);
}
IR::DpdkMovStatement::DpdkMovStatement(JSONLoader & json) : DpdkUnaryStatement(json) {
}
IR::Node * IR::DpdkMovStatement::fromJSON(JSONLoader & json) { return new DpdkMovStatement(json); }
bool IR::DpdkMovStatement::operator==(IR::DpdkUnaryStatement const & a) const { return a == *this; }
bool IR::DpdkMovStatement::operator==(IR::DpdkAssignmentStatement const & a) const { return a == *this; }
bool IR::DpdkMovStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkMovStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 318 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkMovhStatement::DpdkMovhStatement(const IR::Expression* dst, const IR::Expression* src) :
        DpdkUnaryStatement("movh"_cs, dst, src) { }
#line 11183 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkMovhStatement::operator==(IR::DpdkMovhStatement const & a) const {
        return DpdkUnaryStatement::operator==(static_cast<const DpdkUnaryStatement &>(a));
    }
bool IR::DpdkMovhStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkUnaryStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkMovhStatement::toJSON(JSONGenerator & json) const {
    DpdkUnaryStatement::toJSON(json);
}
IR::DpdkMovhStatement::DpdkMovhStatement(JSONLoader & json) : DpdkUnaryStatement(json) {
}
IR::Node * IR::DpdkMovhStatement::fromJSON(JSONLoader & json) { return new DpdkMovhStatement(json); }
bool IR::DpdkMovhStatement::operator==(IR::DpdkUnaryStatement const & a) const { return a == *this; }
bool IR::DpdkMovhStatement::operator==(IR::DpdkAssignmentStatement const & a) const { return a == *this; }
bool IR::DpdkMovhStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkMovhStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 328 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkBinaryStatement::DpdkBinaryStatement(cstring instruction, const IR::Expression* dst, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkAssignmentStatement(instruction, dst), src1(src1), src2(src2) { }
#line 11205 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkBinaryStatement::operator==(IR::DpdkBinaryStatement const & a) const {
        return DpdkAssignmentStatement::operator==(static_cast<const DpdkAssignmentStatement &>(a))
        && src1 == a.src1
        && src2 == a.src2;
    }
bool IR::DpdkBinaryStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAssignmentStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkBinaryStatement &>(a_);
        return (src1 ? a.src1 ? src1->equiv(*a.src1) : false : a.src1 == nullptr)
        && (src2 ? a.src2 ? src2->equiv(*a.src2) : false : a.src2 == nullptr);
    }
void IR::DpdkBinaryStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAssignmentStatement::visit_children(v, n);
    v.visit(src1, "src1");
    v.visit(src2, "src2");
}
void IR::DpdkBinaryStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAssignmentStatement::visit_children(v, n);
    v.visit(src1, "src1");
    v.visit(src2, "src2");
}
void IR::DpdkBinaryStatement::validate() const {
#line 11231 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(src1);
        CHECK_NULL(src2); }
void IR::DpdkBinaryStatement::toJSON(JSONGenerator & json) const {
    DpdkAssignmentStatement::toJSON(json);
    json.emit("src1", src1);
    json.emit("src2", src2);
}
IR::DpdkBinaryStatement::DpdkBinaryStatement(JSONLoader & json) : DpdkAssignmentStatement(json) {
    json.load("src1", src1);
    json.load("src2", src2);
}
bool IR::DpdkBinaryStatement::operator==(IR::DpdkAssignmentStatement const & a) const { return a == *this; }
bool IR::DpdkBinaryStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkBinaryStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 335 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkAddStatement::DpdkAddStatement(const IR::Expression* dst, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkBinaryStatement("add"_cs, dst, src1, src2) { }
#line 11250 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkAddStatement::operator==(IR::DpdkAddStatement const & a) const {
        return DpdkBinaryStatement::operator==(static_cast<const DpdkBinaryStatement &>(a));
    }
bool IR::DpdkAddStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkBinaryStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkAddStatement::toJSON(JSONGenerator & json) const {
    DpdkBinaryStatement::toJSON(json);
}
IR::DpdkAddStatement::DpdkAddStatement(JSONLoader & json) : DpdkBinaryStatement(json) {
}
IR::Node * IR::DpdkAddStatement::fromJSON(JSONLoader & json) { return new DpdkAddStatement(json); }
bool IR::DpdkAddStatement::operator==(IR::DpdkBinaryStatement const & a) const { return a == *this; }
bool IR::DpdkAddStatement::operator==(IR::DpdkAssignmentStatement const & a) const { return a == *this; }
bool IR::DpdkAddStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkAddStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 342 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkAndStatement::DpdkAndStatement(const IR::Expression* dst, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkBinaryStatement("and"_cs, dst, src1, src2) { }
#line 11272 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkAndStatement::operator==(IR::DpdkAndStatement const & a) const {
        return DpdkBinaryStatement::operator==(static_cast<const DpdkBinaryStatement &>(a));
    }
bool IR::DpdkAndStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkBinaryStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkAndStatement::toJSON(JSONGenerator & json) const {
    DpdkBinaryStatement::toJSON(json);
}
IR::DpdkAndStatement::DpdkAndStatement(JSONLoader & json) : DpdkBinaryStatement(json) {
}
IR::Node * IR::DpdkAndStatement::fromJSON(JSONLoader & json) { return new DpdkAndStatement(json); }
bool IR::DpdkAndStatement::operator==(IR::DpdkBinaryStatement const & a) const { return a == *this; }
bool IR::DpdkAndStatement::operator==(IR::DpdkAssignmentStatement const & a) const { return a == *this; }
bool IR::DpdkAndStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkAndStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 349 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkShlStatement::DpdkShlStatement(const IR::Expression* dst, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkBinaryStatement("shl"_cs, dst, src1, src2) { }
#line 11294 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkShlStatement::operator==(IR::DpdkShlStatement const & a) const {
        return DpdkBinaryStatement::operator==(static_cast<const DpdkBinaryStatement &>(a));
    }
bool IR::DpdkShlStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkBinaryStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkShlStatement::toJSON(JSONGenerator & json) const {
    DpdkBinaryStatement::toJSON(json);
}
IR::DpdkShlStatement::DpdkShlStatement(JSONLoader & json) : DpdkBinaryStatement(json) {
}
IR::Node * IR::DpdkShlStatement::fromJSON(JSONLoader & json) { return new DpdkShlStatement(json); }
bool IR::DpdkShlStatement::operator==(IR::DpdkBinaryStatement const & a) const { return a == *this; }
bool IR::DpdkShlStatement::operator==(IR::DpdkAssignmentStatement const & a) const { return a == *this; }
bool IR::DpdkShlStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkShlStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 356 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkShrStatement::DpdkShrStatement(const IR::Expression* dst, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkBinaryStatement("shr"_cs, dst, src1, src2) { }
#line 11316 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkShrStatement::operator==(IR::DpdkShrStatement const & a) const {
        return DpdkBinaryStatement::operator==(static_cast<const DpdkBinaryStatement &>(a));
    }
bool IR::DpdkShrStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkBinaryStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkShrStatement::toJSON(JSONGenerator & json) const {
    DpdkBinaryStatement::toJSON(json);
}
IR::DpdkShrStatement::DpdkShrStatement(JSONLoader & json) : DpdkBinaryStatement(json) {
}
IR::Node * IR::DpdkShrStatement::fromJSON(JSONLoader & json) { return new DpdkShrStatement(json); }
bool IR::DpdkShrStatement::operator==(IR::DpdkBinaryStatement const & a) const { return a == *this; }
bool IR::DpdkShrStatement::operator==(IR::DpdkAssignmentStatement const & a) const { return a == *this; }
bool IR::DpdkShrStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkShrStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 363 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkSubStatement::DpdkSubStatement(const IR::Expression* dst, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkBinaryStatement("sub"_cs, dst, src1, src2) { }
#line 11338 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkSubStatement::operator==(IR::DpdkSubStatement const & a) const {
        return DpdkBinaryStatement::operator==(static_cast<const DpdkBinaryStatement &>(a));
    }
bool IR::DpdkSubStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkBinaryStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkSubStatement::toJSON(JSONGenerator & json) const {
    DpdkBinaryStatement::toJSON(json);
}
IR::DpdkSubStatement::DpdkSubStatement(JSONLoader & json) : DpdkBinaryStatement(json) {
}
IR::Node * IR::DpdkSubStatement::fromJSON(JSONLoader & json) { return new DpdkSubStatement(json); }
bool IR::DpdkSubStatement::operator==(IR::DpdkBinaryStatement const & a) const { return a == *this; }
bool IR::DpdkSubStatement::operator==(IR::DpdkAssignmentStatement const & a) const { return a == *this; }
bool IR::DpdkSubStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkSubStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 370 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkOrStatement::DpdkOrStatement(const IR::Expression* dst, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkBinaryStatement("or"_cs, dst, src1, src2) { }
#line 11360 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkOrStatement::operator==(IR::DpdkOrStatement const & a) const {
        return DpdkBinaryStatement::operator==(static_cast<const DpdkBinaryStatement &>(a));
    }
bool IR::DpdkOrStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkBinaryStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkOrStatement::toJSON(JSONGenerator & json) const {
    DpdkBinaryStatement::toJSON(json);
}
IR::DpdkOrStatement::DpdkOrStatement(JSONLoader & json) : DpdkBinaryStatement(json) {
}
IR::Node * IR::DpdkOrStatement::fromJSON(JSONLoader & json) { return new DpdkOrStatement(json); }
bool IR::DpdkOrStatement::operator==(IR::DpdkBinaryStatement const & a) const { return a == *this; }
bool IR::DpdkOrStatement::operator==(IR::DpdkAssignmentStatement const & a) const { return a == *this; }
bool IR::DpdkOrStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkOrStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 377 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkXorStatement::DpdkXorStatement(const IR::Expression* dst, const IR::Expression* src1, const IR::Expression* src2) :
        DpdkBinaryStatement("xor"_cs, dst, src1, src2) { }
#line 11382 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkXorStatement::operator==(IR::DpdkXorStatement const & a) const {
        return DpdkBinaryStatement::operator==(static_cast<const DpdkBinaryStatement &>(a));
    }
bool IR::DpdkXorStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkBinaryStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkXorStatement::toJSON(JSONGenerator & json) const {
    DpdkBinaryStatement::toJSON(json);
}
IR::DpdkXorStatement::DpdkXorStatement(JSONLoader & json) : DpdkBinaryStatement(json) {
}
IR::Node * IR::DpdkXorStatement::fromJSON(JSONLoader & json) { return new DpdkXorStatement(json); }
bool IR::DpdkXorStatement::operator==(IR::DpdkBinaryStatement const & a) const { return a == *this; }
bool IR::DpdkXorStatement::operator==(IR::DpdkAssignmentStatement const & a) const { return a == *this; }
bool IR::DpdkXorStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkXorStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkRecircidStatement::operator==(IR::DpdkRecircidStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && pass == a.pass;
    }
bool IR::DpdkRecircidStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkRecircidStatement &>(a_);
        return (pass ? a.pass ? pass->equiv(*a.pass) : false : a.pass == nullptr);
    }
void IR::DpdkRecircidStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(pass, "pass");
}
void IR::DpdkRecircidStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(pass, "pass");
}
void IR::DpdkRecircidStatement::validate() const {
#line 11422 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(pass); }
void IR::DpdkRecircidStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("pass", pass);
}
IR::DpdkRecircidStatement::DpdkRecircidStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("pass", pass);
}
IR::Node * IR::DpdkRecircidStatement::fromJSON(JSONLoader & json) { return new DpdkRecircidStatement(json); }
bool IR::DpdkRecircidStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkRecircidStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkReturnStatement::operator==(IR::DpdkReturnStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a));
    }
bool IR::DpdkReturnStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkReturnStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
}
IR::DpdkReturnStatement::DpdkReturnStatement(JSONLoader & json) : DpdkAsmStatement(json) {
}
IR::Node * IR::DpdkReturnStatement::fromJSON(JSONLoader & json) { return new DpdkReturnStatement(json); }
bool IR::DpdkReturnStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkReturnStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkRearmStatement::operator==(IR::DpdkRearmStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && timeout == a.timeout;
    }
bool IR::DpdkRearmStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkRearmStatement &>(a_);
        return (timeout ? a.timeout ? timeout->equiv(*a.timeout) : false : a.timeout == nullptr);
    }
void IR::DpdkRearmStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(timeout, "timeout");
}
void IR::DpdkRearmStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(timeout, "timeout");
}
void IR::DpdkRearmStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    if (timeout != nullptr) json.emit("timeout", timeout);
}
IR::DpdkRearmStatement::DpdkRearmStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("timeout", timeout);
}
IR::Node * IR::DpdkRearmStatement::fromJSON(JSONLoader & json) { return new DpdkRearmStatement(json); }
bool IR::DpdkRearmStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkRearmStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkRecirculateStatement::operator==(IR::DpdkRecirculateStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a));
    }
bool IR::DpdkRecirculateStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkRecirculateStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
}
IR::DpdkRecirculateStatement::DpdkRecirculateStatement(JSONLoader & json) : DpdkAsmStatement(json) {
}
IR::Node * IR::DpdkRecirculateStatement::fromJSON(JSONLoader & json) { return new DpdkRecirculateStatement(json); }
bool IR::DpdkRecirculateStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkRecirculateStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 407 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkLabelStatement::DpdkLabelStatement(cstring l) : label(l.toUpper()) { }
#line 11499 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkLabelStatement::operator==(IR::DpdkLabelStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && label == a.label;
    }
bool IR::DpdkLabelStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkLabelStatement &>(a_);
        return label == a.label;
    }
void IR::DpdkLabelStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " label=" << label;
}
void IR::DpdkLabelStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("label", label);
}
IR::DpdkLabelStatement::DpdkLabelStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("label", label);
}
IR::Node * IR::DpdkLabelStatement::fromJSON(JSONLoader & json) { return new DpdkLabelStatement(json); }
bool IR::DpdkLabelStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkLabelStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkChecksumAddStatement::operator==(IR::DpdkChecksumAddStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && csum == a.csum
        && intermediate_value == a.intermediate_value
        && field == a.field;
    }
bool IR::DpdkChecksumAddStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkChecksumAddStatement &>(a_);
        return csum == a.csum
        && intermediate_value == a.intermediate_value
        && (field ? a.field ? field->equiv(*a.field) : false : a.field == nullptr);
    }
void IR::DpdkChecksumAddStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(field, "field");
}
void IR::DpdkChecksumAddStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(field, "field");
}
void IR::DpdkChecksumAddStatement::validate() const {
#line 11549 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(field); }
void IR::DpdkChecksumAddStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " csum=" << csum;
        out << " intermediate_value=" << intermediate_value;
}
void IR::DpdkChecksumAddStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("csum", csum);
    json.emit("intermediate_value", intermediate_value);
    json.emit("field", field);
}
IR::DpdkChecksumAddStatement::DpdkChecksumAddStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("csum", csum);
    json.load("intermediate_value", intermediate_value);
    json.load("field", field);
}
IR::Node * IR::DpdkChecksumAddStatement::fromJSON(JSONLoader & json) { return new DpdkChecksumAddStatement(json); }
bool IR::DpdkChecksumAddStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkChecksumAddStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkChecksumSubStatement::operator==(IR::DpdkChecksumSubStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && csum == a.csum
        && intermediate_value == a.intermediate_value
        && field == a.field;
    }
bool IR::DpdkChecksumSubStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkChecksumSubStatement &>(a_);
        return csum == a.csum
        && intermediate_value == a.intermediate_value
        && (field ? a.field ? field->equiv(*a.field) : false : a.field == nullptr);
    }
void IR::DpdkChecksumSubStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(field, "field");
}
void IR::DpdkChecksumSubStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(field, "field");
}
void IR::DpdkChecksumSubStatement::validate() const {
#line 11596 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(field); }
void IR::DpdkChecksumSubStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " csum=" << csum;
        out << " intermediate_value=" << intermediate_value;
}
void IR::DpdkChecksumSubStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("csum", csum);
    json.emit("intermediate_value", intermediate_value);
    json.emit("field", field);
}
IR::DpdkChecksumSubStatement::DpdkChecksumSubStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("csum", csum);
    json.load("intermediate_value", intermediate_value);
    json.load("field", field);
}
IR::Node * IR::DpdkChecksumSubStatement::fromJSON(JSONLoader & json) { return new DpdkChecksumSubStatement(json); }
bool IR::DpdkChecksumSubStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkChecksumSubStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkChecksumClearStatement::operator==(IR::DpdkChecksumClearStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && csum == a.csum
        && intermediate_value == a.intermediate_value;
    }
bool IR::DpdkChecksumClearStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkChecksumClearStatement &>(a_);
        return csum == a.csum
        && intermediate_value == a.intermediate_value;
    }
void IR::DpdkChecksumClearStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " csum=" << csum;
        out << " intermediate_value=" << intermediate_value;
}
void IR::DpdkChecksumClearStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("csum", csum);
    json.emit("intermediate_value", intermediate_value);
}
IR::DpdkChecksumClearStatement::DpdkChecksumClearStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("csum", csum);
    json.load("intermediate_value", intermediate_value);
}
IR::Node * IR::DpdkChecksumClearStatement::fromJSON(JSONLoader & json) { return new DpdkChecksumClearStatement(json); }
bool IR::DpdkChecksumClearStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkChecksumClearStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkHashDeclStatement::operator==(IR::DpdkHashDeclStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && hash == a.hash;
    }
bool IR::DpdkHashDeclStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkHashDeclStatement &>(a_);
        return hash == a.hash;
    }
void IR::DpdkHashDeclStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " hash=" << hash;
}
void IR::DpdkHashDeclStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("hash", hash);
}
IR::DpdkHashDeclStatement::DpdkHashDeclStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("hash", hash);
}
IR::Node * IR::DpdkHashDeclStatement::fromJSON(JSONLoader & json) { return new DpdkHashDeclStatement(json); }
bool IR::DpdkHashDeclStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkHashDeclStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkGetHashStatement::operator==(IR::DpdkGetHashStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && instr == a.instr
        && hash == a.hash
        && fields == a.fields
        && dst == a.dst;
    }
bool IR::DpdkGetHashStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkGetHashStatement &>(a_);
        return instr == a.instr
        && hash == a.hash
        && (fields ? a.fields ? fields->equiv(*a.fields) : false : a.fields == nullptr)
        && (dst ? a.dst ? dst->equiv(*a.dst) : false : a.dst == nullptr);
    }
void IR::DpdkGetHashStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(fields, "fields");
    v.visit(dst, "dst");
}
void IR::DpdkGetHashStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(fields, "fields");
    v.visit(dst, "dst");
}
void IR::DpdkGetHashStatement::validate() const {
#line 11700 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(fields);
        CHECK_NULL(dst); }
void IR::DpdkGetHashStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " instr=" << instr;
        out << " hash=" << hash;
}
void IR::DpdkGetHashStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("instr", instr);
    json.emit("hash", hash);
    json.emit("fields", fields);
    json.emit("dst", dst);
}
IR::DpdkGetHashStatement::DpdkGetHashStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("instr", instr);
    json.load("hash", hash);
    json.load("fields", fields);
    json.load("dst", dst);
}
IR::Node * IR::DpdkGetHashStatement::fromJSON(JSONLoader & json) { return new DpdkGetHashStatement(json); }
bool IR::DpdkGetHashStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkGetHashStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkGetChecksumStatement::operator==(IR::DpdkGetChecksumStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && dst == a.dst
        && checksum == a.checksum
        && intermediate_value == a.intermediate_value;
    }
bool IR::DpdkGetChecksumStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkGetChecksumStatement &>(a_);
        return (dst ? a.dst ? dst->equiv(*a.dst) : false : a.dst == nullptr)
        && checksum == a.checksum
        && intermediate_value == a.intermediate_value;
    }
void IR::DpdkGetChecksumStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(dst, "dst");
}
void IR::DpdkGetChecksumStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(dst, "dst");
}
void IR::DpdkGetChecksumStatement::validate() const {
#line 11750 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(dst); }
void IR::DpdkGetChecksumStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " checksum=" << checksum;
        out << " intermediate_value=" << intermediate_value;
}
void IR::DpdkGetChecksumStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("dst", dst);
    json.emit("checksum", checksum);
    json.emit("intermediate_value", intermediate_value);
}
IR::DpdkGetChecksumStatement::DpdkGetChecksumStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("dst", dst);
    json.load("checksum", checksum);
    json.load("intermediate_value", intermediate_value);
}
IR::Node * IR::DpdkGetChecksumStatement::fromJSON(JSONLoader & json) { return new DpdkGetChecksumStatement(json); }
bool IR::DpdkGetChecksumStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkGetChecksumStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkCastStatement::operator==(IR::DpdkCastStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && dst == a.dst
        && src == a.src
        && type == a.type;
    }
bool IR::DpdkCastStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkCastStatement &>(a_);
        return (dst ? a.dst ? dst->equiv(*a.dst) : false : a.dst == nullptr)
        && (src ? a.src ? src->equiv(*a.src) : false : a.src == nullptr)
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr);
    }
void IR::DpdkCastStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(dst, "dst");
    v.visit(src, "src");
    v.visit(type, "type");
}
void IR::DpdkCastStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(dst, "dst");
    v.visit(src, "src");
    v.visit(type, "type");
}
void IR::DpdkCastStatement::validate() const {
#line 11801 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(dst);
        CHECK_NULL(src);
        CHECK_NULL(type); }
void IR::DpdkCastStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("dst", dst);
    json.emit("src", src);
    json.emit("type", type);
}
IR::DpdkCastStatement::DpdkCastStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("dst", dst);
    json.load("src", src);
    json.load("type", type);
}
IR::Node * IR::DpdkCastStatement::fromJSON(JSONLoader & json) { return new DpdkCastStatement(json); }
bool IR::DpdkCastStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkCastStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkVerifyStatement::operator==(IR::DpdkVerifyStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && condition == a.condition
        && error == a.error;
    }
bool IR::DpdkVerifyStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkVerifyStatement &>(a_);
        return (condition ? a.condition ? condition->equiv(*a.condition) : false : a.condition == nullptr)
        && (error ? a.error ? error->equiv(*a.error) : false : a.error == nullptr);
    }
void IR::DpdkVerifyStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(condition, "condition");
    v.visit(error, "error");
}
void IR::DpdkVerifyStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(condition, "condition");
    v.visit(error, "error");
}
void IR::DpdkVerifyStatement::validate() const {
#line 11845 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(condition);
        CHECK_NULL(error); }
void IR::DpdkVerifyStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("condition", condition);
    json.emit("error", error);
}
IR::DpdkVerifyStatement::DpdkVerifyStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("condition", condition);
    json.load("error", error);
}
IR::Node * IR::DpdkVerifyStatement::fromJSON(JSONLoader & json) { return new DpdkVerifyStatement(json); }
bool IR::DpdkVerifyStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkVerifyStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkMeterDeclStatement::operator==(IR::DpdkMeterDeclStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && meter == a.meter
        && size == a.size;
    }
bool IR::DpdkMeterDeclStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkMeterDeclStatement &>(a_);
        return meter == a.meter
        && (size ? a.size ? size->equiv(*a.size) : false : a.size == nullptr);
    }
void IR::DpdkMeterDeclStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(size, "size");
}
void IR::DpdkMeterDeclStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(size, "size");
}
void IR::DpdkMeterDeclStatement::validate() const {
#line 11884 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(size); }
void IR::DpdkMeterDeclStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " meter=" << meter;
}
void IR::DpdkMeterDeclStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("meter", meter);
    json.emit("size", size);
}
IR::DpdkMeterDeclStatement::DpdkMeterDeclStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("meter", meter);
    json.load("size", size);
}
IR::Node * IR::DpdkMeterDeclStatement::fromJSON(JSONLoader & json) { return new DpdkMeterDeclStatement(json); }
bool IR::DpdkMeterDeclStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkMeterDeclStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkMeterExecuteStatement::operator==(IR::DpdkMeterExecuteStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && meter == a.meter
        && index == a.index
        && length == a.length
        && color_in == a.color_in
        && color_out == a.color_out;
    }
bool IR::DpdkMeterExecuteStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkMeterExecuteStatement &>(a_);
        return meter == a.meter
        && (index ? a.index ? index->equiv(*a.index) : false : a.index == nullptr)
        && (length ? a.length ? length->equiv(*a.length) : false : a.length == nullptr)
        && (color_in ? a.color_in ? color_in->equiv(*a.color_in) : false : a.color_in == nullptr)
        && (color_out ? a.color_out ? color_out->equiv(*a.color_out) : false : a.color_out == nullptr);
    }
void IR::DpdkMeterExecuteStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(index, "index");
    v.visit(length, "length");
    v.visit(color_in, "color_in");
    v.visit(color_out, "color_out");
}
void IR::DpdkMeterExecuteStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(index, "index");
    v.visit(length, "length");
    v.visit(color_in, "color_in");
    v.visit(color_out, "color_out");
}
void IR::DpdkMeterExecuteStatement::validate() const {
#line 11938 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(index);
        CHECK_NULL(color_in);
        CHECK_NULL(color_out); }
void IR::DpdkMeterExecuteStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " meter=" << meter;
}
void IR::DpdkMeterExecuteStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("meter", meter);
    json.emit("index", index);
    if (length != nullptr) json.emit("length", length);
    json.emit("color_in", color_in);
    json.emit("color_out", color_out);
}
IR::DpdkMeterExecuteStatement::DpdkMeterExecuteStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("meter", meter);
    json.load("index", index);
    json.load("length", length);
    json.load("color_in", color_in);
    json.load("color_out", color_out);
}
IR::Node * IR::DpdkMeterExecuteStatement::fromJSON(JSONLoader & json) { return new DpdkMeterExecuteStatement(json); }
bool IR::DpdkMeterExecuteStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkMeterExecuteStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkGetTableEntryIndex::operator==(IR::DpdkGetTableEntryIndex const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && index == a.index;
    }
bool IR::DpdkGetTableEntryIndex::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkGetTableEntryIndex &>(a_);
        return (index ? a.index ? index->equiv(*a.index) : false : a.index == nullptr);
    }
void IR::DpdkGetTableEntryIndex::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(index, "index");
}
void IR::DpdkGetTableEntryIndex::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(index, "index");
}
void IR::DpdkGetTableEntryIndex::validate() const {
#line 11986 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(index); }
void IR::DpdkGetTableEntryIndex::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("index", index);
}
IR::DpdkGetTableEntryIndex::DpdkGetTableEntryIndex(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("index", index);
}
IR::Node * IR::DpdkGetTableEntryIndex::fromJSON(JSONLoader & json) { return new DpdkGetTableEntryIndex(json); }
bool IR::DpdkGetTableEntryIndex::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkGetTableEntryIndex::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkCounterCountStatement::operator==(IR::DpdkCounterCountStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && counter == a.counter
        && index == a.index
        && incr == a.incr;
    }
bool IR::DpdkCounterCountStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkCounterCountStatement &>(a_);
        return counter == a.counter
        && (index ? a.index ? index->equiv(*a.index) : false : a.index == nullptr)
        && (incr ? a.incr ? incr->equiv(*a.incr) : false : a.incr == nullptr);
    }
void IR::DpdkCounterCountStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(index, "index");
    v.visit(incr, "incr");
}
void IR::DpdkCounterCountStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(index, "index");
    v.visit(incr, "incr");
}
void IR::DpdkCounterCountStatement::validate() const {
#line 12026 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(index); }
void IR::DpdkCounterCountStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " counter=" << counter;
}
void IR::DpdkCounterCountStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("counter", counter);
    json.emit("index", index);
    if (incr != nullptr) json.emit("incr", incr);
}
IR::DpdkCounterCountStatement::DpdkCounterCountStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("counter", counter);
    json.load("index", index);
    json.load("incr", incr);
}
IR::Node * IR::DpdkCounterCountStatement::fromJSON(JSONLoader & json) { return new DpdkCounterCountStatement(json); }
bool IR::DpdkCounterCountStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkCounterCountStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkRegisterDeclStatement::operator==(IR::DpdkRegisterDeclStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && reg == a.reg
        && size == a.size
        && init_val == a.init_val;
    }
bool IR::DpdkRegisterDeclStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkRegisterDeclStatement &>(a_);
        return reg == a.reg
        && (size ? a.size ? size->equiv(*a.size) : false : a.size == nullptr)
        && (init_val ? a.init_val ? init_val->equiv(*a.init_val) : false : a.init_val == nullptr);
    }
void IR::DpdkRegisterDeclStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(size, "size");
    v.visit(init_val, "init_val");
}
void IR::DpdkRegisterDeclStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(size, "size");
    v.visit(init_val, "init_val");
}
void IR::DpdkRegisterDeclStatement::validate() const {
#line 12074 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(size); }
void IR::DpdkRegisterDeclStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " reg=" << reg;
}
void IR::DpdkRegisterDeclStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("reg", reg);
    json.emit("size", size);
    if (init_val != nullptr) json.emit("init_val", init_val);
}
IR::DpdkRegisterDeclStatement::DpdkRegisterDeclStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("reg", reg);
    json.load("size", size);
    json.load("init_val", init_val);
}
IR::Node * IR::DpdkRegisterDeclStatement::fromJSON(JSONLoader & json) { return new DpdkRegisterDeclStatement(json); }
bool IR::DpdkRegisterDeclStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkRegisterDeclStatement::operator==(IR::Node const & a) const { return a == *this; }
#line 517 "/builddir/build/BUILD/p4c-1.2.5.8/backends/dpdk/dpdk.def"
IR::DpdkRegisterReadStatement::DpdkRegisterReadStatement(const IR::Expression* dst, cstring reg, const IR::Expression* index) :
        DpdkAssignmentStatement("read"_cs, dst), reg(reg), index(index) { }
#line 12098 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::DpdkRegisterReadStatement::operator==(IR::DpdkRegisterReadStatement const & a) const {
        return DpdkAssignmentStatement::operator==(static_cast<const DpdkAssignmentStatement &>(a))
        && reg == a.reg
        && index == a.index;
    }
bool IR::DpdkRegisterReadStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAssignmentStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkRegisterReadStatement &>(a_);
        return reg == a.reg
        && (index ? a.index ? index->equiv(*a.index) : false : a.index == nullptr);
    }
void IR::DpdkRegisterReadStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAssignmentStatement::visit_children(v, n);
    v.visit(index, "index");
}
void IR::DpdkRegisterReadStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAssignmentStatement::visit_children(v, n);
    v.visit(index, "index");
}
void IR::DpdkRegisterReadStatement::validate() const {
#line 12122 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(index); }
void IR::DpdkRegisterReadStatement::dump_fields(std::ostream & out) const {
    DpdkAssignmentStatement::dump_fields(out);
        out << " reg=" << reg;
}
void IR::DpdkRegisterReadStatement::toJSON(JSONGenerator & json) const {
    DpdkAssignmentStatement::toJSON(json);
    json.emit("reg", reg);
    json.emit("index", index);
}
IR::DpdkRegisterReadStatement::DpdkRegisterReadStatement(JSONLoader & json) : DpdkAssignmentStatement(json) {
    json.load("reg", reg);
    json.load("index", index);
}
IR::Node * IR::DpdkRegisterReadStatement::fromJSON(JSONLoader & json) { return new DpdkRegisterReadStatement(json); }
bool IR::DpdkRegisterReadStatement::operator==(IR::DpdkAssignmentStatement const & a) const { return a == *this; }
bool IR::DpdkRegisterReadStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkRegisterReadStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkRegisterWriteStatement::operator==(IR::DpdkRegisterWriteStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && reg == a.reg
        && index == a.index
        && src == a.src;
    }
bool IR::DpdkRegisterWriteStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkRegisterWriteStatement &>(a_);
        return reg == a.reg
        && (index ? a.index ? index->equiv(*a.index) : false : a.index == nullptr)
        && (src ? a.src ? src->equiv(*a.src) : false : a.src == nullptr);
    }
void IR::DpdkRegisterWriteStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(index, "index");
    v.visit(src, "src");
}
void IR::DpdkRegisterWriteStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(index, "index");
    v.visit(src, "src");
}
void IR::DpdkRegisterWriteStatement::validate() const {
#line 12169 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(index);
        CHECK_NULL(src); }
void IR::DpdkRegisterWriteStatement::dump_fields(std::ostream & out) const {
    DpdkAsmStatement::dump_fields(out);
        out << " reg=" << reg;
}
void IR::DpdkRegisterWriteStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("reg", reg);
    json.emit("index", index);
    json.emit("src", src);
}
IR::DpdkRegisterWriteStatement::DpdkRegisterWriteStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("reg", reg);
    json.load("index", index);
    json.load("src", src);
}
IR::Node * IR::DpdkRegisterWriteStatement::fromJSON(JSONLoader & json) { return new DpdkRegisterWriteStatement(json); }
bool IR::DpdkRegisterWriteStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkRegisterWriteStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkValidateStatement::operator==(IR::DpdkValidateStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && header == a.header;
    }
bool IR::DpdkValidateStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkValidateStatement &>(a_);
        return (header ? a.header ? header->equiv(*a.header) : false : a.header == nullptr);
    }
void IR::DpdkValidateStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(header, "header");
}
void IR::DpdkValidateStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(header, "header");
}
void IR::DpdkValidateStatement::validate() const {
#line 12212 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(header); }
void IR::DpdkValidateStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("header", header);
}
IR::DpdkValidateStatement::DpdkValidateStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("header", header);
}
IR::Node * IR::DpdkValidateStatement::fromJSON(JSONLoader & json) { return new DpdkValidateStatement(json); }
bool IR::DpdkValidateStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkValidateStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkInvalidateStatement::operator==(IR::DpdkInvalidateStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a))
        && header == a.header;
    }
bool IR::DpdkInvalidateStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        auto &a = static_cast<const DpdkInvalidateStatement &>(a_);
        return (header ? a.header ? header->equiv(*a.header) : false : a.header == nullptr);
    }
void IR::DpdkInvalidateStatement::visit_children(Visitor & v, char const * n) {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(header, "header");
}
void IR::DpdkInvalidateStatement::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DpdkAsmStatement::visit_children(v, n);
    v.visit(header, "header");
}
void IR::DpdkInvalidateStatement::validate() const {
#line 12246 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(header); }
void IR::DpdkInvalidateStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
    json.emit("header", header);
}
IR::DpdkInvalidateStatement::DpdkInvalidateStatement(JSONLoader & json) : DpdkAsmStatement(json) {
    json.load("header", header);
}
IR::Node * IR::DpdkInvalidateStatement::fromJSON(JSONLoader & json) { return new DpdkInvalidateStatement(json); }
bool IR::DpdkInvalidateStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkInvalidateStatement::operator==(IR::Node const & a) const { return a == *this; }
bool IR::DpdkDropStatement::operator==(IR::DpdkDropStatement const & a) const {
        return DpdkAsmStatement::operator==(static_cast<const DpdkAsmStatement &>(a));
    }
bool IR::DpdkDropStatement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DpdkAsmStatement::equiv(a_)) return false;
        return true;
    }
void IR::DpdkDropStatement::toJSON(JSONGenerator & json) const {
    DpdkAsmStatement::toJSON(json);
}
IR::DpdkDropStatement::DpdkDropStatement(JSONLoader & json) : DpdkAsmStatement(json) {
}
IR::Node * IR::DpdkDropStatement::fromJSON(JSONLoader & json) { return new DpdkDropStatement(json); }
bool IR::DpdkDropStatement::operator==(IR::DpdkAsmStatement const & a) const { return a == *this; }
bool IR::DpdkDropStatement::operator==(IR::Node const & a) const { return a == *this; }

#line 25 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCKernelMetadata::toString() const {
        std::string metaName = "kernel.";
        switch (metaField) {
            case TC::SKBREDIR: {
                absl::StrAppend(&metaName, "skbredir");
                break;
            }
            case TC::SKBIIF: {
                absl::StrAppend(&metaName, "skbiif");
                break;
            }
            case TC::SKBTSTAMP: {
                absl::StrAppend(&metaName, "skbtstamp");
                break;
            }
            case TC::SKBPRIO: {
                absl::StrAppend(&metaName, "skbprio");
                break;
            }
        }
        return metaName;
    }
#line 12299 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 47 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCKernelMetadata::dbprint(std::ostream & out) const { out << toString(); }
#line 12302 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TCKernelMetadata::operator==(IR::TCKernelMetadata const & a) const {
        return this->typeId() == a.typeId()
        && metaField == a.metaField;
    }
bool IR::TCKernelMetadata::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TCKernelMetadata &>(a_);
        return metaField == a.metaField;
    }
void IR::TCKernelMetadata::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " metaField=" << metaField;
}
void IR::TCKernelMetadata::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("metaField", metaField);
}
IR::TCKernelMetadata::TCKernelMetadata(JSONLoader & json) : Node(json) {
    json.load("metaField", metaField);
}
IR::Node * IR::TCKernelMetadata::fromJSON(JSONLoader & json) { return new TCKernelMetadata(json); }
bool IR::TCKernelMetadata::operator==(IR::Node const & a) const { return a == *this; }
#line 55 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCActionParam::setParamName(cstring pN) {
        paramName = pN;
    }
#line 12330 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 58 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCActionParam::setBitSize(unsigned bS) {
        bitSize = bS;
    }
#line 12335 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 61 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCActionParam::setDataType(unsigned d) {
        dataType = d;
    }
#line 12340 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 64 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCActionParam::setDirection(unsigned d) {
        direction = d;
    }
#line 12345 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 67 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
IR::TCActionParam::TCActionParam() {
        paramName = nullptr;
        bitSize = 0;
    }
#line 12351 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 71 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCActionParam::getParamName() const {
        return paramName;
    }
#line 12356 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 74 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCActionParam::getName() const {
        return "param." + paramName;
    }
#line 12361 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 77 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
unsigned IR::TCActionParam::getDirection() const {
        return direction;
    }
#line 12366 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 80 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCActionParam::getParamDecl(cstring placeholderName) const {
        std::string paramDecl = "";
        switch(dataType) {
            case TC::BIT_TYPE :
                if (bitSize == 8 || bitSize == 16 || bitSize == 32 || bitSize == 64) {
                    paramDecl = absl::StrCat("__u", bitSize, " ", placeholderName);
                } else {
                    unsigned byteSize = bitSize / 8 + (bitSize % 8 != 0);

                    paramDecl = absl::StrCat("__u8 ", placeholderName, "[", byteSize, "]");
                }
                break;
            case TC::DEV_TYPE :
                paramDecl = absl::StrCat("__u32 ", placeholderName);
                break;
            case TC::MACADDR_TYPE :
                paramDecl = absl::StrCat("__u8 ", placeholderName, "[6]");
                break;
            case TC::IPV4_TYPE :
                paramDecl = absl::StrCat("__u32 ", placeholderName);
                break;
            case TC::IPV6_TYPE :
                paramDecl = absl::StrCat("__u8 ", placeholderName, "[16]");
                break;
            case TC::BE16_TYPE :
                paramDecl = absl::StrCat("__be16 ", placeholderName);
                break;
            case TC::BE32_TYPE :
                paramDecl = absl::StrCat("__be32 ", placeholderName);
                break;
            case TC::BE64_TYPE :
                paramDecl = absl::StrCat("__be64 ", placeholderName);
                break;
        }
        return paramDecl;
    }
#line 12404 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 116 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCActionParam::toString() const {
        std::string tcActionParam = absl::StrCat("\n\tparam ", paramName, " type ");
        switch(dataType) {
            case TC::BIT_TYPE :
                absl::StrAppend(&tcActionParam, "bit", bitSize);
                break;
            case TC::DEV_TYPE :
                absl::StrAppend(&tcActionParam, "dev");
                break;
            case TC::MACADDR_TYPE :
                absl::StrAppend(&tcActionParam, "macaddr");
                break;
            case TC::IPV4_TYPE :
                absl::StrAppend(&tcActionParam, "ipv4");
                break;
            case TC::IPV6_TYPE :
                absl::StrAppend(&tcActionParam, "ipv6");
                break;
            case TC::BE16_TYPE :
                absl::StrAppend(&tcActionParam, "be16");
                break;
            case TC::BE32_TYPE :
                absl::StrAppend(&tcActionParam, "be32");
                break;
            case TC::BE64_TYPE :
                absl::StrAppend(&tcActionParam, "be64");
                break;
        }
        return tcActionParam;
    }
#line 12436 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 146 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCActionParam::dbprint(std::ostream & out) const { out << toString(); }
#line 12439 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TCActionParam::operator==(IR::TCActionParam const & a) const {
        return this->typeId() == a.typeId()
        && paramName == a.paramName
        && dataType == a.dataType
        && bitSize == a.bitSize
        && direction == a.direction;
    }
bool IR::TCActionParam::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TCActionParam &>(a_);
        return paramName == a.paramName
        && dataType == a.dataType
        && bitSize == a.bitSize
        && direction == a.direction;
    }
void IR::TCActionParam::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " paramName=" << paramName;
        out << " dataType=" << dataType;
        out << " bitSize=" << bitSize;
        out << " direction=" << direction;
}
void IR::TCActionParam::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("paramName", paramName);
    json.emit("dataType", dataType);
    json.emit("bitSize", bitSize);
    json.emit("direction", direction);
}
IR::TCActionParam::TCActionParam(JSONLoader & json) : Node(json) {
    json.load("paramName", paramName);
    json.load("dataType", dataType);
    json.load("bitSize", bitSize);
    json.load("direction", direction);
}
IR::Node * IR::TCActionParam::fromJSON(JSONLoader & json) { return new TCActionParam(json); }
bool IR::TCActionParam::operator==(IR::Node const & a) const { return a == *this; }
#line 152 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCDefaultActionParam::setParamDetail(const IR::TCActionParam* pN) {
        paramDetail = pN;
    }
#line 12482 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 155 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCDefaultActionParam::setDefaultValue(cstring dV) {
        defaultValue = dV;
    }
#line 12487 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 158 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
IR::TCDefaultActionParam::TCDefaultActionParam() {
        paramDetail = nullptr;
        defaultValue = nullptr;
    }
#line 12493 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 162 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCDefaultActionParam::toString() const {
        std::string tcActionParam = absl::StrCat(" ", paramDetail->paramName);
        if (defaultValue != nullptr)
            absl::StrAppend(&tcActionParam, " ", defaultValue);
        return tcActionParam;
    }
#line 12501 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 168 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCDefaultActionParam::dbprint(std::ostream & out) const { out << toString(); }
#line 12504 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TCDefaultActionParam::operator==(IR::TCDefaultActionParam const & a) const {
        return this->typeId() == a.typeId()
        && paramDetail == a.paramDetail
        && defaultValue == a.defaultValue;
    }
bool IR::TCDefaultActionParam::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TCDefaultActionParam &>(a_);
        return (paramDetail ? a.paramDetail ? paramDetail->equiv(*a.paramDetail) : false : a.paramDetail == nullptr)
        && defaultValue == a.defaultValue;
    }
void IR::TCDefaultActionParam::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(paramDetail, "paramDetail");
}
void IR::TCDefaultActionParam::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(paramDetail, "paramDetail");
}
void IR::TCDefaultActionParam::validate() const {
#line 12528 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(paramDetail); }
void IR::TCDefaultActionParam::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " defaultValue=" << defaultValue;
}
void IR::TCDefaultActionParam::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("paramDetail", paramDetail);
    json.emit("defaultValue", defaultValue);
}
IR::TCDefaultActionParam::TCDefaultActionParam(JSONLoader & json) : Node(json) {
    json.load("paramDetail", paramDetail);
    json.load("defaultValue", defaultValue);
}
IR::Node * IR::TCDefaultActionParam::fromJSON(JSONLoader & json) { return new TCDefaultActionParam(json); }
bool IR::TCDefaultActionParam::operator==(IR::Node const & a) const { return a == *this; }
#line 176 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCAction::getName() const {
        if (actionName == "NoAction") {
            return actionName;
        }
        return absl::StrCat(pipelineName, "/", actionName);
    }
#line 12553 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 182 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCAction::getActionName() const {
        return actionName;
    }
#line 12558 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 185 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCAction::setPipelineName(cstring pN) {
        pipelineName = pN;
    }
#line 12563 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 188 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCAction::addActionParams(const IR::TCActionParam* tca) {
        actionParams.push_back(tca);
    }
#line 12568 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 191 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCAction::setActionId(unsigned id) {
        actId = id;
    }
#line 12573 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 194 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
IR::TCAction::TCAction(cstring aN) {
        actionName = aN;
        pipelineName = nullptr;
        actId = 0;
    }
#line 12580 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 199 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCAction::toString() const {
        std::string tcAction = absl::StrCat("\n$TC p4template create action/", pipelineName, "/", actionName);
        if (actId != 0) {
            absl::StrAppend(&tcAction, " actid ", actId);
        }
        if (!actionParams.empty()) {
            for (auto actParam : actionParams) {
                absl::StrAppend(&tcAction, " \\", actParam);
            }
        }
        absl::StrAppend(&tcAction, "\n$TC p4template update action/", pipelineName, "/", actionName, " state active");

        return tcAction;
    }
#line 12596 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 213 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCAction::dbprint(std::ostream & out) const { out << toString(); }
#line 12599 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TCAction::operator==(IR::TCAction const & a) const {
        return this->typeId() == a.typeId()
        && actionName == a.actionName
        && pipelineName == a.pipelineName
        && actId == a.actId
        && actionParams == a.actionParams;
    }
bool IR::TCAction::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TCAction &>(a_);
        return actionName == a.actionName
        && pipelineName == a.pipelineName
        && actId == a.actId
        && actionParams == a.actionParams;
    }
void IR::TCAction::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " actionName=" << actionName;
        out << " pipelineName=" << pipelineName;
        out << " actId=" << actId;
}
void IR::TCAction::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("actionName", actionName);
    json.emit("pipelineName", pipelineName);
    json.emit("actId", actId);
    json.emit("actionParams", actionParams);
}
IR::TCAction::TCAction(JSONLoader & json) : Node(json) {
    json.load("actionName", actionName);
    json.load("pipelineName", pipelineName);
    json.load("actId", actId);
    json.load("actionParams", actionParams);
}
IR::Node * IR::TCAction::fromJSON(JSONLoader & json) { return new TCAction(json); }
bool IR::TCAction::operator==(IR::Node const & a) const { return a == *this; }
#line 219 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCEntry::getActionName() const {
        return action;
    }
#line 12641 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 222 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCEntry::toString() const {
        std::string tcEntry = "";
        for (auto k : keys) {
            tcEntry = absl::StrJoin({tcEntry, k.first.string(), k.second.string()}, " ");
        }
        return tcEntry;
    }
#line 12650 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 229 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCEntry::dbprint(std::ostream & out) const { out << toString(); }
#line 12653 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TCEntry::operator==(IR::TCEntry const & a) const {
        return this->typeId() == a.typeId()
        && action == a.action
        && keys == a.keys;
    }
bool IR::TCEntry::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TCEntry &>(a_);
        return action == a.action
        && keys == a.keys;
    }
void IR::TCEntry::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " action=" << action;
}
void IR::TCEntry::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("action", action);
    json.emit("keys", keys);
}
IR::TCEntry::TCEntry(JSONLoader & json) : Node(json) {
    json.load("action", action);
    json.load("keys", keys);
}
IR::Node * IR::TCEntry::fromJSON(JSONLoader & json) { return new TCEntry(json); }
bool IR::TCEntry::operator==(IR::Node const & a) const { return a == *this; }
#line 261 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setTablePermission(cstring p) {
        permissions = p;
    }
#line 12685 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 264 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setDirectCounter(cstring counter) {
        isDirectCounter = true;
        directCounterInstance = counter;
    }
#line 12691 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 268 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setDirectMeter(cstring meter) {
        isDirectMeter = true;
        directMeterInstance = meter;
    }
#line 12697 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 272 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setKeySize(unsigned k) {
        keySize = k;
    }
#line 12702 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 275 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setTableEntriesCount(unsigned t) {
        tableEntriesCount = t;
    }
#line 12707 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 278 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setNumMask(unsigned n) {
        numMask = n;
    }
#line 12712 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 281 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setMatchType(unsigned m) {
        matchType = m;
    }
#line 12717 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 284 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setDefaultHitAction(const IR::TCAction* d) {
        defaultHitAction = d;
    }
#line 12722 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 287 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setDefaultMissAction(const IR::TCAction* d) {
        defaultMissAction = d;
    }
#line 12727 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 290 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setDefaultHitConst(bool i) {
        isDefaultHitConst = i;
    }
#line 12732 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 293 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setDefaultMissConst(bool i) {
        isDefaultMissConst = i;
    }
#line 12737 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 296 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setTcMayOverrideHit() {
        isTcMayOverrideHit = true;
    }
#line 12742 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 299 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setTcMayOverrideMiss() {
        isTcMayOverrideMiss = true;
    }
#line 12747 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 302 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::setTableAddOnMiss() {
        isTableAddOnMiss = true;
    }
#line 12752 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 305 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::addAction(const IR::TCAction* action, unsigned flag) {
        actionList.emplace(action, flag);
    }
#line 12757 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 308 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::addConstEntries(const IR::TCEntry* entry) {
        const_entries.push_back(entry);
    }
#line 12762 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 311 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::addTimerProfiles(unsigned tp) {
        timerProfiles = tp;
    }
#line 12767 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 314 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCTable::getTableName() const {
        return tableName;
    }
#line 12772 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 317 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCTable::printMatchType(unsigned matchType) const {
        cstring matchTypeString = cstring::empty;
        switch(matchType) {
                case TC::EXACT_TYPE :
                    matchTypeString = "exact"_cs;
                    break;
                case TC::LPM_TYPE :
                    matchTypeString = "lpm"_cs;
                    break;
                case TC::TERNARY_TYPE :
                    matchTypeString = "ternary"_cs;
                    break;
        }
        return matchTypeString;
    }
#line 12789 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 332 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
IR::TCTable::TCTable(unsigned tId, cstring tN, cstring cN, cstring pN) {
        tableID = tId;
        tableName = tN;
        controlName = cN;
        pipelineName = pN;
        keySize = 0;
        timerProfiles = 0;
        tableEntriesCount = TC::DEFAULT_TABLE_ENTRIES;
        numMask = TC::DEFAULT_KEY_MASK;
        matchType = TC::EXACT_TYPE;
        defaultHitAction = nullptr;
        defaultMissAction = nullptr;
        isDefaultHitConst = false;
        isDefaultMissConst = false;
        isTcMayOverrideHit = false;
        isTcMayOverrideMiss = false;
        isTableAddOnMiss = false;
        isTcMayOverride = false;
        isDirectCounter = false;
        isDirectMeter = false;
    }
#line 12812 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 353 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCTable::toString() const {
        std::string tcTable = absl::StrCat("\n$TC p4template create table/",
                        pipelineName, "/", controlName, "/", tableName, " \\",
                        "\n\ttblid ", tableID, " \\",
                        "\n\ttype ", printMatchType(matchType), " \\",
                        "\n\tkeysz ", keySize,
                        " permissions ", permissions,
                        " tentries ", tableEntriesCount);
        if (matchType == TC::EXACT_TYPE) {
            absl::StrAppend(&tcTable, " nummasks ", TC::DEFAULT_KEY_MASK_EXACT);
        } else {
            absl::StrAppend(&tcTable, " nummasks ", TC::DEFAULT_KEY_MASK);
        }

        if (isTableAddOnMiss && timerProfiles > defaultTimerProfiles) {
            absl::StrAppend(&tcTable, " num_timer_profiles ", timerProfiles);
        }
        if (isDirectCounter) {
            absl::StrAppend(&tcTable, " \\\n\tpna_direct_counter DirectCounter/", directCounterInstance);
        }
        if (isDirectMeter) {
            absl::StrAppend(&tcTable, " \\\n\tpna_direct_meter DirectMeter/", directMeterInstance);
        }

        if (!actionList.empty()) {
            absl::StrAppend(&tcTable, " \\", "\n\ttable_acts ");
            for (auto iter = actionList.begin(); iter != actionList.end(); iter++) {
                absl::StrAppend(&tcTable, "act name ", iter->first->getName());
                if (iter->second == TC::TABLEONLY) {
                    absl::StrAppend(&tcTable, " flags tableonly");
                } else if (iter->second == TC::DEFAULTONLY) {
                    absl::StrAppend(&tcTable, " flags defaultonly");
                }
                if (std::next(iter) != actionList.end()) {
                    absl::StrAppend(&tcTable, " \\", "\n\t");
                }
            }
        }
        if (defaultHitAction != nullptr) {
            absl::StrAppend(&tcTable, "\n$TC p4template update table/", pipelineName,
                      "/", controlName, "/", tableName,
                      " default_hit_action");
            if (isDefaultHitConst) {
                absl::StrAppend(&tcTable, " permissions 0x1024");
            }
            absl::StrAppend(&tcTable, " action ", defaultHitAction->getName());
            if (!defaultHitActionParams.empty())
                absl::StrAppend(&tcTable, " param");
            for (auto param : defaultHitActionParams)
                absl::StrAppend(&tcTable, param);
            if (isTcMayOverrideHit)
                absl::StrAppend(&tcTable, " flags runtime");
        }
        if (defaultMissAction != nullptr) {
            absl::StrAppend(&tcTable, "\n$TC p4template update table/", pipelineName,
                      "/", controlName, "/", tableName,
                      " default_miss_action");
            if (isDefaultMissConst) {
                absl::StrAppend(&tcTable, " permissions 0x1024");
            }
            absl::StrAppend(&tcTable, " action ", defaultMissAction->getName());
            if (!defaultMissActionParams.empty())
                absl::StrAppend(&tcTable, " param");
            for (auto param : defaultMissActionParams)
                absl::StrAppend(&tcTable, param);
            if (isTcMayOverrideMiss)
                absl::StrAppend(&tcTable, " flags runtime");
        }
        if (const_entries.size() != 0) {
            for (auto entry : const_entries) {
                absl::StrAppend(&tcTable, "\n$TC p4template create table/", pipelineName,
                                "/", controlName, "/", tableName,
                                " entry", entry,
                                " permissions 0x1024",
                                " action ", pipelineName,
                                "/", controlName, "/", entry->getActionName());
            }
        }
        return tcTable;
    }
#line 12894 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 433 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCTable::dbprint(std::ostream & out) const { out << toString(); }
#line 12897 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TCTable::operator==(IR::TCTable const & a) const {
        return this->typeId() == a.typeId()
        && tableID == a.tableID
        && tableName == a.tableName
        && controlName == a.controlName
        && pipelineName == a.pipelineName
        && permissions == a.permissions
        && directCounterInstance == a.directCounterInstance
        && directMeterInstance == a.directMeterInstance
        && keySize == a.keySize
        && tableEntriesCount == a.tableEntriesCount
        && numMask == a.numMask
        && matchType == a.matchType
        && timerProfiles == a.timerProfiles
        && defaultTimerProfiles == a.defaultTimerProfiles
        && defaultHitAction == a.defaultHitAction
        && isDefaultHitConst == a.isDefaultHitConst
        && isTcMayOverrideHit == a.isTcMayOverrideHit
        && defaultHitActionParams == a.defaultHitActionParams
        && defaultMissAction == a.defaultMissAction
        && isDefaultMissConst == a.isDefaultMissConst
        && isTcMayOverrideMiss == a.isTcMayOverrideMiss
        && defaultMissActionParams == a.defaultMissActionParams
        && isTableAddOnMiss == a.isTableAddOnMiss
        && isTcMayOverride == a.isTcMayOverride
        && isDirectCounter == a.isDirectCounter
        && isDirectMeter == a.isDirectMeter
        && actionList == a.actionList
        && const_entries == a.const_entries;
    }
bool IR::TCTable::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TCTable &>(a_);
        return tableID == a.tableID
        && tableName == a.tableName
        && controlName == a.controlName
        && pipelineName == a.pipelineName
        && permissions == a.permissions
        && directCounterInstance == a.directCounterInstance
        && directMeterInstance == a.directMeterInstance
        && keySize == a.keySize
        && tableEntriesCount == a.tableEntriesCount
        && numMask == a.numMask
        && matchType == a.matchType
        && timerProfiles == a.timerProfiles
        && defaultTimerProfiles == a.defaultTimerProfiles
        && (defaultHitAction ? a.defaultHitAction ? defaultHitAction->equiv(*a.defaultHitAction) : false : a.defaultHitAction == nullptr)
        && isDefaultHitConst == a.isDefaultHitConst
        && isTcMayOverrideHit == a.isTcMayOverrideHit
        && defaultHitActionParams == a.defaultHitActionParams
        && (defaultMissAction ? a.defaultMissAction ? defaultMissAction->equiv(*a.defaultMissAction) : false : a.defaultMissAction == nullptr)
        && isDefaultMissConst == a.isDefaultMissConst
        && isTcMayOverrideMiss == a.isTcMayOverrideMiss
        && defaultMissActionParams == a.defaultMissActionParams
        && isTableAddOnMiss == a.isTableAddOnMiss
        && isTcMayOverride == a.isTcMayOverride
        && isDirectCounter == a.isDirectCounter
        && isDirectMeter == a.isDirectMeter
        && actionList == a.actionList
        && const_entries == a.const_entries;
    }
void IR::TCTable::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(defaultHitAction, "defaultHitAction");
    v.visit(defaultMissAction, "defaultMissAction");
}
void IR::TCTable::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(defaultHitAction, "defaultHitAction");
    v.visit(defaultMissAction, "defaultMissAction");
}
void IR::TCTable::validate() const {
#line 12973 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(defaultHitAction);
        CHECK_NULL(defaultMissAction); }
void IR::TCTable::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " tableID=" << tableID;
        out << " tableName=" << tableName;
        out << " controlName=" << controlName;
        out << " pipelineName=" << pipelineName;
        out << " permissions=" << permissions;
        out << " directCounterInstance=" << directCounterInstance;
        out << " directMeterInstance=" << directMeterInstance;
        out << " keySize=" << keySize;
        out << " tableEntriesCount=" << tableEntriesCount;
        out << " numMask=" << numMask;
        out << " matchType=" << matchType;
        out << " timerProfiles=" << timerProfiles;
        out << " defaultTimerProfiles=" << defaultTimerProfiles;
        out << " isDefaultHitConst=" << isDefaultHitConst;
        out << " isTcMayOverrideHit=" << isTcMayOverrideHit;
        out << " isDefaultMissConst=" << isDefaultMissConst;
        out << " isTcMayOverrideMiss=" << isTcMayOverrideMiss;
        out << " isTableAddOnMiss=" << isTableAddOnMiss;
        out << " isTcMayOverride=" << isTcMayOverride;
        out << " isDirectCounter=" << isDirectCounter;
        out << " isDirectMeter=" << isDirectMeter;
}
void IR::TCTable::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("tableID", tableID);
    json.emit("tableName", tableName);
    json.emit("controlName", controlName);
    json.emit("pipelineName", pipelineName);
    json.emit("permissions", permissions);
    json.emit("directCounterInstance", directCounterInstance);
    json.emit("directMeterInstance", directMeterInstance);
    json.emit("keySize", keySize);
    json.emit("tableEntriesCount", tableEntriesCount);
    json.emit("numMask", numMask);
    json.emit("matchType", matchType);
    json.emit("timerProfiles", timerProfiles);
    json.emit("defaultTimerProfiles", defaultTimerProfiles);
    json.emit("defaultHitAction", defaultHitAction);
    json.emit("isDefaultHitConst", isDefaultHitConst);
    json.emit("isTcMayOverrideHit", isTcMayOverrideHit);
    json.emit("defaultHitActionParams", defaultHitActionParams);
    json.emit("defaultMissAction", defaultMissAction);
    json.emit("isDefaultMissConst", isDefaultMissConst);
    json.emit("isTcMayOverrideMiss", isTcMayOverrideMiss);
    json.emit("defaultMissActionParams", defaultMissActionParams);
    json.emit("isTableAddOnMiss", isTableAddOnMiss);
    json.emit("isTcMayOverride", isTcMayOverride);
    json.emit("isDirectCounter", isDirectCounter);
    json.emit("isDirectMeter", isDirectMeter);
    json.emit("actionList", actionList);
    json.emit("const_entries", const_entries);
}
IR::TCTable::TCTable(JSONLoader & json) : Node(json) {
    json.load("tableID", tableID);
    json.load("tableName", tableName);
    json.load("controlName", controlName);
    json.load("pipelineName", pipelineName);
    json.load("permissions", permissions);
    json.load("directCounterInstance", directCounterInstance);
    json.load("directMeterInstance", directMeterInstance);
    json.load("keySize", keySize);
    json.load("tableEntriesCount", tableEntriesCount);
    json.load("numMask", numMask);
    json.load("matchType", matchType);
    json.load("timerProfiles", timerProfiles);
    json.load("defaultTimerProfiles", defaultTimerProfiles);
    json.load("defaultHitAction", defaultHitAction);
    json.load("isDefaultHitConst", isDefaultHitConst);
    json.load("isTcMayOverrideHit", isTcMayOverrideHit);
    json.load("defaultHitActionParams", defaultHitActionParams);
    json.load("defaultMissAction", defaultMissAction);
    json.load("isDefaultMissConst", isDefaultMissConst);
    json.load("isTcMayOverrideMiss", isTcMayOverrideMiss);
    json.load("defaultMissActionParams", defaultMissActionParams);
    json.load("isTableAddOnMiss", isTableAddOnMiss);
    json.load("isTcMayOverride", isTcMayOverride);
    json.load("isDirectCounter", isDirectCounter);
    json.load("isDirectMeter", isDirectMeter);
    json.load("actionList", actionList);
    json.load("const_entries", const_entries);
}
IR::Node * IR::TCTable::fromJSON(JSONLoader & json) { return new TCTable(json); }
bool IR::TCTable::operator==(IR::Node const & a) const { return a == *this; }
#line 445 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
IR::TCKey::TCKey(unsigned id, unsigned width, cstring ptype, cstring name, cstring attr, bool isID) {
        keyID = id;
        bitwidth = width;
        type = ptype;
        keyName = name;
        keyAttribute = attr;
        emitID = isID;
        emitValue = false;
    }
#line 13072 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 454 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCKey::setValue(unsigned v) {
        value = v;
        emitValue = true;
    }
#line 13078 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 458 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCKey::toString() const {
        std::string tckeyInstance = absl::StrCat(" ", keyAttribute, " ", keyName, " ptype ", type);
        if (emitID) {
            absl::StrAppend(&tckeyInstance, " id ", keyID);
        }
        if (emitValue) {
            absl::StrAppend(&tckeyInstance, " ", value);
        }
        return tckeyInstance;
    }
#line 13090 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 468 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCKey::dbprint(std::ostream & out) const { out << toString(); }
#line 13093 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TCKey::operator==(IR::TCKey const & a) const {
        return this->typeId() == a.typeId()
        && keyID == a.keyID
        && bitwidth == a.bitwidth
        && type == a.type
        && keyName == a.keyName
        && keyAttribute == a.keyAttribute
        && emitID == a.emitID
        && emitValue == a.emitValue
        && value == a.value;
    }
bool IR::TCKey::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TCKey &>(a_);
        return keyID == a.keyID
        && bitwidth == a.bitwidth
        && type == a.type
        && keyName == a.keyName
        && keyAttribute == a.keyAttribute
        && emitID == a.emitID
        && emitValue == a.emitValue
        && value == a.value;
    }
void IR::TCKey::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " keyID=" << keyID;
        out << " bitwidth=" << bitwidth;
        out << " type=" << type;
        out << " keyName=" << keyName;
        out << " keyAttribute=" << keyAttribute;
        out << " emitID=" << emitID;
        out << " emitValue=" << emitValue;
        out << " value=" << value;
}
void IR::TCKey::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("keyID", keyID);
    json.emit("bitwidth", bitwidth);
    json.emit("type", type);
    json.emit("keyName", keyName);
    json.emit("keyAttribute", keyAttribute);
    json.emit("emitID", emitID);
    json.emit("emitValue", emitValue);
    json.emit("value", value);
}
IR::TCKey::TCKey(JSONLoader & json) : Node(json) {
    json.load("keyID", keyID);
    json.load("bitwidth", bitwidth);
    json.load("type", type);
    json.load("keyName", keyName);
    json.load("keyAttribute", keyAttribute);
    json.load("emitID", emitID);
    json.load("emitValue", emitValue);
    json.load("value", value);
}
IR::Node * IR::TCKey::fromJSON(JSONLoader & json) { return new TCKey(json); }
bool IR::TCKey::operator==(IR::Node const & a) const { return a == *this; }
#line 484 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
IR::TCExternInstance::TCExternInstance(unsigned id, cstring name, bool isNe, unsigned ne) {
        instanceID = id;
        instanceName = name;
        isNumelemns = isNe;
        numelemns = ne;
        isControlPath = false;
        isTablBindable = false;
        isInstanceType = false;
        isConstructorKeys = false;
    }
#line 13163 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 494 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCExternInstance::addControlPathKeys(safe_vector<const IR::TCKey *> k) {
        if (k.size() > 0) {
            isControlPath = true;
            controlKeys = k;
        }
    }
#line 13171 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 500 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCExternInstance::addConstructorKeys(safe_vector<const IR::TCKey *> k) {
        if (k.size() > 0) {
            isConstructorKeys = true;
            constructorKeys = k;
        }
    }
#line 13179 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 506 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCExternInstance::setExternTypeInstance(cstring type) {
        isInstanceType = true;
        instanceType = type;
    }
#line 13185 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 510 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCExternInstance::setExternTableBindable(bool flag) {
        isTablBindable = flag;
    }
#line 13190 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 513 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCExternInstance::setNumElements(unsigned ne) {
        isNumelemns = true;
        numelemns = ne;
    }
#line 13196 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 517 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCExternInstance::toString() const {
        std::string tcExternInstance = absl::StrCat(instanceName, " instid ", instanceID, " \\");
        if (isNumelemns) {
            absl::StrAppend(&tcExternInstance, "\ntc_numel ", numelemns, " \\");
        }
        if (isTablBindable) {
            absl::StrAppend(&tcExternInstance, "\ntbl_bindable", " \\");
        }
        if (isConstructorKeys) {
            absl::StrAppend(&tcExternInstance, "\nconstructor");
            for (auto field : constructorKeys) {
                absl::StrAppend(&tcExternInstance, field);
            }
            absl::StrAppend(&tcExternInstance, " \\");
        }
        if (isControlPath) {
            absl::StrAppend(&tcExternInstance, "\ncontrol_path");
            for (auto field : controlKeys) {
                absl::StrAppend(&tcExternInstance, field);
            }
        }
        return tcExternInstance;
    }
#line 13221 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 540 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCExternInstance::dbprint(std::ostream & out) const { out << toString(); }
#line 13224 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TCExternInstance::operator==(IR::TCExternInstance const & a) const {
        return this->typeId() == a.typeId()
        && instanceID == a.instanceID
        && instanceName == a.instanceName
        && isNumelemns == a.isNumelemns
        && numelemns == a.numelemns
        && isControlPath == a.isControlPath
        && isTablBindable == a.isTablBindable
        && isInstanceType == a.isInstanceType
        && isConstructorKeys == a.isConstructorKeys
        && instanceType == a.instanceType
        && controlKeys == a.controlKeys
        && constructorKeys == a.constructorKeys;
    }
bool IR::TCExternInstance::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TCExternInstance &>(a_);
        return instanceID == a.instanceID
        && instanceName == a.instanceName
        && isNumelemns == a.isNumelemns
        && numelemns == a.numelemns
        && isControlPath == a.isControlPath
        && isTablBindable == a.isTablBindable
        && isInstanceType == a.isInstanceType
        && isConstructorKeys == a.isConstructorKeys
        && instanceType == a.instanceType
        && controlKeys == a.controlKeys
        && constructorKeys == a.constructorKeys;
    }
void IR::TCExternInstance::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " instanceID=" << instanceID;
        out << " instanceName=" << instanceName;
        out << " isNumelemns=" << isNumelemns;
        out << " numelemns=" << numelemns;
        out << " isControlPath=" << isControlPath;
        out << " isTablBindable=" << isTablBindable;
        out << " isInstanceType=" << isInstanceType;
        out << " isConstructorKeys=" << isConstructorKeys;
        out << " instanceType=" << instanceType;
}
void IR::TCExternInstance::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("instanceID", instanceID);
    json.emit("instanceName", instanceName);
    json.emit("isNumelemns", isNumelemns);
    json.emit("numelemns", numelemns);
    json.emit("isControlPath", isControlPath);
    json.emit("isTablBindable", isTablBindable);
    json.emit("isInstanceType", isInstanceType);
    json.emit("isConstructorKeys", isConstructorKeys);
    json.emit("instanceType", instanceType);
    json.emit("controlKeys", controlKeys);
    json.emit("constructorKeys", constructorKeys);
}
IR::TCExternInstance::TCExternInstance(JSONLoader & json) : Node(json) {
    json.load("instanceID", instanceID);
    json.load("instanceName", instanceName);
    json.load("isNumelemns", isNumelemns);
    json.load("numelemns", numelemns);
    json.load("isControlPath", isControlPath);
    json.load("isTablBindable", isTablBindable);
    json.load("isInstanceType", isInstanceType);
    json.load("isConstructorKeys", isConstructorKeys);
    json.load("instanceType", instanceType);
    json.load("controlKeys", controlKeys);
    json.load("constructorKeys", constructorKeys);
}
IR::Node * IR::TCExternInstance::fromJSON(JSONLoader & json) { return new TCExternInstance(json); }
bool IR::TCExternInstance::operator==(IR::Node const & a) const { return a == *this; }
#line 551 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
IR::TCExtern::TCExtern(cstring eId, cstring eN, cstring pN, unsigned inst, cstring p, bool exec_method) {
        externID = eId;
        externName = eN;
        pipelineName = pN;
        numinstances = inst;
        acl_permisson = p;
        has_exec_method = exec_method;
    }
#line 13305 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 559 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCExtern::addExternInstance(const IR::TCExternInstance* tei) {
        externInstances.push_back(tei);
    }
#line 13310 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 562 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
const IR::TCExternInstance *IR::TCExtern::getExternInstance(cstring inst_name) const {
        for (auto extnInstDef : externInstances) {
            if(extnInstDef->instanceName == inst_name) {
                return extnInstDef;
            }
        }
        return nullptr;
    }
#line 13320 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 570 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCExtern::toString() const {
        std::string tcExtern = absl::StrCat("\n$TC p4template create extern/", "root/", externName,
                                            " extid ", externID, " numinstances ", numinstances, " tc_acl " + acl_permisson);
        if (has_exec_method) {
             absl::StrAppend(&tcExtern, " has_exec_method");
        }
        for (unsigned iter = 0; iter < numinstances; iter++) {
            absl::StrAppend(&tcExtern, "\n\n$TC p4template create extern_inst/", pipelineName,
                      "/", externName, "/");
            if (externInstances.size() > iter)
                absl::StrAppend(&tcExtern, externInstances[iter]->toString());
        }
        return tcExtern;
    }
#line 13336 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 584 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCExtern::dbprint(std::ostream & out) const { out << toString(); }
#line 13339 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TCExtern::operator==(IR::TCExtern const & a) const {
        return this->typeId() == a.typeId()
        && externName == a.externName
        && pipelineName == a.pipelineName
        && acl_permisson == a.acl_permisson
        && externID == a.externID
        && numinstances == a.numinstances
        && externInstances == a.externInstances
        && has_exec_method == a.has_exec_method;
    }
bool IR::TCExtern::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TCExtern &>(a_);
        return externName == a.externName
        && pipelineName == a.pipelineName
        && acl_permisson == a.acl_permisson
        && externID == a.externID
        && numinstances == a.numinstances
        && externInstances == a.externInstances
        && has_exec_method == a.has_exec_method;
    }
void IR::TCExtern::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " externName=" << externName;
        out << " pipelineName=" << pipelineName;
        out << " acl_permisson=" << acl_permisson;
        out << " externID=" << externID;
        out << " numinstances=" << numinstances;
        out << " has_exec_method=" << has_exec_method;
}
void IR::TCExtern::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("externName", externName);
    json.emit("pipelineName", pipelineName);
    json.emit("acl_permisson", acl_permisson);
    json.emit("externID", externID);
    json.emit("numinstances", numinstances);
    json.emit("externInstances", externInstances);
    json.emit("has_exec_method", has_exec_method);
}
IR::TCExtern::TCExtern(JSONLoader & json) : Node(json) {
    json.load("externName", externName);
    json.load("pipelineName", pipelineName);
    json.load("acl_permisson", acl_permisson);
    json.load("externID", externID);
    json.load("numinstances", numinstances);
    json.load("externInstances", externInstances);
    json.load("has_exec_method", has_exec_method);
}
IR::Node * IR::TCExtern::fromJSON(JSONLoader & json) { return new TCExtern(json); }
bool IR::TCExtern::operator==(IR::Node const & a) const { return a == *this; }
#line 596 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCPipeline::setPipelineName(cstring pName) {
        pipelineName = pName;
    }
#line 13396 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 599 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCPipeline::setNumTables(unsigned n) {
        numTables = n;
    }
#line 13401 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 602 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCPipeline::addNoActionDefinition(const IR::TCAction* actionDef) {
        NoAction = actionDef;
    }
#line 13406 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 605 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCPipeline::addActionDefinition(const IR::TCAction* actionDef) {
        actionDefs.push_back(actionDef);
    }
#line 13411 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 608 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCPipeline::addTableDefinition(const IR::TCTable* tableDef) {
        tableDefs.push_back(tableDef);
    }
#line 13416 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 611 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCPipeline::setPipelinePreAction(const IR::TCAction* action) {
        preaction = action;
    }
#line 13421 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 614 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCPipeline::setPipelinePostAction(const IR::TCAction* action) {
        postaction = action;
    }
#line 13426 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 617 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCPipeline::addExternDefinition(const IR::TCExtern* externDef) {
        externDefs.push_back(externDef);
    }
#line 13431 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 620 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
const IR::TCExtern *IR::TCPipeline::getExternDefinition(cstring eName) const {
        for (auto externDef : externDefs) {
            if(externDef->externName == eName) {
                return externDef;
            }
        }
        return nullptr;
    }
#line 13441 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 628 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
IR::TCPipeline::TCPipeline() {
        Util::SourceInfo* srcinfo = new Util::SourceInfo();
        Node::srcInfo = *srcinfo;
        pipelineName = nullptr;
    }
#line 13448 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 633 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
cstring IR::TCPipeline::toString() const {
        std::string tcCode = absl::StrCat("#!/bin/bash -x\n",
                                        "\nset -e\n", "\n: \"${TC:=\"tc\"}\"",
                                        "\n$TC p4template create pipeline/", pipelineName,
                                        " numtables ", numTables);
        if (!actionDefs.empty()) {
            for (auto a : actionDefs) {
                absl::StrAppend(&tcCode, "\n", a);
            }
        }
        if (!externDefs.empty()) {
            for (auto e : externDefs) {
                absl::StrAppend(&tcCode, "\n", e);
            }
        }
        if (!tableDefs.empty()) {
            for (auto t : tableDefs) {
                absl::StrAppend(&tcCode, "\n", t);
            }
        }
        if (preaction != nullptr) {
            absl::StrAppend(&tcCode, "\n", preaction,
                            "\n$TC p4template update pipeline/", pipelineName,
                            " preactions action ", pipelineName, "/preaction");
        }
        if (postaction != nullptr) {
            absl::StrAppend(&tcCode, "\n", postaction,
                            "\n$TC p4template update pipeline/", pipelineName,
                            " postactions action ", pipelineName, "/postaction");
        }
        absl::StrAppend(&tcCode, "\n$TC p4template update pipeline/", pipelineName, " state ready");
        return tcCode;
    }
#line 13483 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 666 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tc/tc.def"
void IR::TCPipeline::dbprint(std::ostream & out) const { out << toString(); }
#line 13486 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TCPipeline::operator==(IR::TCPipeline const & a) const {
        return this->typeId() == a.typeId()
        && pipelineName == a.pipelineName
        && numTables == a.numTables
        && NoAction == a.NoAction
        && actionDefs == a.actionDefs
        && tableDefs == a.tableDefs
        && externDefs == a.externDefs
        && preaction == a.preaction
        && postaction == a.postaction;
    }
bool IR::TCPipeline::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TCPipeline &>(a_);
        return pipelineName == a.pipelineName
        && numTables == a.numTables
        && (NoAction ? a.NoAction ? NoAction->equiv(*a.NoAction) : false : a.NoAction == nullptr)
        && actionDefs == a.actionDefs
        && tableDefs == a.tableDefs
        && externDefs == a.externDefs
        && (preaction ? a.preaction ? preaction->equiv(*a.preaction) : false : a.preaction == nullptr)
        && (postaction ? a.postaction ? postaction->equiv(*a.postaction) : false : a.postaction == nullptr);
    }
void IR::TCPipeline::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(NoAction, "NoAction");
    v.visit(preaction, "preaction");
    v.visit(postaction, "postaction");
}
void IR::TCPipeline::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(NoAction, "NoAction");
    v.visit(preaction, "preaction");
    v.visit(postaction, "postaction");
}
void IR::TCPipeline::validate() const {
#line 13526 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(NoAction);
        CHECK_NULL(preaction);
        CHECK_NULL(postaction); }
void IR::TCPipeline::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " pipelineName=" << pipelineName;
        out << " numTables=" << numTables;
}
void IR::TCPipeline::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("pipelineName", pipelineName);
    json.emit("numTables", numTables);
    json.emit("NoAction", NoAction);
    json.emit("actionDefs", actionDefs);
    json.emit("tableDefs", tableDefs);
    json.emit("externDefs", externDefs);
    json.emit("preaction", preaction);
    json.emit("postaction", postaction);
}
IR::TCPipeline::TCPipeline(JSONLoader & json) : Node(json) {
    json.load("pipelineName", pipelineName);
    json.load("numTables", numTables);
    json.load("NoAction", NoAction);
    json.load("actionDefs", actionDefs);
    json.load("tableDefs", tableDefs);
    json.load("externDefs", externDefs);
    json.load("preaction", preaction);
    json.load("postaction", postaction);
}
IR::Node * IR::TCPipeline::fromJSON(JSONLoader & json) { return new TCPipeline(json); }
bool IR::TCPipeline::operator==(IR::Node const & a) const { return a == *this; }
/// The wrapped reference.
/// The expression type is derived from the member.
/// Implicit conversions to allow implementations to be treated like a Member*.
#line 16 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
IR::Expression const & IR::StateVariable::operator*() const { return *ref; }
#line 13564 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 17 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
const IR::Expression *IR::StateVariable::operator->() const { return ref; }
#line 13567 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// StateVariables can be Members, PathExpressions, or ArrayIndices.
#line 20 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
IR::StateVariable::StateVariable(const IR::Member* member) : Expression(member->getSourceInfo(), member->type), ref(member) {}
#line 13571 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The expression type is derived from the path.
#line 22 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
IR::StateVariable::StateVariable(const IR::PathExpression* path) : Expression(path->getSourceInfo(), path->type), ref(path) {}
#line 13575 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 23 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
IR::StateVariable::StateVariable(const IR::ArrayIndex* arr) : Expression(arr->getSourceInfo(), arr->type), ref(arr) {}
#line 13578 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// Implements comparisons so that StateVariables can be used as map keys.

#line 27 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
bool IR::StateVariable::operator==(IR::StateVariable const & other) const {
        return *ref == *other.ref;
    }
#line 13585 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// Implements comparisons so that StateVariables can be used as map keys.
/// Note that we ignore the type of the variable in the comparison.
#line 33 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
bool IR::StateVariable::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const StateVariable &>(a_);

#line 33 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
{
        // We use a custom compare function.
        // TODO: Is there a faster way to implement this comparison?
        return compare(ref, a.ref) == 0;
    }    }
#line 13600 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// Implements comparisons so that StateVariables can be used as map keys.
/// Note that we ignore the type of the variable in the comparison.
#line 41 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
bool IR::StateVariable::operator<(IR::StateVariable const & other) const {
        // We use a custom compare function.
        // TODO: Is there a faster way to implement this comparison?
        return compare(ref, other.ref) < 0;
    }
#line 13609 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 47 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
int IR::StateVariable::compare(IR::Expression const * e1, IR::Expression const * e2) const {
        // e1 is a Member.
        if (const auto *m1 = e1->to<Member>()) {
            if (const auto *m2 = e2->to<Member>()) {
                return compare(m1, m2);
            }
            if (e2->is<PathExpression>()) {
                return 1;
            }
            if (e2->is<ArrayIndex>()) {
                return 1;
            }
        }
        // e1 is a PathExpression.
        if (const auto *p1 = e1->to<PathExpression>()) {
            if (const auto *p2 = e2->to<PathExpression>()) {
                return compare(p1, p2);
            }
            if (e2->is<Member>()) {
                return -1;
            }
            if (e2->is<ArrayIndex>()) {
                return 1;
            }
        }
        // e1 is a ArrayIndex.
        if (const auto *a1 = e1->to<ArrayIndex>()) {
            if (const auto *a2 = e2->to<ArrayIndex>()) {
                return compare(a1, a2);
            }
            if (e2->is<Member>()) {
                return -1;
            }
            if (e2->is<PathExpression>()) {
                return -1;
            }
        }
        BUG("Either %1% of type %2% or %3% of type %4% is not a valid StateVariable", e1, e1->node_type_name(), e2, e2->node_type_name());
    }
#line 13650 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 87 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
int IR::StateVariable::compare(IR::Member const * m1, IR::Member const * m2) const {
        auto result = compare(m1->expr, m2->expr);
        if (result != 0) {
            return result;
        }
        if (m1->member.name < m2->member.name) {
            return -1;
        }
        if (m1->member.name > m2->member.name) {
            return 1;
        }
        return 0;
    }
#line 13665 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 101 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
int IR::StateVariable::compare(IR::PathExpression const * p1, IR::PathExpression const * p2) const {
        if (p1->path->name.name < p2->path->name.name) {
            return -1;
        }
        if (p1->path->name.name > p2->path->name.name) {
            return 1;
        }
        return 0;
    }
#line 13676 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 111 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
int IR::StateVariable::compare(IR::ArrayIndex const * a1, IR::ArrayIndex const * a2) const {
        auto result = compare(a1->left, a2->left);
        if (result != 0) {
            return result;
        }
        auto a1Val = a1->right->to<Constant>();
        BUG_CHECK(a1Val != nullptr, "Value %1% is not a constant. Only constants are supported as part of a state variable.", a1->right);
        auto a2Val = a2->right->to<Constant>();
        BUG_CHECK(a2Val != nullptr, "Value %1% is not a constant. Only constants are supported as part of a state variable.", a2->right);
        if (a1Val->value < a2Val->value) {
            return -1;
        }
        if (a1Val->value > a2Val->value) {
            return 1;
        }
        return 0;
    }
#line 13695 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 129 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
cstring IR::StateVariable::toString() const { return ref->toString(); }
#line 13698 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 131 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
void IR::StateVariable::dbprint(std::ostream & out) const { ref->dbprint(out); }
#line 13701 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
void IR::StateVariable::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " ref=" << ref;
}
void IR::StateVariable::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("ref", ref);
}
IR::StateVariable::StateVariable(JSONLoader & json) : Expression(json) {
    json.load("ref", ref);
}
IR::Node * IR::StateVariable::fromJSON(JSONLoader & json) { return new StateVariable(json); }
bool IR::StateVariable::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::StateVariable::operator==(IR::Node const & a) const { return a == *this; }
#line 137 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
cstring IR::TaintExpression::toString() const { return "TaintedExpression(" + type->toString() + ")"; }
#line 13718 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 139 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
void IR::TaintExpression::dbprint(std::ostream & out) const { out << "TaintedExpression(" << type << ")"; }
#line 13721 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TaintExpression::operator==(IR::TaintExpression const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a));
    }
bool IR::TaintExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        return true;
    }
void IR::TaintExpression::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
}
IR::TaintExpression::TaintExpression(JSONLoader & json) : Expression(json) {
}
IR::Node * IR::TaintExpression::fromJSON(JSONLoader & json) { return new TaintExpression(json); }
bool IR::TaintExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::TaintExpression::operator==(IR::Node const & a) const { return a == *this; }
/// The assigned size of this varbit (assigned by extract calls).
/// Returns the current width this variable bit type has.
#line 158 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
int IR::Extracted_Varbits::width_bits() const { return assignedSize; }
#line 13742 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 160 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
bool IR::Extracted_Varbits::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Bits::equiv(a_)) return false;
        auto &a = static_cast<const Extracted_Varbits &>(a_);

#line 160 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
{
        // Ignore the current size and only check for equivalence of the assigned size.
        return size == a.size;
    }    }
#line 13754 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 165 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
bool IR::Extracted_Varbits::operator==(IR::Extracted_Varbits const & a) const {
        // Ignore the current size and only check for equivalence of the assigned size.
        return size == a.size;
    }
#line 13760 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 170 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
cstring IR::Extracted_Varbits::baseName() const { return "SizedVarbit"_cs; }
#line 13763 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 172 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
cstring IR::Extracted_Varbits::toString() const {
        return baseName() + "<"_cs + Util::toString(size) + ">"_cs + "<"_cs + Util::toString(assignedSize) +
               ">"_cs;
    }
#line 13769 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 177 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
void IR::Extracted_Varbits::dbprint(std::ostream & out) const { out << toString(); }
#line 13772 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
void IR::Extracted_Varbits::dump_fields(std::ostream & out) const {
    Type_Bits::dump_fields(out);
        out << " assignedSize=" << assignedSize;
}
void IR::Extracted_Varbits::toJSON(JSONGenerator & json) const {
    Type_Bits::toJSON(json);
    json.emit("assignedSize", assignedSize);
}
IR::Extracted_Varbits::Extracted_Varbits(JSONLoader & json) : Type_Bits(json) {
    json.load("assignedSize", assignedSize);
}
IR::Node * IR::Extracted_Varbits::fromJSON(JSONLoader & json) { return new Extracted_Varbits(json); }
bool IR::Extracted_Varbits::operator==(IR::Type_Bits const & a) const { return a == *this; }
bool IR::Extracted_Varbits::operator==(IR::Type_Base const & a) const { return a == *this; }
bool IR::Extracted_Varbits::operator==(IR::Type const & a) const { return a == *this; }
bool IR::Extracted_Varbits::operator==(IR::Node const & a) const { return a == *this; }
#line 186 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
cstring IR::ConcolicVariable::produceLabel(cstring methodName, int srcIdentifier, int concolicId) {
        std::stringstream sstr;
        sstr << methodName << "_" << srcIdentifier << "_" << concolicId;
        return sstr.str();
    }
#line 13795 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// Arguments to the concolic method.
/// The unique identifier of the source the produces this concolic variable.
/// Some identifiers produce multiple concolic variables. This ID keeps track of the offsets.
/// Nodes that are associated with this concolic variable. This may be declarations.
/// The name of the concolic method that this variable targets.
#line 208 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
cstring IR::ConcolicVariable::toString() const {
        std::string argumentStr = "";
        std::string sep = "";
        for (const auto *arg : *arguments) {
            argumentStr += sep + arg->toString();
            sep = ", ";
        }
        return "Concolic_" + label + "(" + argumentStr + ")";
    }
#line 13811 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 218 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
void IR::ConcolicVariable::dbprint(std::ostream & out) const {
        out << "Concolic_" << label << "(" << arguments << ")";
    }
#line 13816 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 222 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
void IR::ConcolicVariable::visit_children(Visitor & v, char const * n) { (void)n; v.visit(type, "type"); }
#line 222 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
void IR::ConcolicVariable::visit_children(Visitor & v, char const * n) const { (void)n; v.visit(type, "type"); }
#line 13821 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 224 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
IR::ConcolicVariable::ConcolicVariable(IR::Type const * type, cstring methodName, IR::Vector<IR::Argument> const * arguments, int srcIdentifier, int concolicId, IR::IndexedVector<IR::Node> const & associatedNodes) : SymbolicVariable(type, produceLabel(methodName, srcIdentifier, concolicId)),
          arguments(arguments),
          srcIdentifier(srcIdentifier),
          concolicId(concolicId),
          associatedNodes(associatedNodes),
          concolicMethodName(methodName) {}
#line 13829 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 234 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
IR::ConcolicVariable::ConcolicVariable(IR::Type const * type, cstring methodName, IR::Vector<IR::Argument> const * arguments, int srcIdentifier, int concolicId) : SymbolicVariable(type, produceLabel(methodName, srcIdentifier, concolicId)),
          arguments(arguments),
          srcIdentifier(srcIdentifier),
          concolicId(concolicId),
          concolicMethodName(methodName) {}
#line 13836 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::ConcolicVariable::operator==(IR::ConcolicVariable const & a) const {
        return SymbolicVariable::operator==(static_cast<const SymbolicVariable &>(a))
        && arguments == a.arguments
        && srcIdentifier == a.srcIdentifier
        && concolicId == a.concolicId
        && associatedNodes == a.associatedNodes
        && concolicMethodName == a.concolicMethodName;
    }
bool IR::ConcolicVariable::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!SymbolicVariable::equiv(a_)) return false;
        auto &a = static_cast<const ConcolicVariable &>(a_);
        return (arguments ? a.arguments ? arguments->equiv(*a.arguments) : false : a.arguments == nullptr)
        && srcIdentifier == a.srcIdentifier
        && concolicId == a.concolicId
        && associatedNodes.equiv(a.associatedNodes)
        && concolicMethodName == a.concolicMethodName;
    }
void IR::ConcolicVariable::validate() const {
#line 13856 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(arguments);
        associatedNodes.validate(); }
void IR::ConcolicVariable::dump_fields(std::ostream & out) const {
    SymbolicVariable::dump_fields(out);
        out << " srcIdentifier=" << srcIdentifier;
        out << " concolicId=" << concolicId;
        out << " concolicMethodName=" << concolicMethodName;
}
void IR::ConcolicVariable::toJSON(JSONGenerator & json) const {
    SymbolicVariable::toJSON(json);
    json.emit("arguments", arguments);
    json.emit("srcIdentifier", srcIdentifier);
    json.emit("concolicId", concolicId);
    json.emit("associatedNodes", associatedNodes);
    json.emit("concolicMethodName", concolicMethodName);
}
IR::ConcolicVariable::ConcolicVariable(JSONLoader & json) : SymbolicVariable(json) {
    json.load("arguments", arguments);
    json.load("srcIdentifier", srcIdentifier);
    json.load("concolicId", concolicId);
    json.load("associatedNodes", associatedNodes);
    json.load("concolicMethodName", concolicMethodName);
}
IR::Node * IR::ConcolicVariable::fromJSON(JSONLoader & json) { return new ConcolicVariable(json); }
bool IR::ConcolicVariable::operator==(IR::SymbolicVariable const & a) const { return a == *this; }
bool IR::ConcolicVariable::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::ConcolicVariable::operator==(IR::Node const & a) const { return a == *this; }
/// Whether this struct expression is valid. Mostly relevant to header expressions.
#line 248 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
void IR::HeaderExpression::dbprint(std::ostream & out) const {
        out <<"{" << Log::endl << IndentCtl::indent;
        out << "$headerValid:" << validity <<";" << Log::endl;
        for (auto &field : components) {
            out << Log::endl << field << ';';
        }
        out << Log::endl << IndentCtl::unindent << "}";
    }
#line 13895 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 257 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
void IR::HeaderExpression::validate() const {
#line 13898 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(validity);
#line 257 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
{
        components.check_null(); components.validate();
        BUG_CHECK(structType == nullptr || structType->is<IR::Type_Header>(), "%1%: unexpected header type", structType->node_type_name());
    } }
#line 13906 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::HeaderExpression::operator==(IR::HeaderExpression const & a) const {
        return StructExpression::operator==(static_cast<const StructExpression &>(a))
        && validity == a.validity;
    }
bool IR::HeaderExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!StructExpression::equiv(a_)) return false;
        auto &a = static_cast<const HeaderExpression &>(a_);
        return (validity ? a.validity ? validity->equiv(*a.validity) : false : a.validity == nullptr);
    }
void IR::HeaderExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    StructExpression::visit_children(v, n);
    v.visit(validity, "validity");
}
void IR::HeaderExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    StructExpression::visit_children(v, n);
    v.visit(validity, "validity");
}
void IR::HeaderExpression::toJSON(JSONGenerator & json) const {
    StructExpression::toJSON(json);
    json.emit("validity", validity);
}
IR::HeaderExpression::HeaderExpression(JSONLoader & json) : StructExpression(json) {
    json.load("validity", validity);
}
IR::Node * IR::HeaderExpression::fromJSON(JSONLoader & json) { return new HeaderExpression(json); }
bool IR::HeaderExpression::operator==(IR::StructExpression const & a) const { return a == *this; }
bool IR::HeaderExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::HeaderExpression::operator==(IR::Node const & a) const { return a == *this; }
/// Because we do not use a reference for "ref" and also inline the member the irgenerator produces
/// a constructor that does not compile. We need to write the constructor manually instead.
/// The original reference.
/// The value of the reference after it was resolved.
#line 275 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
IR::InOutReference::InOutReference(JSONLoader & json) : Expression(json), ref(json) {
        json.load("resolvedRef", resolvedRef);
    }
#line 13946 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 279 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
IR::InOutReference::InOutReference(Util::SourceInfo srcInfo, IR::StateVariable & ref, IR::Expression const * resolvedRef) :
        Expression(srcInfo, ref.type), ref(ref), resolvedRef(resolvedRef)
        { validate(); }
#line 13951 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 283 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
IR::InOutReference::InOutReference(IR::StateVariable & ref, IR::Expression const * resolvedRef) :
        Expression(ref.type), ref(ref), resolvedRef(resolvedRef)
        { validate(); }
#line 13956 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 287 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
cstring IR::InOutReference::toString() const { return ref->toString() + "("_cs + resolvedRef->toString() + ")"_cs; }
#line 13959 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 289 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/p4tools.def"
void IR::InOutReference::dbprint(std::ostream & out) const { out << ref << "(" << resolvedRef << ")"; }
#line 13962 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::InOutReference::operator==(IR::InOutReference const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && ref == a.ref
        && resolvedRef == a.resolvedRef;
    }
bool IR::InOutReference::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const InOutReference &>(a_);
        return ref.equiv(a.ref)
        && (resolvedRef ? a.resolvedRef ? resolvedRef->equiv(*a.resolvedRef) : false : a.resolvedRef == nullptr);
    }
void IR::InOutReference::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    ref.visit_children(v, "ref");
    v.visit(resolvedRef, "resolvedRef");
}
void IR::InOutReference::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    ref.visit_children(v, "ref");
    v.visit(resolvedRef, "resolvedRef");
}
void IR::InOutReference::validate() const {
#line 13988 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        ref.validate();
        CHECK_NULL(resolvedRef); }
void IR::InOutReference::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("ref", ref);
    json.emit("resolvedRef", resolvedRef);
}
IR::Node * IR::InOutReference::fromJSON(JSONLoader & json) { return new InOutReference(json); }
bool IR::InOutReference::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::InOutReference::operator==(IR::Node const & a) const { return a == *this; }
#line 7 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/modules/testgen/targets/tofino/testgen_tofino.def"
cstring IR::UninitializedTaintExpression::toString() const { return "UninitializedTaintExpression(" + type->toString() + ")"; }
#line 14002 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 9 "/builddir/build/BUILD/p4c-1.2.5.8/backends/p4tools/modules/testgen/targets/tofino/testgen_tofino.def"
void IR::UninitializedTaintExpression::dbprint(std::ostream & out) const { out << "UninitializedTaintExpression(" << type << ")"; }
#line 14005 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::UninitializedTaintExpression::operator==(IR::UninitializedTaintExpression const & a) const {
        return TaintExpression::operator==(static_cast<const TaintExpression &>(a));
    }
bool IR::UninitializedTaintExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!TaintExpression::equiv(a_)) return false;
        return true;
    }
void IR::UninitializedTaintExpression::toJSON(JSONGenerator & json) const {
    TaintExpression::toJSON(json);
}
IR::UninitializedTaintExpression::UninitializedTaintExpression(JSONLoader & json) : TaintExpression(json) {
}
IR::Node * IR::UninitializedTaintExpression::fromJSON(JSONLoader & json) { return new UninitializedTaintExpression(json); }
bool IR::UninitializedTaintExpression::operator==(IR::TaintExpression const & a) const { return a == *this; }
bool IR::UninitializedTaintExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::UninitializedTaintExpression::operator==(IR::Node const & a) const { return a == *this; }
/**
 * Copyright (C) 2024 Intel Corporation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License.  You may obtain a copy
 * of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 * 
 * 
 * SPDX-License-Identifier: Apache-2.0
 */
/* -*-C++-*- */
#line 51 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
#include "backends/tofino/bf-p4c/ir/thread_visitor.h"
#line 14043 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::HardwareConstrainedField::operator==(IR::BFN::HardwareConstrainedField const & a) const {
        return this->typeId() == a.typeId()
        && expr == a.expr
        && constraint_type == a.constraint_type;
    }
bool IR::BFN::HardwareConstrainedField::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const HardwareConstrainedField &>(a_);
        return (expr ? a.expr ? expr->equiv(*a.expr) : false : a.expr == nullptr)
        && constraint_type == a.constraint_type;
    }
void IR::BFN::HardwareConstrainedField::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(expr, "expr");
}
void IR::BFN::HardwareConstrainedField::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(expr, "expr");
}
void IR::BFN::HardwareConstrainedField::validate() const {
#line 14067 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(expr); }
void IR::BFN::HardwareConstrainedField::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " constraint_type=" << constraint_type;
}
void IR::BFN::HardwareConstrainedField::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("expr", expr);
    json.emit("constraint_type", constraint_type);
}
IR::BFN::HardwareConstrainedField::HardwareConstrainedField(JSONLoader & json) : Node(json) {
    json.load("expr", expr);
    json.load("constraint_type", constraint_type);
}
IR::Node * IR::BFN::HardwareConstrainedField::fromJSON(JSONLoader & json) { return new HardwareConstrainedField(json); }
bool IR::BFN::HardwareConstrainedField::operator==(IR::Node const & a) const { return a == *this; }




#line 81 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
IR::BFN::Pipe::Pipe() {
        ids = {0};
        names = {cstring()};
    }
#line 14094 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 85 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
IR::BFN::Pipe::Pipe(std::vector<cstring> names, std::vector<int> ids) : names(names), ids(ids) {}
#line 14097 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"


#line 94 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
bool IR::BFN::Pipe::thread_t::operator==(IR::BFN::Pipe::thread_t const & a) const {
            return parsers == a.parsers && mau == a.mau && deparser == a.deparser &&
                hw_constrained_fields == a.hw_constrained_fields;
        }
#line 14105 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 98 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::Pipe::thread_t::toJSON(JSONGenerator & json) const {
            json.emit("parser", parsers);
            json.emit("mau", mau);
            json.emit("deparser", deparser);
            json.emit("hw_constrained_fields",  hw_constrained_fields); }
#line 14112 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 103 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
IR::BFN::Pipe::thread_t * IR::BFN::Pipe::thread_t::fromJSON(JSONLoader & json) {
            IR::BFN::Pipe::thread_t * thread = new IR::BFN::Pipe::thread_t();
            json.load("parsers", thread->parsers);
            json.load("mau", thread->mau);
            json.load("deparser", thread->deparser);
            json.load("hw_constrained_fields", thread->hw_constrained_fields);
            return thread;
        }
#line 14122 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IR::BFN::Pipe::thread_t::thread_t(JSONLoader & json)  {
    json.load("hw_constrained_fields", hw_constrained_fields);
    json.load("parsers", parsers);
    json.load("mau", mau);
    json.load("deparser", deparser);
}

#line 117 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
bool IR::BFN::Pipe::ghost_thread_t::operator==(IR::BFN::Pipe::ghost_thread_t const & a) const { return ghost_mau == a.ghost_mau
                        && ghost_parser == a.ghost_parser; }
#line 14133 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 119 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::Pipe::ghost_thread_t::dump_fields(std::ostream & out) const { out << "ghost_mau=" << ghost_mau;
                      out << "ghost_parser=" << ghost_parser; }
#line 14137 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 121 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::Pipe::ghost_thread_t::dbprint(std::ostream & out) const { out << "ghost_mau=" << ghost_mau << " "
                      << "ghost_parser=" << ghost_parser; }
#line 14141 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
void IR::BFN::Pipe::ghost_thread_t::toJSON(JSONGenerator & json) const {
    if (ghost_mau != nullptr) json.emit("ghost_mau", ghost_mau);
    json.emit("ghost_parser", ghost_parser);
}
IR::BFN::Pipe::ghost_thread_t::ghost_thread_t(JSONLoader & json)  {
    json.load("ghost_mau", ghost_mau);
    json.load("ghost_parser", ghost_parser);
}
IR::BFN::Pipe::ghost_thread_t * IR::BFN::Pipe::ghost_thread_t::fromJSON(JSONLoader & json) { return new ghost_thread_t(json); }
IRNODE_DEFINE_APPLY_OVERLOAD(BFN::Pipe, , )
#line 150 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
bool IR::BFN::Pipe::operator==(IR::BFN::Pipe const & a) const { return metadata == a.metadata
                        && global_pragmas == a.global_pragmas
                        && headerStackInfo == a.headerStackInfo
                        && thread[0] == a.thread[0]
                        && thread[1] == a.thread[1]
                        && ghost_thread.ghost_parser == a.ghost_thread.ghost_parser
                        && ghost_thread.ghost_mau == a.ghost_thread.ghost_mau; }
#line 14160 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 157 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::Pipe::visit_children(Visitor & v, char const * n) {
        if (auto *th = dynamic_cast<ThreadVisitor *>(&v)) {
            if (th->thread == GHOST) {
                v.visit(ghost_thread.ghost_parser, "ghost_parser");
                v.visit(ghost_thread.ghost_mau, "ghost_mau");
            } else {
                thread[th->thread].parsers.parallel_visit_children(v, n);
                ControlFlowVisitor::GuardGlobal guard_except(v, "-EXIT-"_cs);
                v.visit(thread[th->thread].mau, "mau");
                v.flow_merge_global_from("-EXIT-"_cs);
                v.visit(ghost_thread.ghost_parser, "ghost_parser");
                v.visit(ghost_thread.ghost_mau, "ghost_mau");
                v.visit(thread[th->thread].deparser, "deparser"); }
        } else {
            // FIXME -- doesn't do flow_clone/merge properly, but fixing will
            // break ParserGraph (at least) as it doesn't do merge properly
            for (auto &th : thread) {
                th.parsers.parallel_visit_children(v, n);
                ControlFlowVisitor::GuardGlobal guard_except(v, "-EXIT-"_cs);
                v.visit(th.mau, "mau");
                v.flow_merge_global_from("-EXIT-"_cs);
                v.visit(th.deparser, "deparser"); }
            v.visit(ghost_thread.ghost_parser, "ghost_parser");
            v.visit(ghost_thread.ghost_mau, "ghost_mau"); } }
#line 157 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::Pipe::visit_children(Visitor & v, char const * n) const {
        if (auto *th = dynamic_cast<ThreadVisitor *>(&v)) {
            if (th->thread == GHOST) {
                v.visit(ghost_thread.ghost_parser, "ghost_parser");
                v.visit(ghost_thread.ghost_mau, "ghost_mau");
            } else {
                thread[th->thread].parsers.parallel_visit_children(v, n);
                ControlFlowVisitor::GuardGlobal guard_except(v, "-EXIT-"_cs);
                v.visit(thread[th->thread].mau, "mau");
                v.flow_merge_global_from("-EXIT-"_cs);
                v.visit(ghost_thread.ghost_parser, "ghost_parser");
                v.visit(ghost_thread.ghost_mau, "ghost_mau");
                v.visit(thread[th->thread].deparser, "deparser"); }
        } else {
            // FIXME -- doesn't do flow_clone/merge properly, but fixing will
            // break ParserGraph (at least) as it doesn't do merge properly
            for (auto &th : thread) {
                th.parsers.parallel_visit_children(v, n);
                ControlFlowVisitor::GuardGlobal guard_except(v, "-EXIT-"_cs);
                v.visit(th.mau, "mau");
                v.flow_merge_global_from("-EXIT-"_cs);
                v.visit(th.deparser, "deparser"); }
            v.visit(ghost_thread.ghost_parser, "ghost_parser");
            v.visit(ghost_thread.ghost_mau, "ghost_mau"); } }
#line 14211 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::Pipe::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Pipe &>(a_);
        return names == a.names
        && ids == a.ids
        && metadata.equiv(a.metadata)
        && global_pragmas == a.global_pragmas
        && thread == a.thread
        && ghost_thread == a.ghost_thread;
    }
void IR::BFN::Pipe::validate() const {
#line 14224 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        metadata.validate(); }
void IR::BFN::Pipe::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " thread=" << thread;
        out << " ghost_thread=" << ghost_thread;
}
void IR::BFN::Pipe::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("names", names);
    json.emit("ids", ids);
    json.emit("metadata", metadata);
    json.emit("global_pragmas", global_pragmas);
    json.emit("thread", thread);
    json.emit("ghost_thread", ghost_thread);
}
IR::BFN::Pipe::Pipe(JSONLoader & json) : Node(json) {
    json.load("names", names);
    json.load("ids", ids);
    json.load("metadata", metadata);
    json.load("global_pragmas", global_pragmas);
    json.load("thread", thread);
    json.load("ghost_thread", ghost_thread);
}
IR::Node * IR::BFN::Pipe::fromJSON(JSONLoader & json) { return new Pipe(json); }
bool IR::BFN::Pipe::operator==(IR::Node const & a) const { return a == *this; }
#line 195 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
IR::BFN::BridgePath::BridgePath() = default;
#line 14253 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 196 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::BridgePath::visit_children(Visitor & v, char const * n) {
        // does not have support for ghost thread yet.
        for (auto &th : threads.at(INGRESS)) {
            th.parsers.parallel_visit_children(v, n);
            ControlFlowVisitor::GuardGlobal guard_except(v, "-EXIT-"_cs);
            v.visit(th.mau, "mau");
            v.flow_merge_global_from("-EXIT-"_cs);
            v.visit(th.deparser, "deparser"); }
        for (auto &th : threads.at(EGRESS)) {
            th.parsers.parallel_visit_children(v, n);
            ControlFlowVisitor::GuardGlobal guard_except(v, "-EXIT-"_cs);
            v.visit(th.mau, "mau");
            v.flow_merge_global_from("-EXIT-"_cs);
            v.visit(th.deparser, "deparser"); } }
#line 196 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::BridgePath::visit_children(Visitor & v, char const * n) const {
        // does not have support for ghost thread yet.
        for (auto &th : threads.at(INGRESS)) {
            th.parsers.parallel_visit_children(v, n);
            ControlFlowVisitor::GuardGlobal guard_except(v, "-EXIT-"_cs);
            v.visit(th.mau, "mau");
            v.flow_merge_global_from("-EXIT-"_cs);
            v.visit(th.deparser, "deparser"); }
        for (auto &th : threads.at(EGRESS)) {
            th.parsers.parallel_visit_children(v, n);
            ControlFlowVisitor::GuardGlobal guard_except(v, "-EXIT-"_cs);
            v.visit(th.mau, "mau");
            v.flow_merge_global_from("-EXIT-"_cs);
            v.visit(th.deparser, "deparser"); } }
#line 14284 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::BridgePath::operator==(IR::BFN::BridgePath const & a) const {
        return Pipe::operator==(static_cast<const Pipe &>(a))
        && threads == a.threads;
    }
bool IR::BFN::BridgePath::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Pipe::equiv(a_)) return false;
        auto &a = static_cast<const BridgePath &>(a_);
        return threads == a.threads;
    }
void IR::BFN::BridgePath::toJSON(JSONGenerator & json) const {
    Pipe::toJSON(json);
    json.emit("threads", threads);
}
IR::BFN::BridgePath::BridgePath(JSONLoader & json) : Pipe(json) {
    json.load("threads", threads);
}
IR::Node * IR::BFN::BridgePath::fromJSON(JSONLoader & json) { return new BridgePath(json); }
bool IR::BFN::BridgePath::operator==(IR::BFN::Pipe const & a) const { return a == *this; }
bool IR::BFN::BridgePath::operator==(IR::Node const & a) const { return a == *this; }






IRNODE_DEFINE_APPLY_OVERLOAD(BFN::Toplevel, , )
#line 233 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::Toplevel::visit_children(Visitor & v, char const * n) {
        v.visit(pipes, n);
    }
#line 233 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::Toplevel::visit_children(Visitor & v, char const * n) const {
        v.visit(pipes, n);
    }
#line 14320 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::Toplevel::operator==(IR::BFN::Toplevel const & a) const {
        return this->typeId() == a.typeId()
        && program == a.program
        && pipes == a.pipes
        && bridge_pipes == a.bridge_pipes;
    }
bool IR::BFN::Toplevel::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Toplevel &>(a_);
        return (program ? a.program ? program->equiv(*a.program) : false : a.program == nullptr)
        && (pipes ? a.pipes ? pipes->equiv(*a.pipes) : false : a.pipes == nullptr)
        && bridge_pipes == a.bridge_pipes;
    }
void IR::BFN::Toplevel::validate() const {
#line 14336 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(program);
        CHECK_NULL(pipes); }
void IR::BFN::Toplevel::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("program", program);
    json.emit("pipes", pipes);
    json.emit("bridge_pipes", bridge_pipes);
}
IR::BFN::Toplevel::Toplevel(JSONLoader & json) : Node(json) {
    json.load("program", program);
    json.load("pipes", pipes);
    json.load("bridge_pipes", bridge_pipes);
}
IR::Node * IR::BFN::Toplevel::fromJSON(JSONLoader & json) { return new Toplevel(json); }
bool IR::BFN::Toplevel::operator==(IR::Node const & a) const { return a == *this; }

bool IR::BFN::Unit::operator==(IR::BFN::Unit const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::BFN::Unit::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::BFN::Unit::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::BFN::Unit::Unit(JSONLoader & json) : Node(json) {
}
bool IR::BFN::Unit::operator==(IR::Node const & a) const { return a == *this; }
#line 256 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
IR::InstanceRef::InstanceRef(IR::ID n, const IR::Type* t, const IR::Vector<IR::Annotation>* ann) : InstanceRef(cstring(), n, t, ann) {}
#line 14370 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 258 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
IR::InstanceRef::InstanceRef(const IR::Type* t) : HeaderRef(t) { }
#line 14373 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 259 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
const IR::HeaderOrMetadata *IR::InstanceRef::baseRef() const { return obj; }
#line 14376 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 260 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
cstring IR::InstanceRef::toString() const { return obj ? obj->name : name; }
#line 14379 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 261 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::InstanceRef::dbprint(std::ostream & out) const { out << toString(); }
#line 14382 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::InstanceRef::operator==(IR::InstanceRef const & a) const {
        return HeaderRef::operator==(static_cast<const HeaderRef &>(a))
        && name == a.name
        && obj == a.obj
        && nested == a.nested;
    }
bool IR::InstanceRef::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!HeaderRef::equiv(a_)) return false;
        auto &a = static_cast<const InstanceRef &>(a_);
        return name == a.name
        && (obj ? a.obj ? obj->equiv(*a.obj) : false : a.obj == nullptr)
        && nested.equiv(a.nested);
    }
void IR::InstanceRef::visit_children(Visitor & v, char const * n) {
    (void)n;
    HeaderRef::visit_children(v, n);
    v.visit(obj, "obj");
    nested.visit_children(v, "nested");
}
void IR::InstanceRef::visit_children(Visitor & v, char const * n) const {
    (void)n;
    HeaderRef::visit_children(v, n);
    v.visit(obj, "obj");
    nested.visit_children(v, "nested");
}
void IR::InstanceRef::validate() const {
#line 14410 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        nested.validate(); }
void IR::InstanceRef::dump_fields(std::ostream & out) const {
    HeaderRef::dump_fields(out);
        out << " name=" << name;
}
void IR::InstanceRef::toJSON(JSONGenerator & json) const {
    HeaderRef::toJSON(json);
    json.emit("name", name);
    if (obj != nullptr) json.emit("obj", obj);
    json.emit("nested", nested);
}
IR::InstanceRef::InstanceRef(JSONLoader & json) : HeaderRef(json) {
    json.load("name", name);
    json.load("obj", obj);
    json.load("nested", nested);
}
IR::Node * IR::InstanceRef::fromJSON(JSONLoader & json) { return new InstanceRef(json); }
bool IR::InstanceRef::operator==(IR::HeaderRef const & a) const { return a == *this; }
bool IR::InstanceRef::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::InstanceRef::operator==(IR::Node const & a) const { return a == *this; }
#line 283 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
IR::V1InstanceRef::V1InstanceRef(IR::ID n, const IR::Type* t, const IR::Vector<IR::Annotation>* ann) : V1InstanceRef(cstring(), n, t, ann) {}
#line 14434 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 285 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
cstring IR::V1InstanceRef::toString() const { return obj ? obj->name : name; }
#line 14437 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 286 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::V1InstanceRef::dbprint(std::ostream & out) const { out << toString(); }
#line 14440 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::V1InstanceRef::operator==(IR::V1InstanceRef const & a) const {
        return InstanceRef::operator==(static_cast<const InstanceRef &>(a));
    }
bool IR::V1InstanceRef::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!InstanceRef::equiv(a_)) return false;
        return true;
    }
void IR::V1InstanceRef::toJSON(JSONGenerator & json) const {
    InstanceRef::toJSON(json);
}
IR::V1InstanceRef::V1InstanceRef(JSONLoader & json) : InstanceRef(json) {
}
IR::Node * IR::V1InstanceRef::fromJSON(JSONLoader & json) { return new V1InstanceRef(json); }
bool IR::V1InstanceRef::operator==(IR::InstanceRef const & a) const { return a == *this; }
bool IR::V1InstanceRef::operator==(IR::HeaderRef const & a) const { return a == *this; }
bool IR::V1InstanceRef::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::V1InstanceRef::operator==(IR::Node const & a) const { return a == *this; }
#line 295 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::MetadataInfo::dbprint(std::ostream & out) const { out << instance_name << "." << field_name; }
#line 14461 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MetadataInfo::operator==(IR::MetadataInfo const & a) const {
        return this->typeId() == a.typeId()
        && forTM == a.forTM
        && gress == a.gress
        && type_name == a.type_name
        && instance_name == a.instance_name
        && field_name == a.field_name;
    }
bool IR::MetadataInfo::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const MetadataInfo &>(a_);
        return forTM == a.forTM
        && gress == a.gress
        && type_name == a.type_name
        && instance_name == a.instance_name
        && field_name == a.field_name;
    }
void IR::MetadataInfo::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " forTM=" << forTM;
        out << " gress=" << gress;
        out << " type_name=" << type_name;
        out << " instance_name=" << instance_name;
        out << " field_name=" << field_name;
}
void IR::MetadataInfo::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("forTM", forTM);
    json.emit("gress", gress);
    json.emit("type_name", type_name);
    json.emit("instance_name", instance_name);
    json.emit("field_name", field_name);
}
IR::MetadataInfo::MetadataInfo(JSONLoader & json) : Node(json) {
    json.load("forTM", forTM);
    json.load("gress", gress);
    json.load("type_name", type_name);
    json.load("instance_name", instance_name);
    json.load("field_name", field_name);
}
IR::Node * IR::MetadataInfo::fromJSON(JSONLoader & json) { return new MetadataInfo(json); }
bool IR::MetadataInfo::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::Value::operator==(IR::BFN::Value const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a));
    }
bool IR::BFN::Value::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        return true;
    }
void IR::BFN::Value::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
}
IR::BFN::Value::Value(JSONLoader & json) : Expression(json) {
}
bool IR::BFN::Value::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::Value::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::Reference::operator==(IR::BFN::Reference const & a) const {
        return Value::operator==(static_cast<const Value &>(a));
    }
bool IR::BFN::Reference::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Value::equiv(a_)) return false;
        return true;
    }
void IR::BFN::Reference::toJSON(JSONGenerator & json) const {
    Value::toJSON(json);
}
IR::BFN::Reference::Reference(JSONLoader & json) : Value(json) {
}
bool IR::BFN::Reference::operator==(IR::BFN::Value const & a) const { return a == *this; }
bool IR::BFN::Reference::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::Reference::operator==(IR::Node const & a) const { return a == *this; }


#line 344 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
IR::TempVar::TempVar(const IR::Type* t, bool p, cstring n) : BFN::Reference(t), POV(p), name(n) {
        deparsed_zero = false;
        if (!name) {
            std::string n("$tmp");
            n += std::to_string(++uid);
            name = n; } }
#line 14545 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 350 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
IR::TempVar::TempVar(const IR::Type* t, cstring n) : TempVar(t, false, n) {
        deparsed_zero = false;
    }
#line 14550 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 353 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
cstring IR::TempVar::toString() const { return name; }
#line 14553 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 354 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::TempVar::dbprint(std::ostream & out) const { out << name; }
#line 14556 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::TempVar::operator==(IR::TempVar const & a) const {
        return BFN::Reference::operator==(static_cast<const BFN::Reference &>(a))
        && POV == a.POV
        && deparsed_zero == a.deparsed_zero
        && name == a.name;
    }
bool IR::TempVar::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!BFN::Reference::equiv(a_)) return false;
        auto &a = static_cast<const TempVar &>(a_);
        return POV == a.POV
        && deparsed_zero == a.deparsed_zero
        && name == a.name;
    }
void IR::TempVar::dump_fields(std::ostream & out) const {
    BFN::Reference::dump_fields(out);
        out << " POV=" << POV;
        out << " deparsed_zero=" << deparsed_zero;
        out << " name=" << name;
}
void IR::TempVar::toJSON(JSONGenerator & json) const {
    BFN::Reference::toJSON(json);
    json.emit("POV", POV);
    json.emit("deparsed_zero", deparsed_zero);
    json.emit("name", name);
}
IR::TempVar::TempVar(JSONLoader & json) : BFN::Reference(json) {
    json.load("POV", POV);
    json.load("deparsed_zero", deparsed_zero);
    json.load("name", name);
}
IR::Node * IR::TempVar::fromJSON(JSONLoader & json) { return new TempVar(json); }
bool IR::TempVar::operator==(IR::BFN::Reference const & a) const { return a == *this; }
bool IR::TempVar::operator==(IR::BFN::Value const & a) const { return a == *this; }
bool IR::TempVar::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::TempVar::operator==(IR::Node const & a) const { return a == *this; }
#line 366 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
IR::Padding::Padding(const IR::Type* t, cstring n) :
        BFN::Reference(t), name(n) {
        if (!name) {
            std::string n("$pad");
            n += std::to_string(++uid);
            name = n; } }
#line 14600 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 372 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
cstring IR::Padding::toString() const { return name; }
#line 14603 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 373 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::Padding::dbprint(std::ostream & out) const { out << name; }
#line 14606 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::Padding::operator==(IR::Padding const & a) const {
        return BFN::Reference::operator==(static_cast<const BFN::Reference &>(a))
        && name == a.name;
    }
bool IR::Padding::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!BFN::Reference::equiv(a_)) return false;
        auto &a = static_cast<const Padding &>(a_);
        return name == a.name;
    }
void IR::Padding::dump_fields(std::ostream & out) const {
    BFN::Reference::dump_fields(out);
        out << " name=" << name;
}
void IR::Padding::toJSON(JSONGenerator & json) const {
    BFN::Reference::toJSON(json);
    json.emit("name", name);
}
IR::Padding::Padding(JSONLoader & json) : BFN::Reference(json) {
    json.load("name", name);
}
IR::Node * IR::Padding::fromJSON(JSONLoader & json) { return new Padding(json); }
bool IR::Padding::operator==(IR::BFN::Reference const & a) const { return a == *this; }
bool IR::Padding::operator==(IR::BFN::Value const & a) const { return a == *this; }
bool IR::Padding::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::Padding::operator==(IR::Node const & a) const { return a == *this; }
#line 383 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
cstring IR::BFN::Type_FixedSizeHeader::toString() const { return cstring("fixed_size_header ") + externalName(); }
#line 14635 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::Type_FixedSizeHeader::operator==(IR::BFN::Type_FixedSizeHeader const & a) const {
        return Type_Header::operator==(static_cast<const Type_Header &>(a))
        && fixed_size == a.fixed_size;
    }
bool IR::BFN::Type_FixedSizeHeader::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Type_Header::equiv(a_)) return false;
        auto &a = static_cast<const Type_FixedSizeHeader &>(a_);
        return fixed_size == a.fixed_size;
    }
void IR::BFN::Type_FixedSizeHeader::dump_fields(std::ostream & out) const {
    Type_Header::dump_fields(out);
        out << " fixed_size=" << fixed_size;
}
void IR::BFN::Type_FixedSizeHeader::toJSON(JSONGenerator & json) const {
    Type_Header::toJSON(json);
    json.emit("fixed_size", fixed_size);
}
IR::BFN::Type_FixedSizeHeader::Type_FixedSizeHeader(JSONLoader & json) : Type_Header(json) {
    json.load("fixed_size", fixed_size);
}
IR::Node * IR::BFN::Type_FixedSizeHeader::fromJSON(JSONLoader & json) { return new Type_FixedSizeHeader(json); }
bool IR::BFN::Type_FixedSizeHeader::operator==(IR::Type_Header const & a) const { return a == *this; }
bool IR::BFN::Type_FixedSizeHeader::operator==(IR::Type_StructLike const & a) const { return a == *this; }
bool IR::BFN::Type_FixedSizeHeader::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::BFN::Type_FixedSizeHeader::operator==(IR::Type const & a) const { return a == *this; }
bool IR::BFN::Type_FixedSizeHeader::operator==(IR::Node const & a) const { return a == *this; }
#line 408 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
IR::BFN::ContainerRef::ContainerRef(PHV::Container c) : BFN::Reference(), container(c), id(nextId++) { }
#line 14665 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 409 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
IR::BFN::ContainerRef::ContainerRef(Util::SourceInfo si, PHV::Container c) :
        BFN::Reference(si), container(c), id(nextId++) { }
#line 14669 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 412 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::ContainerRef::validate() const {
#line 14672 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 412 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
{
        BUG_CHECK(bool(container), "Invalid PHV container in ContainerRef");
    } }
#line 14678 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 415 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
cstring IR::BFN::ContainerRef::toString() const {
        auto rv = container.toString();
        std::stringstream ss;
        if (range) {
            const le_bitrange lerange = (*range)
                .toOrder<Endian::Little>(container.size());

            if (lerange.lo != 0 || lerange.hi != (int)container.size() - 1) {
                ss << "(" << lerange.lo << ".." << lerange.hi << ")";
                rv += ss.str();
            }
        }
        return rv;
    }
#line 14694 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 429 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::ContainerRef::dbprint(std::ostream & out) const { out << toString(); }
#line 14697 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// @return a clone of this ContainerRef which does not compare equal
/// to the original or to any other existing ContainerRef.
#line 433 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
const IR::BFN::ContainerRef *IR::BFN::ContainerRef::makeFresh() const {
        auto* newRef = this->clone();
        newRef->id = nextId++;
        return newRef;
    }
#line 14706 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ContainerRef::operator==(IR::BFN::ContainerRef const & a) const {
        return Reference::operator==(static_cast<const Reference &>(a))
        && container == a.container
        && range == a.range
        && id == a.id;
    }
bool IR::BFN::ContainerRef::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Reference::equiv(a_)) return false;
        auto &a = static_cast<const ContainerRef &>(a_);
        return container == a.container
        && range == a.range
        && id == a.id;
    }
void IR::BFN::ContainerRef::dump_fields(std::ostream & out) const {
    Reference::dump_fields(out);
        out << " container=" << container;
        out << " id=" << id;
}
void IR::BFN::ContainerRef::toJSON(JSONGenerator & json) const {
    Reference::toJSON(json);
    json.emit("container", container);
    json.emit("range", range);
    json.emit("id", id);
}
IR::BFN::ContainerRef::ContainerRef(JSONLoader & json) : Reference(json) {
    json.load("container", container);
    json.load("range", range);
    json.load("id", id);
}
IR::Node * IR::BFN::ContainerRef::fromJSON(JSONLoader & json) { return new ContainerRef(json); }
bool IR::BFN::ContainerRef::operator==(IR::BFN::Reference const & a) const { return a == *this; }
bool IR::BFN::ContainerRef::operator==(IR::BFN::Value const & a) const { return a == *this; }
bool IR::BFN::ContainerRef::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::ContainerRef::operator==(IR::Node const & a) const { return a == *this; }
#line 442 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
cstring IR::BFN::ContainerBitRef::toString() const { return container->container.toString() + "(" + std::to_string(bit) + ")"; }
#line 14744 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 443 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::ContainerBitRef::dbprint(std::ostream & out) const { out << toString(); }
#line 14747 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The container containing the bit.
bool IR::BFN::ContainerBitRef::operator==(IR::BFN::ContainerBitRef const & a) const {
        return Reference::operator==(static_cast<const Reference &>(a))
        && container == a.container
        && bit == a.bit;
    }
bool IR::BFN::ContainerBitRef::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Reference::equiv(a_)) return false;
        auto &a = static_cast<const ContainerBitRef &>(a_);
        return (container ? a.container ? container->equiv(*a.container) : false : a.container == nullptr)
        && bit == a.bit;
    }
void IR::BFN::ContainerBitRef::visit_children(Visitor & v, char const * n) {
    (void)n;
    Reference::visit_children(v, n);
    v.visit(container, "container");
}
void IR::BFN::ContainerBitRef::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Reference::visit_children(v, n);
    v.visit(container, "container");
}
void IR::BFN::ContainerBitRef::validate() const {
#line 14772 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(container); }
void IR::BFN::ContainerBitRef::dump_fields(std::ostream & out) const {
    Reference::dump_fields(out);
        out << " bit=" << bit;
}
void IR::BFN::ContainerBitRef::toJSON(JSONGenerator & json) const {
    Reference::toJSON(json);
    json.emit("container", container);
    json.emit("bit", bit);
}
IR::BFN::ContainerBitRef::ContainerBitRef(JSONLoader & json) : Reference(json) {
    json.load("container", container);
    json.load("bit", bit);
}
IR::Node * IR::BFN::ContainerBitRef::fromJSON(JSONLoader & json) { return new ContainerBitRef(json); }
bool IR::BFN::ContainerBitRef::operator==(IR::BFN::Reference const & a) const { return a == *this; }
bool IR::BFN::ContainerBitRef::operator==(IR::BFN::Value const & a) const { return a == *this; }
bool IR::BFN::ContainerBitRef::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::ContainerBitRef::operator==(IR::Node const & a) const { return a == *this; }
/// The original expression (i.e. the alias source) that has been replaced
/// by this AliasMember.  Not visited in IR traversals.
#line 464 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::AliasMember::dbprint(std::ostream & out) const {
        IR::Member::dbprint(out);
        out << " alias: " << source;
    }
#line 14800 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 469 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::AliasMember::visit_children(Visitor & v, char const * n) {
        IR::Member::visit_children(v, n);
    }
#line 469 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::AliasMember::visit_children(Visitor & v, char const * n) const {
        IR::Member::visit_children(v, n);
    }
#line 14809 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// do not use the auto-generated equiv function.
/// do not compare the 'source' field of the AliasMember.
#line 476 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
bool IR::BFN::AliasMember::equiv(IR::Node const & a_) const { return Member::equiv(a_); }
#line 14814 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::AliasMember::operator==(IR::BFN::AliasMember const & a) const {
        return Member::operator==(static_cast<const Member &>(a))
        && source == a.source;
    }
void IR::BFN::AliasMember::validate() const {
#line 14820 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source); }
void IR::BFN::AliasMember::toJSON(JSONGenerator & json) const {
    Member::toJSON(json);
    json.emit("source", source);
}
IR::BFN::AliasMember::AliasMember(JSONLoader & json) : Member(json) {
    json.load("source", source);
}
IR::Node * IR::BFN::AliasMember::fromJSON(JSONLoader & json) { return new AliasMember(json); }
bool IR::BFN::AliasMember::operator==(IR::Member const & a) const { return a == *this; }
bool IR::BFN::AliasMember::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::BFN::AliasMember::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::BFN::AliasMember::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::AliasMember::operator==(IR::Node const & a) const { return a == *this; }
/// The original expression (i.e. the alias source) that has been replaced
/// by this AliasSlice.  Not visited in IR traversals.
#line 501 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::AliasSlice::dbprint(std::ostream & out) const {
        IR::Slice::dbprint(out);
        out << "alias: " << source;
    }
#line 14843 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 506 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::AliasSlice::visit_children(Visitor & v, char const * n) {
        IR::Slice::visit_children(v, n);
    }
#line 506 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::AliasSlice::visit_children(Visitor & v, char const * n) const {
        IR::Slice::visit_children(v, n);
    }
#line 14852 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::AliasSlice::operator==(IR::BFN::AliasSlice const & a) const {
        return Slice::operator==(static_cast<const Slice &>(a))
        && source == a.source;
    }
bool IR::BFN::AliasSlice::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Slice::equiv(a_)) return false;
        auto &a = static_cast<const AliasSlice &>(a_);
        return (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr);
    }
void IR::BFN::AliasSlice::validate() const {
#line 14864 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source); }
void IR::BFN::AliasSlice::toJSON(JSONGenerator & json) const {
    Slice::toJSON(json);
    json.emit("source", source);
}
IR::BFN::AliasSlice::AliasSlice(JSONLoader & json) : Slice(json) {
    json.load("source", source);
}
IR::Node * IR::BFN::AliasSlice::fromJSON(JSONLoader & json) { return new AliasSlice(json); }
bool IR::BFN::AliasSlice::operator==(IR::Slice const & a) const { return a == *this; }
bool IR::BFN::AliasSlice::operator==(IR::AbstractSlice const & a) const { return a == *this; }
bool IR::BFN::AliasSlice::operator==(IR::Operation_Ternary const & a) const { return a == *this; }
bool IR::BFN::AliasSlice::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::BFN::AliasSlice::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::AliasSlice::operator==(IR::Node const & a) const { return a == *this; }
bool IR::HashListExpression::operator==(IR::HashListExpression const & a) const {
        return ListExpression::operator==(static_cast<const ListExpression &>(a))
        && fieldListCalcName == a.fieldListCalcName
        && outputWidth == a.outputWidth
        && fieldListNames == a.fieldListNames
        && algorithms == a.algorithms;
    }
bool IR::HashListExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ListExpression::equiv(a_)) return false;
        auto &a = static_cast<const HashListExpression &>(a_);
        return fieldListCalcName == a.fieldListCalcName
        && outputWidth == a.outputWidth
        && (fieldListNames ? a.fieldListNames ? fieldListNames->equiv(*a.fieldListNames) : false : a.fieldListNames == nullptr)
        && (algorithms ? a.algorithms ? algorithms->equiv(*a.algorithms) : false : a.algorithms == nullptr);
    }
void IR::HashListExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    ListExpression::visit_children(v, n);
    v.visit(fieldListNames, "fieldListNames");
    v.visit(algorithms, "algorithms");
}
void IR::HashListExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ListExpression::visit_children(v, n);
    v.visit(fieldListNames, "fieldListNames");
    v.visit(algorithms, "algorithms");
}
void IR::HashListExpression::dump_fields(std::ostream & out) const {
    ListExpression::dump_fields(out);
        out << " fieldListCalcName=" << fieldListCalcName;
        out << " outputWidth=" << outputWidth;
}
void IR::HashListExpression::toJSON(JSONGenerator & json) const {
    ListExpression::toJSON(json);
    json.emit("fieldListCalcName", fieldListCalcName);
    json.emit("outputWidth", outputWidth);
    if (fieldListNames != nullptr) json.emit("fieldListNames", fieldListNames);
    if (algorithms != nullptr) json.emit("algorithms", algorithms);
}
IR::HashListExpression::HashListExpression(JSONLoader & json) : ListExpression(json) {
    json.load("fieldListCalcName", fieldListCalcName);
    json.load("outputWidth", outputWidth);
    json.load("fieldListNames", fieldListNames);
    json.load("algorithms", algorithms);
}
IR::Node * IR::HashListExpression::fromJSON(JSONLoader & json) { return new HashListExpression(json); }
bool IR::HashListExpression::operator==(IR::ListExpression const & a) const { return a == *this; }
bool IR::HashListExpression::operator==(IR::BaseListExpression const & a) const { return a == *this; }
bool IR::HashListExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::HashListExpression::operator==(IR::Node const & a) const { return a == *this; }
bool IR::HashStructExpression::operator==(IR::HashStructExpression const & a) const {
        return StructExpression::operator==(static_cast<const StructExpression &>(a))
        && fieldListCalcName == a.fieldListCalcName
        && outputWidth == a.outputWidth
        && fieldListNames == a.fieldListNames
        && algorithms == a.algorithms;
    }
bool IR::HashStructExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!StructExpression::equiv(a_)) return false;
        auto &a = static_cast<const HashStructExpression &>(a_);
        return fieldListCalcName == a.fieldListCalcName
        && outputWidth == a.outputWidth
        && (fieldListNames ? a.fieldListNames ? fieldListNames->equiv(*a.fieldListNames) : false : a.fieldListNames == nullptr)
        && (algorithms ? a.algorithms ? algorithms->equiv(*a.algorithms) : false : a.algorithms == nullptr);
    }
void IR::HashStructExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    StructExpression::visit_children(v, n);
    v.visit(fieldListNames, "fieldListNames");
    v.visit(algorithms, "algorithms");
}
void IR::HashStructExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    StructExpression::visit_children(v, n);
    v.visit(fieldListNames, "fieldListNames");
    v.visit(algorithms, "algorithms");
}
void IR::HashStructExpression::dump_fields(std::ostream & out) const {
    StructExpression::dump_fields(out);
        out << " fieldListCalcName=" << fieldListCalcName;
        out << " outputWidth=" << outputWidth;
}
void IR::HashStructExpression::toJSON(JSONGenerator & json) const {
    StructExpression::toJSON(json);
    json.emit("fieldListCalcName", fieldListCalcName);
    json.emit("outputWidth", outputWidth);
    if (fieldListNames != nullptr) json.emit("fieldListNames", fieldListNames);
    if (algorithms != nullptr) json.emit("algorithms", algorithms);
}
IR::HashStructExpression::HashStructExpression(JSONLoader & json) : StructExpression(json) {
    json.load("fieldListCalcName", fieldListCalcName);
    json.load("outputWidth", outputWidth);
    json.load("fieldListNames", fieldListNames);
    json.load("algorithms", algorithms);
}
IR::Node * IR::HashStructExpression::fromJSON(JSONLoader & json) { return new HashStructExpression(json); }
bool IR::HashStructExpression::operator==(IR::StructExpression const & a) const { return a == *this; }
bool IR::HashStructExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::HashStructExpression::operator==(IR::Node const & a) const { return a == *this; }
#line 548 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
cstring IR::BFN::ReinterpretCast::toString() const { return "reinterpret_cast"_cs; }
#line 14984 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 549 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::ReinterpretCast::validate() const {
#line 14987 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 549 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
{ BUG_CHECK(!destType->is<Type_Unknown>(), "%1%: Cannot cast to unknown type", this); } }
#line 14991 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ReinterpretCast::operator==(IR::BFN::ReinterpretCast const & a) const {
        return Cast::operator==(static_cast<const Cast &>(a));
    }
bool IR::BFN::ReinterpretCast::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Cast::equiv(a_)) return false;
        return true;
    }
void IR::BFN::ReinterpretCast::toJSON(JSONGenerator & json) const {
    Cast::toJSON(json);
}
IR::BFN::ReinterpretCast::ReinterpretCast(JSONLoader & json) : Cast(json) {
}
IR::Node * IR::BFN::ReinterpretCast::fromJSON(JSONLoader & json) { return new ReinterpretCast(json); }
bool IR::BFN::ReinterpretCast::operator==(IR::Cast const & a) const { return a == *this; }
bool IR::BFN::ReinterpretCast::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::BFN::ReinterpretCast::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::BFN::ReinterpretCast::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::ReinterpretCast::operator==(IR::Node const & a) const { return a == *this; }
#line 556 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
cstring IR::BFN::SignExtend::toString() const { return "sign_extend"_cs; }
#line 15013 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 557 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
void IR::BFN::SignExtend::validate() const {
#line 15016 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 557 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/tofino.def"
{ BUG_CHECK(!destType->is<Type_Unknown>(), "%1%: Cannot cast to unknown type", this); } }
#line 15020 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::SignExtend::operator==(IR::BFN::SignExtend const & a) const {
        return Cast::operator==(static_cast<const Cast &>(a));
    }
bool IR::BFN::SignExtend::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Cast::equiv(a_)) return false;
        return true;
    }
void IR::BFN::SignExtend::toJSON(JSONGenerator & json) const {
    Cast::toJSON(json);
}
IR::BFN::SignExtend::SignExtend(JSONLoader & json) : Cast(json) {
}
IR::Node * IR::BFN::SignExtend::fromJSON(JSONLoader & json) { return new SignExtend(json); }
bool IR::BFN::SignExtend::operator==(IR::Cast const & a) const { return a == *this; }
bool IR::BFN::SignExtend::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::BFN::SignExtend::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::BFN::SignExtend::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::SignExtend::operator==(IR::Node const & a) const { return a == *this; }
/**
 * Copyright (C) 2024 Intel Corporation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License.  You may obtain a copy
 * of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 * 
 * 
 * SPDX-License-Identifier: Apache-2.0
 */
/* -*-C++-*- */
#line 32 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/arch.def"
cstring IR::BFN::TnaControl::toString() const { return P4Control::toString() + " on " + cstring::to_cstring(thread); }
#line 15060 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// A map from standard TNA parameter names to the names actually used in
/// the P4 program. For example, a TNA control prototype may include a
/// parameter "inout M ig_md", but the user program instantiating the
/// control may use "inout M user_metadata"; this map would include a
/// mapping from "ig_md" to "user_metadata".
/// The thread this control is associated with.
/// The pipe name this control belongs to
bool IR::BFN::TnaControl::operator==(IR::BFN::TnaControl const & a) const {
        return P4Control::operator==(static_cast<const P4Control &>(a))
        && tnaParams == a.tnaParams
        && thread == a.thread
        && pipeName == a.pipeName;
    }
bool IR::BFN::TnaControl::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!P4Control::equiv(a_)) return false;
        auto &a = static_cast<const TnaControl &>(a_);
        return tnaParams == a.tnaParams
        && thread == a.thread
        && pipeName == a.pipeName;
    }
void IR::BFN::TnaControl::dump_fields(std::ostream & out) const {
    P4Control::dump_fields(out);
        out << " thread=" << thread;
        out << " pipeName=" << pipeName;
}
void IR::BFN::TnaControl::toJSON(JSONGenerator & json) const {
    P4Control::toJSON(json);
    json.emit("tnaParams", tnaParams);
    json.emit("thread", thread);
    if (pipeName != nullptr) json.emit("pipeName", pipeName);
}
IR::BFN::TnaControl::TnaControl(JSONLoader & json) : P4Control(json) {
    json.load("tnaParams", tnaParams);
    json.load("thread", thread);
    json.load("pipeName", pipeName);
}
IR::Node * IR::BFN::TnaControl::fromJSON(JSONLoader & json) { return new TnaControl(json); }
bool IR::BFN::TnaControl::operator==(IR::P4Control const & a) const { return a == *this; }
bool IR::BFN::TnaControl::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::BFN::TnaControl::operator==(IR::Type const & a) const { return a == *this; }
bool IR::BFN::TnaControl::operator==(IR::Node const & a) const { return a == *this; }
#line 52 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/arch.def"
cstring IR::BFN::TnaDeparser::toString() const { return P4Control::toString() + " on " + cstring::to_cstring(thread); }
#line 15105 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// A map from standard TNA parameter names to the names actually used in
/// the P4 program. For example, a TNA control prototype may include a
/// parameter "inout M ig_md", but the user program instantiating the
/// control may use "inout M user_metadata"; this map would include a
/// mapping from "ig_md" to "user_metadata".
/// The thread this control is associated with.
/// The pipe name this control belongs to
bool IR::BFN::TnaDeparser::operator==(IR::BFN::TnaDeparser const & a) const {
        return P4Control::operator==(static_cast<const P4Control &>(a))
        && tnaParams == a.tnaParams
        && thread == a.thread
        && pipeName == a.pipeName;
    }
bool IR::BFN::TnaDeparser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!P4Control::equiv(a_)) return false;
        auto &a = static_cast<const TnaDeparser &>(a_);
        return tnaParams == a.tnaParams
        && thread == a.thread
        && pipeName == a.pipeName;
    }
void IR::BFN::TnaDeparser::dump_fields(std::ostream & out) const {
    P4Control::dump_fields(out);
        out << " thread=" << thread;
        out << " pipeName=" << pipeName;
}
void IR::BFN::TnaDeparser::toJSON(JSONGenerator & json) const {
    P4Control::toJSON(json);
    json.emit("tnaParams", tnaParams);
    json.emit("thread", thread);
    if (pipeName != nullptr) json.emit("pipeName", pipeName);
}
IR::BFN::TnaDeparser::TnaDeparser(JSONLoader & json) : P4Control(json) {
    json.load("tnaParams", tnaParams);
    json.load("thread", thread);
    json.load("pipeName", pipeName);
}
IR::Node * IR::BFN::TnaDeparser::fromJSON(JSONLoader & json) { return new TnaDeparser(json); }
bool IR::BFN::TnaDeparser::operator==(IR::P4Control const & a) const { return a == *this; }
bool IR::BFN::TnaDeparser::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::BFN::TnaDeparser::operator==(IR::Type const & a) const { return a == *this; }
bool IR::BFN::TnaDeparser::operator==(IR::Node const & a) const { return a == *this; }
#line 72 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/arch.def"
cstring IR::BFN::TnaParser::toString() const { return P4Parser::toString() + " on " + cstring::to_cstring(thread); }
#line 15150 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 73 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/arch.def"
void IR::BFN::TnaParser::dbprint(std::ostream & out) const { out << thread << " "; P4Parser::dbprint(out); }
#line 15153 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// A map from standard TNA parameter names to the names actually used in
/// the P4 program. @see TnaControl::tnaParams.
/// The thread this parser is associated with.
/// Phase0 Info
/// The pipe name this parser belongs to
/// The port to configure at initialization time.
bool IR::BFN::TnaParser::operator==(IR::BFN::TnaParser const & a) const {
        return P4Parser::operator==(static_cast<const P4Parser &>(a))
        && tnaParams == a.tnaParams
        && thread == a.thread
        && phase0 == a.phase0
        && pipeName == a.pipeName
        && portmap == a.portmap;
    }
bool IR::BFN::TnaParser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!P4Parser::equiv(a_)) return false;
        auto &a = static_cast<const TnaParser &>(a_);
        return tnaParams == a.tnaParams
        && thread == a.thread
        && (phase0 ? a.phase0 ? phase0->equiv(*a.phase0) : false : a.phase0 == nullptr)
        && pipeName == a.pipeName
        && portmap == a.portmap;
    }
void IR::BFN::TnaParser::visit_children(Visitor & v, char const * n) {
    (void)n;
    P4Parser::visit_children(v, n);
    v.visit(phase0, "phase0");
}
void IR::BFN::TnaParser::visit_children(Visitor & v, char const * n) const {
    (void)n;
    P4Parser::visit_children(v, n);
    v.visit(phase0, "phase0");
}
void IR::BFN::TnaParser::dump_fields(std::ostream & out) const {
    P4Parser::dump_fields(out);
        out << " thread=" << thread;
        out << " pipeName=" << pipeName;
}
void IR::BFN::TnaParser::toJSON(JSONGenerator & json) const {
    P4Parser::toJSON(json);
    json.emit("tnaParams", tnaParams);
    json.emit("thread", thread);
    if (phase0 != nullptr) json.emit("phase0", phase0);
    if (pipeName != nullptr) json.emit("pipeName", pipeName);
    json.emit("portmap", portmap);
}
IR::BFN::TnaParser::TnaParser(JSONLoader & json) : P4Parser(json) {
    json.load("tnaParams", tnaParams);
    json.load("thread", thread);
    json.load("phase0", phase0);
    json.load("pipeName", pipeName);
    json.load("portmap", portmap);
}
IR::Node * IR::BFN::TnaParser::fromJSON(JSONLoader & json) { return new TnaParser(json); }
bool IR::BFN::TnaParser::operator==(IR::P4Parser const & a) const { return a == *this; }
bool IR::BFN::TnaParser::operator==(IR::Type_Declaration const & a) const { return a == *this; }
bool IR::BFN::TnaParser::operator==(IR::Type const & a) const { return a == *this; }
bool IR::BFN::TnaParser::operator==(IR::Node const & a) const { return a == *this; }
#line 96 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/arch.def"
cstring IR::BFN::P4Thread::toString() const { return "P4Thread"_cs; }
#line 15215 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 97 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/arch.def"
void IR::BFN::P4Thread::dbprint(std::ostream & out) const { out << toString(); }
#line 15218 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 99 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/arch.def"
IR::BFN::P4Thread::P4Thread() = default;
#line 15221 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 103 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/arch.def"
void IR::BFN::P4Thread::toJSON(JSONGenerator & json) const {
        json.emit("parsers", parsers);
        json.emit("mau", mau);
        json.emit("deparser", deparser); }
#line 15227 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 107 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/arch.def"
IR::Node * IR::BFN::P4Thread::fromJSON(JSONLoader & json) {
        IR::BFN::P4Thread * thread = new IR::BFN::P4Thread();
        json.load("parsers", thread->parsers);
        json.load("mau", thread->mau);
        json.load("deparser", thread->deparser);
        return thread;
    }
#line 15236 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(BFN::P4Thread, , )
#line 115 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/arch.def"
void IR::BFN::P4Thread::visit_children(Visitor & v, char const * n) {
        parsers.parallel_visit_children(v, n);
        v.visit(mau);
        v.visit(deparser);
    }
#line 115 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/arch.def"
void IR::BFN::P4Thread::visit_children(Visitor & v, char const * n) const {
        parsers.parallel_visit_children(v, n);
        v.visit(mau);
        v.visit(deparser);
    }
#line 15250 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::P4Thread::operator==(IR::BFN::P4Thread const & a) const {
        return this->typeId() == a.typeId()
        && parsers == a.parsers
        && mau == a.mau
        && deparser == a.deparser;
    }
bool IR::BFN::P4Thread::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const P4Thread &>(a_);
        return parsers.equiv(a.parsers)
        && mau == a.mau
        && deparser == a.deparser;
    }
void IR::BFN::P4Thread::validate() const {
#line 15266 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        parsers.validate(); }
void IR::BFN::P4Thread::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " mau=" << mau;
        out << " deparser=" << deparser;
}
IR::BFN::P4Thread::P4Thread(JSONLoader & json) : Node(json) {
    json.load("parsers", parsers);
    json.load("mau", mau);
    json.load("deparser", deparser);
}
bool IR::BFN::P4Thread::operator==(IR::Node const & a) const { return a == *this; }

#line 33 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
#include "backends/tofino/bf-p4c/mau/resource.h"
#line 15283 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"





/// The stage to which this table is assigned, if any.
/// The logical ID within the stage to which this table is assigned.

/**
     * each gateway_row is an expression for a single row of the gateway,
     * with the string being a tag to run in 'next' for next table, or null
     * if the match_table should run.  The last row (only) may have a NULL
     * expression for 'always' -- corresponding to the gateway miss action
     * (taken if no gateway rows match).  If this last row NULL expression
     * is not present, the default action is to run the table.
     */
/**
     * If the gateway inhibits the match_table (gateway_row.second != NULL), then the gateway
     * can also run an action directly, via the payload.  All rows must be the same action on
     * tofino1, but tofino2 allows different actions for different rows.
     *
     * The gateway payload key value pair is:
     *     the key corresponds to gateway_row.second which inhibits the table.  These also
     *     appear as the keys in the next map.
     *     the value is a pair of two items.  The first item is the key of the corresponding
     *     action in the actions map.  The vector of Constants is the value of parameters
     *     of the action
     */












#line 127 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::Table::IndirectAddress::total_bits() const {
           int rv = address_bits + meter_type_bits;
           rv = per_flow_enable ? rv + 1 : rv;
           return rv;
        }
#line 15330 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::Table::IndirectAddress::operator==(IR::MAU::Table::IndirectAddress const & a) const {
        return shifter_enabled == a.shifter_enabled
        && address_bits == a.address_bits
        && per_flow_enable == a.per_flow_enable
        && meter_type_bits == a.meter_type_bits;
    }
void IR::MAU::Table::IndirectAddress::toJSON(JSONGenerator & json) const {
    json.emit("shifter_enabled", shifter_enabled);
    json.emit("address_bits", address_bits);
    json.emit("per_flow_enable", per_flow_enable);
    json.emit("meter_type_bits", meter_type_bits);
}
IR::MAU::Table::IndirectAddress::IndirectAddress(JSONLoader & json)  {
    json.load("shifter_enabled", shifter_enabled);
    json.load("address_bits", address_bits);
    json.load("per_flow_enable", per_flow_enable);
    json.load("meter_type_bits", meter_type_bits);
}
IR::MAU::Table::IndirectAddress * IR::MAU::Table::IndirectAddress::fromJSON(JSONLoader & json) { return new IndirectAddress(json); }


#line 178 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Table::Layout::is_lamb_direct() const { return is_lamb && is_direct; }
#line 15354 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 179 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::Table::Layout::get_subword_bits() const {
            if (is_lamb) return ceil_log2(sets_per_word);
            return 0;
        }
#line 15360 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 184 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::Table::Layout::get_sram_depth() const {
            if (is_lamb) return Memories::LAMB_DEPTH;
            return Memories::SRAM_DEPTH;
        }
#line 15366 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/**
         * @brief Equivalent to get_sram_depth() for non-ATCAM tables.
         *
         * @return min(partition_count, get_sram_depth()) for ATCAM tables
         */
#line 194 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::Table::Layout::words_per_sram() const {
            return std::min(atcam ? partition_count : INT_MAX, get_sram_depth());
        }
#line 15376 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 198 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::Table::Layout::get_ram_line_select_bits() const {
            if (is_lamb) return IXBar::LAMB_LINE_SELECT_BITS;
            return IXBar::RAM_LINE_SELECT_BITS;
        }
#line 15382 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 203 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::Table::Layout::get_ram_ghost_bits() const {
            return get_ram_line_select_bits();
        }
#line 15387 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 207 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Table::Layout::direct_ad_required() const {
            if (action_addr.address_bits > 0)
                return false;
            if (action_data_bytes == 0)
                return false;
            return action_data_bytes_in_table > 0;
        }
#line 15396 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 214 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Table::Layout::ternary_indirect_required() const { return ternary && overhead_bits > 0; }
#line 15399 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 215 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Table::Layout::no_match_data() const {
            return ixbar_width_bits == 0;
        }
#line 15404 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 218 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Table::Layout::no_match_rams() const {
            return no_match_data() || hash_action;
        }
#line 15409 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/** no match tables that need hash_dist need to go through the hit path; all others
         *  can use the miss path
         */
#line 224 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Table::Layout::no_match_hit_path() const {
            return no_match_rams() && hash_action;
        }
#line 15417 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/** Any keyless table that has parameters that can be adjusted during runtime, such
         *  as action or action data must go through the miss pathway, as those are the registers
         *  configured by the driver when the control plane requests a change
         */
#line 231 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Table::Layout::no_match_miss_path() const {
            return no_match_rams() && !no_match_hit_path() && !gateway_match;
        }
#line 15426 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::Table::Layout::operator==(IR::MAU::Table::Layout const & a) const {
        return entries == a.entries
        && pre_classifier == a.pre_classifier
        && gateway == a.gateway
        && exact == a.exact
        && ternary == a.ternary
        && hash_action == a.hash_action
        && gateway_match == a.gateway_match
        && atcam == a.atcam
        && alpm == a.alpm
        && has_range == a.has_range
        && proxy_hash == a.proxy_hash
        && requires_versioning == a.requires_versioning
        && is_lamb == a.is_lamb
        && is_direct == a.is_direct
        && is_local_tind == a.is_local_tind
        && entries_per_set == a.entries_per_set
        && sets_per_word == a.sets_per_word
        && ixbar_bytes == a.ixbar_bytes
        && match_bytes == a.match_bytes
        && ixbar_width_bits == a.ixbar_width_bits
        && match_width_bits == a.match_width_bits
        && ghost_bytes == a.ghost_bytes
        && action_data_bytes == a.action_data_bytes
        && action_data_bytes_in_table == a.action_data_bytes_in_table
        && pre_classifer_number_entries == a.pre_classifer_number_entries
        && overhead_bits == a.overhead_bits
        && immediate_bits == a.immediate_bits
        && valid_bits == a.valid_bits
        && partition_bits == a.partition_bits
        && partition_count == a.partition_count
        && subtrees_per_partition == a.subtrees_per_partition
        && atcam_subset_width == a.atcam_subset_width
        && shift_granularity == a.shift_granularity
        && total_actions == a.total_actions
        && sel_len_bits == a.sel_len_bits
        && proxy_hash_width == a.proxy_hash_width
        && proxy_hash_algorithm == a.proxy_hash_algorithm
        && meter_addr == a.meter_addr
        && stats_addr == a.stats_addr
        && action_addr == a.action_addr
        && excluded_field_msb_bits == a.excluded_field_msb_bits;
    }
void IR::MAU::Table::Layout::toJSON(JSONGenerator & json) const {
    json.emit("entries", entries);
    json.emit("pre_classifier", pre_classifier);
    json.emit("gateway", gateway);
    json.emit("exact", exact);
    json.emit("ternary", ternary);
    json.emit("hash_action", hash_action);
    json.emit("gateway_match", gateway_match);
    json.emit("atcam", atcam);
    json.emit("alpm", alpm);
    json.emit("has_range", has_range);
    json.emit("proxy_hash", proxy_hash);
    json.emit("requires_versioning", requires_versioning);
    json.emit("is_lamb", is_lamb);
    json.emit("is_direct", is_direct);
    json.emit("is_local_tind", is_local_tind);
    json.emit("entries_per_set", entries_per_set);
    json.emit("sets_per_word", sets_per_word);
    json.emit("ixbar_bytes", ixbar_bytes);
    json.emit("match_bytes", match_bytes);
    json.emit("ixbar_width_bits", ixbar_width_bits);
    json.emit("match_width_bits", match_width_bits);
    json.emit("ghost_bytes", ghost_bytes);
    json.emit("action_data_bytes", action_data_bytes);
    json.emit("action_data_bytes_in_table", action_data_bytes_in_table);
    json.emit("pre_classifer_number_entries", pre_classifer_number_entries);
    json.emit("overhead_bits", overhead_bits);
    json.emit("immediate_bits", immediate_bits);
    json.emit("valid_bits", valid_bits);
    json.emit("partition_bits", partition_bits);
    json.emit("partition_count", partition_count);
    json.emit("subtrees_per_partition", subtrees_per_partition);
    json.emit("atcam_subset_width", atcam_subset_width);
    json.emit("shift_granularity", shift_granularity);
    json.emit("total_actions", total_actions);
    json.emit("sel_len_bits", sel_len_bits);
    json.emit("proxy_hash_width", proxy_hash_width);
    json.emit("proxy_hash_algorithm", proxy_hash_algorithm);
    json.emit("meter_addr", meter_addr);
    json.emit("stats_addr", stats_addr);
    json.emit("action_addr", action_addr);
    json.emit("excluded_field_msb_bits", excluded_field_msb_bits);
}
IR::MAU::Table::Layout::Layout(JSONLoader & json)  {
    json.load("entries", entries);
    json.load("pre_classifier", pre_classifier);
    json.load("gateway", gateway);
    json.load("exact", exact);
    json.load("ternary", ternary);
    json.load("hash_action", hash_action);
    json.load("gateway_match", gateway_match);
    json.load("atcam", atcam);
    json.load("alpm", alpm);
    json.load("has_range", has_range);
    json.load("proxy_hash", proxy_hash);
    json.load("requires_versioning", requires_versioning);
    json.load("is_lamb", is_lamb);
    json.load("is_direct", is_direct);
    json.load("is_local_tind", is_local_tind);
    json.load("entries_per_set", entries_per_set);
    json.load("sets_per_word", sets_per_word);
    json.load("ixbar_bytes", ixbar_bytes);
    json.load("match_bytes", match_bytes);
    json.load("ixbar_width_bits", ixbar_width_bits);
    json.load("match_width_bits", match_width_bits);
    json.load("ghost_bytes", ghost_bytes);
    json.load("action_data_bytes", action_data_bytes);
    json.load("action_data_bytes_in_table", action_data_bytes_in_table);
    json.load("pre_classifer_number_entries", pre_classifer_number_entries);
    json.load("overhead_bits", overhead_bits);
    json.load("immediate_bits", immediate_bits);
    json.load("valid_bits", valid_bits);
    json.load("partition_bits", partition_bits);
    json.load("partition_count", partition_count);
    json.load("subtrees_per_partition", subtrees_per_partition);
    json.load("atcam_subset_width", atcam_subset_width);
    json.load("shift_granularity", shift_granularity);
    json.load("total_actions", total_actions);
    json.load("sel_len_bits", sel_len_bits);
    json.load("proxy_hash_width", proxy_hash_width);
    json.load("proxy_hash_algorithm", proxy_hash_algorithm);
    json.load("meter_addr", meter_addr);
    json.load("stats_addr", stats_addr);
    json.load("action_addr", action_addr);
    json.load("excluded_field_msb_bits", excluded_field_msb_bits);
}
IR::MAU::Table::Layout * IR::MAU::Table::Layout::fromJSON(JSONLoader & json) { return new Layout(json); }

bool IR::MAU::Table::Way::operator==(IR::MAU::Table::Way const & a) const {
        return match_groups == a.match_groups
        && entries == a.entries
        && width == a.width;
    }
void IR::MAU::Table::Way::toJSON(JSONGenerator & json) const {
    json.emit("match_groups", match_groups);
    json.emit("entries", entries);
    json.emit("width", width);
}
IR::MAU::Table::Way::Way(JSONLoader & json)  {
    json.load("match_groups", match_groups);
    json.load("entries", entries);
    json.load("width", width);
}
IR::MAU::Table::Way * IR::MAU::Table::Way::fromJSON(JSONLoader & json) { return new Way(json); }
#line 299 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Table::is_placed() const { return resources != nullptr; }
#line 15576 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"


#line 327 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Table::is_always_run_action() const { return always_run == IR::MAU::AlwaysRun::ACTION; }
#line 15581 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 357 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::Table::Table(cstring n, gress_t gr) : name(n), gress(gr) {}
#line 15585 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 358 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::Table::Table(cstring n, gress_t gr, const IR::P4Table* t) : BFN::Unit(t->srcInfo), name(n), gress(gr), match_table(t) {}
#line 15588 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/** Returns the global logical ID for this table. Each stage has 16 slots
     * for tables.  Global logical IDs are numbered from 0 (Stage 0, Slot 0) to
     * N * 16 + 15 (Stage N, Slot 15).
     */
#line 371 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
std::optional<int> IR::MAU::Table::global_id() const {
        if (stage_ && logical_id) return *stage_ * StageUse::MAX_LOGICAL_IDS + *logical_id;
        return std::nullopt;
    }
#line 15598 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 375 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::Table::set_global_id(int id) {
        stage_ = id / StageUse::MAX_LOGICAL_IDS;
        logical_id = id % StageUse::MAX_LOGICAL_IDS;
    }
#line 15604 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 379 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::Table::logical_order() const { return *global_id() + gress * 4096; }
#line 15607 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 380 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::Table * IR::MAU::Table::clone_rename(cstring ext) const {
        Table *rv = clone();
        rv->name += ext;
        return rv; }
#line 15613 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 391 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Table::uses_gateway() const { return !gateway_rows.empty(); }
#line 15616 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 392 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Table::uses_gateway_payload() const {
        for (auto row : gateway_rows) {
            if (row.second) return true;
        }

        return false;
    }
#line 15625 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 401 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::Table::build_gateway_name() const {
        auto gName = gateway_name.isNullOrEmpty() ? name : gateway_name;
        // Add '-gateway' suffix to table name if gateway_name is not unique
        if (!gName.startsWith("cond")) gName = gName + "-gateway";
        return gName; }
#line 15632 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 406 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Table::uses_idletime() const {
        for (auto back_at : attached)
            if (back_at->attached->is<IR::MAU::IdleTime>()) return true;
        return false; }
#line 15638 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 410 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::Table::stage() const { return stage_ ? *stage_ : -1; }
#line 15641 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 411 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
gress_t IR::MAU::Table::thread() const { return gress; }
#line 15644 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 412 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::Table::toString() const { return "table "_cs + externalName(); }
#line 15647 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 413 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::Table::validate() const {
#line 15650 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        gateway_constant_entries_key.validate();
        attached.validate();
        match_key.validate();
#line 413 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
{
        std::set<cstring> gw_next;
        if (always_run == AlwaysRun::ACTION) {
            // This table represents an always-run action, so a few things should be empty here.
            BUG_CHECK(gateway_rows.empty(),
                "always-run action %s is ill-formed: cannot have gateway rows", name);
            BUG_CHECK(attached.empty(),
                "always-run action %s is ill-formed: cannot have attached tables", name);
            BUG_CHECK(match_key.empty(),
                "always-run action %s is ill-formed: cannot have match keys", name);
            BUG_CHECK(next.empty(),
                "always-run action %s is ill-formed: cannot have next tables", name);
            BUG_CHECK(gateway_payload.empty(),
                "always-run action %s is ill-formed: cannot have gateway payloads", name);

            // Also ensure we just have a single action in the table.
            BUG_CHECK(actions.size() == 1,
                "always-run action %s is ill-formed: must have exactly one action", name);
        }
        for (auto &gw : gateway_rows) {
            if (&gw != &gateway_rows.back())
                CHECK_NULL(gw.first);
            else if (gw.first)
                gw_next.emplace(cstring());
            gw_next.emplace(gw.second); }
        if (gateway_rows.empty())
            gw_next.emplace(cstring());
        if (created_during_tp && !conditional_gateway_only())
            BUG("starter-pistol table %s cannot have a match table", name);
        if (created_during_tp && uses_gateway_payload())
            BUG("starter-pistol table %s cannot have a gateway payload", name);
        if (!created_during_tp && conditional_gateway_only() && gw_next.count(cstring()))
            BUG("gateway with no match table wants to run match table");
        if (!is_a_gateway_table_only() && uses_gateway() && !gw_next.count(cstring()))
            BUG("gateway always overrides match table");
        std::set<cstring> payload_act;
        for (auto &payload : gateway_payload) {
            BUG_CHECK(gw_next.count(payload.first), "payload tag %s not in gateway", payload.first);
            BUG_CHECK(actions.count(payload.second.first), "payload action %s not in table",
                      payload.second.first);
            payload_act.insert(payload.second.first); }
        if (!gw_next.count(cstring()) && 0) {
            // FIXME -- we currently sometimes get dead actions in tables.  They should
            // eventually get deadcode eliminated, but might be in the IR until then
            for (auto &act : actions)
                BUG_CHECK(payload_act.count(act.first), "Unused action in gateway table"); }
        for (auto &nxt : next) {
            if (nxt.first == "$default") {
            } else if (nxt.first == "$hit" || nxt.first == "$miss"
                       || nxt.first == "$try_next_stage") {
                if (!match_table)
                    BUG("%s next on table with no match_table", nxt.first);
            } else if (!gw_next.count(nxt.first) && !actions.count(nxt.first)) {
                BUG("%s next on table with no such action or gw result", nxt.first);
            }
            if (!conditional_gateway_only() && !layout.gateway_match) {
                if (next.count("$hit"_cs) || next.count("$miss"_cs)) {
                    BUG_CHECK(!action_chain() && !has_default_path(),
                              "Somehow have a hit/miss table combined with an action chain");
                }
            }
        }
        for (auto &act : actions) {
            BUG_CHECK(act.first == act.second->name.originalName,
                      "name mismatch '%s' != '%s' for action %s",
                      act.first, act.second->name.originalName, act.second->name.name);
            if (act.second->exitAction)
                BUG_CHECK(next.count(act.first) == 0,
                          "action %s is an early exit and has next", act.first); }
    } }
#line 15726 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"



IRNODE_DEFINE_APPLY_OVERLOAD(MAU::Table, , )
bool IR::MAU::Table::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!BFN::Unit::equiv(a_)) return false;
        auto &a = static_cast<const Table &>(a_);
        return name == a.name
        && gress == a.gress
        && gateway_name == a.gateway_name
        && gateway_cond == a.gateway_cond
        && gateway_result_tag == a.gateway_result_tag
        && stage_ == a.stage_
        && logical_id == a.logical_id
        && stage_split == a.stage_split
        && logical_split == a.logical_split
        && logical_tables_in_stage == a.logical_tables_in_stage
        && atcam_entries_in_stage == a.atcam_entries_in_stage
        && gateway_rows == a.gateway_rows
        && gateway_payload == a.gateway_payload
        && gateway_constant_entries_key.equiv(a.gateway_constant_entries_key)
        && (match_table ? a.match_table ? match_table->equiv(*a.match_table) : false : a.match_table == nullptr)
        && attached.equiv(a.attached)
        && actions == a.actions
        && next == a.next
        && match_key.equiv(a.match_key)
        && sel_symmetric_keys == a.sel_symmetric_keys
        && random_seed == a.random_seed
        && dynamic_key_masks == a.dynamic_key_masks
        && (entries_list ? a.entries_list ? entries_list->equiv(*a.entries_list) : false : a.entries_list == nullptr)
        && first_entry_list_priority == a.first_entry_list_priority
        && created_during_tp == a.created_during_tp
        && is_compiler_generated == a.is_compiler_generated
        && has_dark_init == a.has_dark_init
        && always_run == a.always_run
        && suppress_context_json == a.suppress_context_json
        && run_before_exit == a.run_before_exit
        && is_detached_attached_tbl == a.is_detached_attached_tbl
        && layout == a.layout
        && ways == a.ways
        && resources == a.resources;
    }
void IR::MAU::Table::dump_fields(std::ostream & out) const {
    BFN::Unit::dump_fields(out);
        out << " name=" << name;
        out << " gress=" << gress;
        out << " gateway_name=" << gateway_name;
        out << " gateway_cond=" << gateway_cond;
        out << " gateway_result_tag=" << gateway_result_tag;
        out << " stage_split=" << stage_split;
        out << " logical_split=" << logical_split;
        out << " logical_tables_in_stage=" << logical_tables_in_stage;
        out << " atcam_entries_in_stage=" << atcam_entries_in_stage;
        out << " sel_symmetric_keys=" << sel_symmetric_keys;
        out << " random_seed=" << random_seed;
        out << " dynamic_key_masks=" << dynamic_key_masks;
        out << " first_entry_list_priority=" << first_entry_list_priority;
        out << " created_during_tp=" << created_during_tp;
        out << " is_compiler_generated=" << is_compiler_generated;
        out << " has_dark_init=" << has_dark_init;
        out << " always_run=" << always_run;
        out << " suppress_context_json=" << suppress_context_json;
        out << " run_before_exit=" << run_before_exit;
        out << " is_detached_attached_tbl=" << is_detached_attached_tbl;
        out << " layout=" << layout;
        out << " resources=" << resources;
}
void IR::MAU::Table::toJSON(JSONGenerator & json) const {
    BFN::Unit::toJSON(json);
    json.emit("name", name);
    json.emit("gress", gress);
    json.emit("gateway_name", gateway_name);
    json.emit("gateway_cond", gateway_cond);
    json.emit("gateway_result_tag", gateway_result_tag);
    json.emit("stage_", stage_);
    json.emit("logical_id", logical_id);
    json.emit("stage_split", stage_split);
    json.emit("logical_split", logical_split);
    json.emit("logical_tables_in_stage", logical_tables_in_stage);
    json.emit("atcam_entries_in_stage", atcam_entries_in_stage);
    json.emit("gateway_rows", gateway_rows);
    json.emit("gateway_payload", gateway_payload);
    json.emit("gateway_constant_entries_key", gateway_constant_entries_key);
    if (match_table != nullptr) json.emit("match_table", match_table);
    json.emit("attached", attached);
    json.emit("actions", actions);
    json.emit("next", next);
    json.emit("match_key", match_key);
    json.emit("sel_symmetric_keys", sel_symmetric_keys);
    json.emit("random_seed", random_seed);
    json.emit("dynamic_key_masks", dynamic_key_masks);
    if (entries_list != nullptr) json.emit("entries_list", entries_list);
    json.emit("first_entry_list_priority", first_entry_list_priority);
    json.emit("created_during_tp", created_during_tp);
    json.emit("is_compiler_generated", is_compiler_generated);
    json.emit("has_dark_init", has_dark_init);
    json.emit("always_run", always_run);
    json.emit("suppress_context_json", suppress_context_json);
    json.emit("run_before_exit", run_before_exit);
    json.emit("is_detached_attached_tbl", is_detached_attached_tbl);
    json.emit("layout", layout);
    json.emit("ways", ways);
    json.emit("resources", resources);
}
IR::MAU::Table::Table(JSONLoader & json) : BFN::Unit(json) {
    json.load("name", name);
    json.load("gress", gress);
    json.load("gateway_name", gateway_name);
    json.load("gateway_cond", gateway_cond);
    json.load("gateway_result_tag", gateway_result_tag);
    json.load("stage_", stage_);
    json.load("logical_id", logical_id);
    json.load("stage_split", stage_split);
    json.load("logical_split", logical_split);
    json.load("logical_tables_in_stage", logical_tables_in_stage);
    json.load("atcam_entries_in_stage", atcam_entries_in_stage);
    json.load("gateway_rows", gateway_rows);
    json.load("gateway_payload", gateway_payload);
    json.load("gateway_constant_entries_key", gateway_constant_entries_key);
    json.load("match_table", match_table);
    json.load("attached", attached);
    json.load("actions", actions);
    json.load("next", next);
    json.load("match_key", match_key);
    json.load("sel_symmetric_keys", sel_symmetric_keys);
    json.load("random_seed", random_seed);
    json.load("dynamic_key_masks", dynamic_key_masks);
    json.load("entries_list", entries_list);
    json.load("first_entry_list_priority", first_entry_list_priority);
    json.load("created_during_tp", created_during_tp);
    json.load("is_compiler_generated", is_compiler_generated);
    json.load("has_dark_init", has_dark_init);
    json.load("always_run", always_run);
    json.load("suppress_context_json", suppress_context_json);
    json.load("run_before_exit", run_before_exit);
    json.load("is_detached_attached_tbl", is_detached_attached_tbl);
    json.load("layout", layout);
    json.load("ways", ways);
    json.load("resources", resources);
}
IR::Node * IR::MAU::Table::fromJSON(JSONLoader & json) { return new Table(json); }
bool IR::MAU::BackendAttached::operator==(IR::MAU::BackendAttached const & a) const {
        return this->typeId() == a.typeId()
        && hash_dist == a.hash_dist
        && entries == a.entries
        && use == a.use
        && addr_location == a.addr_location
        && pfe_location == a.pfe_location
        && type_location == a.type_location
        && attached == a.attached
        && chain_vpn == a.chain_vpn;
    }
bool IR::MAU::BackendAttached::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const BackendAttached &>(a_);
        return (hash_dist ? a.hash_dist ? hash_dist->equiv(*a.hash_dist) : false : a.hash_dist == nullptr)
        && entries == a.entries
        && use == a.use
        && addr_location == a.addr_location
        && pfe_location == a.pfe_location
        && type_location == a.type_location
        && (attached ? a.attached ? attached->equiv(*a.attached) : false : a.attached == nullptr)
        && chain_vpn == a.chain_vpn;
    }
void IR::MAU::BackendAttached::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(hash_dist, "hash_dist");
    v.visit(attached, "attached");
}
void IR::MAU::BackendAttached::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(hash_dist, "hash_dist");
    v.visit(attached, "attached");
}
void IR::MAU::BackendAttached::validate() const {
#line 15906 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(attached); }
void IR::MAU::BackendAttached::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " entries=" << entries;
        out << " use=" << use;
        out << " addr_location=" << addr_location;
        out << " pfe_location=" << pfe_location;
        out << " type_location=" << type_location;
        out << " chain_vpn=" << chain_vpn;
}
void IR::MAU::BackendAttached::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    if (hash_dist != nullptr) json.emit("hash_dist", hash_dist);
    json.emit("entries", entries);
    json.emit("use", use);
    json.emit("addr_location", addr_location);
    json.emit("pfe_location", pfe_location);
    json.emit("type_location", type_location);
    json.emit("attached", attached);
    json.emit("chain_vpn", chain_vpn);
}
IR::MAU::BackendAttached::BackendAttached(JSONLoader & json) : Node(json) {
    json.load("hash_dist", hash_dist);
    json.load("entries", entries);
    json.load("use", use);
    json.load("addr_location", addr_location);
    json.load("pfe_location", pfe_location);
    json.load("type_location", type_location);
    json.load("attached", attached);
    json.load("chain_vpn", chain_vpn);
}
IR::Node * IR::MAU::BackendAttached::fromJSON(JSONLoader & json) { return new BackendAttached(json); }
bool IR::MAU::BackendAttached::operator==(IR::Node const & a) const { return a == *this; }
#line 528 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::AttachedMemory::indexed() const { return !direct; }
#line 15943 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 530 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
UniqueAttachedId IR::MAU::AttachedMemory::unique_id() const { return UniqueAttachedId(name, direct, get_id_type()); }
#line 15946 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 532 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::AttachedMemory::toString() const { return externType() + " " + name.toString(); }
#line 15949 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 533 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::AttachedMemory::usesMeterBus() const { return false; }
#line 15952 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 534 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::AttachedMemory::usesStatsBus() const { return false; }
#line 15955 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 535 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::AttachedMemory::get_min_size() const { return Memories::SRAM_DEPTH; }
#line 15958 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::AttachedMemory::operator==(IR::MAU::AttachedMemory const & a) const {
        return Attached::operator==(static_cast<const Attached &>(a))
        && direct == a.direct
        && size == a.size;
    }
bool IR::MAU::AttachedMemory::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Attached::equiv(a_)) return false;
        auto &a = static_cast<const AttachedMemory &>(a_);
        return direct == a.direct
        && size == a.size;
    }
void IR::MAU::AttachedMemory::dump_fields(std::ostream & out) const {
    Attached::dump_fields(out);
        out << " direct=" << direct;
        out << " size=" << size;
}
void IR::MAU::AttachedMemory::toJSON(JSONGenerator & json) const {
    Attached::toJSON(json);
    json.emit("direct", direct);
    json.emit("size", size);
}
IR::MAU::AttachedMemory::AttachedMemory(JSONLoader & json) : Attached(json) {
    json.load("direct", direct);
    json.load("size", size);
}
bool IR::MAU::AttachedMemory::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::MAU::AttachedMemory::operator==(IR::Node const & a) const { return a == *this; }
#line 546 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::Synth2Port::settype(cstring t) {
        if (strcasecmp(t.c_str(), "packets") == 0) type = DataAggregation::PACKETS;
        else if (strcasecmp(t.c_str(), "bytes") == 0) type = DataAggregation::BYTES;
        else if (strcasecmp(t.c_str(), "packets_and_bytes") == 0 ||
                 strcasecmp(t.c_str(), "packetsandbytes") == 0) type = DataAggregation::BOTH;
        else error("%s: Unknown type %s", srcInfo, t); }
#line 15994 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 552 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::Synth2Port::get_bytecount_adjust() const {
        if (type == DataAggregation::BYTES || type == DataAggregation::BOTH)
            return bytecount_adjust;
        return 0;
    }
#line 16002 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 557 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::Synth2Port::get_min_size() const {
        return ((TableFormat::SINGLE_RAM_BITS / width) * Memories::SRAM_DEPTH); 
    }
#line 16007 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::Synth2Port::operator==(IR::MAU::Synth2Port const & a) const {
        return AttachedMemory::operator==(static_cast<const AttachedMemory &>(a))
        && table == a.table
        && saturating == a.saturating
        && width == a.width
        && bytecount_adjust == a.bytecount_adjust
        && type == a.type;
    }
bool IR::MAU::Synth2Port::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!AttachedMemory::equiv(a_)) return false;
        auto &a = static_cast<const Synth2Port &>(a_);
        return table == a.table
        && saturating == a.saturating
        && width == a.width
        && bytecount_adjust == a.bytecount_adjust
        && type == a.type;
    }
void IR::MAU::Synth2Port::dump_fields(std::ostream & out) const {
    AttachedMemory::dump_fields(out);
        out << " table=" << table;
        out << " saturating=" << saturating;
        out << " width=" << width;
        out << " bytecount_adjust=" << bytecount_adjust;
        out << " type=" << type;
}
void IR::MAU::Synth2Port::toJSON(JSONGenerator & json) const {
    AttachedMemory::toJSON(json);
    json.emit("table", table);
    json.emit("saturating", saturating);
    json.emit("width", width);
    json.emit("bytecount_adjust", bytecount_adjust);
    json.emit("type", type);
}
IR::MAU::Synth2Port::Synth2Port(JSONLoader & json) : AttachedMemory(json) {
    json.load("table", table);
    json.load("saturating", saturating);
    json.load("width", width);
    json.load("bytecount_adjust", bytecount_adjust);
    json.load("type", type);
}
bool IR::MAU::Synth2Port::operator==(IR::MAU::AttachedMemory const & a) const { return a == *this; }
bool IR::MAU::Synth2Port::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::MAU::Synth2Port::operator==(IR::Node const & a) const { return a == *this; }

#line 569 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
char const * IR::MAU::Counter::kind() const { return "stats"; }
#line 16055 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 571 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::Type const * IR::MAU::Counter::getType() const { return Type_Counter::get(); }
#line 16058 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 572 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
UniqueAttachedId::type_t IR::MAU::Counter::get_id_type() const { return UniqueAttachedId::COUNTER; }
#line 16061 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 573 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::Counter::externType() const { return direct ? "DirectCounter"_cs : "Counter"_cs; }
#line 16064 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 574 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Counter::usesStatsBus() const { return true; }
#line 16067 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::Counter::operator==(IR::MAU::Counter const & a) const {
        return Synth2Port::operator==(static_cast<const Synth2Port &>(a))
        && min_width == a.min_width
        && max_width == a.max_width
        && threshold == a.threshold
        && interval == a.interval
        && true_egress_accounting == a.true_egress_accounting;
    }
bool IR::MAU::Counter::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Synth2Port::equiv(a_)) return false;
        auto &a = static_cast<const Counter &>(a_);
        return min_width == a.min_width
        && max_width == a.max_width
        && threshold == a.threshold
        && interval == a.interval
        && true_egress_accounting == a.true_egress_accounting;
    }
void IR::MAU::Counter::dump_fields(std::ostream & out) const {
    Synth2Port::dump_fields(out);
        out << " min_width=" << min_width;
        out << " max_width=" << max_width;
        out << " threshold=" << threshold;
        out << " interval=" << interval;
        out << " true_egress_accounting=" << true_egress_accounting;
}
void IR::MAU::Counter::toJSON(JSONGenerator & json) const {
    Synth2Port::toJSON(json);
    json.emit("min_width", min_width);
    json.emit("max_width", max_width);
    json.emit("threshold", threshold);
    json.emit("interval", interval);
    json.emit("true_egress_accounting", true_egress_accounting);
}
IR::MAU::Counter::Counter(JSONLoader & json) : Synth2Port(json) {
    json.load("min_width", min_width);
    json.load("max_width", max_width);
    json.load("threshold", threshold);
    json.load("interval", interval);
    json.load("true_egress_accounting", true_egress_accounting);
}
IR::Node * IR::MAU::Counter::fromJSON(JSONLoader & json) { return new Counter(json); }
bool IR::MAU::Counter::operator==(IR::MAU::Synth2Port const & a) const { return a == *this; }
bool IR::MAU::Counter::operator==(IR::MAU::AttachedMemory const & a) const { return a == *this; }
bool IR::MAU::Counter::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::MAU::Counter::operator==(IR::Node const & a) const { return a == *this; }

#line 578 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::MeterBus2Port::usesMeterBus() const { return true; }
#line 16117 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::MeterBus2Port::operator==(IR::MAU::MeterBus2Port const & a) const {
        return Synth2Port::operator==(static_cast<const Synth2Port &>(a));
    }
bool IR::MAU::MeterBus2Port::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Synth2Port::equiv(a_)) return false;
        return true;
    }
void IR::MAU::MeterBus2Port::toJSON(JSONGenerator & json) const {
    Synth2Port::toJSON(json);
}
IR::MAU::MeterBus2Port::MeterBus2Port(JSONLoader & json) : Synth2Port(json) {
}
bool IR::MAU::MeterBus2Port::operator==(IR::MAU::Synth2Port const & a) const { return a == *this; }
bool IR::MAU::MeterBus2Port::operator==(IR::MAU::AttachedMemory const & a) const { return a == *this; }
bool IR::MAU::MeterBus2Port::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::MAU::MeterBus2Port::operator==(IR::Node const & a) const { return a == *this; }






#line 599 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
char const * IR::MAU::Meter::kind() const { return "meter"; }
#line 16143 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 600 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::Type const * IR::MAU::Meter::getType() const { return Type_Meter::get(); }
#line 16146 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 601 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
UniqueAttachedId::type_t IR::MAU::Meter::get_id_type() const { return UniqueAttachedId::METER; }
#line 16149 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 602 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Meter::alu_output() const { return implementation.name == "lpf"
                               || implementation.name == "wred"; }
#line 16153 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 604 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Meter::color_output() const { return implementation.name == "normal"
                                 || implementation.name.isNull(); }
#line 16157 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 606 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Meter::mapram_possible(IR::MAU::ColorMapramAddress cma) const {
        return possible_mapram_address & (1U << static_cast<int>(cma));
    }
#line 16162 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 609 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::Meter::externType() const { return direct ? "DirectMeter"_cs : "Meter"_cs; }
#line 16165 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::Meter::operator==(IR::MAU::Meter const & a) const {
        return MeterBus2Port::operator==(static_cast<const MeterBus2Port &>(a))
        && red_drop_value == a.red_drop_value
        && red_nodrop_value == a.red_nodrop_value
        && green_value == a.green_value
        && yellow_value == a.yellow_value
        && red_value == a.red_value
        && profile == a.profile
        && sweep_interval == a.sweep_interval
        && true_egress_accounting == a.true_egress_accounting
        && result == a.result
        && pre_color == a.pre_color
        && input == a.input
        && implementation == a.implementation
        && possible_mapram_address == a.possible_mapram_address;
    }
bool IR::MAU::Meter::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!MeterBus2Port::equiv(a_)) return false;
        auto &a = static_cast<const Meter &>(a_);
        return red_drop_value == a.red_drop_value
        && red_nodrop_value == a.red_nodrop_value
        && green_value == a.green_value
        && yellow_value == a.yellow_value
        && red_value == a.red_value
        && profile == a.profile
        && sweep_interval == a.sweep_interval
        && true_egress_accounting == a.true_egress_accounting
        && (result ? a.result ? result->equiv(*a.result) : false : a.result == nullptr)
        && (pre_color ? a.pre_color ? pre_color->equiv(*a.pre_color) : false : a.pre_color == nullptr)
        && (input ? a.input ? input->equiv(*a.input) : false : a.input == nullptr)
        && implementation == a.implementation
        && possible_mapram_address == a.possible_mapram_address;
    }
void IR::MAU::Meter::visit_children(Visitor & v, char const * n) {
    (void)n;
    MeterBus2Port::visit_children(v, n);
    v.visit(result, "result");
    v.visit(pre_color, "pre_color");
    v.visit(input, "input");
}
void IR::MAU::Meter::visit_children(Visitor & v, char const * n) const {
    (void)n;
    MeterBus2Port::visit_children(v, n);
    v.visit(result, "result");
    v.visit(pre_color, "pre_color");
    v.visit(input, "input");
}
void IR::MAU::Meter::dump_fields(std::ostream & out) const {
    MeterBus2Port::dump_fields(out);
        out << " red_drop_value=" << red_drop_value;
        out << " red_nodrop_value=" << red_nodrop_value;
        out << " green_value=" << green_value;
        out << " yellow_value=" << yellow_value;
        out << " red_value=" << red_value;
        out << " profile=" << profile;
        out << " sweep_interval=" << sweep_interval;
        out << " true_egress_accounting=" << true_egress_accounting;
        out << " implementation=" << implementation;
        out << " possible_mapram_address=" << possible_mapram_address;
}
void IR::MAU::Meter::toJSON(JSONGenerator & json) const {
    MeterBus2Port::toJSON(json);
    json.emit("red_drop_value", red_drop_value);
    json.emit("red_nodrop_value", red_nodrop_value);
    json.emit("green_value", green_value);
    json.emit("yellow_value", yellow_value);
    json.emit("red_value", red_value);
    json.emit("profile", profile);
    json.emit("sweep_interval", sweep_interval);
    json.emit("true_egress_accounting", true_egress_accounting);
    if (result != nullptr) json.emit("result", result);
    if (pre_color != nullptr) json.emit("pre_color", pre_color);
    if (input != nullptr) json.emit("input", input);
    json.emit("implementation", implementation);
    json.emit("possible_mapram_address", possible_mapram_address);
}
IR::MAU::Meter::Meter(JSONLoader & json) : MeterBus2Port(json) {
    json.load("red_drop_value", red_drop_value);
    json.load("red_nodrop_value", red_nodrop_value);
    json.load("green_value", green_value);
    json.load("yellow_value", yellow_value);
    json.load("red_value", red_value);
    json.load("profile", profile);
    json.load("sweep_interval", sweep_interval);
    json.load("true_egress_accounting", true_egress_accounting);
    json.load("result", result);
    json.load("pre_color", pre_color);
    json.load("input", input);
    json.load("implementation", implementation);
    json.load("possible_mapram_address", possible_mapram_address);
}
IR::Node * IR::MAU::Meter::fromJSON(JSONLoader & json) { return new Meter(json); }
bool IR::MAU::Meter::operator==(IR::MAU::MeterBus2Port const & a) const { return a == *this; }
bool IR::MAU::Meter::operator==(IR::MAU::Synth2Port const & a) const { return a == *this; }
bool IR::MAU::Meter::operator==(IR::MAU::AttachedMemory const & a) const { return a == *this; }
bool IR::MAU::Meter::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::MAU::Meter::operator==(IR::Node const & a) const { return a == *this; }

















#line 657 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::StatefulAlu::MathUnit::dbprint(std::ostream & out) const { out << valid; }
#line 16283 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::StatefulAlu::MathUnit::operator==(IR::MAU::StatefulAlu::MathUnit const & a) const {
        return valid == a.valid
        && exp_invert == a.exp_invert
        && exp_shift == a.exp_shift
        && scale == a.scale
        && table[0] == a.table[0]
        && table[1] == a.table[1]
        && table[2] == a.table[2]
        && table[3] == a.table[3]
        && table[4] == a.table[4]
        && table[5] == a.table[5]
        && table[6] == a.table[6]
        && table[7] == a.table[7]
        && table[8] == a.table[8]
        && table[9] == a.table[9]
        && table[10] == a.table[10]
        && table[11] == a.table[11]
        && table[12] == a.table[12]
        && table[13] == a.table[13]
        && table[14] == a.table[14]
        && table[15] == a.table[15];
    }
void IR::MAU::StatefulAlu::MathUnit::toJSON(JSONGenerator & json) const {
    json.emit("valid", valid);
    json.emit("exp_invert", exp_invert);
    json.emit("exp_shift", exp_shift);
    json.emit("scale", scale);
    json.emit("table", table);
}
IR::MAU::StatefulAlu::MathUnit::MathUnit(JSONLoader & json)  {
    json.load("valid", valid);
    json.load("exp_invert", exp_invert);
    json.load("exp_shift", exp_shift);
    json.load("scale", scale);
    json.load("table", table);
}
IR::MAU::StatefulAlu::MathUnit * IR::MAU::StatefulAlu::MathUnit::fromJSON(JSONLoader & json) { return new MathUnit(json); }
#line 667 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
char const * IR::MAU::StatefulAlu::kind() const { return "stateful"; }
#line 16323 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 668 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::Type const * IR::MAU::StatefulAlu::getType() const { return Type_Register::get(); }
#line 16326 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 669 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
UniqueAttachedId::type_t IR::MAU::StatefulAlu::get_id_type() const { return UniqueAttachedId::STATEFUL_ALU; }
#line 16329 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 672 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::StatefulAlu::for_dleft() const {
        for (auto *act : Values(instruction))
            if (act->learn_action) return true;
        return false; }
#line 16336 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 678 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::StatefulAlu::alu_width() const { return std::min(32, source_width()); }
#line 16339 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 679 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::StatefulAlu::externType() const { return direct ? "DirectRegister"_cs : "Register"_cs; }
#line 16342 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::StatefulAlu::operator==(IR::MAU::StatefulAlu const & a) const {
        return MeterBus2Port::operator==(static_cast<const MeterBus2Port &>(a))
        && reg == a.reg
        && selector == a.selector
        && dual == a.dual
        && instruction == a.instruction
        && action_map == a.action_map
        && regfile == a.regfile
        && overflow == a.overflow
        && underflow == a.underflow
        && chain_vpn == a.chain_vpn
        && chain_total_size == a.chain_total_size
        && init_reg_lo == a.init_reg_lo
        && init_reg_hi == a.init_reg_hi
        && reduction_or_group == a.reduction_or_group
        && learn_action == a.learn_action
        && synthetic_for_selector == a.synthetic_for_selector
        && pred_shift == a.pred_shift
        && pred_comb_shift == a.pred_comb_shift
        && clear_value == a.clear_value
        && busy_value == a.busy_value
        && math == a.math;
    }
bool IR::MAU::StatefulAlu::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!MeterBus2Port::equiv(a_)) return false;
        auto &a = static_cast<const StatefulAlu &>(a_);
        return (reg ? a.reg ? reg->equiv(*a.reg) : false : a.reg == nullptr)
        && (selector ? a.selector ? selector->equiv(*a.selector) : false : a.selector == nullptr)
        && dual == a.dual
        && instruction.equiv(a.instruction)
        && action_map == a.action_map
        && regfile == a.regfile
        && overflow == a.overflow
        && underflow == a.underflow
        && chain_vpn == a.chain_vpn
        && chain_total_size == a.chain_total_size
        && init_reg_lo == a.init_reg_lo
        && init_reg_hi == a.init_reg_hi
        && reduction_or_group == a.reduction_or_group
        && learn_action == a.learn_action
        && synthetic_for_selector == a.synthetic_for_selector
        && pred_shift == a.pred_shift
        && pred_comb_shift == a.pred_comb_shift
        && clear_value == a.clear_value
        && busy_value == a.busy_value
        && math == a.math;
    }
void IR::MAU::StatefulAlu::visit_children(Visitor & v, char const * n) {
    (void)n;
    MeterBus2Port::visit_children(v, n);
    v.visit(reg, "reg");
    v.visit(selector, "selector");
    instruction.visit_children(v, "instruction");
}
void IR::MAU::StatefulAlu::visit_children(Visitor & v, char const * n) const {
    (void)n;
    MeterBus2Port::visit_children(v, n);
    v.visit(reg, "reg");
    v.visit(selector, "selector");
    instruction.visit_children(v, "instruction");
}
void IR::MAU::StatefulAlu::validate() const {
#line 16406 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        instruction.validate(); }
void IR::MAU::StatefulAlu::dump_fields(std::ostream & out) const {
    MeterBus2Port::dump_fields(out);
        out << " dual=" << dual;
        out << " overflow=" << overflow;
        out << " underflow=" << underflow;
        out << " chain_vpn=" << chain_vpn;
        out << " chain_total_size=" << chain_total_size;
        out << " init_reg_lo=" << init_reg_lo;
        out << " init_reg_hi=" << init_reg_hi;
        out << " reduction_or_group=" << reduction_or_group;
        out << " learn_action=" << learn_action;
        out << " synthetic_for_selector=" << synthetic_for_selector;
        out << " pred_shift=" << pred_shift;
        out << " pred_comb_shift=" << pred_comb_shift;
        out << " clear_value=" << clear_value;
        out << " busy_value=" << busy_value;
        out << " math=" << math;
}
void IR::MAU::StatefulAlu::toJSON(JSONGenerator & json) const {
    MeterBus2Port::toJSON(json);
    if (reg != nullptr) json.emit("reg", reg);
    if (selector != nullptr) json.emit("selector", selector);
    json.emit("dual", dual);
    json.emit("instruction", instruction);
    json.emit("action_map", action_map);
    json.emit("regfile", regfile);
    json.emit("overflow", overflow);
    json.emit("underflow", underflow);
    json.emit("chain_vpn", chain_vpn);
    json.emit("chain_total_size", chain_total_size);
    json.emit("init_reg_lo", init_reg_lo);
    json.emit("init_reg_hi", init_reg_hi);
    json.emit("reduction_or_group", reduction_or_group);
    json.emit("learn_action", learn_action);
    json.emit("synthetic_for_selector", synthetic_for_selector);
    json.emit("pred_shift", pred_shift);
    json.emit("pred_comb_shift", pred_comb_shift);
    json.emit("clear_value", clear_value);
    json.emit("busy_value", busy_value);
    json.emit("math", math);
}
IR::MAU::StatefulAlu::StatefulAlu(JSONLoader & json) : MeterBus2Port(json) {
    json.load("reg", reg);
    json.load("selector", selector);
    json.load("dual", dual);
    json.load("instruction", instruction);
    json.load("action_map", action_map);
    json.load("regfile", regfile);
    json.load("overflow", overflow);
    json.load("underflow", underflow);
    json.load("chain_vpn", chain_vpn);
    json.load("chain_total_size", chain_total_size);
    json.load("init_reg_lo", init_reg_lo);
    json.load("init_reg_hi", init_reg_hi);
    json.load("reduction_or_group", reduction_or_group);
    json.load("learn_action", learn_action);
    json.load("synthetic_for_selector", synthetic_for_selector);
    json.load("pred_shift", pred_shift);
    json.load("pred_comb_shift", pred_comb_shift);
    json.load("clear_value", clear_value);
    json.load("busy_value", busy_value);
    json.load("math", math);
}
IR::Node * IR::MAU::StatefulAlu::fromJSON(JSONLoader & json) { return new StatefulAlu(json); }
bool IR::MAU::StatefulAlu::operator==(IR::MAU::MeterBus2Port const & a) const { return a == *this; }
bool IR::MAU::StatefulAlu::operator==(IR::MAU::Synth2Port const & a) const { return a == *this; }
bool IR::MAU::StatefulAlu::operator==(IR::MAU::AttachedMemory const & a) const { return a == *this; }
bool IR::MAU::StatefulAlu::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::MAU::StatefulAlu::operator==(IR::Node const & a) const { return a == *this; }
#line 686 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
char const * IR::MAU::Selector::kind() const { return "selector"; }
#line 16480 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 687 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
UniqueAttachedId::type_t IR::MAU::Selector::get_id_type() const { return UniqueAttachedId::SELECTOR; }
#line 16483 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 692 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::Selector::externType() const { return "ActionSelector"_cs; }
#line 16487 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 693 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Selector::usesMeterBus() const { return true; }
#line 16490 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::Selector::operator==(IR::MAU::Selector const & a) const {
        return AttachedMemory::operator==(static_cast<const AttachedMemory &>(a))
        && type == a.type
        && mode == a.mode
        && algorithm == a.algorithm
        && num_pools == a.num_pools
        && max_pool_size == a.max_pool_size
        && sps_scramble == a.sps_scramble
        && hash_mod == a.hash_mod;
    }
bool IR::MAU::Selector::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!AttachedMemory::equiv(a_)) return false;
        auto &a = static_cast<const Selector &>(a_);
        return type == a.type
        && mode == a.mode
        && algorithm == a.algorithm
        && num_pools == a.num_pools
        && max_pool_size == a.max_pool_size
        && sps_scramble == a.sps_scramble
        && (hash_mod ? a.hash_mod ? hash_mod->equiv(*a.hash_mod) : false : a.hash_mod == nullptr);
    }
void IR::MAU::Selector::visit_children(Visitor & v, char const * n) {
    (void)n;
    AttachedMemory::visit_children(v, n);
    v.visit(hash_mod, "hash_mod");
}
void IR::MAU::Selector::visit_children(Visitor & v, char const * n) const {
    (void)n;
    AttachedMemory::visit_children(v, n);
    v.visit(hash_mod, "hash_mod");
}
void IR::MAU::Selector::dump_fields(std::ostream & out) const {
    AttachedMemory::dump_fields(out);
        out << " type=" << type;
        out << " mode=" << mode;
        out << " algorithm=" << algorithm;
        out << " num_pools=" << num_pools;
        out << " max_pool_size=" << max_pool_size;
        out << " sps_scramble=" << sps_scramble;
}
void IR::MAU::Selector::toJSON(JSONGenerator & json) const {
    AttachedMemory::toJSON(json);
    json.emit("type", type);
    json.emit("mode", mode);
    json.emit("algorithm", algorithm);
    json.emit("num_pools", num_pools);
    json.emit("max_pool_size", max_pool_size);
    json.emit("sps_scramble", sps_scramble);
    if (hash_mod != nullptr) json.emit("hash_mod", hash_mod);
}
IR::MAU::Selector::Selector(JSONLoader & json) : AttachedMemory(json) {
    json.load("type", type);
    json.load("mode", mode);
    json.load("algorithm", algorithm);
    json.load("num_pools", num_pools);
    json.load("max_pool_size", max_pool_size);
    json.load("sps_scramble", sps_scramble);
    json.load("hash_mod", hash_mod);
}
IR::Node * IR::MAU::Selector::fromJSON(JSONLoader & json) { return new Selector(json); }
bool IR::MAU::Selector::operator==(IR::MAU::AttachedMemory const & a) const { return a == *this; }
bool IR::MAU::Selector::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::MAU::Selector::operator==(IR::Node const & a) const { return a == *this; }
#line 705 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::TernaryIndirect::TernaryIndirect(cstring tbl_name) { name = IR::ID(tbl_name + "$tind"_cs); direct = true; }
#line 16557 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 706 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
char const * IR::MAU::TernaryIndirect::kind() const { return "indirect"; }
#line 16560 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 707 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
UniqueAttachedId::type_t IR::MAU::TernaryIndirect::get_id_type() const {
        return UniqueAttachedId::TERNARY_INDIRECT;
    }
#line 16565 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 710 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::TernaryIndirect::externType() const { return "ternary indirect"_cs; }
#line 16568 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::TernaryIndirect::operator==(IR::MAU::TernaryIndirect const & a) const {
        return AttachedMemory::operator==(static_cast<const AttachedMemory &>(a));
    }
bool IR::MAU::TernaryIndirect::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!AttachedMemory::equiv(a_)) return false;
        return true;
    }
void IR::MAU::TernaryIndirect::toJSON(JSONGenerator & json) const {
    AttachedMemory::toJSON(json);
}
IR::MAU::TernaryIndirect::TernaryIndirect(JSONLoader & json) : AttachedMemory(json) {
}
IR::Node * IR::MAU::TernaryIndirect::fromJSON(JSONLoader & json) { return new TernaryIndirect(json); }
bool IR::MAU::TernaryIndirect::operator==(IR::MAU::AttachedMemory const & a) const { return a == *this; }
bool IR::MAU::TernaryIndirect::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::MAU::TernaryIndirect::operator==(IR::Node const & a) const { return a == *this; }
#line 716 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
char const * IR::MAU::ActionData::kind() const { return "action"; }
#line 16588 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 717 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::ActionData::indexed() const { return !direct; }
#line 16591 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 718 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
UniqueAttachedId::type_t IR::MAU::ActionData::get_id_type() const {
        return UniqueAttachedId::ACTION_DATA;
    }
#line 16596 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 721 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::ActionData::externType() const { return "ActionProfile"_cs; }
#line 16599 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::ActionData::operator==(IR::MAU::ActionData const & a) const {
        return AttachedMemory::operator==(static_cast<const AttachedMemory &>(a));
    }
bool IR::MAU::ActionData::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!AttachedMemory::equiv(a_)) return false;
        return true;
    }
void IR::MAU::ActionData::toJSON(JSONGenerator & json) const {
    AttachedMemory::toJSON(json);
}
IR::MAU::ActionData::ActionData(JSONLoader & json) : AttachedMemory(json) {
}
IR::Node * IR::MAU::ActionData::fromJSON(JSONLoader & json) { return new ActionData(json); }
bool IR::MAU::ActionData::operator==(IR::MAU::AttachedMemory const & a) const { return a == *this; }
bool IR::MAU::ActionData::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::MAU::ActionData::operator==(IR::Node const & a) const { return a == *this; }
#line 734 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::IdleTime::IdleTime(cstring tbl_name) { name = IR::ID(tbl_name + "$idletime"_cs); direct = true; }
#line 16619 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 735 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
char const * IR::MAU::IdleTime::kind() const { return "idletime"; }
#line 16622 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 736 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
UniqueAttachedId::type_t IR::MAU::IdleTime::get_id_type() const {
        return UniqueAttachedId::IDLETIME;
    }
#line 16627 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 739 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::IdleTime::externType() const { return "idletime"_cs; }
#line 16630 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::IdleTime::operator==(IR::MAU::IdleTime const & a) const {
        return AttachedMemory::operator==(static_cast<const AttachedMemory &>(a))
        && precision == a.precision
        && interval == a.interval
        && per_flow_idletime == a.per_flow_idletime;
    }
bool IR::MAU::IdleTime::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!AttachedMemory::equiv(a_)) return false;
        auto &a = static_cast<const IdleTime &>(a_);
        return precision == a.precision
        && interval == a.interval
        && per_flow_idletime == a.per_flow_idletime;
    }
void IR::MAU::IdleTime::dump_fields(std::ostream & out) const {
    AttachedMemory::dump_fields(out);
        out << " precision=" << precision;
        out << " interval=" << interval;
        out << " per_flow_idletime=" << per_flow_idletime;
}
void IR::MAU::IdleTime::toJSON(JSONGenerator & json) const {
    AttachedMemory::toJSON(json);
    json.emit("precision", precision);
    json.emit("interval", interval);
    json.emit("per_flow_idletime", per_flow_idletime);
}
IR::MAU::IdleTime::IdleTime(JSONLoader & json) : AttachedMemory(json) {
    json.load("precision", precision);
    json.load("interval", interval);
    json.load("per_flow_idletime", per_flow_idletime);
}
IR::Node * IR::MAU::IdleTime::fromJSON(JSONLoader & json) { return new IdleTime(json); }
bool IR::MAU::IdleTime::operator==(IR::MAU::AttachedMemory const & a) const { return a == *this; }
bool IR::MAU::IdleTime::operator==(IR::Attached const & a) const { return a == *this; }
bool IR::MAU::IdleTime::operator==(IR::Node const & a) const { return a == *this; }

#line 751 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::TableSeq::TableSeq() { seq_uid = ++uid_ctr; }
#line 16669 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 752 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::TableSeq::TableSeq(const IR::MAU::Table* a) {
        seq_uid = ++uid_ctr;
        if (a) tables.push_back(a); }
#line 16674 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 755 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::TableSeq::TableSeq(const IR::MAU::Table* a, const IR::MAU::Table* b) {
        seq_uid = ++uid_ctr;
        if (a) tables.push_back(a);
        if (b) tables.push_back(b); }
#line 16680 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 759 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::TableSeq::TableSeq(const IR::MAU::TableSeq* a, const IR::MAU::Table* b) {
        seq_uid = ++uid_ctr;
        if (a) tables.insert(tables.end(), a->tables.begin(), a->tables.end());
        if (b) tables.push_back(b); }
#line 16686 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 763 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::TableSeq::empty() const { return tables.empty(); }
#line 16689 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 764 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
const IR::MAU::Table *IR::MAU::TableSeq::front() const { return tables.empty() ? nullptr : tables.front(); }
#line 16692 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 765 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
const IR::MAU::Table *IR::MAU::TableSeq::back() const { return tables.empty() ? nullptr : tables.back(); }
#line 16695 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 766 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
size_t IR::MAU::TableSeq::size() const { return tables.size(); }
#line 16698 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 767 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::TableSeq::validate() const {
#line 16701 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        tables.validate();
#line 767 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
{
        bool first = true;
        gress_t gress;
        for (auto t : tables) {
            if (first) { gress = t->gress; first = false; }
            else if (t->gress != gress) BUG("TableSeq consists of tables of different gress?"); }
    } }
#line 16712 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"


#line 777 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::TableSeq::operator==(IR::MAU::TableSeq const & a) const { return tables == a.tables && deps == a.deps; }
#line 16717 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 778 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::TableSeq::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const TableSeq &>(a_);

#line 778 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
{ return tables.equiv(a.tables) && deps == a.deps; }    }
#line 16726 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(MAU::TableSeq, , )
void IR::MAU::TableSeq::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    tables.visit_children(v, "tables");
}
void IR::MAU::TableSeq::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    tables.visit_children(v, "tables");
}
void IR::MAU::TableSeq::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " deps=" << deps;
        out << " seq_uid=" << seq_uid;
}
void IR::MAU::TableSeq::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("tables", tables);
    json.emit("deps", deps);
    json.emit("seq_uid", seq_uid);
}
IR::MAU::TableSeq::TableSeq(JSONLoader & json) : Node(json) {
    json.load("tables", tables);
    json.load("deps", deps);
    json.load("seq_uid", seq_uid);
}
IR::Node * IR::MAU::TableSeq::fromJSON(JSONLoader & json) { return new TableSeq(json); }
bool IR::MAU::TableSeq::operator==(IR::Node const & a) const { return a == *this; }
IRNODE_DEFINE_APPLY_OVERLOAD(MAU::Primitive, , )
bool IR::MAU::Primitive::operator==(IR::MAU::Primitive const & a) const {
        return IR::Primitive::operator==(static_cast<const IR::Primitive &>(a))
        && in_hash == a.in_hash;
    }
bool IR::MAU::Primitive::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!IR::Primitive::equiv(a_)) return false;
        auto &a = static_cast<const Primitive &>(a_);
        return in_hash == a.in_hash;
    }
void IR::MAU::Primitive::dump_fields(std::ostream & out) const {
    IR::Primitive::dump_fields(out);
        out << " in_hash=" << in_hash;
}
void IR::MAU::Primitive::toJSON(JSONGenerator & json) const {
    IR::Primitive::toJSON(json);
    json.emit("in_hash", in_hash);
}
IR::MAU::Primitive::Primitive(JSONLoader & json) : IR::Primitive(json) {
    json.load("in_hash", in_hash);
}
IR::Node * IR::MAU::Primitive::fromJSON(JSONLoader & json) { return new Primitive(json); }
bool IR::MAU::Primitive::operator==(IR::Primitive const & a) const { return a == *this; }
bool IR::MAU::Primitive::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::MAU::Primitive::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::Primitive::operator==(IR::Node const & a) const { return a == *this; }
#line 798 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::TypedPrimitive::TypedPrimitive(cstring name) : Primitive(name), method_type(nullptr) { }
#line 16785 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 800 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::TypedPrimitive::TypedPrimitive(Util::SourceInfo srcInfo, IR::Type const * return_type, IR::Type const * m_type, cstring name) : Primitive(srcInfo, return_type, name), method_type(m_type) { }
#line 16788 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 803 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::TypedPrimitive::visit_children(Visitor & v, char const * n) { Primitive::visit_children(v, n); }
#line 803 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::TypedPrimitive::visit_children(Visitor & v, char const * n) const { Primitive::visit_children(v, n); }
#line 16793 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::TypedPrimitive::operator==(IR::MAU::TypedPrimitive const & a) const {
        return Primitive::operator==(static_cast<const Primitive &>(a))
        && method_type == a.method_type
        && op_names == a.op_names;
    }
bool IR::MAU::TypedPrimitive::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Primitive::equiv(a_)) return false;
        auto &a = static_cast<const TypedPrimitive &>(a_);
        return (method_type ? a.method_type ? method_type->equiv(*a.method_type) : false : a.method_type == nullptr)
        && op_names == a.op_names;
    }
void IR::MAU::TypedPrimitive::validate() const {
#line 16807 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(method_type); }
void IR::MAU::TypedPrimitive::toJSON(JSONGenerator & json) const {
    Primitive::toJSON(json);
    json.emit("method_type", method_type);
    json.emit("op_names", op_names);
}
IR::MAU::TypedPrimitive::TypedPrimitive(JSONLoader & json) : Primitive(json) {
    json.load("method_type", method_type);
    json.load("op_names", op_names);
}
IR::Node * IR::MAU::TypedPrimitive::fromJSON(JSONLoader & json) { return new TypedPrimitive(json); }
bool IR::MAU::TypedPrimitive::operator==(IR::MAU::Primitive const & a) const { return a == *this; }
bool IR::MAU::TypedPrimitive::operator==(IR::Primitive const & a) const { return a == *this; }
bool IR::MAU::TypedPrimitive::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::MAU::TypedPrimitive::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::TypedPrimitive::operator==(IR::Node const & a) const { return a == *this; }



#line 820 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::Instruction::Instruction(IR::MAU::Primitive const & p) : Primitive(p) {}
#line 16830 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 821 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::Instruction::Instruction(cstring op, std::vector<const IR::Expression *> const & args) : Primitive(op) {
        for (auto a : args) operands.push_back(a); }
#line 16834 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 823 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::Instruction::Instruction(Util::SourceInfo si, cstring op, std::vector<const IR::Expression *> const & args) : Primitive(si, op) { for (auto a : args) operands.push_back(a); }
#line 16837 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 825 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Instruction::isOutput(int operand_index) const { return operand_index == 0; }
#line 16840 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 826 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
const IR::Expression *IR::MAU::Instruction::getOutput() const { return operands.empty() ? nullptr : operands.at(0); }
#line 16843 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 827 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::Instruction::validate() const {
#line 16846 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 827 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
{ BUG_CHECK(name, "empty name in instruction"); } }
#line 16850 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::Instruction::operator==(IR::MAU::Instruction const & a) const {
        return Primitive::operator==(static_cast<const Primitive &>(a))
        && copy_propagated == a.copy_propagated;
    }
bool IR::MAU::Instruction::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Primitive::equiv(a_)) return false;
        auto &a = static_cast<const Instruction &>(a_);
        return copy_propagated == a.copy_propagated;
    }
void IR::MAU::Instruction::dump_fields(std::ostream & out) const {
    Primitive::dump_fields(out);
        out << " copy_propagated=" << copy_propagated;
}
void IR::MAU::Instruction::toJSON(JSONGenerator & json) const {
    Primitive::toJSON(json);
    json.emit("copy_propagated", copy_propagated);
}
IR::MAU::Instruction::Instruction(JSONLoader & json) : Primitive(json) {
    json.load("copy_propagated", copy_propagated);
}
IR::Node * IR::MAU::Instruction::fromJSON(JSONLoader & json) { return new Instruction(json); }
bool IR::MAU::Instruction::operator==(IR::MAU::Primitive const & a) const { return a == *this; }
bool IR::MAU::Instruction::operator==(IR::Primitive const & a) const { return a == *this; }
bool IR::MAU::Instruction::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::MAU::Instruction::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::Instruction::operator==(IR::Node const & a) const { return a == *this; }




#line 839 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::SaluInstruction::SaluInstruction(cstring op) : Instruction(op), output_operand(-1) {}
#line 16884 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 840 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::SaluInstruction::SaluInstruction(cstring op, int out, std::vector<const IR::Expression *> const & args) : Instruction(op, args), output_operand(out) {}
#line 16887 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 842 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::SaluInstruction::SaluInstruction(cstring op, int out, const IR::Vector<IR::Expression>* args) : Instruction(op, args), output_operand(out) {}
#line 16890 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 844 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::SaluInstruction::SaluInstruction(Util::SourceInfo si, cstring op, int out, std::vector<const IR::Expression *> const & args) : Instruction(si, op, args), output_operand(out) {}
#line 16893 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 846 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::SaluInstruction::SaluInstruction(Util::SourceInfo si, cstring op, int out, const IR::Vector<IR::Expression>* args) : Instruction(si, op, args), output_operand(out) {}
#line 16896 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 848 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::SaluInstruction::SaluInstruction(cstring op, int out, const IR::Expression* a1) : Instruction(op, a1), output_operand(out) {}
#line 16899 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 850 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::SaluInstruction::SaluInstruction(cstring op, int out, const IR::Expression* a1, const IR::Expression* a2) : Instruction(op, a1, a2), output_operand(out) {}
#line 16902 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 852 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::SaluInstruction::SaluInstruction(cstring op, int out, const IR::Expression* a1, const IR::Expression* a2, const IR::Expression* a3) : Instruction(op, a1, a2, a3), output_operand(out) {}
#line 16905 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 855 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::SaluInstruction::isOutput(int operand_index) const { return operand_index == output_operand; }
#line 16908 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 856 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
const IR::Expression *IR::MAU::SaluInstruction::getOutput() const {
        return output_operand >= 0.? operands.at(output_operand) : nullptr; }
#line 16912 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::SaluInstruction::operator==(IR::MAU::SaluInstruction const & a) const {
        return Instruction::operator==(static_cast<const Instruction &>(a))
        && output_operand == a.output_operand;
    }
bool IR::MAU::SaluInstruction::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Instruction::equiv(a_)) return false;
        auto &a = static_cast<const SaluInstruction &>(a_);
        return output_operand == a.output_operand;
    }
void IR::MAU::SaluInstruction::dump_fields(std::ostream & out) const {
    Instruction::dump_fields(out);
        out << " output_operand=" << output_operand;
}
void IR::MAU::SaluInstruction::toJSON(JSONGenerator & json) const {
    Instruction::toJSON(json);
    json.emit("output_operand", output_operand);
}
IR::MAU::SaluInstruction::SaluInstruction(JSONLoader & json) : Instruction(json) {
    json.load("output_operand", output_operand);
}
IR::Node * IR::MAU::SaluInstruction::fromJSON(JSONLoader & json) { return new SaluInstruction(json); }
bool IR::MAU::SaluInstruction::operator==(IR::MAU::Instruction const & a) const { return a == *this; }
bool IR::MAU::SaluInstruction::operator==(IR::MAU::Primitive const & a) const { return a == *this; }
bool IR::MAU::SaluInstruction::operator==(IR::Primitive const & a) const { return a == *this; }
bool IR::MAU::SaluInstruction::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::MAU::SaluInstruction::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::SaluInstruction::operator==(IR::Node const & a) const { return a == *this; }
#line 866 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::Vector<IR::Annotation> const & IR::MAU::ActionArg::getAnnotations() const { return annotations; }
#line 16943 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 867 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::Vector<IR::Annotation> & IR::MAU::ActionArg::getAnnotations() { return annotations; }
#line 16946 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 869 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::ActionArg::dbprint(std::ostream & out) const { out << action_name << ':' << name; }
#line 16949 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 870 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::ActionArg::toString() const { return name.toString(); }
#line 16952 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::ActionArg::operator==(IR::MAU::ActionArg const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && action_name == a.action_name
        && name == a.name
        && annotations == a.annotations;
    }
bool IR::MAU::ActionArg::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const ActionArg &>(a_);
        return action_name == a.action_name
        && name == a.name
        && annotations.equiv(a.annotations);
    }
void IR::MAU::ActionArg::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    annotations.visit_children(v, "annotations");
}
void IR::MAU::ActionArg::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    annotations.visit_children(v, "annotations");
}
void IR::MAU::ActionArg::validate() const {
#line 16978 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        annotations.validate(); }
void IR::MAU::ActionArg::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " action_name=" << action_name;
        out << " name=" << name;
}
void IR::MAU::ActionArg::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("action_name", action_name);
    json.emit("name", name);
    json.emit("annotations", annotations);
}
IR::MAU::ActionArg::ActionArg(JSONLoader & json) : Expression(json) {
    json.load("action_name", action_name);
    json.load("name", name);
    json.load("annotations", annotations);
}
IR::Node * IR::MAU::ActionArg::fromJSON(JSONLoader & json) { return new ActionArg(json); }
bool IR::MAU::ActionArg::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::ActionArg::operator==(IR::Node const & a) const { return a == *this; }
#line 875 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::ConditionalArg::dbprint(std::ostream & out) const { ActionArg::dbprint(out);  out << " = orig " << orig_arg; }
#line 17002 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::ConditionalArg::operator==(IR::MAU::ConditionalArg const & a) const {
        return ActionArg::operator==(static_cast<const ActionArg &>(a))
        && orig_arg == a.orig_arg
        && one_on_true == a.one_on_true;
    }
bool IR::MAU::ConditionalArg::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ActionArg::equiv(a_)) return false;
        auto &a = static_cast<const ConditionalArg &>(a_);
        return (orig_arg ? a.orig_arg ? orig_arg->equiv(*a.orig_arg) : false : a.orig_arg == nullptr)
        && one_on_true == a.one_on_true;
    }
void IR::MAU::ConditionalArg::visit_children(Visitor & v, char const * n) {
    (void)n;
    ActionArg::visit_children(v, n);
    v.visit(orig_arg, "orig_arg");
}
void IR::MAU::ConditionalArg::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ActionArg::visit_children(v, n);
    v.visit(orig_arg, "orig_arg");
}
void IR::MAU::ConditionalArg::validate() const {
#line 17026 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(orig_arg); }
void IR::MAU::ConditionalArg::dump_fields(std::ostream & out) const {
    ActionArg::dump_fields(out);
        out << " one_on_true=" << one_on_true;
}
void IR::MAU::ConditionalArg::toJSON(JSONGenerator & json) const {
    ActionArg::toJSON(json);
    json.emit("orig_arg", orig_arg);
    json.emit("one_on_true", one_on_true);
}
IR::MAU::ConditionalArg::ConditionalArg(JSONLoader & json) : ActionArg(json) {
    json.load("orig_arg", orig_arg);
    json.load("one_on_true", one_on_true);
}
IR::Node * IR::MAU::ConditionalArg::fromJSON(JSONLoader & json) { return new ConditionalArg(json); }
bool IR::MAU::ConditionalArg::operator==(IR::MAU::ActionArg const & a) const { return a == *this; }
bool IR::MAU::ConditionalArg::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::ConditionalArg::operator==(IR::Node const & a) const { return a == *this; }
#line 881 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::StatefulCounter::dbprint(std::ostream & out) const { out << "stful_cnt(" << type << ")"; }
#line 17048 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::StatefulCounter::operator==(IR::MAU::StatefulCounter const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && attached == a.attached;
    }
bool IR::MAU::StatefulCounter::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const StatefulCounter &>(a_);
        return (attached ? a.attached ? attached->equiv(*a.attached) : false : a.attached == nullptr);
    }
void IR::MAU::StatefulCounter::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(attached, "attached");
}
void IR::MAU::StatefulCounter::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(attached, "attached");
}
void IR::MAU::StatefulCounter::validate() const {
#line 17070 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(attached); }
void IR::MAU::StatefulCounter::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("attached", attached);
}
IR::MAU::StatefulCounter::StatefulCounter(JSONLoader & json) : Expression(json) {
    json.load("attached", attached);
}
IR::Node * IR::MAU::StatefulCounter::fromJSON(JSONLoader & json) { return new StatefulCounter(json); }
bool IR::MAU::StatefulCounter::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::StatefulCounter::operator==(IR::Node const & a) const { return a == *this; }


bool IR::MAU::StatefulCall::operator==(IR::MAU::StatefulCall const & a) const {
        return this->typeId() == a.typeId()
        && prim == a.prim
        && attached_callee == a.attached_callee
        && index == a.index;
    }
bool IR::MAU::StatefulCall::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const StatefulCall &>(a_);
        return (prim ? a.prim ? prim->equiv(*a.prim) : false : a.prim == nullptr)
        && (attached_callee ? a.attached_callee ? attached_callee->equiv(*a.attached_callee) : false : a.attached_callee == nullptr)
        && (index ? a.index ? index->equiv(*a.index) : false : a.index == nullptr);
    }
void IR::MAU::StatefulCall::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(prim, "prim");
    v.visit(attached_callee, "attached_callee");
    v.visit(index, "index");
}
void IR::MAU::StatefulCall::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(prim, "prim");
    v.visit(attached_callee, "attached_callee");
    v.visit(index, "index");
}
void IR::MAU::StatefulCall::validate() const {
#line 17114 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(attached_callee); }
void IR::MAU::StatefulCall::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    if (prim != nullptr) json.emit("prim", prim);
    json.emit("attached_callee", attached_callee);
    if (index != nullptr) json.emit("index", index);
}
IR::MAU::StatefulCall::StatefulCall(JSONLoader & json) : Node(json) {
    json.load("prim", prim);
    json.load("attached_callee", attached_callee);
    json.load("index", index);
}
IR::Node * IR::MAU::StatefulCall::fromJSON(JSONLoader & json) { return new StatefulCall(json); }
bool IR::MAU::StatefulCall::operator==(IR::Node const & a) const { return a == *this; }
bool IR::MAU::VLIWInstruction::operator==(IR::MAU::VLIWInstruction const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::MAU::VLIWInstruction::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::MAU::VLIWInstruction::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::MAU::VLIWInstruction::VLIWInstruction(JSONLoader & json) : Node(json) {
}
bool IR::MAU::VLIWInstruction::operator==(IR::Node const & a) const { return a == *this; }










#line 928 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::StatefulCall const * IR::MAU::Action::stateful_call(cstring name) const {
        for (auto sc : stateful_calls)
            if (sc->attached_callee->name == name)
                return sc;
        return nullptr; }
#line 17160 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 935 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Action::hit_only() const { return !default_allowed; }
#line 17163 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 936 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::Action::miss_only() const { return !hit_allowed; }
#line 17166 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 938 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::Vector<IR::Annotation> const & IR::MAU::Action::getAnnotations() const { return annotations; }
#line 17169 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 939 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::Vector<IR::Annotation> & IR::MAU::Action::getAnnotations() { return annotations; }
#line 17172 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 941 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::Action::toString() const { return name.toString(); }
#line 17175 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 942 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::Action::validate() const {
#line 17178 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        action.validate();
        annotations.validate();
        default_params.validate();
        stateful_calls.validate();
#line 942 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
{
        ERROR_CHECK(!(hit_only() && miss_only()), "Action %s cannot be both a hit only and a "
                    "miss only action.  Can only be at most one of those categories", name);
    } }
#line 17189 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 947 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::Action::visit_children(Visitor & v, char const * n) {
        VLIWInstruction::visit_children(v, n);
        if (parallel)
            action.parallel_visit_children(v, n);
        else
            action.visit_children(v, n);
        v.visit(annotations, "annotations");
        default_params.visit_children(v, n);
        stateful_calls.visit_children(v, n);
    }
#line 947 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::Action::visit_children(Visitor & v, char const * n) const {
        VLIWInstruction::visit_children(v, n);
        if (parallel)
            action.parallel_visit_children(v, n);
        else
            action.visit_children(v, n);
        v.visit(annotations, "annotations");
        default_params.visit_children(v, n);
        stateful_calls.visit_children(v, n);
    }
#line 17212 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(MAU::Action, , )
bool IR::MAU::Action::operator==(IR::MAU::Action const & a) const {
        return VLIWInstruction::operator==(static_cast<const VLIWInstruction &>(a))
        && name == a.name
        && handle == a.handle
        && action == a.action
        && args == a.args
        && annotations == a.annotations
        && init_default == a.init_default
        && hit_allowed == a.hit_allowed
        && default_allowed == a.default_allowed
        && is_constant_action == a.is_constant_action
        && hit_path_imp_only == a.hit_path_imp_only
        && default_params == a.default_params
        && exitAction == a.exitAction
        && parallel == a.parallel
        && per_flow_enables == a.per_flow_enables
        && meter_types == a.meter_types
        && stateful_calls == a.stateful_calls;
    }
bool IR::MAU::Action::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!VLIWInstruction::equiv(a_)) return false;
        auto &a = static_cast<const Action &>(a_);
        return name == a.name
        && handle == a.handle
        && action.equiv(a.action)
        && args == a.args
        && annotations.equiv(a.annotations)
        && init_default == a.init_default
        && hit_allowed == a.hit_allowed
        && default_allowed == a.default_allowed
        && is_constant_action == a.is_constant_action
        && hit_path_imp_only == a.hit_path_imp_only
        && default_params.equiv(a.default_params)
        && exitAction == a.exitAction
        && parallel == a.parallel
        && per_flow_enables == a.per_flow_enables
        && meter_types == a.meter_types
        && stateful_calls.equiv(a.stateful_calls);
    }
void IR::MAU::Action::dump_fields(std::ostream & out) const {
    VLIWInstruction::dump_fields(out);
        out << " name=" << name;
        out << " handle=" << handle;
        out << " init_default=" << init_default;
        out << " hit_allowed=" << hit_allowed;
        out << " default_allowed=" << default_allowed;
        out << " is_constant_action=" << is_constant_action;
        out << " hit_path_imp_only=" << hit_path_imp_only;
        out << " exitAction=" << exitAction;
        out << " parallel=" << parallel;
}
void IR::MAU::Action::toJSON(JSONGenerator & json) const {
    VLIWInstruction::toJSON(json);
    json.emit("name", name);
    json.emit("handle", handle);
    json.emit("action", action);
    json.emit("args", args);
    json.emit("annotations", annotations);
    json.emit("init_default", init_default);
    json.emit("hit_allowed", hit_allowed);
    json.emit("default_allowed", default_allowed);
    json.emit("is_constant_action", is_constant_action);
    json.emit("hit_path_imp_only", hit_path_imp_only);
    json.emit("default_params", default_params);
    json.emit("exitAction", exitAction);
    json.emit("parallel", parallel);
    json.emit("per_flow_enables", per_flow_enables);
    json.emit("meter_types", meter_types);
    json.emit("stateful_calls", stateful_calls);
}
IR::MAU::Action::Action(JSONLoader & json) : VLIWInstruction(json) {
    json.load("name", name);
    json.load("handle", handle);
    json.load("action", action);
    json.load("args", args);
    json.load("annotations", annotations);
    json.load("init_default", init_default);
    json.load("hit_allowed", hit_allowed);
    json.load("default_allowed", default_allowed);
    json.load("is_constant_action", is_constant_action);
    json.load("hit_path_imp_only", hit_path_imp_only);
    json.load("default_params", default_params);
    json.load("exitAction", exitAction);
    json.load("parallel", parallel);
    json.load("per_flow_enables", per_flow_enables);
    json.load("meter_types", meter_types);
    json.load("stateful_calls", stateful_calls);
}
IR::Node * IR::MAU::Action::fromJSON(JSONLoader & json) { return new Action(json); }
bool IR::MAU::Action::operator==(IR::MAU::VLIWInstruction const & a) const { return a == *this; }
bool IR::MAU::Action::operator==(IR::Node const & a) const { return a == *this; }
#line 969 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::Vector<IR::Annotation> const & IR::MAU::SaluAction::getAnnotations() const { return annotations; }
#line 17308 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 970 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::Vector<IR::Annotation> & IR::MAU::SaluAction::getAnnotations() { return annotations; }
#line 17311 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"



















bool IR::MAU::SaluAction::ReturnEnumEncoding::operator==(IR::MAU::SaluAction::ReturnEnumEncoding const & a) const {
        return return_type == a.return_type
        && tag_used == a.tag_used
        && cmp_used == a.cmp_used;
    }
void IR::MAU::SaluAction::ReturnEnumEncoding::toJSON(JSONGenerator & json) const {
    json.emit("return_type", return_type);
    json.emit("tag_used", tag_used);
    json.emit("cmp_used", cmp_used);
}
IR::MAU::SaluAction::ReturnEnumEncoding::ReturnEnumEncoding(JSONLoader & json)  {
    json.load("return_type", return_type);
    json.load("tag_used", tag_used);
    json.load("cmp_used", cmp_used);
}
IR::MAU::SaluAction::ReturnEnumEncoding * IR::MAU::SaluAction::ReturnEnumEncoding::fromJSON(JSONLoader & json) { return new ReturnEnumEncoding(json); }





#line 1006 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::SaluAction::visit_children(Visitor & v, char const * n) {
        VLIWInstruction::visit_children(v, n);
        action.visit_children(v, n);
        v.visit(annotations, "annotations");
        v.visit(output_dst, "output_dst");
    }
#line 1006 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::SaluAction::visit_children(Visitor & v, char const * n) const {
        VLIWInstruction::visit_children(v, n);
        action.visit_children(v, n);
        v.visit(annotations, "annotations");
        v.visit(output_dst, "output_dst");
    }
#line 17366 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1013 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::SaluAction::toString() const {
        return cstring("action ") + name + " {\n" +
                 cstring::join(action.begin(), action.end(), ";\n") +
                 " }"; }
#line 17372 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::SaluAction::operator==(IR::MAU::SaluAction const & a) const {
        return VLIWInstruction::operator==(static_cast<const VLIWInstruction &>(a))
        && name == a.name
        && inst_code == a.inst_code
        && action == a.action
        && args == a.args
        && annotations == a.annotations
        && output_dst == a.output_dst
        && learn_action == a.learn_action
        && p4func == a.p4func
        && return_encoding == a.return_encoding
        && return_predicate_words == a.return_predicate_words
        && output_param_to_alu == a.output_param_to_alu;
    }
bool IR::MAU::SaluAction::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!VLIWInstruction::equiv(a_)) return false;
        auto &a = static_cast<const SaluAction &>(a_);
        return name == a.name
        && inst_code == a.inst_code
        && action.equiv(a.action)
        && args == a.args
        && annotations.equiv(a.annotations)
        && (output_dst ? a.output_dst ? output_dst->equiv(*a.output_dst) : false : a.output_dst == nullptr)
        && learn_action == a.learn_action
        && (p4func ? a.p4func ? p4func->equiv(*a.p4func) : false : a.p4func == nullptr)
        && return_encoding == a.return_encoding
        && return_predicate_words == a.return_predicate_words
        && output_param_to_alu == a.output_param_to_alu;
    }
void IR::MAU::SaluAction::validate() const {
#line 17404 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        action.validate();
        annotations.validate(); }
void IR::MAU::SaluAction::dump_fields(std::ostream & out) const {
    VLIWInstruction::dump_fields(out);
        out << " name=" << name;
        out << " inst_code=" << inst_code;
        out << " learn_action=" << learn_action;
        out << " return_encoding=" << return_encoding;
        out << " return_predicate_words=" << return_predicate_words;
}
void IR::MAU::SaluAction::toJSON(JSONGenerator & json) const {
    VLIWInstruction::toJSON(json);
    json.emit("name", name);
    json.emit("inst_code", inst_code);
    json.emit("action", action);
    json.emit("args", args);
    json.emit("annotations", annotations);
    if (output_dst != nullptr) json.emit("output_dst", output_dst);
    json.emit("learn_action", learn_action);
    if (p4func != nullptr) json.emit("p4func", p4func);
    json.emit("return_encoding", return_encoding);
    json.emit("return_predicate_words", return_predicate_words);
    json.emit("output_param_to_alu", output_param_to_alu);
}
IR::MAU::SaluAction::SaluAction(JSONLoader & json) : VLIWInstruction(json) {
    json.load("name", name);
    json.load("inst_code", inst_code);
    json.load("action", action);
    json.load("args", args);
    json.load("annotations", annotations);
    json.load("output_dst", output_dst);
    json.load("learn_action", learn_action);
    json.load("p4func", p4func);
    json.load("return_encoding", return_encoding);
    json.load("return_predicate_words", return_predicate_words);
    json.load("output_param_to_alu", output_param_to_alu);
}
IR::Node * IR::MAU::SaluAction::fromJSON(JSONLoader & json) { return new SaluAction(json); }
bool IR::MAU::SaluAction::operator==(IR::MAU::VLIWInstruction const & a) const { return a == *this; }
bool IR::MAU::SaluAction::operator==(IR::Node const & a) const { return a == *this; }
#line 1026 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::FieldListExpression::dbprint(std::ostream & out) const { out << id << " : ";  ListExpression::dbprint(out); }
#line 17448 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::FieldListExpression::operator==(IR::MAU::FieldListExpression const & a) const {
        return ListExpression::operator==(static_cast<const ListExpression &>(a))
        && id == a.id
        && rotateable == a.rotateable
        && permutable == a.permutable
        && symmetric_keys == a.symmetric_keys;
    }
bool IR::MAU::FieldListExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ListExpression::equiv(a_)) return false;
        auto &a = static_cast<const FieldListExpression &>(a_);
        return id == a.id
        && rotateable == a.rotateable
        && permutable == a.permutable
        && symmetric_keys == a.symmetric_keys;
    }
void IR::MAU::FieldListExpression::dump_fields(std::ostream & out) const {
    ListExpression::dump_fields(out);
        out << " id=" << id;
        out << " rotateable=" << rotateable;
        out << " permutable=" << permutable;
        out << " symmetric_keys=" << symmetric_keys;
}
void IR::MAU::FieldListExpression::toJSON(JSONGenerator & json) const {
    ListExpression::toJSON(json);
    json.emit("id", id);
    json.emit("rotateable", rotateable);
    json.emit("permutable", permutable);
    json.emit("symmetric_keys", symmetric_keys);
}
IR::MAU::FieldListExpression::FieldListExpression(JSONLoader & json) : ListExpression(json) {
    json.load("id", id);
    json.load("rotateable", rotateable);
    json.load("permutable", permutable);
    json.load("symmetric_keys", symmetric_keys);
}
IR::Node * IR::MAU::FieldListExpression::fromJSON(JSONLoader & json) { return new FieldListExpression(json); }
bool IR::MAU::FieldListExpression::operator==(IR::ListExpression const & a) const { return a == *this; }
bool IR::MAU::FieldListExpression::operator==(IR::BaseListExpression const & a) const { return a == *this; }
bool IR::MAU::FieldListExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::FieldListExpression::operator==(IR::Node const & a) const { return a == *this; }

#line 1045 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::HashGenExpression::HashGenExpression(Util::SourceInfo si, IR::Type const * type, const IR::Expression* e, HashFunction a) : Expression(si, type), expr(e), id("$hash" + std::to_string(nextId++)), algorithm(a) {}
#line 17493 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1047 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::HashGenExpression::HashGenExpression(const IR::Expression* e, HashFunction a) : HashGenExpression(e->srcInfo, e->type, e, a) {}
#line 17496 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1050 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::HashGenExpression::dbprint(std::ostream & out) const { out << id << "(" << expr << ")"; }
#line 17499 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1052 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::HashGenExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const HashGenExpression &>(a_);

#line 1052 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
{
        bool rv = expr->equiv(*a.expr) && dynamic == a.dynamic &&
                  algorithm == a.algorithm && hash_output_width == a.hash_output_width;
        rv &= alg_names
                  ?  (a.alg_names ? alg_names->equiv(*a.alg_names) : false)
                  : a.alg_names == nullptr;
        rv &= dynamic && a.dynamic ? id == a.id : true;
        return rv;
    }    }
#line 17516 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/**
     * For static hashes, each hash call is a unique ID, in order potentially for later
     * comparisons and tracking.  With each statc HashGeneExpression created, the nextId
     * increases
     */
bool IR::MAU::HashGenExpression::operator==(IR::MAU::HashGenExpression const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && expr == a.expr
        && id == a.id
        && dynamic == a.dynamic
        && algorithm == a.algorithm
        && any_alg_allowed == a.any_alg_allowed
        && alg_names == a.alg_names
        && hash_output_width == a.hash_output_width;
    }
void IR::MAU::HashGenExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(expr, "expr");
    v.visit(alg_names, "alg_names");
}
void IR::MAU::HashGenExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(expr, "expr");
    v.visit(alg_names, "alg_names");
}
void IR::MAU::HashGenExpression::validate() const {
#line 17545 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(expr); }
void IR::MAU::HashGenExpression::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " id=" << id;
        out << " dynamic=" << dynamic;
        out << " algorithm=" << algorithm;
        out << " any_alg_allowed=" << any_alg_allowed;
        out << " hash_output_width=" << hash_output_width;
}
void IR::MAU::HashGenExpression::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("expr", expr);
    json.emit("id", id);
    json.emit("dynamic", dynamic);
    json.emit("algorithm", algorithm);
    json.emit("any_alg_allowed", any_alg_allowed);
    if (alg_names != nullptr) json.emit("alg_names", alg_names);
    json.emit("hash_output_width", hash_output_width);
}
IR::MAU::HashGenExpression::HashGenExpression(JSONLoader & json) : Expression(json) {
    json.load("expr", expr);
    json.load("id", id);
    json.load("dynamic", dynamic);
    json.load("algorithm", algorithm);
    json.load("any_alg_allowed", any_alg_allowed);
    json.load("alg_names", alg_names);
    json.load("hash_output_width", hash_output_width);
}
IR::Node * IR::MAU::HashGenExpression::fromJSON(JSONLoader & json) { return new HashGenExpression(json); }
bool IR::MAU::HashGenExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::HashGenExpression::operator==(IR::Node const & a) const { return a == *this; }

#line 1079 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::HashDist::allocated() const { return !units.empty(); }
#line 17581 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1080 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::HashDist::dbprint(std::ostream & out) const {
        out << "hash_dist(" << expr << ")";
        if (allocated())
            out << units;
    }
#line 17588 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::HashDist::operator==(IR::MAU::HashDist const & a) const {
        return Operation_Unary::operator==(static_cast<const Operation_Unary &>(a))
        && units == a.units;
    }
bool IR::MAU::HashDist::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Unary::equiv(a_)) return false;
        auto &a = static_cast<const HashDist &>(a_);
        return units == a.units;
    }
void IR::MAU::HashDist::toJSON(JSONGenerator & json) const {
    Operation_Unary::toJSON(json);
    json.emit("units", units);
}
IR::MAU::HashDist::HashDist(JSONLoader & json) : Operation_Unary(json) {
    json.load("units", units);
}
IR::Node * IR::MAU::HashDist::fromJSON(JSONLoader & json) { return new HashDist(json); }
bool IR::MAU::HashDist::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::MAU::HashDist::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::MAU::HashDist::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::HashDist::operator==(IR::Node const & a) const { return a == *this; }

#line 1103 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::IXBarExpression::IXBarExpression(const IR::Expression* e, int b) : Expression(e->srcInfo, e->type), expr(e), bit(b) {}
#line 17614 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1104 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::IXBarExpression::dbprint(std::ostream & out) const { out << "ixbar(" << expr << ")"; }
#line 17617 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::IXBarExpression::operator==(IR::MAU::IXBarExpression const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && expr == a.expr
        && bit == a.bit;
    }
bool IR::MAU::IXBarExpression::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const IXBarExpression &>(a_);
        return (expr ? a.expr ? expr->equiv(*a.expr) : false : a.expr == nullptr)
        && bit == a.bit;
    }
void IR::MAU::IXBarExpression::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(expr, "expr");
}
void IR::MAU::IXBarExpression::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(expr, "expr");
}
void IR::MAU::IXBarExpression::validate() const {
#line 17641 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(expr); }
void IR::MAU::IXBarExpression::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " bit=" << bit;
}
void IR::MAU::IXBarExpression::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("expr", expr);
    json.emit("bit", bit);
}
IR::MAU::IXBarExpression::IXBarExpression(JSONLoader & json) : Expression(json) {
    json.load("expr", expr);
    json.load("bit", bit);
}
IR::Node * IR::MAU::IXBarExpression::fromJSON(JSONLoader & json) { return new IXBarExpression(json); }
bool IR::MAU::IXBarExpression::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::IXBarExpression::operator==(IR::Node const & a) const { return a == *this; }
#line 1112 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::ActionDataConstant::dbprint(std::ostream & out) const { out << "action data constant " << constant; }
#line 17662 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::ActionDataConstant::operator==(IR::MAU::ActionDataConstant const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && name == a.name
        && constant == a.constant;
    }
bool IR::MAU::ActionDataConstant::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const ActionDataConstant &>(a_);
        return name == a.name
        && (constant ? a.constant ? constant->equiv(*a.constant) : false : a.constant == nullptr);
    }
void IR::MAU::ActionDataConstant::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(constant, "constant");
}
void IR::MAU::ActionDataConstant::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(constant, "constant");
}
void IR::MAU::ActionDataConstant::validate() const {
#line 17686 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(constant); }
void IR::MAU::ActionDataConstant::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " name=" << name;
}
void IR::MAU::ActionDataConstant::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("name", name);
    json.emit("constant", constant);
}
IR::MAU::ActionDataConstant::ActionDataConstant(JSONLoader & json) : Expression(json) {
    json.load("name", name);
    json.load("constant", constant);
}
IR::Node * IR::MAU::ActionDataConstant::fromJSON(JSONLoader & json) { return new ActionDataConstant(json); }
bool IR::MAU::ActionDataConstant::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::ActionDataConstant::operator==(IR::Node const & a) const { return a == *this; }
/**
     * 2 units per stage, one per logical table, on a byte by byte basis
     */
#line 1121 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::RandomNumber::allocated() const { return rng_unit >= 0; }
#line 17710 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1122 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::RandomNumber::dbprint(std::ostream & out) const { out << "rng(" << name << ", " << type << ")"; }
#line 17713 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::RandomNumber::operator==(IR::MAU::RandomNumber const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && name == a.name
        && rng_unit == a.rng_unit;
    }
bool IR::MAU::RandomNumber::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const RandomNumber &>(a_);
        return name == a.name
        && rng_unit == a.rng_unit;
    }
void IR::MAU::RandomNumber::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " name=" << name;
        out << " rng_unit=" << rng_unit;
}
void IR::MAU::RandomNumber::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("name", name);
    json.emit("rng_unit", rng_unit);
}
IR::MAU::RandomNumber::RandomNumber(JSONLoader & json) : Expression(json) {
    json.load("name", name);
    json.load("rng_unit", rng_unit);
}
IR::Node * IR::MAU::RandomNumber::fromJSON(JSONLoader & json) { return new RandomNumber(json); }
bool IR::MAU::RandomNumber::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::RandomNumber::operator==(IR::Node const & a) const { return a == *this; }
#line 1128 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::AttachedOutput::dbprint(std::ostream & out) const { out << attached->name; }
#line 17745 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::AttachedOutput::operator==(IR::MAU::AttachedOutput const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && attached == a.attached;
    }
bool IR::MAU::AttachedOutput::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const AttachedOutput &>(a_);
        return (attached ? a.attached ? attached->equiv(*a.attached) : false : a.attached == nullptr);
    }
void IR::MAU::AttachedOutput::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(attached, "attached");
}
void IR::MAU::AttachedOutput::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(attached, "attached");
}
void IR::MAU::AttachedOutput::validate() const {
#line 17767 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(attached); }
void IR::MAU::AttachedOutput::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("attached", attached);
}
IR::MAU::AttachedOutput::AttachedOutput(JSONLoader & json) : Expression(json) {
    json.load("attached", attached);
}
IR::Node * IR::MAU::AttachedOutput::fromJSON(JSONLoader & json) { return new AttachedOutput(json); }
bool IR::MAU::AttachedOutput::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::AttachedOutput::operator==(IR::Node const & a) const { return a == *this; }

#line 1136 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::SaluReg::dbprint(std::ostream & out) const { out << name << (hi ? " {hi}" : ""); }
#line 17783 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1137 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
cstring IR::MAU::SaluReg::toString() const { return name; }
#line 17786 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::SaluReg::operator==(IR::MAU::SaluReg const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && name == a.name
        && hi == a.hi
        && phv_src == a.phv_src;
    }
bool IR::MAU::SaluReg::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const SaluReg &>(a_);
        return name == a.name
        && hi == a.hi
        && (phv_src ? a.phv_src ? phv_src->equiv(*a.phv_src) : false : a.phv_src == nullptr);
    }
void IR::MAU::SaluReg::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(phv_src, "phv_src");
}
void IR::MAU::SaluReg::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(phv_src, "phv_src");
}
void IR::MAU::SaluReg::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " name=" << name;
        out << " hi=" << hi;
}
void IR::MAU::SaluReg::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("name", name);
    json.emit("hi", hi);
    if (phv_src != nullptr) json.emit("phv_src", phv_src);
}
IR::MAU::SaluReg::SaluReg(JSONLoader & json) : Expression(json) {
    json.load("name", name);
    json.load("hi", hi);
    json.load("phv_src", phv_src);
}
IR::Node * IR::MAU::SaluReg::fromJSON(JSONLoader & json) { return new SaluReg(json); }
bool IR::MAU::SaluReg::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::SaluReg::operator==(IR::Node const & a) const { return a == *this; }
#line 1141 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::SaluCmpReg::SaluCmpReg(cstring name, int idx) : SaluReg(Type::Bits::get(1), name, idx & 1), index(idx) {}
#line 17832 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::SaluCmpReg::operator==(IR::MAU::SaluCmpReg const & a) const {
        return SaluReg::operator==(static_cast<const SaluReg &>(a))
        && index == a.index;
    }
bool IR::MAU::SaluCmpReg::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!SaluReg::equiv(a_)) return false;
        auto &a = static_cast<const SaluCmpReg &>(a_);
        return index == a.index;
    }
void IR::MAU::SaluCmpReg::dump_fields(std::ostream & out) const {
    SaluReg::dump_fields(out);
        out << " index=" << index;
}
void IR::MAU::SaluCmpReg::toJSON(JSONGenerator & json) const {
    SaluReg::toJSON(json);
    json.emit("index", index);
}
IR::MAU::SaluCmpReg::SaluCmpReg(JSONLoader & json) : SaluReg(json) {
    json.load("index", index);
}
IR::Node * IR::MAU::SaluCmpReg::fromJSON(JSONLoader & json) { return new SaluCmpReg(json); }
bool IR::MAU::SaluCmpReg::operator==(IR::MAU::SaluReg const & a) const { return a == *this; }
bool IR::MAU::SaluCmpReg::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::SaluCmpReg::operator==(IR::Node const & a) const { return a == *this; }

#line 1148 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::SaluFunction::dbprint(std::ostream & out) const { out << name << "(" << expr << ")"; }
#line 17861 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::SaluFunction::operator==(IR::MAU::SaluFunction const & a) const {
        return Operation_Unary::operator==(static_cast<const Operation_Unary &>(a))
        && name == a.name;
    }
bool IR::MAU::SaluFunction::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Unary::equiv(a_)) return false;
        auto &a = static_cast<const SaluFunction &>(a_);
        return name == a.name;
    }
void IR::MAU::SaluFunction::dump_fields(std::ostream & out) const {
    Operation_Unary::dump_fields(out);
        out << " name=" << name;
}
void IR::MAU::SaluFunction::toJSON(JSONGenerator & json) const {
    Operation_Unary::toJSON(json);
    json.emit("name", name);
}
IR::MAU::SaluFunction::SaluFunction(JSONLoader & json) : Operation_Unary(json) {
    json.load("name", name);
}
IR::Node * IR::MAU::SaluFunction::fromJSON(JSONLoader & json) { return new SaluFunction(json); }
bool IR::MAU::SaluFunction::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::MAU::SaluFunction::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::MAU::SaluFunction::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::SaluFunction::operator==(IR::Node const & a) const { return a == *this; }




#line 1161 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::SaluRegfileRow::dbprint(std::ostream & out) const { out << "regfile(index=" << index << ", "
                  << "initial_value=" << initial_value << ", "
                  << "name=" << name.name << ", "
                  << "externalName=" << externalName << ")"; }
#line 17897 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::SaluRegfileRow::operator==(IR::MAU::SaluRegfileRow const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && index == a.index
        && initial_value == a.initial_value
        && name == a.name
        && externalName == a.externalName;
    }
bool IR::MAU::SaluRegfileRow::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const SaluRegfileRow &>(a_);
        return index == a.index
        && initial_value == a.initial_value
        && name == a.name
        && externalName == a.externalName;
    }
void IR::MAU::SaluRegfileRow::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " index=" << index;
        out << " initial_value=" << initial_value;
        out << " name=" << name;
        out << " externalName=" << externalName;
}
void IR::MAU::SaluRegfileRow::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("index", index);
    json.emit("initial_value", initial_value);
    json.emit("name", name);
    json.emit("externalName", externalName);
}
IR::MAU::SaluRegfileRow::SaluRegfileRow(JSONLoader & json) : Expression(json) {
    json.load("index", index);
    json.load("initial_value", initial_value);
    json.load("name", name);
    json.load("externalName", externalName);
}
IR::Node * IR::MAU::SaluRegfileRow::fromJSON(JSONLoader & json) { return new SaluRegfileRow(json); }
bool IR::MAU::SaluRegfileRow::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::SaluRegfileRow::operator==(IR::Node const & a) const { return a == *this; }
#line 1168 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::MultiOperand::dbprint(std::ostream & out) const { out << name; }
#line 17939 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1171 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::MultiOperand::MultiOperand(IR::ListExpression & le, cstring n, bool ip) : ListExpression(le), name(n), is_phv(ip) {}
#line 17942 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::MultiOperand::operator==(IR::MAU::MultiOperand const & a) const {
        return ListExpression::operator==(static_cast<const ListExpression &>(a))
        && name == a.name
        && is_phv == a.is_phv;
    }
bool IR::MAU::MultiOperand::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ListExpression::equiv(a_)) return false;
        auto &a = static_cast<const MultiOperand &>(a_);
        return name == a.name
        && is_phv == a.is_phv;
    }
void IR::MAU::MultiOperand::dump_fields(std::ostream & out) const {
    ListExpression::dump_fields(out);
        out << " name=" << name;
        out << " is_phv=" << is_phv;
}
void IR::MAU::MultiOperand::toJSON(JSONGenerator & json) const {
    ListExpression::toJSON(json);
    json.emit("name", name);
    json.emit("is_phv", is_phv);
}
IR::MAU::MultiOperand::MultiOperand(JSONLoader & json) : ListExpression(json) {
    json.load("name", name);
    json.load("is_phv", is_phv);
}
IR::Node * IR::MAU::MultiOperand::fromJSON(JSONLoader & json) { return new MultiOperand(json); }
bool IR::MAU::MultiOperand::operator==(IR::ListExpression const & a) const { return a == *this; }
bool IR::MAU::MultiOperand::operator==(IR::BaseListExpression const & a) const { return a == *this; }
bool IR::MAU::MultiOperand::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::MultiOperand::operator==(IR::Node const & a) const { return a == *this; }
#line 1178 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::WrappedSlice::validate() const {
#line 17976 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 1178 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
{
        if (e1 && e2 && e1->is<IR::Constant>() && e2->is<IR::Constant>()) {
            BUG_CHECK(e1->is<IR::Constant>() && e2->is<IR::Constant>(),
                      "Wrapped Slices must have constants as slice portion");
            BUG_CHECK(e1->to<IR::Constant>()->asUnsigned() < e2->to<IR::Constant>()->asUnsigned(),
                      "Wrapped Slices must have a higher lo than hi");
            BUG_CHECK(e0->is<IR::MAU::MultiOperand>(), "Wrapped Slices only make sense on "
                      "wrapped values");
        }
    } }
#line 17989 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1188 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::MAU::WrappedSlice::WrappedSlice(const IR::Expression* a, int hi, int lo, int wrap_size) : Operation_Ternary(IR::Type::Bits::get(wrap_size - (lo - hi) + 1), a, new Constant(hi),
                       new Constant(lo)) {}
#line 17993 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1191 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::WrappedSlice::getL() const { return e2->to<IR::Constant>()->asInt(); }
#line 17996 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1192 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::WrappedSlice::dbprint(std::ostream & out) const { out << e0 << "(" << e1 << ".." << e2 << ")"; }
#line 17999 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::WrappedSlice::operator==(IR::MAU::WrappedSlice const & a) const {
        return Operation_Ternary::operator==(static_cast<const Operation_Ternary &>(a));
    }
bool IR::MAU::WrappedSlice::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Ternary::equiv(a_)) return false;
        return true;
    }
void IR::MAU::WrappedSlice::toJSON(JSONGenerator & json) const {
    Operation_Ternary::toJSON(json);
}
IR::MAU::WrappedSlice::WrappedSlice(JSONLoader & json) : Operation_Ternary(json) {
}
IR::Node * IR::MAU::WrappedSlice::fromJSON(JSONLoader & json) { return new WrappedSlice(json); }
bool IR::MAU::WrappedSlice::operator==(IR::Operation_Ternary const & a) const { return a == *this; }
bool IR::MAU::WrappedSlice::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::MAU::WrappedSlice::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::WrappedSlice::operator==(IR::Node const & a) const { return a == *this; }

#line 1209 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::TableKey::for_selection() const { return match_type.name == "selector"; }
#line 18021 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1210 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::TableKey::for_dleft() const { return match_type.name == "dleft_hash"; }
#line 18024 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1211 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::TableKey::for_atcam_partition_index() const { return match_type.name == "atcam_partition_index"; }
#line 18027 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1212 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::TableKey::for_alpm_partition_index() const { return match_type.name == "atcam_partition_index" &&
                                                   used_in_alpm == true; }
#line 18031 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1214 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::TableKey::for_range() const { return match_type.name == "range"; }
#line 18034 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1215 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
bool IR::MAU::TableKey::for_match() const { return !for_selection() && !for_dleft(); }
#line 18037 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1216 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
int IR::MAU::TableKey::get_ixbar_group_num() const { return ixbar_group_num; }
#line 18040 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1218 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
void IR::MAU::TableKey::dbprint(std::ostream & out) const { out << expr << ": " << match_type.name; }
#line 18043 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1221 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::Vector<IR::Annotation> const & IR::MAU::TableKey::getAnnotations() const { return annotations; }
#line 18046 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 1222 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/mau.def"
IR::Vector<IR::Annotation> & IR::MAU::TableKey::getAnnotations() { return annotations; }
#line 18049 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::MAU::TableKey::operator==(IR::MAU::TableKey const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && expr == a.expr
        && match_type == a.match_type
        && p4_param_order == a.p4_param_order
        && from_mask == a.from_mask
        && partition_index == a.partition_index
        && used_in_alpm == a.used_in_alpm
        && ixbar_group_num == a.ixbar_group_num
        && annotations == a.annotations;
    }
bool IR::MAU::TableKey::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const TableKey &>(a_);
        return (expr ? a.expr ? expr->equiv(*a.expr) : false : a.expr == nullptr)
        && match_type == a.match_type
        && p4_param_order == a.p4_param_order
        && from_mask == a.from_mask
        && partition_index == a.partition_index
        && used_in_alpm == a.used_in_alpm
        && ixbar_group_num == a.ixbar_group_num
        && annotations.equiv(a.annotations);
    }
void IR::MAU::TableKey::visit_children(Visitor & v, char const * n) {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(expr, "expr");
    annotations.visit_children(v, "annotations");
}
void IR::MAU::TableKey::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Expression::visit_children(v, n);
    v.visit(expr, "expr");
    annotations.visit_children(v, "annotations");
}
void IR::MAU::TableKey::validate() const {
#line 18087 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(expr);
        annotations.validate(); }
void IR::MAU::TableKey::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " match_type=" << match_type;
        out << " p4_param_order=" << p4_param_order;
        out << " from_mask=" << from_mask;
        out << " partition_index=" << partition_index;
        out << " used_in_alpm=" << used_in_alpm;
        out << " ixbar_group_num=" << ixbar_group_num;
}
void IR::MAU::TableKey::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("expr", expr);
    json.emit("match_type", match_type);
    json.emit("p4_param_order", p4_param_order);
    json.emit("from_mask", from_mask);
    json.emit("partition_index", partition_index);
    json.emit("used_in_alpm", used_in_alpm);
    json.emit("ixbar_group_num", ixbar_group_num);
    json.emit("annotations", annotations);
}
IR::MAU::TableKey::TableKey(JSONLoader & json) : Expression(json) {
    json.load("expr", expr);
    json.load("match_type", match_type);
    json.load("p4_param_order", p4_param_order);
    json.load("from_mask", from_mask);
    json.load("partition_index", partition_index);
    json.load("used_in_alpm", used_in_alpm);
    json.load("ixbar_group_num", ixbar_group_num);
    json.load("annotations", annotations);
}
IR::Node * IR::MAU::TableKey::fromJSON(JSONLoader & json) { return new TableKey(json); }
bool IR::MAU::TableKey::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::MAU::TableKey::operator==(IR::Node const & a) const { return a == *this; }


bool IR::RangeMatch::operator==(IR::RangeMatch const & a) const {
        return Operation_Unary::operator==(static_cast<const Operation_Unary &>(a))
        && data == a.data;
    }
bool IR::RangeMatch::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Operation_Unary::equiv(a_)) return false;
        auto &a = static_cast<const RangeMatch &>(a_);
        return data == a.data;
    }
void IR::RangeMatch::dump_fields(std::ostream & out) const {
    Operation_Unary::dump_fields(out);
        out << " data=" << data;
}
void IR::RangeMatch::toJSON(JSONGenerator & json) const {
    Operation_Unary::toJSON(json);
    json.emit("data", data);
}
IR::RangeMatch::RangeMatch(JSONLoader & json) : Operation_Unary(json) {
    json.load("data", data);
}
IR::Node * IR::RangeMatch::fromJSON(JSONLoader & json) { return new RangeMatch(json); }
bool IR::RangeMatch::operator==(IR::Operation_Unary const & a) const { return a == *this; }
bool IR::RangeMatch::operator==(IR::Operation const & a) const { return a == *this; }
bool IR::RangeMatch::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::RangeMatch::operator==(IR::Node const & a) const { return a == *this; }
/**
 * Copyright (C) 2024 Intel Corporation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License.  You may obtain a copy
 * of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 * 
 * 
 * SPDX-License-Identifier: Apache-2.0
 */
/* -*-C++-*- */
#line 36 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::UnresolvedHeaderStackIndex::validate() const {
#line 18172 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 36 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
{
        BUG_CHECK(index == "next" || index == "last",
            "Unexpected header stack index %1%", index);
    } }
#line 18179 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 41 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::UnresolvedHeaderStackIndex::dbprint(std::ostream & out) const { out << index; }
#line 18182 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::UnresolvedHeaderStackIndex::operator==(IR::BFN::UnresolvedHeaderStackIndex const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a))
        && index == a.index;
    }
bool IR::BFN::UnresolvedHeaderStackIndex::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        auto &a = static_cast<const UnresolvedHeaderStackIndex &>(a_);
        return index == a.index;
    }
void IR::BFN::UnresolvedHeaderStackIndex::dump_fields(std::ostream & out) const {
    Expression::dump_fields(out);
        out << " index=" << index;
}
void IR::BFN::UnresolvedHeaderStackIndex::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
    json.emit("index", index);
}
IR::BFN::UnresolvedHeaderStackIndex::UnresolvedHeaderStackIndex(JSONLoader & json) : Expression(json) {
    json.load("index", index);
}
IR::Node * IR::BFN::UnresolvedHeaderStackIndex::fromJSON(JSONLoader & json) { return new UnresolvedHeaderStackIndex(json); }
bool IR::BFN::UnresolvedHeaderStackIndex::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::UnresolvedHeaderStackIndex::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::ParserRVal::operator==(IR::BFN::ParserRVal const & a) const {
        return Expression::operator==(static_cast<const Expression &>(a));
    }
bool IR::BFN::ParserRVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Expression::equiv(a_)) return false;
        return true;
    }
void IR::BFN::ParserRVal::toJSON(JSONGenerator & json) const {
    Expression::toJSON(json);
}
IR::BFN::ParserRVal::ParserRVal(JSONLoader & json) : Expression(json) {
}
bool IR::BFN::ParserRVal::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::ParserRVal::operator==(IR::Node const & a) const { return a == *this; }
/// @return this buffer range as a half-open range of bits in network order.
#line 53 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
nw_bitinterval IR::BFN::InputBufferRVal::interval() const { return toHalfOpenRange(range); }
#line 18225 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The region represented by this `InputBufferRVal`, expressed as a range of
/// bits in network order.
bool IR::BFN::InputBufferRVal::operator==(IR::BFN::InputBufferRVal const & a) const {
        return ParserRVal::operator==(static_cast<const ParserRVal &>(a))
        && range == a.range;
    }
bool IR::BFN::InputBufferRVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserRVal::equiv(a_)) return false;
        auto &a = static_cast<const InputBufferRVal &>(a_);
        return range == a.range;
    }
void IR::BFN::InputBufferRVal::dump_fields(std::ostream & out) const {
    ParserRVal::dump_fields(out);
        out << " range=" << range;
}
void IR::BFN::InputBufferRVal::toJSON(JSONGenerator & json) const {
    ParserRVal::toJSON(json);
    json.emit("range", range);
}
IR::BFN::InputBufferRVal::InputBufferRVal(JSONLoader & json) : ParserRVal(json) {
    json.load("range", range);
}
bool IR::BFN::InputBufferRVal::operator==(IR::BFN::ParserRVal const & a) const { return a == *this; }
bool IR::BFN::InputBufferRVal::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::InputBufferRVal::operator==(IR::Node const & a) const { return a == *this; }
/// Enables and disables the partial_hdr_err_proc bit in the parser
/// transition.  Controls the extraction behavior when packets
/// with incomplete headers are received (i.e. greedy extract).
#line 68 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::PacketRVal::dbprint(std::ostream & out) const {
        out << "inbuf " << range;
    }
#line 18259 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::PacketRVal::operator==(IR::BFN::PacketRVal const & a) const {
        return InputBufferRVal::operator==(static_cast<const InputBufferRVal &>(a))
        && partial_hdr_err_proc == a.partial_hdr_err_proc;
    }
bool IR::BFN::PacketRVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!InputBufferRVal::equiv(a_)) return false;
        auto &a = static_cast<const PacketRVal &>(a_);
        return partial_hdr_err_proc == a.partial_hdr_err_proc;
    }
void IR::BFN::PacketRVal::dump_fields(std::ostream & out) const {
    InputBufferRVal::dump_fields(out);
        out << " partial_hdr_err_proc=" << partial_hdr_err_proc;
}
void IR::BFN::PacketRVal::toJSON(JSONGenerator & json) const {
    InputBufferRVal::toJSON(json);
    json.emit("partial_hdr_err_proc", partial_hdr_err_proc);
}
IR::BFN::PacketRVal::PacketRVal(JSONLoader & json) : InputBufferRVal(json) {
    json.load("partial_hdr_err_proc", partial_hdr_err_proc);
}
IR::Node * IR::BFN::PacketRVal::fromJSON(JSONLoader & json) { return new PacketRVal(json); }
bool IR::BFN::PacketRVal::operator==(IR::BFN::InputBufferRVal const & a) const { return a == *this; }
bool IR::BFN::PacketRVal::operator==(IR::BFN::ParserRVal const & a) const { return a == *this; }
bool IR::BFN::PacketRVal::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::PacketRVal::operator==(IR::Node const & a) const { return a == *this; }
#line 75 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::MetadataRVal::dbprint(std::ostream & out) const {
        out << "inbuf(metadata) " << range;
    }
#line 18290 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::MetadataRVal::operator==(IR::BFN::MetadataRVal const & a) const {
        return InputBufferRVal::operator==(static_cast<const InputBufferRVal &>(a));
    }
bool IR::BFN::MetadataRVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!InputBufferRVal::equiv(a_)) return false;
        return true;
    }
void IR::BFN::MetadataRVal::toJSON(JSONGenerator & json) const {
    InputBufferRVal::toJSON(json);
}
IR::BFN::MetadataRVal::MetadataRVal(JSONLoader & json) : InputBufferRVal(json) {
}
IR::Node * IR::BFN::MetadataRVal::fromJSON(JSONLoader & json) { return new MetadataRVal(json); }
bool IR::BFN::MetadataRVal::operator==(IR::BFN::InputBufferRVal const & a) const { return a == *this; }
bool IR::BFN::MetadataRVal::operator==(IR::BFN::ParserRVal const & a) const { return a == *this; }
bool IR::BFN::MetadataRVal::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::MetadataRVal::operator==(IR::Node const & a) const { return a == *this; }
#line 82 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::ConstantRVal::ConstantRVal(big_int value) : constant(new IR::Constant(value)) { }
#line 18311 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 83 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::ConstantRVal::ConstantRVal(int value) : constant(new IR::Constant(value)) { }
#line 18314 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 84 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::ConstantRVal::ConstantRVal(IR::Type const * type, big_int value) : constant(new IR::Constant(type, value)) { }
#line 18317 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 86 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::ConstantRVal::ConstantRVal(Util::SourceInfo si, IR::Type const * type, big_int value) : ParserRVal(si), constant(new IR::Constant(type, value)) { }
#line 18320 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 89 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ConstantRVal::dbprint(std::ostream & out) const { out << "constant " << constant; }
#line 18323 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ConstantRVal::operator==(IR::BFN::ConstantRVal const & a) const {
        return ParserRVal::operator==(static_cast<const ParserRVal &>(a))
        && constant == a.constant;
    }
bool IR::BFN::ConstantRVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserRVal::equiv(a_)) return false;
        auto &a = static_cast<const ConstantRVal &>(a_);
        return (constant ? a.constant ? constant->equiv(*a.constant) : false : a.constant == nullptr);
    }
void IR::BFN::ConstantRVal::visit_children(Visitor & v, char const * n) {
    (void)n;
    ParserRVal::visit_children(v, n);
    v.visit(constant, "constant");
}
void IR::BFN::ConstantRVal::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ParserRVal::visit_children(v, n);
    v.visit(constant, "constant");
}
void IR::BFN::ConstantRVal::validate() const {
#line 18345 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(constant); }
void IR::BFN::ConstantRVal::toJSON(JSONGenerator & json) const {
    ParserRVal::toJSON(json);
    json.emit("constant", constant);
}
IR::BFN::ConstantRVal::ConstantRVal(JSONLoader & json) : ParserRVal(json) {
    json.load("constant", constant);
}
IR::Node * IR::BFN::ConstantRVal::fromJSON(JSONLoader & json) { return new ConstantRVal(json); }
bool IR::BFN::ConstantRVal::operator==(IR::BFN::ParserRVal const & a) const { return a == *this; }
bool IR::BFN::ConstantRVal::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::ConstantRVal::operator==(IR::Node const & a) const { return a == *this; }
#line 96 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::SavedRVal::dbprint(std::ostream & out) const {
        out << "[ ";
        if (reg_slices.empty()) {
            out << source;
        } else {
            for (const auto& rs : reg_slices)
                out << " " << rs.first << " : " << rs.second;
        }
        out << " ]";
    }
#line 18370 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// An expression which should evaluate to an r-value usable in the parser.
/// The registers to match on.
bool IR::BFN::SavedRVal::operator==(IR::BFN::SavedRVal const & a) const {
        return ParserRVal::operator==(static_cast<const ParserRVal &>(a))
        && source == a.source
        && reg_slices == a.reg_slices;
    }
bool IR::BFN::SavedRVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserRVal::equiv(a_)) return false;
        auto &a = static_cast<const SavedRVal &>(a_);
        return (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr)
        && reg_slices == a.reg_slices;
    }
void IR::BFN::SavedRVal::visit_children(Visitor & v, char const * n) {
    (void)n;
    ParserRVal::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::SavedRVal::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ParserRVal::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::SavedRVal::validate() const {
#line 18396 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source); }
void IR::BFN::SavedRVal::toJSON(JSONGenerator & json) const {
    ParserRVal::toJSON(json);
    json.emit("source", source);
    json.emit("reg_slices", reg_slices);
}
IR::BFN::SavedRVal::SavedRVal(JSONLoader & json) : ParserRVal(json) {
    json.load("source", source);
    json.load("reg_slices", reg_slices);
}
IR::Node * IR::BFN::SavedRVal::fromJSON(JSONLoader & json) { return new SavedRVal(json); }
bool IR::BFN::SavedRVal::operator==(IR::BFN::ParserRVal const & a) const { return a == *this; }
bool IR::BFN::SavedRVal::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::SavedRVal::operator==(IR::Node const & a) const { return a == *this; }
#line 116 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
cstring IR::BFN::ParserLVal::toString() const { return field->toString(); }
#line 18414 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 118 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
int IR::BFN::ParserLVal::size() const { return field->type->width_bits(); }
#line 18417 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 120 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserLVal::validate() const {
#line 18420 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(field);
#line 120 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
{
        CHECK_NULL(field);
        BUG_CHECK(field->is<IR::Member>() || field->is<IR::TempVar>() ||
                  field->is<IR::Slice>() || field->is<IR::ConcreteHeaderRef>() ||
                  field->is<IR::Concat>() || field->is<IR::Padding>(),
                  "FieldLVal contains unexpected value: %1%", field);
    } }
#line 18431 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserLVal::operator==(IR::BFN::ParserLVal const & a) const {
        return this->typeId() == a.typeId()
        && field == a.field;
    }
bool IR::BFN::ParserLVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const ParserLVal &>(a_);
        return (field ? a.field ? field->equiv(*a.field) : false : a.field == nullptr);
    }
void IR::BFN::ParserLVal::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(field, "field");
}
void IR::BFN::ParserLVal::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(field, "field");
}
void IR::BFN::ParserLVal::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("field", field);
}
IR::BFN::ParserLVal::ParserLVal(JSONLoader & json) : Node(json) {
    json.load("field", field);
}
bool IR::BFN::ParserLVal::operator==(IR::Node const & a) const { return a == *this; }
#line 133 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::FieldLVal::dbprint(std::ostream & out) const { out << field; }
#line 18462 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::FieldLVal::operator==(IR::BFN::FieldLVal const & a) const {
        return ParserLVal::operator==(static_cast<const ParserLVal &>(a));
    }
bool IR::BFN::FieldLVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserLVal::equiv(a_)) return false;
        return true;
    }
void IR::BFN::FieldLVal::toJSON(JSONGenerator & json) const {
    ParserLVal::toJSON(json);
}
IR::BFN::FieldLVal::FieldLVal(JSONLoader & json) : ParserLVal(json) {
}
IR::Node * IR::BFN::FieldLVal::fromJSON(JSONLoader & json) { return new FieldLVal(json); }
bool IR::BFN::FieldLVal::operator==(IR::BFN::ParserLVal const & a) const { return a == *this; }
bool IR::BFN::FieldLVal::operator==(IR::Node const & a) const { return a == *this; }
#line 137 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::MatchLVal::dbprint(std::ostream & out) const { out << field; }
#line 18481 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::MatchLVal::operator==(IR::BFN::MatchLVal const & a) const {
        return ParserLVal::operator==(static_cast<const ParserLVal &>(a));
    }
bool IR::BFN::MatchLVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserLVal::equiv(a_)) return false;
        return true;
    }
void IR::BFN::MatchLVal::toJSON(JSONGenerator & json) const {
    ParserLVal::toJSON(json);
}
IR::BFN::MatchLVal::MatchLVal(JSONLoader & json) : ParserLVal(json) {
}
IR::Node * IR::BFN::MatchLVal::fromJSON(JSONLoader & json) { return new MatchLVal(json); }
bool IR::BFN::MatchLVal::operator==(IR::BFN::ParserLVal const & a) const { return a == *this; }
bool IR::BFN::MatchLVal::operator==(IR::Node const & a) const { return a == *this; }
#line 142 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ChecksumLVal::dbprint(std::ostream & out) const { out << "checksummed " << field; }
#line 18500 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ChecksumLVal::operator==(IR::BFN::ChecksumLVal const & a) const {
        return FieldLVal::operator==(static_cast<const FieldLVal &>(a));
    }
bool IR::BFN::ChecksumLVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!FieldLVal::equiv(a_)) return false;
        return true;
    }
void IR::BFN::ChecksumLVal::toJSON(JSONGenerator & json) const {
    FieldLVal::toJSON(json);
}
IR::BFN::ChecksumLVal::ChecksumLVal(JSONLoader & json) : FieldLVal(json) {
}
IR::Node * IR::BFN::ChecksumLVal::fromJSON(JSONLoader & json) { return new ChecksumLVal(json); }
bool IR::BFN::ChecksumLVal::operator==(IR::BFN::FieldLVal const & a) const { return a == *this; }
bool IR::BFN::ChecksumLVal::operator==(IR::BFN::ParserLVal const & a) const { return a == *this; }
bool IR::BFN::ChecksumLVal::operator==(IR::Node const & a) const { return a == *this; }
#line 148 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::TotalContainerSize::dbprint(std::ostream & out) const { out << toString(); }
#line 18520 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 149 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
cstring IR::BFN::TotalContainerSize::toString() const { return "total size of containers for fields"_cs; }
#line 18523 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::TotalContainerSize::operator==(IR::BFN::TotalContainerSize const & a) const {
        return ParserRVal::operator==(static_cast<const ParserRVal &>(a))
        && fields == a.fields;
    }
bool IR::BFN::TotalContainerSize::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserRVal::equiv(a_)) return false;
        auto &a = static_cast<const TotalContainerSize &>(a_);
        return fields.equiv(a.fields);
    }
void IR::BFN::TotalContainerSize::visit_children(Visitor & v, char const * n) {
    (void)n;
    ParserRVal::visit_children(v, n);
    fields.visit_children(v, "fields");
}
void IR::BFN::TotalContainerSize::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ParserRVal::visit_children(v, n);
    fields.visit_children(v, "fields");
}
void IR::BFN::TotalContainerSize::validate() const {
#line 18545 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        fields.validate(); }
void IR::BFN::TotalContainerSize::toJSON(JSONGenerator & json) const {
    ParserRVal::toJSON(json);
    json.emit("fields", fields);
}
IR::BFN::TotalContainerSize::TotalContainerSize(JSONLoader & json) : ParserRVal(json) {
    json.load("fields", fields);
}
IR::Node * IR::BFN::TotalContainerSize::fromJSON(JSONLoader & json) { return new TotalContainerSize(json); }
bool IR::BFN::TotalContainerSize::operator==(IR::BFN::ParserRVal const & a) const { return a == *this; }
bool IR::BFN::TotalContainerSize::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::TotalContainerSize::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::ParserPrimitive::operator==(IR::BFN::ParserPrimitive const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::BFN::ParserPrimitive::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::BFN::ParserPrimitive::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::BFN::ParserPrimitive::ParserPrimitive(JSONLoader & json) : Node(json) {
}
bool IR::BFN::ParserPrimitive::operator==(IR::Node const & a) const { return a == *this; }
#line 175 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::Extract::Extract(const IR::Expression* dest, const IR::BFN::ParserRVal* source) : dest(new FieldLVal(dest)), source(source) { }
#line 18575 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 177 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::Extract::Extract(Util::SourceInfo si, const IR::Expression* dest, const IR::BFN::ParserRVal* source) : ParserPrimitive(si), dest(new FieldLVal(dest)), source(source) { }
#line 18578 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 180 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::Extract::dbprint(std::ostream & out) const {
        out << "extract " << source << " to " << dest;
    }
#line 18583 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 184 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
const IR::BFN::ParserLVal *IR::BFN::Extract::getWriteDest() const { return dest; }
#line 18586 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 185 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::ParserWriteMode IR::BFN::Extract::getWriteMode() const { return write_mode; }
#line 18589 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The location to be written to.
/// The location that will be read from.
/// The original state this extract originated from (before parser state splitting).
/// This extract expects that data on the wire is marshaled from a `field`.

#line 201 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::Extract::dump_fields(std::ostream & out) const {
       ParserPrimitive::dump_fields(out);
       out << " write_mode=" << write_mode;
       out << " original_state=" << DBPrint::Brief << original_state; }
#line 18600 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::Extract::operator==(IR::BFN::Extract const & a) const {
        return ParserPrimitive::operator==(static_cast<const ParserPrimitive &>(a))
        && dest == a.dest
        && source == a.source
        && write_mode == a.write_mode
        && original_state == a.original_state
        && marshaled_from == a.marshaled_from;
    }
bool IR::BFN::Extract::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const Extract &>(a_);
        return (dest ? a.dest ? dest->equiv(*a.dest) : false : a.dest == nullptr)
        && (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr)
        && write_mode == a.write_mode
        && original_state == a.original_state
        && marshaled_from == a.marshaled_from;
    }
void IR::BFN::Extract::visit_children(Visitor & v, char const * n) {
    (void)n;
    ParserPrimitive::visit_children(v, n);
    v.visit(dest, "dest");
    v.visit(source, "source");
}
void IR::BFN::Extract::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ParserPrimitive::visit_children(v, n);
    v.visit(dest, "dest");
    v.visit(source, "source");
}
void IR::BFN::Extract::validate() const {
#line 18632 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(dest);
        CHECK_NULL(source); }
void IR::BFN::Extract::toJSON(JSONGenerator & json) const {
    ParserPrimitive::toJSON(json);
    json.emit("dest", dest);
    json.emit("source", source);
    json.emit("write_mode", write_mode);
    json.emit("original_state", original_state);
    json.emit("marshaled_from", marshaled_from);
}
IR::BFN::Extract::Extract(JSONLoader & json) : ParserPrimitive(json) {
    json.load("dest", dest);
    json.load("source", source);
    json.load("write_mode", write_mode);
    json.load("original_state", original_state);
    json.load("marshaled_from", marshaled_from);
}
IR::Node * IR::BFN::Extract::fromJSON(JSONLoader & json) { return new Extract(json); }
bool IR::BFN::Extract::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::Extract::operator==(IR::Node const & a) const { return a == *this; }
#line 212 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ExtractPhv::dbprint(std::ostream & out) const { out << "extract " << source << " to PHV-allocated " << dest; }
#line 18656 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ExtractPhv::operator==(IR::BFN::ExtractPhv const & a) const {
        return Extract::operator==(static_cast<const Extract &>(a));
    }
bool IR::BFN::ExtractPhv::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Extract::equiv(a_)) return false;
        return true;
    }
void IR::BFN::ExtractPhv::toJSON(JSONGenerator & json) const {
    Extract::toJSON(json);
}
IR::BFN::ExtractPhv::ExtractPhv(JSONLoader & json) : Extract(json) {
}
IR::Node * IR::BFN::ExtractPhv::fromJSON(JSONLoader & json) { return new ExtractPhv(json); }
bool IR::BFN::ExtractPhv::operator==(IR::BFN::Extract const & a) const { return a == *this; }
bool IR::BFN::ExtractPhv::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ExtractPhv::operator==(IR::Node const & a) const { return a == *this; }
#line 220 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ExtractClot::dbprint(std::ostream & out) const {
        out << "extract " << source << " to CLOT-allocated " << dest << " from "
            << (original_state ? original_state->name : "(nullptr)"_cs);
    }
#line 18679 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 225 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ExtractClot::validate() const {
#line 18682 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 225 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
{
        BUG_CHECK(source->to<IR::BFN::PacketRVal>(),
                  "ExtractClot source for %1% is not a PacketRVal: %2%",
                  dest, source);
    } }
#line 18690 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ExtractClot::operator==(IR::BFN::ExtractClot const & a) const {
        return Extract::operator==(static_cast<const Extract &>(a));
    }
bool IR::BFN::ExtractClot::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Extract::equiv(a_)) return false;
        return true;
    }
void IR::BFN::ExtractClot::toJSON(JSONGenerator & json) const {
    Extract::toJSON(json);
}
IR::BFN::ExtractClot::ExtractClot(JSONLoader & json) : Extract(json) {
}
IR::Node * IR::BFN::ExtractClot::fromJSON(JSONLoader & json) { return new ExtractClot(json); }
bool IR::BFN::ExtractClot::operator==(IR::BFN::Extract const & a) const { return a == *this; }
bool IR::BFN::ExtractClot::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ExtractClot::operator==(IR::Node const & a) const { return a == *this; }
#line 238 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::SaveToRegister::dbprint(std::ostream & out) const { out << "save " << source << " to " << dest.name; }
#line 18710 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The location that will be read from.
/// The register this save writes to.
bool IR::BFN::SaveToRegister::operator==(IR::BFN::SaveToRegister const & a) const {
        return ParserPrimitive::operator==(static_cast<const ParserPrimitive &>(a))
        && source == a.source
        && dest == a.dest;
    }
bool IR::BFN::SaveToRegister::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const SaveToRegister &>(a_);
        return (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr)
        && dest == a.dest;
    }
void IR::BFN::SaveToRegister::visit_children(Visitor & v, char const * n) {
    (void)n;
    ParserPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::SaveToRegister::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ParserPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::SaveToRegister::validate() const {
#line 18736 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source); }
void IR::BFN::SaveToRegister::dump_fields(std::ostream & out) const {
    ParserPrimitive::dump_fields(out);
        out << " dest=" << dest;
}
void IR::BFN::SaveToRegister::toJSON(JSONGenerator & json) const {
    ParserPrimitive::toJSON(json);
    json.emit("source", source);
    json.emit("dest", dest);
}
IR::BFN::SaveToRegister::SaveToRegister(JSONLoader & json) : ParserPrimitive(json) {
    json.load("source", source);
    json.load("dest", dest);
}
IR::Node * IR::BFN::SaveToRegister::fromJSON(JSONLoader & json) { return new SaveToRegister(json); }
bool IR::BFN::SaveToRegister::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::SaveToRegister::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::ParserCounterPrimitive::operator==(IR::BFN::ParserCounterPrimitive const & a) const {
        return ParserPrimitive::operator==(static_cast<const ParserPrimitive &>(a))
        && declName == a.declName;
    }
bool IR::BFN::ParserCounterPrimitive::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ParserCounterPrimitive &>(a_);
        return declName == a.declName;
    }
void IR::BFN::ParserCounterPrimitive::dump_fields(std::ostream & out) const {
    ParserPrimitive::dump_fields(out);
        out << " declName=" << declName;
}
void IR::BFN::ParserCounterPrimitive::toJSON(JSONGenerator & json) const {
    ParserPrimitive::toJSON(json);
    json.emit("declName", declName);
}
IR::BFN::ParserCounterPrimitive::ParserCounterPrimitive(JSONLoader & json) : ParserPrimitive(json) {
    json.load("declName", declName);
}
bool IR::BFN::ParserCounterPrimitive::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserCounterPrimitive::operator==(IR::Node const & a) const { return a == *this; }
#line 253 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserZeroInit::dbprint(std::ostream & out) const { out << "Parser zero init: " << field; }
#line 18780 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserZeroInit::operator==(IR::BFN::ParserZeroInit const & a) const {
        return ParserPrimitive::operator==(static_cast<const ParserPrimitive &>(a))
        && field == a.field;
    }
bool IR::BFN::ParserZeroInit::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ParserZeroInit &>(a_);
        return (field ? a.field ? field->equiv(*a.field) : false : a.field == nullptr);
    }
void IR::BFN::ParserZeroInit::visit_children(Visitor & v, char const * n) {
    (void)n;
    ParserPrimitive::visit_children(v, n);
    v.visit(field, "field");
}
void IR::BFN::ParserZeroInit::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ParserPrimitive::visit_children(v, n);
    v.visit(field, "field");
}
void IR::BFN::ParserZeroInit::validate() const {
#line 18802 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(field); }
void IR::BFN::ParserZeroInit::toJSON(JSONGenerator & json) const {
    ParserPrimitive::toJSON(json);
    json.emit("field", field);
}
IR::BFN::ParserZeroInit::ParserZeroInit(JSONLoader & json) : ParserPrimitive(json) {
    json.load("field", field);
}
IR::Node * IR::BFN::ParserZeroInit::fromJSON(JSONLoader & json) { return new ParserZeroInit(json); }
bool IR::BFN::ParserZeroInit::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserZeroInit::operator==(IR::Node const & a) const { return a == *this; }
#line 257 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserCounterLoadImm::dbprint(std::ostream & out) const { out << "parser counter load imm " << imm; }
#line 18817 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserCounterLoadImm::operator==(IR::BFN::ParserCounterLoadImm const & a) const {
        return ParserCounterPrimitive::operator==(static_cast<const ParserCounterPrimitive &>(a))
        && imm == a.imm
        && push == a.push
        && update_with_top == a.update_with_top;
    }
bool IR::BFN::ParserCounterLoadImm::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserCounterPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ParserCounterLoadImm &>(a_);
        return imm == a.imm
        && push == a.push
        && update_with_top == a.update_with_top;
    }
void IR::BFN::ParserCounterLoadImm::dump_fields(std::ostream & out) const {
    ParserCounterPrimitive::dump_fields(out);
        out << " imm=" << imm;
        out << " push=" << push;
        out << " update_with_top=" << update_with_top;
}
void IR::BFN::ParserCounterLoadImm::toJSON(JSONGenerator & json) const {
    ParserCounterPrimitive::toJSON(json);
    json.emit("imm", imm);
    json.emit("push", push);
    json.emit("update_with_top", update_with_top);
}
IR::BFN::ParserCounterLoadImm::ParserCounterLoadImm(JSONLoader & json) : ParserCounterPrimitive(json) {
    json.load("imm", imm);
    json.load("push", push);
    json.load("update_with_top", update_with_top);
}
IR::Node * IR::BFN::ParserCounterLoadImm::fromJSON(JSONLoader & json) { return new ParserCounterLoadImm(json); }
bool IR::BFN::ParserCounterLoadImm::operator==(IR::BFN::ParserCounterPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserCounterLoadImm::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserCounterLoadImm::operator==(IR::Node const & a) const { return a == *this; }
#line 266 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::ParserCounterLoadPkt::ParserCounterLoadPkt(cstring n, IR::BFN::SavedRVal const * s) :
        ParserCounterPrimitive(n), source(s) { }
#line 18856 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 269 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserCounterLoadPkt::dbprint(std::ostream & out) const { out << "parser counter set " << source; }
#line 18859 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserCounterLoadPkt::operator==(IR::BFN::ParserCounterLoadPkt const & a) const {
        return ParserCounterPrimitive::operator==(static_cast<const ParserCounterPrimitive &>(a))
        && source == a.source
        && max == a.max
        && rotate == a.rotate
        && mask == a.mask
        && add == a.add
        && push == a.push
        && update_with_top == a.update_with_top;
    }
bool IR::BFN::ParserCounterLoadPkt::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserCounterPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ParserCounterLoadPkt &>(a_);
        return (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr)
        && max == a.max
        && rotate == a.rotate
        && mask == a.mask
        && add == a.add
        && push == a.push
        && update_with_top == a.update_with_top;
    }
void IR::BFN::ParserCounterLoadPkt::visit_children(Visitor & v, char const * n) {
    (void)n;
    ParserCounterPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::ParserCounterLoadPkt::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ParserCounterPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::ParserCounterLoadPkt::validate() const {
#line 18893 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source); }
void IR::BFN::ParserCounterLoadPkt::dump_fields(std::ostream & out) const {
    ParserCounterPrimitive::dump_fields(out);
        out << " push=" << push;
        out << " update_with_top=" << update_with_top;
}
void IR::BFN::ParserCounterLoadPkt::toJSON(JSONGenerator & json) const {
    ParserCounterPrimitive::toJSON(json);
    json.emit("source", source);
    json.emit("max", max);
    json.emit("rotate", rotate);
    json.emit("mask", mask);
    json.emit("add", add);
    json.emit("push", push);
    json.emit("update_with_top", update_with_top);
}
IR::BFN::ParserCounterLoadPkt::ParserCounterLoadPkt(JSONLoader & json) : ParserCounterPrimitive(json) {
    json.load("source", source);
    json.load("max", max);
    json.load("rotate", rotate);
    json.load("mask", mask);
    json.load("add", add);
    json.load("push", push);
    json.load("update_with_top", update_with_top);
}
IR::Node * IR::BFN::ParserCounterLoadPkt::fromJSON(JSONLoader & json) { return new ParserCounterLoadPkt(json); }
bool IR::BFN::ParserCounterLoadPkt::operator==(IR::BFN::ParserCounterPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserCounterLoadPkt::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserCounterLoadPkt::operator==(IR::Node const & a) const { return a == *this; }
#line 283 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserCounterIncrement::dbprint(std::ostream & out) const { out << "parser counter inc " << value; }
#line 18926 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserCounterIncrement::operator==(IR::BFN::ParserCounterIncrement const & a) const {
        return ParserCounterPrimitive::operator==(static_cast<const ParserCounterPrimitive &>(a))
        && value == a.value;
    }
bool IR::BFN::ParserCounterIncrement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserCounterPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ParserCounterIncrement &>(a_);
        return value == a.value;
    }
void IR::BFN::ParserCounterIncrement::dump_fields(std::ostream & out) const {
    ParserCounterPrimitive::dump_fields(out);
        out << " value=" << value;
}
void IR::BFN::ParserCounterIncrement::toJSON(JSONGenerator & json) const {
    ParserCounterPrimitive::toJSON(json);
    json.emit("value", value);
}
IR::BFN::ParserCounterIncrement::ParserCounterIncrement(JSONLoader & json) : ParserCounterPrimitive(json) {
    json.load("value", value);
}
IR::Node * IR::BFN::ParserCounterIncrement::fromJSON(JSONLoader & json) { return new ParserCounterIncrement(json); }
bool IR::BFN::ParserCounterIncrement::operator==(IR::BFN::ParserCounterPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserCounterIncrement::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserCounterIncrement::operator==(IR::Node const & a) const { return a == *this; }
#line 288 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserCounterDecrement::dbprint(std::ostream & out) const { out << "parser counter dec " << value; }
#line 18954 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserCounterDecrement::operator==(IR::BFN::ParserCounterDecrement const & a) const {
        return ParserCounterPrimitive::operator==(static_cast<const ParserCounterPrimitive &>(a))
        && value == a.value;
    }
bool IR::BFN::ParserCounterDecrement::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserCounterPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ParserCounterDecrement &>(a_);
        return value == a.value;
    }
void IR::BFN::ParserCounterDecrement::dump_fields(std::ostream & out) const {
    ParserCounterPrimitive::dump_fields(out);
        out << " value=" << value;
}
void IR::BFN::ParserCounterDecrement::toJSON(JSONGenerator & json) const {
    ParserCounterPrimitive::toJSON(json);
    json.emit("value", value);
}
IR::BFN::ParserCounterDecrement::ParserCounterDecrement(JSONLoader & json) : ParserCounterPrimitive(json) {
    json.load("value", value);
}
IR::Node * IR::BFN::ParserCounterDecrement::fromJSON(JSONLoader & json) { return new ParserCounterDecrement(json); }
bool IR::BFN::ParserCounterDecrement::operator==(IR::BFN::ParserCounterPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserCounterDecrement::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserCounterDecrement::operator==(IR::Node const & a) const { return a == *this; }
#line 293 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserCounterPop::dbprint(std::ostream & out) const { out << "parser counter pop"; }
#line 18982 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserCounterPop::operator==(IR::BFN::ParserCounterPop const & a) const {
        return ParserCounterPrimitive::operator==(static_cast<const ParserCounterPrimitive &>(a));
    }
bool IR::BFN::ParserCounterPop::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserCounterPrimitive::equiv(a_)) return false;
        return true;
    }
void IR::BFN::ParserCounterPop::toJSON(JSONGenerator & json) const {
    ParserCounterPrimitive::toJSON(json);
}
IR::BFN::ParserCounterPop::ParserCounterPop(JSONLoader & json) : ParserCounterPrimitive(json) {
}
IR::Node * IR::BFN::ParserCounterPop::fromJSON(JSONLoader & json) { return new ParserCounterPop(json); }
bool IR::BFN::ParserCounterPop::operator==(IR::BFN::ParserCounterPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserCounterPop::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserCounterPop::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::ParserCounterRVal::operator==(IR::BFN::ParserCounterRVal const & a) const {
        return ParserRVal::operator==(static_cast<const ParserRVal &>(a))
        && declName == a.declName;
    }
bool IR::BFN::ParserCounterRVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserRVal::equiv(a_)) return false;
        auto &a = static_cast<const ParserCounterRVal &>(a_);
        return declName == a.declName;
    }
void IR::BFN::ParserCounterRVal::dump_fields(std::ostream & out) const {
    ParserRVal::dump_fields(out);
        out << " declName=" << declName;
}
void IR::BFN::ParserCounterRVal::toJSON(JSONGenerator & json) const {
    ParserRVal::toJSON(json);
    json.emit("declName", declName);
}
IR::BFN::ParserCounterRVal::ParserCounterRVal(JSONLoader & json) : ParserRVal(json) {
    json.load("declName", declName);
}
bool IR::BFN::ParserCounterRVal::operator==(IR::BFN::ParserRVal const & a) const { return a == *this; }
bool IR::BFN::ParserCounterRVal::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::ParserCounterRVal::operator==(IR::Node const & a) const { return a == *this; }
#line 301 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserCounterIsZero::dbprint(std::ostream & out) const { out << "parser counter is zero? "; }
#line 19026 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserCounterIsZero::operator==(IR::BFN::ParserCounterIsZero const & a) const {
        return ParserCounterRVal::operator==(static_cast<const ParserCounterRVal &>(a));
    }
bool IR::BFN::ParserCounterIsZero::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserCounterRVal::equiv(a_)) return false;
        return true;
    }
void IR::BFN::ParserCounterIsZero::toJSON(JSONGenerator & json) const {
    ParserCounterRVal::toJSON(json);
}
IR::BFN::ParserCounterIsZero::ParserCounterIsZero(JSONLoader & json) : ParserCounterRVal(json) {
}
IR::Node * IR::BFN::ParserCounterIsZero::fromJSON(JSONLoader & json) { return new ParserCounterIsZero(json); }
bool IR::BFN::ParserCounterIsZero::operator==(IR::BFN::ParserCounterRVal const & a) const { return a == *this; }
bool IR::BFN::ParserCounterIsZero::operator==(IR::BFN::ParserRVal const & a) const { return a == *this; }
bool IR::BFN::ParserCounterIsZero::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::ParserCounterIsZero::operator==(IR::Node const & a) const { return a == *this; }
#line 305 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserCounterIsNegative::dbprint(std::ostream & out) const { out << "parser counter is negative? "; }
#line 19047 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserCounterIsNegative::operator==(IR::BFN::ParserCounterIsNegative const & a) const {
        return ParserCounterRVal::operator==(static_cast<const ParserCounterRVal &>(a));
    }
bool IR::BFN::ParserCounterIsNegative::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserCounterRVal::equiv(a_)) return false;
        return true;
    }
void IR::BFN::ParserCounterIsNegative::toJSON(JSONGenerator & json) const {
    ParserCounterRVal::toJSON(json);
}
IR::BFN::ParserCounterIsNegative::ParserCounterIsNegative(JSONLoader & json) : ParserCounterRVal(json) {
}
IR::Node * IR::BFN::ParserCounterIsNegative::fromJSON(JSONLoader & json) { return new ParserCounterIsNegative(json); }
bool IR::BFN::ParserCounterIsNegative::operator==(IR::BFN::ParserCounterRVal const & a) const { return a == *this; }
bool IR::BFN::ParserCounterIsNegative::operator==(IR::BFN::ParserRVal const & a) const { return a == *this; }
bool IR::BFN::ParserCounterIsNegative::operator==(IR::Expression const & a) const { return a == *this; }
bool IR::BFN::ParserCounterIsNegative::operator==(IR::Node const & a) const { return a == *this; }
#line 309 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserPrioritySet::dbprint(std::ostream & out) const { out << "priority set " << val; }
#line 19068 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserPrioritySet::operator==(IR::BFN::ParserPrioritySet const & a) const {
        return ParserPrimitive::operator==(static_cast<const ParserPrimitive &>(a))
        && val == a.val;
    }
bool IR::BFN::ParserPrioritySet::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ParserPrioritySet &>(a_);
        return val == a.val;
    }
void IR::BFN::ParserPrioritySet::dump_fields(std::ostream & out) const {
    ParserPrimitive::dump_fields(out);
        out << " val=" << val;
}
void IR::BFN::ParserPrioritySet::toJSON(JSONGenerator & json) const {
    ParserPrimitive::toJSON(json);
    json.emit("val", val);
}
IR::BFN::ParserPrioritySet::ParserPrioritySet(JSONLoader & json) : ParserPrimitive(json) {
    json.load("val", val);
}
IR::Node * IR::BFN::ParserPrioritySet::fromJSON(JSONLoader & json) { return new ParserPrioritySet(json); }
bool IR::BFN::ParserPrioritySet::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserPrioritySet::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::ParserChecksumPrimitive::operator==(IR::BFN::ParserChecksumPrimitive const & a) const {
        return ParserPrimitive::operator==(static_cast<const ParserPrimitive &>(a))
        && declName == a.declName;
    }
bool IR::BFN::ParserChecksumPrimitive::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ParserChecksumPrimitive &>(a_);
        return declName == a.declName;
    }
void IR::BFN::ParserChecksumPrimitive::dump_fields(std::ostream & out) const {
    ParserPrimitive::dump_fields(out);
        out << " declName=" << declName;
}
void IR::BFN::ParserChecksumPrimitive::toJSON(JSONGenerator & json) const {
    ParserPrimitive::toJSON(json);
    json.emit("declName", declName);
}
IR::BFN::ParserChecksumPrimitive::ParserChecksumPrimitive(JSONLoader & json) : ParserPrimitive(json) {
    json.load("declName", declName);
}
bool IR::BFN::ParserChecksumPrimitive::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserChecksumPrimitive::operator==(IR::Node const & a) const { return a == *this; }

#line 319 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
const IR::BFN::FieldLVal *IR::BFN::ParserChecksumWritePrimitive::getWriteDest() const {
        BUG("calling getWriteDest from abtract base, this should have beed overwritten");
    }
#line 19121 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 323 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::ParserWriteMode IR::BFN::ParserChecksumWritePrimitive::getWriteMode() const { return write_mode; }
#line 19124 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserChecksumWritePrimitive::operator==(IR::BFN::ParserChecksumWritePrimitive const & a) const {
        return ParserChecksumPrimitive::operator==(static_cast<const ParserChecksumPrimitive &>(a))
        && write_mode == a.write_mode;
    }
bool IR::BFN::ParserChecksumWritePrimitive::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserChecksumPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ParserChecksumWritePrimitive &>(a_);
        return write_mode == a.write_mode;
    }
void IR::BFN::ParserChecksumWritePrimitive::dump_fields(std::ostream & out) const {
    ParserChecksumPrimitive::dump_fields(out);
        out << " write_mode=" << write_mode;
}
void IR::BFN::ParserChecksumWritePrimitive::toJSON(JSONGenerator & json) const {
    ParserChecksumPrimitive::toJSON(json);
    json.emit("write_mode", write_mode);
}
IR::BFN::ParserChecksumWritePrimitive::ParserChecksumWritePrimitive(JSONLoader & json) : ParserChecksumPrimitive(json) {
    json.load("write_mode", write_mode);
}
bool IR::BFN::ParserChecksumWritePrimitive::operator==(IR::BFN::ParserChecksumPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserChecksumWritePrimitive::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ParserChecksumWritePrimitive::operator==(IR::Node const & a) const { return a == *this; }
#line 329 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ChecksumVerify::dbprint(std::ostream & out) const { out << "verify checksum " << declName; }
#line 19151 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 331 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
const IR::BFN::FieldLVal *IR::BFN::ChecksumVerify::getWriteDest() const { return dest; }
#line 19154 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ChecksumVerify::operator==(IR::BFN::ChecksumVerify const & a) const {
        return ParserChecksumWritePrimitive::operator==(static_cast<const ParserChecksumWritePrimitive &>(a))
        && dest == a.dest;
    }
bool IR::BFN::ChecksumVerify::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserChecksumWritePrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ChecksumVerify &>(a_);
        return (dest ? a.dest ? dest->equiv(*a.dest) : false : a.dest == nullptr);
    }
void IR::BFN::ChecksumVerify::visit_children(Visitor & v, char const * n) {
    (void)n;
    ParserChecksumWritePrimitive::visit_children(v, n);
    v.visit(dest, "dest");
}
void IR::BFN::ChecksumVerify::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ParserChecksumWritePrimitive::visit_children(v, n);
    v.visit(dest, "dest");
}
void IR::BFN::ChecksumVerify::toJSON(JSONGenerator & json) const {
    ParserChecksumWritePrimitive::toJSON(json);
    if (dest != nullptr) json.emit("dest", dest);
}
IR::BFN::ChecksumVerify::ChecksumVerify(JSONLoader & json) : ParserChecksumWritePrimitive(json) {
    json.load("dest", dest);
}
IR::Node * IR::BFN::ChecksumVerify::fromJSON(JSONLoader & json) { return new ChecksumVerify(json); }
bool IR::BFN::ChecksumVerify::operator==(IR::BFN::ParserChecksumWritePrimitive const & a) const { return a == *this; }
bool IR::BFN::ChecksumVerify::operator==(IR::BFN::ParserChecksumPrimitive const & a) const { return a == *this; }
bool IR::BFN::ChecksumVerify::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ChecksumVerify::operator==(IR::Node const & a) const { return a == *this; }
#line 337 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ChecksumResidualDeposit::dbprint(std::ostream & out) const { out << "subtract and deposit checksum " << declName; }
#line 19189 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 339 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
const IR::BFN::FieldLVal *IR::BFN::ChecksumResidualDeposit::getWriteDest() const { return dest; }
#line 19192 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

bool IR::BFN::ChecksumResidualDeposit::operator==(IR::BFN::ChecksumResidualDeposit const & a) const {
        return ParserChecksumWritePrimitive::operator==(static_cast<const ParserChecksumWritePrimitive &>(a))
        && dest == a.dest
        && header_end_byte == a.header_end_byte;
    }
bool IR::BFN::ChecksumResidualDeposit::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserChecksumWritePrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ChecksumResidualDeposit &>(a_);
        return (dest ? a.dest ? dest->equiv(*a.dest) : false : a.dest == nullptr)
        && (header_end_byte ? a.header_end_byte ? header_end_byte->equiv(*a.header_end_byte) : false : a.header_end_byte == nullptr);
    }
void IR::BFN::ChecksumResidualDeposit::visit_children(Visitor & v, char const * n) {
    (void)n;
    ParserChecksumWritePrimitive::visit_children(v, n);
    v.visit(dest, "dest");
    v.visit(header_end_byte, "header_end_byte");
}
void IR::BFN::ChecksumResidualDeposit::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ParserChecksumWritePrimitive::visit_children(v, n);
    v.visit(dest, "dest");
    v.visit(header_end_byte, "header_end_byte");
}
void IR::BFN::ChecksumResidualDeposit::validate() const {
#line 19219 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(dest);
        CHECK_NULL(header_end_byte); }
void IR::BFN::ChecksumResidualDeposit::toJSON(JSONGenerator & json) const {
    ParserChecksumWritePrimitive::toJSON(json);
    json.emit("dest", dest);
    json.emit("header_end_byte", header_end_byte);
}
IR::BFN::ChecksumResidualDeposit::ChecksumResidualDeposit(JSONLoader & json) : ParserChecksumWritePrimitive(json) {
    json.load("dest", dest);
    json.load("header_end_byte", header_end_byte);
}
IR::Node * IR::BFN::ChecksumResidualDeposit::fromJSON(JSONLoader & json) { return new ChecksumResidualDeposit(json); }
bool IR::BFN::ChecksumResidualDeposit::operator==(IR::BFN::ParserChecksumWritePrimitive const & a) const { return a == *this; }
bool IR::BFN::ChecksumResidualDeposit::operator==(IR::BFN::ParserChecksumPrimitive const & a) const { return a == *this; }
bool IR::BFN::ChecksumResidualDeposit::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ChecksumResidualDeposit::operator==(IR::Node const & a) const { return a == *this; }
#line 347 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ChecksumDepositToClot::dbprint(std::ostream & out) const { out << "deposit checksum " << declName << " to clot " << clot->tag; }
#line 19239 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ChecksumDepositToClot::operator==(IR::BFN::ChecksumDepositToClot const & a) const {
        return ParserChecksumPrimitive::operator==(static_cast<const ParserChecksumPrimitive &>(a));
    }
bool IR::BFN::ChecksumDepositToClot::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserChecksumPrimitive::equiv(a_)) return false;
        return true;
    }
void IR::BFN::ChecksumDepositToClot::toJSON(JSONGenerator & json) const {
    ParserChecksumPrimitive::toJSON(json);
}
IR::BFN::ChecksumDepositToClot::ChecksumDepositToClot(JSONLoader & json) : ParserChecksumPrimitive(json) {
}
IR::Node * IR::BFN::ChecksumDepositToClot::fromJSON(JSONLoader & json) { return new ChecksumDepositToClot(json); }
bool IR::BFN::ChecksumDepositToClot::operator==(IR::BFN::ParserChecksumPrimitive const & a) const { return a == *this; }
bool IR::BFN::ChecksumDepositToClot::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ChecksumDepositToClot::operator==(IR::Node const & a) const { return a == *this; }
#line 355 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ChecksumAdd::dbprint(std::ostream & out) const { out << "add " << source << " to checksum " << declName; }
#line 19259 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

bool IR::BFN::ChecksumAdd::operator==(IR::BFN::ChecksumAdd const & a) const {
        return ParserChecksumPrimitive::operator==(static_cast<const ParserChecksumPrimitive &>(a))
        && source == a.source
        && swap == a.swap
        && isHeaderChecksum == a.isHeaderChecksum;
    }
bool IR::BFN::ChecksumAdd::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserChecksumPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ChecksumAdd &>(a_);
        return (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr)
        && swap == a.swap
        && isHeaderChecksum == a.isHeaderChecksum;
    }
void IR::BFN::ChecksumAdd::visit_children(Visitor & v, char const * n) {
    (void)n;
    ParserChecksumPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::ChecksumAdd::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ParserChecksumPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::ChecksumAdd::validate() const {
#line 19286 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source); }
void IR::BFN::ChecksumAdd::dump_fields(std::ostream & out) const {
    ParserChecksumPrimitive::dump_fields(out);
        out << " swap=" << swap;
        out << " isHeaderChecksum=" << isHeaderChecksum;
}
void IR::BFN::ChecksumAdd::toJSON(JSONGenerator & json) const {
    ParserChecksumPrimitive::toJSON(json);
    json.emit("source", source);
    json.emit("swap", swap);
    json.emit("isHeaderChecksum", isHeaderChecksum);
}
IR::BFN::ChecksumAdd::ChecksumAdd(JSONLoader & json) : ParserChecksumPrimitive(json) {
    json.load("source", source);
    json.load("swap", swap);
    json.load("isHeaderChecksum", isHeaderChecksum);
}
IR::Node * IR::BFN::ChecksumAdd::fromJSON(JSONLoader & json) { return new ChecksumAdd(json); }
bool IR::BFN::ChecksumAdd::operator==(IR::BFN::ParserChecksumPrimitive const & a) const { return a == *this; }
bool IR::BFN::ChecksumAdd::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ChecksumAdd::operator==(IR::Node const & a) const { return a == *this; }
#line 363 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ChecksumSubtract::dbprint(std::ostream & out) const { out << "subtract " << source << " from checksum " << declName; }
#line 19311 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

bool IR::BFN::ChecksumSubtract::operator==(IR::BFN::ChecksumSubtract const & a) const {
        return ParserChecksumPrimitive::operator==(static_cast<const ParserChecksumPrimitive &>(a))
        && source == a.source
        && swap == a.swap
        && isPayloadChecksum == a.isPayloadChecksum;
    }
bool IR::BFN::ChecksumSubtract::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserChecksumPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const ChecksumSubtract &>(a_);
        return (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr)
        && swap == a.swap
        && isPayloadChecksum == a.isPayloadChecksum;
    }
void IR::BFN::ChecksumSubtract::visit_children(Visitor & v, char const * n) {
    (void)n;
    ParserChecksumPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::ChecksumSubtract::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ParserChecksumPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::ChecksumSubtract::validate() const {
#line 19338 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source); }
void IR::BFN::ChecksumSubtract::dump_fields(std::ostream & out) const {
    ParserChecksumPrimitive::dump_fields(out);
        out << " swap=" << swap;
        out << " isPayloadChecksum=" << isPayloadChecksum;
}
void IR::BFN::ChecksumSubtract::toJSON(JSONGenerator & json) const {
    ParserChecksumPrimitive::toJSON(json);
    json.emit("source", source);
    json.emit("swap", swap);
    json.emit("isPayloadChecksum", isPayloadChecksum);
}
IR::BFN::ChecksumSubtract::ChecksumSubtract(JSONLoader & json) : ParserChecksumPrimitive(json) {
    json.load("source", source);
    json.load("swap", swap);
    json.load("isPayloadChecksum", isPayloadChecksum);
}
IR::Node * IR::BFN::ChecksumSubtract::fromJSON(JSONLoader & json) { return new ChecksumSubtract(json); }
bool IR::BFN::ChecksumSubtract::operator==(IR::BFN::ParserChecksumPrimitive const & a) const { return a == *this; }
bool IR::BFN::ChecksumSubtract::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::ChecksumSubtract::operator==(IR::Node const & a) const { return a == *this; }
#line 372 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::HdrLenIncStop::dbprint(std::ostream & out) const { out << "hdr_len_inc_stop " << source; }
#line 19363 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::HdrLenIncStop::operator==(IR::BFN::HdrLenIncStop const & a) const {
        return ParserPrimitive::operator==(static_cast<const ParserPrimitive &>(a))
        && source == a.source;
    }
bool IR::BFN::HdrLenIncStop::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const HdrLenIncStop &>(a_);
        return (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr);
    }
void IR::BFN::HdrLenIncStop::visit_children(Visitor & v, char const * n) {
    (void)n;
    ParserPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::HdrLenIncStop::visit_children(Visitor & v, char const * n) const {
    (void)n;
    ParserPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::HdrLenIncStop::validate() const {
#line 19385 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source); }
void IR::BFN::HdrLenIncStop::toJSON(JSONGenerator & json) const {
    ParserPrimitive::toJSON(json);
    json.emit("source", source);
}
IR::BFN::HdrLenIncStop::HdrLenIncStop(JSONLoader & json) : ParserPrimitive(json) {
    json.load("source", source);
}
IR::Node * IR::BFN::HdrLenIncStop::fromJSON(JSONLoader & json) { return new HdrLenIncStop(json); }
bool IR::BFN::HdrLenIncStop::operator==(IR::BFN::ParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::HdrLenIncStop::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::ParserMatchValue::operator==(IR::BFN::ParserMatchValue const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::BFN::ParserMatchValue::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::BFN::ParserMatchValue::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::BFN::ParserMatchValue::ParserMatchValue(JSONLoader & json) : Node(json) {
}
bool IR::BFN::ParserMatchValue::operator==(IR::Node const & a) const { return a == *this; }
#line 380 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserConstMatchValue::dbprint(std::ostream & out) const { out << "const: " << value; }
#line 19414 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 381 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
cstring IR::BFN::ParserConstMatchValue::toString() const { std::stringstream ss; ss << value; return cstring(ss.str()); }
#line 19417 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserConstMatchValue::operator==(IR::BFN::ParserConstMatchValue const & a) const {
        return ParserMatchValue::operator==(static_cast<const ParserMatchValue &>(a))
        && value == a.value;
    }
bool IR::BFN::ParserConstMatchValue::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserMatchValue::equiv(a_)) return false;
        auto &a = static_cast<const ParserConstMatchValue &>(a_);
        return value == a.value;
    }
void IR::BFN::ParserConstMatchValue::dump_fields(std::ostream & out) const {
    ParserMatchValue::dump_fields(out);
        out << " value=" << value;
}
void IR::BFN::ParserConstMatchValue::toJSON(JSONGenerator & json) const {
    ParserMatchValue::toJSON(json);
    json.emit("value", value);
}
IR::BFN::ParserConstMatchValue::ParserConstMatchValue(JSONLoader & json) : ParserMatchValue(json) {
    json.load("value", value);
}
IR::Node * IR::BFN::ParserConstMatchValue::fromJSON(JSONLoader & json) { return new ParserConstMatchValue(json); }
bool IR::BFN::ParserConstMatchValue::operator==(IR::BFN::ParserMatchValue const & a) const { return a == *this; }
bool IR::BFN::ParserConstMatchValue::operator==(IR::Node const & a) const { return a == *this; }
#line 393 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserPvsMatchValue::dbprint(std::ostream & out) const { out << "name: " << name << " " << size; }
#line 19444 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 394 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
cstring IR::BFN::ParserPvsMatchValue::toString() const { return name; }
#line 19447 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::ParserPvsMatchValue::operator==(IR::BFN::ParserPvsMatchValue const & a) const {
        return ParserMatchValue::operator==(static_cast<const ParserMatchValue &>(a))
        && name == a.name
        && size == a.size;
    }
bool IR::BFN::ParserPvsMatchValue::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!ParserMatchValue::equiv(a_)) return false;
        auto &a = static_cast<const ParserPvsMatchValue &>(a_);
        return name == a.name
        && size == a.size;
    }
void IR::BFN::ParserPvsMatchValue::dump_fields(std::ostream & out) const {
    ParserMatchValue::dump_fields(out);
        out << " name=" << name;
        out << " size=" << size;
}
void IR::BFN::ParserPvsMatchValue::toJSON(JSONGenerator & json) const {
    ParserMatchValue::toJSON(json);
    json.emit("name", name);
    json.emit("size", size);
}
IR::BFN::ParserPvsMatchValue::ParserPvsMatchValue(JSONLoader & json) : ParserMatchValue(json) {
    json.load("name", name);
    json.load("size", size);
}
IR::Node * IR::BFN::ParserPvsMatchValue::fromJSON(JSONLoader & json) { return new ParserPvsMatchValue(json); }
bool IR::BFN::ParserPvsMatchValue::operator==(IR::BFN::ParserMatchValue const & a) const { return a == *this; }
bool IR::BFN::ParserPvsMatchValue::operator==(IR::Node const & a) const { return a == *this; }

#line 410 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::Phase0::dbprint(std::ostream & out) const {
        out << "phase0: tableName: " << tableName
                << " size: " << size << " handle: 0x " << hex(handle) << std::endl;
        out << "actionName: " << actionName << " keyName: " << keyName << " fields: " << fields; }
#line 19483 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::Phase0::operator==(IR::BFN::Phase0 const & a) const {
        return this->typeId() == a.typeId()
        && fields == a.fields
        && size == a.size
        && handle == a.handle
        && tableName == a.tableName
        && actionName == a.actionName
        && keyName == a.keyName
        && namedByAnnotation == a.namedByAnnotation;
    }
bool IR::BFN::Phase0::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Phase0 &>(a_);
        return (fields ? a.fields ? fields->equiv(*a.fields) : false : a.fields == nullptr)
        && size == a.size
        && handle == a.handle
        && tableName == a.tableName
        && actionName == a.actionName
        && keyName == a.keyName
        && namedByAnnotation == a.namedByAnnotation;
    }
void IR::BFN::Phase0::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(fields, "fields");
}
void IR::BFN::Phase0::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(fields, "fields");
}
void IR::BFN::Phase0::validate() const {
#line 19517 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(fields); }
void IR::BFN::Phase0::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " size=" << size;
        out << " handle=" << handle;
        out << " tableName=" << tableName;
        out << " actionName=" << actionName;
        out << " keyName=" << keyName;
        out << " namedByAnnotation=" << namedByAnnotation;
}
void IR::BFN::Phase0::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("fields", fields);
    json.emit("size", size);
    json.emit("handle", handle);
    json.emit("tableName", tableName);
    json.emit("actionName", actionName);
    json.emit("keyName", keyName);
    json.emit("namedByAnnotation", namedByAnnotation);
}
IR::BFN::Phase0::Phase0(JSONLoader & json) : Node(json) {
    json.load("fields", fields);
    json.load("size", size);
    json.load("handle", handle);
    json.load("tableName", tableName);
    json.load("actionName", actionName);
    json.load("keyName", keyName);
    json.load("namedByAnnotation", namedByAnnotation);
}
IR::Node * IR::BFN::Phase0::fromJSON(JSONLoader & json) { return new Phase0(json); }
bool IR::BFN::Phase0::operator==(IR::Node const & a) const { return a == *this; }
#line 418 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::Transition::validate() const {
#line 19552 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(value);
        saves.validate();
#line 418 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
{
        if (next && loop) {
            BUG("Transition cannot have both loop and non-loop next states");
        }
    } }
#line 19562 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The owning state's select expressions evaluate to a value which
/// matches this...
/// number of bytes to shift out of the input buffer
/// ...and transition to this state (or, if null, end the parser program).
/// The save to registers need to perform for this transition.
/// registers to scratch pad on write
bool IR::BFN::Transition::operator==(IR::BFN::Transition const & a) const {
        return this->typeId() == a.typeId()
        && value == a.value
        && shift == a.shift
        && next == a.next
        && saves == a.saves
        && scratches == a.scratches;
    }
bool IR::BFN::Transition::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Transition &>(a_);
        return (value ? a.value ? value->equiv(*a.value) : false : a.value == nullptr)
        && shift == a.shift
        && (next ? a.next ? next->equiv(*a.next) : false : a.next == nullptr)
        && saves.equiv(a.saves)
        && scratches == a.scratches;
    }
void IR::BFN::Transition::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(value, "value");
    v.visit(next, "next");
    saves.visit_children(v, "saves");
}
void IR::BFN::Transition::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(value, "value");
    v.visit(next, "next");
    saves.visit_children(v, "saves");
}
void IR::BFN::Transition::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " shift=" << shift;
}
void IR::BFN::Transition::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("value", value);
    json.emit("shift", shift);
    if (next != nullptr) json.emit("next", next);
    json.emit("saves", saves);
    json.emit("scratches", scratches);
}
IR::BFN::Transition::Transition(JSONLoader & json) : Node(json) {
    json.load("value", value);
    json.load("shift", shift);
    json.load("next", next);
    json.load("saves", saves);
    json.load("scratches", scratches);
}
IR::Node * IR::BFN::Transition::fromJSON(JSONLoader & json) { return new Transition(json); }
bool IR::BFN::Transition::operator==(IR::Node const & a) const { return a == *this; }
#line 458 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::Select::Select(const IR::BFN::ParserRVal* source, const IR::Expression* p4Source) : source(source), p4Source(p4Source) { }
#line 19624 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 461 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::Select::validate() const {
#line 19627 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source);
#line 461 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
{
        CHECK_NULL(source);
        BUG_CHECK(source->is<IR::BFN::SavedRVal>() ||
                  source->is<IR::BFN::ParserCounterRVal>(),
                  "Select source can only be match register or parser counter: %1%", source);
    } }
#line 19637 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The value to match against.
bool IR::BFN::Select::operator==(IR::BFN::Select const & a) const {
        return this->typeId() == a.typeId()
        && source == a.source;
    }
bool IR::BFN::Select::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Select &>(a_);
        return (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr);
    }
void IR::BFN::Select::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::Select::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::Select::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("source", source);
}
IR::BFN::Select::Select(JSONLoader & json) : Node(json) {
    json.load("source", source);
}
IR::Node * IR::BFN::Select::fromJSON(JSONLoader & json) { return new Select(json); }
bool IR::BFN::Select::operator==(IR::Node const & a) const { return a == *this; }
#line 480 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::ParserState::ParserState(const IR::ParserState* p4State, cstring name, gress_t gr) : name(name), gress(gr) { if (p4State) p4States.emplace(p4State); }
#line 19670 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 482 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::ParserState::ParserState(std::set<IR::ParserState const *> p4States, cstring name, gress_t gr) : p4States(p4States), name(name), gress(gr) { }
#line 19673 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 485 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
int IR::BFN::ParserState::stage() const { return -1; }
#line 19676 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 486 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
gress_t IR::BFN::ParserState::thread() const { return gress; }
#line 19679 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 487 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
cstring IR::BFN::ParserState::toString() const { return name; }
#line 19682 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 488 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserState::visit_children(Visitor & v, char const * n) {
        statements.visit_children(v, n);
        selects.visit_children(v, n);
        transitions.parallel_visit_children(v, n); }
#line 488 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ParserState::visit_children(Visitor & v, char const * n) const {
        statements.visit_children(v, n);
        selects.visit_children(v, n);
        transitions.parallel_visit_children(v, n); }
#line 19693 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 492 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::ParserState const * IR::BFN::ParserState::p4State() const {
        BUG_CHECK(p4States.size() == 1, "Expecing exactly one state in p4States but saw %1%",
                  p4States.size());
        return *p4States.begin();
    }
#line 19700 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// If true, extract at stridded offset on state re-entry

bool IR::BFN::ParserState::operator==(IR::BFN::ParserState const & a) const {
        return Unit::operator==(static_cast<const Unit &>(a))
        && name == a.name
        && gress == a.gress
        && stride == a.stride
        && dontMerge == a.dontMerge
        && statements == a.statements
        && selects == a.selects
        && transitions == a.transitions;
    }
bool IR::BFN::ParserState::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Unit::equiv(a_)) return false;
        auto &a = static_cast<const ParserState &>(a_);
        return name == a.name
        && gress == a.gress
        && stride == a.stride
        && dontMerge == a.dontMerge
        && statements.equiv(a.statements)
        && selects.equiv(a.selects)
        && transitions.equiv(a.transitions);
    }
void IR::BFN::ParserState::validate() const {
#line 19726 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        statements.validate();
        selects.validate();
        transitions.validate(); }
void IR::BFN::ParserState::dump_fields(std::ostream & out) const {
    Unit::dump_fields(out);
        out << " name=" << name;
        out << " gress=" << gress;
        out << " stride=" << stride;
        out << " dontMerge=" << dontMerge;
}
void IR::BFN::ParserState::toJSON(JSONGenerator & json) const {
    Unit::toJSON(json);
    json.emit("name", name);
    json.emit("gress", gress);
    json.emit("stride", stride);
    json.emit("dontMerge", dontMerge);
    json.emit("statements", statements);
    json.emit("selects", selects);
    json.emit("transitions", transitions);
}
IR::BFN::ParserState::ParserState(JSONLoader & json) : Unit(json) {
    json.load("name", name);
    json.load("gress", gress);
    json.load("stride", stride);
    json.load("dontMerge", dontMerge);
    json.load("statements", statements);
    json.load("selects", selects);
    json.load("transitions", transitions);
}
IR::Node * IR::BFN::ParserState::fromJSON(JSONLoader & json) { return new ParserState(json); }
bool IR::BFN::ParserState::operator==(IR::BFN::Unit const & a) const { return a == *this; }
bool IR::BFN::ParserState::operator==(IR::Node const & a) const { return a == *this; }
IRNODE_DEFINE_APPLY_OVERLOAD(BFN::AbstractParser, , )
#line 526 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
int IR::BFN::AbstractParser::stage() const { return -1; }
#line 19763 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 527 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
gress_t IR::BFN::AbstractParser::thread() const { return gress; }
#line 19766 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::AbstractParser::operator==(IR::BFN::AbstractParser const & a) const {
        return Unit::operator==(static_cast<const Unit &>(a))
        && gress == a.gress;
    }
bool IR::BFN::AbstractParser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Unit::equiv(a_)) return false;
        auto &a = static_cast<const AbstractParser &>(a_);
        return gress == a.gress;
    }
void IR::BFN::AbstractParser::dump_fields(std::ostream & out) const {
    Unit::dump_fields(out);
        out << " gress=" << gress;
}
void IR::BFN::AbstractParser::toJSON(JSONGenerator & json) const {
    Unit::toJSON(json);
    json.emit("gress", gress);
}
IR::BFN::AbstractParser::AbstractParser(JSONLoader & json) : Unit(json) {
    json.load("gress", gress);
}
bool IR::BFN::AbstractParser::operator==(IR::BFN::Unit const & a) const { return a == *this; }
bool IR::BFN::AbstractParser::operator==(IR::Node const & a) const { return a == *this; }
IRNODE_DEFINE_APPLY_OVERLOAD(BFN::Parser, , )
#line 540 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
cstring IR::BFN::Parser::toString() const { return gress ? "egress parser"_cs : "ingress parser"_cs; }
#line 19793 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"


bool IR::BFN::Parser::operator==(IR::BFN::Parser const & a) const {
        return AbstractParser::operator==(static_cast<const AbstractParser &>(a))
        && start == a.start
        && name == a.name
        && pipeName == a.pipeName
        && phase0 == a.phase0
        && portmap == a.portmap;
    }
bool IR::BFN::Parser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!AbstractParser::equiv(a_)) return false;
        auto &a = static_cast<const Parser &>(a_);
        return (start ? a.start ? start->equiv(*a.start) : false : a.start == nullptr)
        && name == a.name
        && pipeName == a.pipeName
        && (phase0 ? a.phase0 ? phase0->equiv(*a.phase0) : false : a.phase0 == nullptr)
        && portmap == a.portmap;
    }
void IR::BFN::Parser::visit_children(Visitor & v, char const * n) {
    (void)n;
    AbstractParser::visit_children(v, n);
    v.visit(start, "start");
    v.visit(phase0, "phase0");
}
void IR::BFN::Parser::visit_children(Visitor & v, char const * n) const {
    (void)n;
    AbstractParser::visit_children(v, n);
    v.visit(start, "start");
    v.visit(phase0, "phase0");
}
void IR::BFN::Parser::validate() const {
#line 19827 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(start); }
void IR::BFN::Parser::dump_fields(std::ostream & out) const {
    AbstractParser::dump_fields(out);
        out << " name=" << name;
        out << " pipeName=" << pipeName;
}
void IR::BFN::Parser::toJSON(JSONGenerator & json) const {
    AbstractParser::toJSON(json);
    json.emit("start", start);
    json.emit("name", name);
    if (pipeName != nullptr) json.emit("pipeName", pipeName);
    if (phase0 != nullptr) json.emit("phase0", phase0);
    json.emit("portmap", portmap);
}
IR::BFN::Parser::Parser(JSONLoader & json) : AbstractParser(json) {
    json.load("start", start);
    json.load("name", name);
    json.load("pipeName", pipeName);
    json.load("phase0", phase0);
    json.load("portmap", portmap);
}
IR::Node * IR::BFN::Parser::fromJSON(JSONLoader & json) { return new Parser(json); }
bool IR::BFN::Parser::operator==(IR::BFN::AbstractParser const & a) const { return a == *this; }
bool IR::BFN::Parser::operator==(IR::BFN::Unit const & a) const { return a == *this; }
bool IR::BFN::Parser::operator==(IR::Node const & a) const { return a == *this; }
#line 556 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
cstring IR::BFN::GhostParser::toString() const { return "ghost parser"_cs; }
#line 19856 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"


bool IR::BFN::GhostParser::operator==(IR::BFN::GhostParser const & a) const {
        return AbstractParser::operator==(static_cast<const AbstractParser &>(a))
        && ghost_md == a.ghost_md
        && name == a.name
        && pipeName == a.pipeName;
    }
bool IR::BFN::GhostParser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!AbstractParser::equiv(a_)) return false;
        auto &a = static_cast<const GhostParser &>(a_);
        return (ghost_md ? a.ghost_md ? ghost_md->equiv(*a.ghost_md) : false : a.ghost_md == nullptr)
        && name == a.name
        && pipeName == a.pipeName;
    }
void IR::BFN::GhostParser::visit_children(Visitor & v, char const * n) {
    (void)n;
    AbstractParser::visit_children(v, n);
    v.visit(ghost_md, "ghost_md");
}
void IR::BFN::GhostParser::visit_children(Visitor & v, char const * n) const {
    (void)n;
    AbstractParser::visit_children(v, n);
    v.visit(ghost_md, "ghost_md");
}
void IR::BFN::GhostParser::validate() const {
#line 19884 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(ghost_md); }
void IR::BFN::GhostParser::dump_fields(std::ostream & out) const {
    AbstractParser::dump_fields(out);
        out << " name=" << name;
        out << " pipeName=" << pipeName;
}
void IR::BFN::GhostParser::toJSON(JSONGenerator & json) const {
    AbstractParser::toJSON(json);
    json.emit("ghost_md", ghost_md);
    json.emit("name", name);
    if (pipeName != nullptr) json.emit("pipeName", pipeName);
}
IR::BFN::GhostParser::GhostParser(JSONLoader & json) : AbstractParser(json) {
    json.load("ghost_md", ghost_md);
    json.load("name", name);
    json.load("pipeName", pipeName);
}
IR::Node * IR::BFN::GhostParser::fromJSON(JSONLoader & json) { return new GhostParser(json); }
bool IR::BFN::GhostParser::operator==(IR::BFN::AbstractParser const & a) const { return a == *this; }
bool IR::BFN::GhostParser::operator==(IR::BFN::Unit const & a) const { return a == *this; }
bool IR::BFN::GhostParser::operator==(IR::Node const & a) const { return a == *this; }
/// Index of this field list in the digest table
/// The fields to be copied into the buffer.
/// Learning needs control plane name for the field list.
bool IR::BFN::DigestFieldList::operator==(IR::BFN::DigestFieldList const & a) const {
        return this->typeId() == a.typeId()
        && idx == a.idx
        && sources == a.sources
        && type == a.type
        && controlPlaneName == a.controlPlaneName;
    }
bool IR::BFN::DigestFieldList::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const DigestFieldList &>(a_);
        return idx == a.idx
        && sources.equiv(a.sources)
        && (type ? a.type ? type->equiv(*a.type) : false : a.type == nullptr)
        && controlPlaneName == a.controlPlaneName;
    }
void IR::BFN::DigestFieldList::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    sources.visit_children(v, "sources");
    v.visit(type, "type");
}
void IR::BFN::DigestFieldList::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    sources.visit_children(v, "sources");
    v.visit(type, "type");
}
void IR::BFN::DigestFieldList::validate() const {
#line 19939 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        sources.validate(); }
void IR::BFN::DigestFieldList::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " idx=" << idx;
        out << " controlPlaneName=" << controlPlaneName;
}
void IR::BFN::DigestFieldList::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("idx", idx);
    json.emit("sources", sources);
    if (type != nullptr) json.emit("type", type);
    json.emit("controlPlaneName", controlPlaneName);
}
IR::BFN::DigestFieldList::DigestFieldList(JSONLoader & json) : Node(json) {
    json.load("idx", idx);
    json.load("sources", sources);
    json.load("type", type);
    json.load("controlPlaneName", controlPlaneName);
}
IR::Node * IR::BFN::DigestFieldList::fromJSON(JSONLoader & json) { return new DigestFieldList(json); }
bool IR::BFN::DigestFieldList::operator==(IR::Node const & a) const { return a == *this; }
#line 582 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::Digest::Digest(cstring name) : name(name) { }
#line 19964 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 583 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::Digest::Digest(cstring name, const IR::BFN::FieldLVal* selector) : name(name), selector(selector) { }
#line 19967 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 584 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::Digest::Digest(cstring name, const IR::Expression* selector) : name(name), selector(new FieldLVal(selector)) { }
#line 19970 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The name of the digest, identifying a special buffer to be filled in by
/// the deparser hardware. The name is chosen to match the corresponding
/// assembly directive, which in turn is named based on definitions in the
/// CSRs.
/// The field which controls which field list will be written into the
/// buffer. Selector is nullptr when only one field list is supported.
/// For example, tofino2 pktgen only supports one field list.
/// The available field lists
/// If present, the POV bit controlling whether the contents of the source
/// location are valid. Not used on Tofino.
bool IR::BFN::Digest::operator==(IR::BFN::Digest const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && selector == a.selector
        && fieldLists == a.fieldLists
        && povBit == a.povBit;
    }
bool IR::BFN::Digest::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const Digest &>(a_);
        return name == a.name
        && (selector ? a.selector ? selector->equiv(*a.selector) : false : a.selector == nullptr)
        && fieldLists.equiv(a.fieldLists)
        && (povBit ? a.povBit ? povBit->equiv(*a.povBit) : false : a.povBit == nullptr);
    }
void IR::BFN::Digest::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(selector, "selector");
    fieldLists.visit_children(v, "fieldLists");
    v.visit(povBit, "povBit");
}
void IR::BFN::Digest::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(selector, "selector");
    fieldLists.visit_children(v, "fieldLists");
    v.visit(povBit, "povBit");
}
void IR::BFN::Digest::validate() const {
#line 20012 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        fieldLists.validate(); }
void IR::BFN::Digest::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
}
void IR::BFN::Digest::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    if (selector != nullptr) json.emit("selector", selector);
    json.emit("fieldLists", fieldLists);
    if (povBit != nullptr) json.emit("povBit", povBit);
}
IR::BFN::Digest::Digest(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("selector", selector);
    json.load("fieldLists", fieldLists);
    json.load("povBit", povBit);
}
IR::Node * IR::BFN::Digest::fromJSON(JSONLoader & json) { return new Digest(json); }
bool IR::BFN::Digest::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::DeparserPrimitive::operator==(IR::BFN::DeparserPrimitive const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::BFN::DeparserPrimitive::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::BFN::DeparserPrimitive::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::BFN::DeparserPrimitive::DeparserPrimitive(JSONLoader & json) : Node(json) {
}
bool IR::BFN::DeparserPrimitive::operator==(IR::Node const & a) const { return a == *this; }
#line 612 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::Emit::Emit(const IR::Expression* povBit) : povBit(new FieldLVal(povBit)) { }
#line 20050 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 614 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::Emit::Emit(Util::SourceInfo si, const IR::Expression* povBit) : DeparserPrimitive(si), povBit(new FieldLVal(povBit)) { }
#line 20053 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The POV bit this emit is predicated on
bool IR::BFN::Emit::operator==(IR::BFN::Emit const & a) const {
        return DeparserPrimitive::operator==(static_cast<const DeparserPrimitive &>(a))
        && povBit == a.povBit;
    }
bool IR::BFN::Emit::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DeparserPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const Emit &>(a_);
        return (povBit ? a.povBit ? povBit->equiv(*a.povBit) : false : a.povBit == nullptr);
    }
void IR::BFN::Emit::visit_children(Visitor & v, char const * n) {
    (void)n;
    DeparserPrimitive::visit_children(v, n);
    v.visit(povBit, "povBit");
}
void IR::BFN::Emit::visit_children(Visitor & v, char const * n) const {
    (void)n;
    DeparserPrimitive::visit_children(v, n);
    v.visit(povBit, "povBit");
}
void IR::BFN::Emit::validate() const {
#line 20076 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(povBit); }
void IR::BFN::Emit::toJSON(JSONGenerator & json) const {
    DeparserPrimitive::toJSON(json);
    json.emit("povBit", povBit);
}
IR::BFN::Emit::Emit(JSONLoader & json) : DeparserPrimitive(json) {
    json.load("povBit", povBit);
}
bool IR::BFN::Emit::operator==(IR::BFN::DeparserPrimitive const & a) const { return a == *this; }
bool IR::BFN::Emit::operator==(IR::Node const & a) const { return a == *this; }
#line 625 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::EmitField::EmitField(const IR::Expression* source, const IR::Expression* povBit) : Emit(povBit), source(new FieldLVal(source)) { }
#line 20090 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 627 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::EmitField::EmitField(Util::SourceInfo si, const IR::Expression* source, const IR::Expression* povBit) : Emit(si, povBit), source(new FieldLVal(source)) { }
#line 20093 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 630 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::EmitField::dbprint(std::ostream & out) const {
        out << "emit " << source->field->toString()
            << " if " << povBit->field->toString();
    }
#line 20099 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The location to be copied into the output packet.
bool IR::BFN::EmitField::operator==(IR::BFN::EmitField const & a) const {
        return Emit::operator==(static_cast<const Emit &>(a))
        && source == a.source;
    }
bool IR::BFN::EmitField::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Emit::equiv(a_)) return false;
        auto &a = static_cast<const EmitField &>(a_);
        return (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr);
    }
void IR::BFN::EmitField::visit_children(Visitor & v, char const * n) {
    (void)n;
    Emit::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::EmitField::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Emit::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::EmitField::validate() const {
#line 20122 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source); }
void IR::BFN::EmitField::toJSON(JSONGenerator & json) const {
    Emit::toJSON(json);
    json.emit("source", source);
}
IR::BFN::EmitField::EmitField(JSONLoader & json) : Emit(json) {
    json.load("source", source);
}
IR::Node * IR::BFN::EmitField::fromJSON(JSONLoader & json) { return new EmitField(json); }
bool IR::BFN::EmitField::operator==(IR::BFN::Emit const & a) const { return a == *this; }
bool IR::BFN::EmitField::operator==(IR::BFN::DeparserPrimitive const & a) const { return a == *this; }
bool IR::BFN::EmitField::operator==(IR::Node const & a) const { return a == *this; }
#line 640 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::EmitConstant::dbprint(std::ostream & out) const {
        out << "emit " << constant << " if " << povBit->field->toString();
    }
#line 20140 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::EmitConstant::operator==(IR::BFN::EmitConstant const & a) const {
        return Emit::operator==(static_cast<const Emit &>(a))
        && constant == a.constant;
    }
bool IR::BFN::EmitConstant::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Emit::equiv(a_)) return false;
        auto &a = static_cast<const EmitConstant &>(a_);
        return (constant ? a.constant ? constant->equiv(*a.constant) : false : a.constant == nullptr);
    }
void IR::BFN::EmitConstant::visit_children(Visitor & v, char const * n) {
    (void)n;
    Emit::visit_children(v, n);
    v.visit(constant, "constant");
}
void IR::BFN::EmitConstant::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Emit::visit_children(v, n);
    v.visit(constant, "constant");
}
void IR::BFN::EmitConstant::validate() const {
#line 20162 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(constant); }
void IR::BFN::EmitConstant::toJSON(JSONGenerator & json) const {
    Emit::toJSON(json);
    json.emit("constant", constant);
}
IR::BFN::EmitConstant::EmitConstant(JSONLoader & json) : Emit(json) {
    json.load("constant", constant);
}
IR::Node * IR::BFN::EmitConstant::fromJSON(JSONLoader & json) { return new EmitConstant(json); }
bool IR::BFN::EmitConstant::operator==(IR::BFN::Emit const & a) const { return a == *this; }
bool IR::BFN::EmitConstant::operator==(IR::BFN::DeparserPrimitive const & a) const { return a == *this; }
bool IR::BFN::EmitConstant::operator==(IR::Node const & a) const { return a == *this; }
#line 651 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::EmitClot::dbprint(std::ostream & out) const {
        out << "emit clot " << clot->tag << " if " << povBit->field->toString();
    }
#line 20180 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::EmitClot::operator==(IR::BFN::EmitClot const & a) const {
        return Emit::operator==(static_cast<const Emit &>(a));
    }
bool IR::BFN::EmitClot::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Emit::equiv(a_)) return false;
        return true;
    }
void IR::BFN::EmitClot::toJSON(JSONGenerator & json) const {
    Emit::toJSON(json);
}
IR::BFN::EmitClot::EmitClot(JSONLoader & json) : Emit(json) {
}
IR::Node * IR::BFN::EmitClot::fromJSON(JSONLoader & json) { return new EmitClot(json); }
bool IR::BFN::EmitClot::operator==(IR::BFN::Emit const & a) const { return a == *this; }
bool IR::BFN::EmitClot::operator==(IR::BFN::DeparserPrimitive const & a) const { return a == *this; }
bool IR::BFN::EmitClot::operator==(IR::Node const & a) const { return a == *this; }
#line 661 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::ChecksumEntry::dbprint(std::ostream & out) const {
        out << "checksum entry " << field->field->toString() << " if "
            << povBit->field->toString();
    }
#line 20203 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// Corresponds to an offset of the FieldVal in the checksum list.
bool IR::BFN::ChecksumEntry::operator==(IR::BFN::ChecksumEntry const & a) const {
        return this->typeId() == a.typeId()
        && field == a.field
        && povBit == a.povBit
        && offset == a.offset;
    }
bool IR::BFN::ChecksumEntry::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const ChecksumEntry &>(a_);
        return (field ? a.field ? field->equiv(*a.field) : false : a.field == nullptr)
        && (povBit ? a.povBit ? povBit->equiv(*a.povBit) : false : a.povBit == nullptr)
        && offset == a.offset;
    }
void IR::BFN::ChecksumEntry::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(field, "field");
    v.visit(povBit, "povBit");
}
void IR::BFN::ChecksumEntry::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(field, "field");
    v.visit(povBit, "povBit");
}
void IR::BFN::ChecksumEntry::validate() const {
#line 20232 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(field);
        CHECK_NULL(povBit); }
void IR::BFN::ChecksumEntry::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " offset=" << offset;
}
void IR::BFN::ChecksumEntry::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("field", field);
    json.emit("povBit", povBit);
    json.emit("offset", offset);
}
IR::BFN::ChecksumEntry::ChecksumEntry(JSONLoader & json) : Node(json) {
    json.load("field", field);
    json.load("povBit", povBit);
    json.load("offset", offset);
}
IR::Node * IR::BFN::ChecksumEntry::fromJSON(JSONLoader & json) { return new ChecksumEntry(json); }
bool IR::BFN::ChecksumEntry::operator==(IR::Node const & a) const { return a == *this; }
/// The fields from which the checksum will be computed.



bool IR::BFN::EmitChecksum::operator==(IR::BFN::EmitChecksum const & a) const {
        return Emit::operator==(static_cast<const Emit &>(a))
        && sources == a.sources
        && zeros_as_ones == a.zeros_as_ones
        && dest == a.dest
        && nested_checksum == a.nested_checksum;
    }
bool IR::BFN::EmitChecksum::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Emit::equiv(a_)) return false;
        auto &a = static_cast<const EmitChecksum &>(a_);
        return sources.equiv(a.sources)
        && zeros_as_ones == a.zeros_as_ones
        && (dest ? a.dest ? dest->equiv(*a.dest) : false : a.dest == nullptr)
        && nested_checksum.equiv(a.nested_checksum);
    }
void IR::BFN::EmitChecksum::visit_children(Visitor & v, char const * n) {
    (void)n;
    Emit::visit_children(v, n);
    sources.visit_children(v, "sources");
    v.visit(dest, "dest");
    nested_checksum.visit_children(v, "nested_checksum");
}
void IR::BFN::EmitChecksum::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Emit::visit_children(v, n);
    sources.visit_children(v, "sources");
    v.visit(dest, "dest");
    nested_checksum.visit_children(v, "nested_checksum");
}
void IR::BFN::EmitChecksum::validate() const {
#line 20288 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        sources.validate();
        CHECK_NULL(dest);
        nested_checksum.validate(); }
void IR::BFN::EmitChecksum::dump_fields(std::ostream & out) const {
    Emit::dump_fields(out);
        out << " zeros_as_ones=" << zeros_as_ones;
}
void IR::BFN::EmitChecksum::toJSON(JSONGenerator & json) const {
    Emit::toJSON(json);
    json.emit("sources", sources);
    json.emit("zeros_as_ones", zeros_as_ones);
    json.emit("dest", dest);
    json.emit("nested_checksum", nested_checksum);
}
IR::BFN::EmitChecksum::EmitChecksum(JSONLoader & json) : Emit(json) {
    json.load("sources", sources);
    json.load("zeros_as_ones", zeros_as_ones);
    json.load("dest", dest);
    json.load("nested_checksum", nested_checksum);
}
IR::Node * IR::BFN::EmitChecksum::fromJSON(JSONLoader & json) { return new EmitChecksum(json); }
bool IR::BFN::EmitChecksum::operator==(IR::BFN::Emit const & a) const { return a == *this; }
bool IR::BFN::EmitChecksum::operator==(IR::BFN::DeparserPrimitive const & a) const { return a == *this; }
bool IR::BFN::EmitChecksum::operator==(IR::Node const & a) const { return a == *this; }
#line 692 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::DeparserParameter::DeparserParameter(cstring name, const IR::Expression* source) : name(name), source(new FieldLVal(source)), povBit(nullptr) { }
#line 20316 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 694 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::DeparserParameter::DeparserParameter(Util::SourceInfo si, cstring name, const IR::Expression* source) : Node(si), name(name), source(new FieldLVal(source)), povBit(nullptr) { }
#line 20319 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 697 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::DeparserParameter::dbprint(std::ostream & out) const {
        out << "parameter " << name << " = ";
        if (source) {
            out << source->field->toString();
            if (povBit) out << " if " << povBit->field->toString();
        } else {
            out << "pov " << povBit->field->toString();
        }
    }
#line 20330 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The deparser parameter to configure. The name is the one used in the
/// assembly, which in turn derives from the CSR register name.
/// The location to read the parameter value from.
/// If present, the POV bit controlling whether the contents of the source
/// location are valid. Not used on Tofino.
/// A source is required. If this is false, then a DeparserParameter can
/// exist that is just a povBit.
#line 722 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
void IR::BFN::DeparserParameter::validate() const {
#line 20340 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 722 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
{
        if (sourceReq)
            CHECK_NULL(source);
        else
            BUG_CHECK(source || povBit,
                      "DeparserParameter %1% must have either a source or a povBit", name);
    } }
#line 20350 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::DeparserParameter::operator==(IR::BFN::DeparserParameter const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && source == a.source
        && povBit == a.povBit
        && sourceReq == a.sourceReq;
    }
bool IR::BFN::DeparserParameter::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const DeparserParameter &>(a_);
        return name == a.name
        && (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr)
        && (povBit ? a.povBit ? povBit->equiv(*a.povBit) : false : a.povBit == nullptr)
        && sourceReq == a.sourceReq;
    }
void IR::BFN::DeparserParameter::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(source, "source");
    v.visit(povBit, "povBit");
}
void IR::BFN::DeparserParameter::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(source, "source");
    v.visit(povBit, "povBit");
}
void IR::BFN::DeparserParameter::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
        out << " sourceReq=" << sourceReq;
}
void IR::BFN::DeparserParameter::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    if (source != nullptr) json.emit("source", source);
    if (povBit != nullptr) json.emit("povBit", povBit);
    json.emit("sourceReq", sourceReq);
}
IR::BFN::DeparserParameter::DeparserParameter(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("source", source);
    json.load("povBit", povBit);
    json.load("sourceReq", sourceReq);
}
IR::Node * IR::BFN::DeparserParameter::fromJSON(JSONLoader & json) { return new DeparserParameter(json); }
bool IR::BFN::DeparserParameter::operator==(IR::Node const & a) const { return a == *this; }
IRNODE_DEFINE_APPLY_OVERLOAD(BFN::AbstractDeparser, , )
#line 734 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
int IR::BFN::AbstractDeparser::stage() const { return std::numeric_limits<int>::max(); }
#line 20402 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 735 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
gress_t IR::BFN::AbstractDeparser::thread() const { return gress; }
#line 20405 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::AbstractDeparser::operator==(IR::BFN::AbstractDeparser const & a) const {
        return Unit::operator==(static_cast<const Unit &>(a))
        && gress == a.gress;
    }
bool IR::BFN::AbstractDeparser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Unit::equiv(a_)) return false;
        auto &a = static_cast<const AbstractDeparser &>(a_);
        return gress == a.gress;
    }
void IR::BFN::AbstractDeparser::dump_fields(std::ostream & out) const {
    Unit::dump_fields(out);
        out << " gress=" << gress;
}
void IR::BFN::AbstractDeparser::toJSON(JSONGenerator & json) const {
    Unit::toJSON(json);
    json.emit("gress", gress);
}
IR::BFN::AbstractDeparser::AbstractDeparser(JSONLoader & json) : Unit(json) {
    json.load("gress", gress);
}
bool IR::BFN::AbstractDeparser::operator==(IR::BFN::Unit const & a) const { return a == *this; }
bool IR::BFN::AbstractDeparser::operator==(IR::Node const & a) const { return a == *this; }
#line 742 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
IR::BFN::Deparser::Deparser(gress_t gr) : AbstractDeparser(gr) {}
#line 20431 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(BFN::Deparser, , )
#line 744 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde.def"
cstring IR::BFN::Deparser::toString() const { return gress ? "egress deparser"_cs : "ingress deparser"_cs; }
#line 20435 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::Deparser::operator==(IR::BFN::Deparser const & a) const {
        return AbstractDeparser::operator==(static_cast<const AbstractDeparser &>(a))
        && emits == a.emits
        && params == a.params
        && digests == a.digests;
    }
bool IR::BFN::Deparser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!AbstractDeparser::equiv(a_)) return false;
        auto &a = static_cast<const Deparser &>(a_);
        return emits.equiv(a.emits)
        && params.equiv(a.params)
        && digests.equiv(a.digests);
    }
void IR::BFN::Deparser::visit_children(Visitor & v, char const * n) {
    (void)n;
    AbstractDeparser::visit_children(v, n);
    emits.visit_children(v, "emits");
    params.visit_children(v, "params");
    digests.visit_children(v, "digests");
}
void IR::BFN::Deparser::visit_children(Visitor & v, char const * n) const {
    (void)n;
    AbstractDeparser::visit_children(v, n);
    emits.visit_children(v, "emits");
    params.visit_children(v, "params");
    digests.visit_children(v, "digests");
}
void IR::BFN::Deparser::validate() const {
#line 20465 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        emits.validate();
        params.validate();
        digests.validate(); }
void IR::BFN::Deparser::toJSON(JSONGenerator & json) const {
    AbstractDeparser::toJSON(json);
    json.emit("emits", emits);
    json.emit("params", params);
    json.emit("digests", digests);
}
IR::BFN::Deparser::Deparser(JSONLoader & json) : AbstractDeparser(json) {
    json.load("emits", emits);
    json.load("params", params);
    json.load("digests", digests);
}
IR::Node * IR::BFN::Deparser::fromJSON(JSONLoader & json) { return new Deparser(json); }
bool IR::BFN::Deparser::operator==(IR::BFN::AbstractDeparser const & a) const { return a == *this; }
bool IR::BFN::Deparser::operator==(IR::BFN::Unit const & a) const { return a == *this; }
bool IR::BFN::Deparser::operator==(IR::Node const & a) const { return a == *this; }

bool IR::BFN::LoweredParserRVal::operator==(IR::BFN::LoweredParserRVal const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::BFN::LoweredParserRVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::BFN::LoweredParserRVal::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::BFN::LoweredParserRVal::LoweredParserRVal(JSONLoader & json) : Node(json) {
}
bool IR::BFN::LoweredParserRVal::operator==(IR::Node const & a) const { return a == *this; }
/// @return this buffer range as a half-open range of bytes in network order.
#line 47 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
nw_byteinterval IR::BFN::LoweredInputBufferRVal::byteInterval() const { return toHalfOpenRange(range); }
#line 20503 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The region represented by this `LoweredInputBufferRVal`, expressed as a
/// range of bytes in network order.
bool IR::BFN::LoweredInputBufferRVal::operator==(IR::BFN::LoweredInputBufferRVal const & a) const {
        return LoweredParserRVal::operator==(static_cast<const LoweredParserRVal &>(a))
        && range == a.range;
    }
bool IR::BFN::LoweredInputBufferRVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoweredParserRVal::equiv(a_)) return false;
        auto &a = static_cast<const LoweredInputBufferRVal &>(a_);
        return range == a.range;
    }
void IR::BFN::LoweredInputBufferRVal::dump_fields(std::ostream & out) const {
    LoweredParserRVal::dump_fields(out);
        out << " range=" << range;
}
void IR::BFN::LoweredInputBufferRVal::toJSON(JSONGenerator & json) const {
    LoweredParserRVal::toJSON(json);
    json.emit("range", range);
}
IR::BFN::LoweredInputBufferRVal::LoweredInputBufferRVal(JSONLoader & json) : LoweredParserRVal(json) {
    json.load("range", range);
}
bool IR::BFN::LoweredInputBufferRVal::operator==(IR::BFN::LoweredParserRVal const & a) const { return a == *this; }
bool IR::BFN::LoweredInputBufferRVal::operator==(IR::Node const & a) const { return a == *this; }
#line 58 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredPacketRVal::dbprint(std::ostream & out) const {
        out << "inbuf " << range;
    }
#line 20533 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::LoweredPacketRVal::operator==(IR::BFN::LoweredPacketRVal const & a) const {
        return LoweredInputBufferRVal::operator==(static_cast<const LoweredInputBufferRVal &>(a));
    }
bool IR::BFN::LoweredPacketRVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoweredInputBufferRVal::equiv(a_)) return false;
        return true;
    }
void IR::BFN::LoweredPacketRVal::toJSON(JSONGenerator & json) const {
    LoweredInputBufferRVal::toJSON(json);
}
IR::BFN::LoweredPacketRVal::LoweredPacketRVal(JSONLoader & json) : LoweredInputBufferRVal(json) {
}
IR::Node * IR::BFN::LoweredPacketRVal::fromJSON(JSONLoader & json) { return new LoweredPacketRVal(json); }
bool IR::BFN::LoweredPacketRVal::operator==(IR::BFN::LoweredInputBufferRVal const & a) const { return a == *this; }
bool IR::BFN::LoweredPacketRVal::operator==(IR::BFN::LoweredParserRVal const & a) const { return a == *this; }
bool IR::BFN::LoweredPacketRVal::operator==(IR::Node const & a) const { return a == *this; }
#line 67 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredMetadataRVal::dbprint(std::ostream & out) const {
        out << "inbuf(metadata) " << range;
    }
#line 20555 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::LoweredMetadataRVal::operator==(IR::BFN::LoweredMetadataRVal const & a) const {
        return LoweredInputBufferRVal::operator==(static_cast<const LoweredInputBufferRVal &>(a));
    }
bool IR::BFN::LoweredMetadataRVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoweredInputBufferRVal::equiv(a_)) return false;
        return true;
    }
void IR::BFN::LoweredMetadataRVal::toJSON(JSONGenerator & json) const {
    LoweredInputBufferRVal::toJSON(json);
}
IR::BFN::LoweredMetadataRVal::LoweredMetadataRVal(JSONLoader & json) : LoweredInputBufferRVal(json) {
}
IR::Node * IR::BFN::LoweredMetadataRVal::fromJSON(JSONLoader & json) { return new LoweredMetadataRVal(json); }
bool IR::BFN::LoweredMetadataRVal::operator==(IR::BFN::LoweredInputBufferRVal const & a) const { return a == *this; }
bool IR::BFN::LoweredMetadataRVal::operator==(IR::BFN::LoweredParserRVal const & a) const { return a == *this; }
bool IR::BFN::LoweredMetadataRVal::operator==(IR::Node const & a) const { return a == *this; }
#line 74 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredConstantRVal::dbprint(std::ostream & out) const { out << "constant " << constant; }
#line 20575 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The constant to extract.
bool IR::BFN::LoweredConstantRVal::operator==(IR::BFN::LoweredConstantRVal const & a) const {
        return LoweredParserRVal::operator==(static_cast<const LoweredParserRVal &>(a))
        && constant == a.constant;
    }
bool IR::BFN::LoweredConstantRVal::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoweredParserRVal::equiv(a_)) return false;
        auto &a = static_cast<const LoweredConstantRVal &>(a_);
        return constant == a.constant;
    }
void IR::BFN::LoweredConstantRVal::dump_fields(std::ostream & out) const {
    LoweredParserRVal::dump_fields(out);
        out << " constant=" << constant;
}
void IR::BFN::LoweredConstantRVal::toJSON(JSONGenerator & json) const {
    LoweredParserRVal::toJSON(json);
    json.emit("constant", constant);
}
IR::BFN::LoweredConstantRVal::LoweredConstantRVal(JSONLoader & json) : LoweredParserRVal(json) {
    json.load("constant", constant);
}
IR::Node * IR::BFN::LoweredConstantRVal::fromJSON(JSONLoader & json) { return new LoweredConstantRVal(json); }
bool IR::BFN::LoweredConstantRVal::operator==(IR::BFN::LoweredParserRVal const & a) const { return a == *this; }
bool IR::BFN::LoweredConstantRVal::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::LoweredParserPrimitive::operator==(IR::BFN::LoweredParserPrimitive const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::BFN::LoweredParserPrimitive::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::BFN::LoweredParserPrimitive::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::BFN::LoweredParserPrimitive::LoweredParserPrimitive(JSONLoader & json) : Node(json) {
}
bool IR::BFN::LoweredParserPrimitive::operator==(IR::Node const & a) const { return a == *this; }
#line 94 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredExtractPhv::dbprint(std::ostream & out) const {
        out << "extract " << source << " to " << dest;
    }
#line 20619 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 98 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredExtractPhv::validate() const {
#line 20622 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source);
        CHECK_NULL(dest);
#line 98 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
{ BUG_CHECK(bool(dest->container), "Invalid PHV container"); } }
#line 20628 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The location that will be read from.
/// The PHV container this extract writes to.
bool IR::BFN::LoweredExtractPhv::operator==(IR::BFN::LoweredExtractPhv const & a) const {
        return LoweredParserPrimitive::operator==(static_cast<const LoweredParserPrimitive &>(a))
        && source == a.source
        && dest == a.dest
        && write_mode == a.write_mode;
    }
bool IR::BFN::LoweredExtractPhv::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoweredParserPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const LoweredExtractPhv &>(a_);
        return (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr)
        && (dest ? a.dest ? dest->equiv(*a.dest) : false : a.dest == nullptr)
        && write_mode == a.write_mode;
    }
void IR::BFN::LoweredExtractPhv::visit_children(Visitor & v, char const * n) {
    (void)n;
    LoweredParserPrimitive::visit_children(v, n);
    v.visit(source, "source");
    v.visit(dest, "dest");
}
void IR::BFN::LoweredExtractPhv::visit_children(Visitor & v, char const * n) const {
    (void)n;
    LoweredParserPrimitive::visit_children(v, n);
    v.visit(source, "source");
    v.visit(dest, "dest");
}
void IR::BFN::LoweredExtractPhv::dump_fields(std::ostream & out) const {
    LoweredParserPrimitive::dump_fields(out);
        out << " write_mode=" << write_mode;
}
void IR::BFN::LoweredExtractPhv::toJSON(JSONGenerator & json) const {
    LoweredParserPrimitive::toJSON(json);
    json.emit("source", source);
    json.emit("dest", dest);
    json.emit("write_mode", write_mode);
}
IR::BFN::LoweredExtractPhv::LoweredExtractPhv(JSONLoader & json) : LoweredParserPrimitive(json) {
    json.load("source", source);
    json.load("dest", dest);
    json.load("write_mode", write_mode);
}
IR::Node * IR::BFN::LoweredExtractPhv::fromJSON(JSONLoader & json) { return new LoweredExtractPhv(json); }
bool IR::BFN::LoweredExtractPhv::operator==(IR::BFN::LoweredParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::LoweredExtractPhv::operator==(IR::Node const & a) const { return a == *this; }
#line 116 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredExtractClot::dbprint(std::ostream & out) const { out << "extract " << source << " to " << dest; }
#line 20677 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// Indicates that this is the first extract into the CLOT
/// The location that will be read from.
/// The CLOT this extract writes to.
/// The higher order parser state this CLOT started in.
/// Used to get the proper length of the CLOT from dest->length_in_bytes(parser_state).
#line 136 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredExtractClot::dump_fields(std::ostream & out) const {
        LoweredParserPrimitive::dump_fields(out);
        out << " is_start=" << is_start;
        out << " dest=" << dest;
        out << " higher_parser_state=" << DBPrint::Brief << higher_parser_state;
    }
#line 20690 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::LoweredExtractClot::operator==(IR::BFN::LoweredExtractClot const & a) const {
        return LoweredParserPrimitive::operator==(static_cast<const LoweredParserPrimitive &>(a))
        && is_start == a.is_start
        && source == a.source
        && dest == a.dest
        && higher_parser_state == a.higher_parser_state;
    }
bool IR::BFN::LoweredExtractClot::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoweredParserPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const LoweredExtractClot &>(a_);
        return is_start == a.is_start
        && (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr)
        && dest == a.dest
        && higher_parser_state == a.higher_parser_state;
    }
void IR::BFN::LoweredExtractClot::visit_children(Visitor & v, char const * n) {
    (void)n;
    LoweredParserPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::LoweredExtractClot::visit_children(Visitor & v, char const * n) const {
    (void)n;
    LoweredParserPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::LoweredExtractClot::validate() const {
#line 20718 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source); }
void IR::BFN::LoweredExtractClot::toJSON(JSONGenerator & json) const {
    LoweredParserPrimitive::toJSON(json);
    json.emit("is_start", is_start);
    json.emit("source", source);
    json.emit("dest", dest);
    json.emit("higher_parser_state", higher_parser_state);
}
IR::BFN::LoweredExtractClot::LoweredExtractClot(JSONLoader & json) : LoweredParserPrimitive(json) {
    json.load("is_start", is_start);
    json.load("source", source);
    json.load("dest", dest);
    json.load("higher_parser_state", higher_parser_state);
}
IR::Node * IR::BFN::LoweredExtractClot::fromJSON(JSONLoader & json) { return new LoweredExtractClot(json); }
bool IR::BFN::LoweredExtractClot::operator==(IR::BFN::LoweredParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::LoweredExtractClot::operator==(IR::Node const & a) const { return a == *this; }
#line 145 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredParserChecksum::dbprint(std::ostream & out) const {
        out << "parser checksum: ";
        out << "unit = "  << unit_id << ", ";
        out << "mask = [";
        for (auto& r : masked_ranges) {
            out << r << ", "; }
        out << "] ";
        out << "swap = " << swap << ", ";
        out << "start = " << start << ", ";
        out << "end = " << end << ", ";
        out << "end_pos = " << end_pos << ", ";
        out << "type = " << type << ", ";
        out << "write_mode = " << write_mode << ".";
    }
#line 20752 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::LoweredParserChecksum::operator==(IR::BFN::LoweredParserChecksum const & a) const {
        return LoweredParserPrimitive::operator==(static_cast<const LoweredParserPrimitive &>(a))
        && unit_id == a.unit_id
        && masked_ranges == a.masked_ranges
        && swap == a.swap
        && start == a.start
        && end == a.end
        && end_pos == a.end_pos
        && type == a.type
        && multiply_2 == a.multiply_2
        && csum_err == a.csum_err
        && clot_dest == a.clot_dest
        && phv_dest == a.phv_dest
        && write_mode == a.write_mode;
    }
bool IR::BFN::LoweredParserChecksum::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoweredParserPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const LoweredParserChecksum &>(a_);
        return unit_id == a.unit_id
        && masked_ranges == a.masked_ranges
        && swap == a.swap
        && start == a.start
        && end == a.end
        && end_pos == a.end_pos
        && type == a.type
        && multiply_2 == a.multiply_2
        && (csum_err ? a.csum_err ? csum_err->equiv(*a.csum_err) : false : a.csum_err == nullptr)
        && clot_dest == a.clot_dest
        && (phv_dest ? a.phv_dest ? phv_dest->equiv(*a.phv_dest) : false : a.phv_dest == nullptr)
        && write_mode == a.write_mode;
    }
void IR::BFN::LoweredParserChecksum::visit_children(Visitor & v, char const * n) {
    (void)n;
    LoweredParserPrimitive::visit_children(v, n);
    v.visit(csum_err, "csum_err");
    v.visit(phv_dest, "phv_dest");
}
void IR::BFN::LoweredParserChecksum::visit_children(Visitor & v, char const * n) const {
    (void)n;
    LoweredParserPrimitive::visit_children(v, n);
    v.visit(csum_err, "csum_err");
    v.visit(phv_dest, "phv_dest");
}
void IR::BFN::LoweredParserChecksum::dump_fields(std::ostream & out) const {
    LoweredParserPrimitive::dump_fields(out);
        out << " unit_id=" << unit_id;
        out << " swap=" << swap;
        out << " start=" << start;
        out << " end=" << end;
        out << " end_pos=" << end_pos;
        out << " type=" << type;
        out << " multiply_2=" << multiply_2;
        out << " clot_dest=" << clot_dest;
        out << " write_mode=" << write_mode;
}
void IR::BFN::LoweredParserChecksum::toJSON(JSONGenerator & json) const {
    LoweredParserPrimitive::toJSON(json);
    json.emit("unit_id", unit_id);
    json.emit("masked_ranges", masked_ranges);
    json.emit("swap", swap);
    json.emit("start", start);
    json.emit("end", end);
    json.emit("end_pos", end_pos);
    json.emit("type", type);
    json.emit("multiply_2", multiply_2);
    if (csum_err != nullptr) json.emit("csum_err", csum_err);
    json.emit("clot_dest", clot_dest);
    if (phv_dest != nullptr) json.emit("phv_dest", phv_dest);
    json.emit("write_mode", write_mode);
}
IR::BFN::LoweredParserChecksum::LoweredParserChecksum(JSONLoader & json) : LoweredParserPrimitive(json) {
    json.load("unit_id", unit_id);
    json.load("masked_ranges", masked_ranges);
    json.load("swap", swap);
    json.load("start", start);
    json.load("end", end);
    json.load("end_pos", end_pos);
    json.load("type", type);
    json.load("multiply_2", multiply_2);
    json.load("csum_err", csum_err);
    json.load("clot_dest", clot_dest);
    json.load("phv_dest", phv_dest);
    json.load("write_mode", write_mode);
}
IR::Node * IR::BFN::LoweredParserChecksum::fromJSON(JSONLoader & json) { return new LoweredParserChecksum(json); }
bool IR::BFN::LoweredParserChecksum::operator==(IR::BFN::LoweredParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::LoweredParserChecksum::operator==(IR::Node const & a) const { return a == *this; }
/// Convenience constructors for accepting a raw source
#line 177 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
IR::BFN::LoweredSave::LoweredSave(MatchRegister const & reg, IR::BFN::LoweredInputBufferRVal const * source) : source(source), dest(reg) { }
#line 20844 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 180 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredSave::dbprint(std::ostream & out) const { out << "save " << source << " to " << dest.name; }
#line 20847 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The location that will be read from.
/// The register this save writes to.
bool IR::BFN::LoweredSave::operator==(IR::BFN::LoweredSave const & a) const {
        return LoweredParserPrimitive::operator==(static_cast<const LoweredParserPrimitive &>(a))
        && source == a.source
        && dest == a.dest;
    }
bool IR::BFN::LoweredSave::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoweredParserPrimitive::equiv(a_)) return false;
        auto &a = static_cast<const LoweredSave &>(a_);
        return (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr)
        && dest == a.dest;
    }
void IR::BFN::LoweredSave::visit_children(Visitor & v, char const * n) {
    (void)n;
    LoweredParserPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::LoweredSave::visit_children(Visitor & v, char const * n) const {
    (void)n;
    LoweredParserPrimitive::visit_children(v, n);
    v.visit(source, "source");
}
void IR::BFN::LoweredSave::validate() const {
#line 20873 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source); }
void IR::BFN::LoweredSave::dump_fields(std::ostream & out) const {
    LoweredParserPrimitive::dump_fields(out);
        out << " dest=" << dest;
}
void IR::BFN::LoweredSave::toJSON(JSONGenerator & json) const {
    LoweredParserPrimitive::toJSON(json);
    json.emit("source", source);
    json.emit("dest", dest);
}
IR::BFN::LoweredSave::LoweredSave(JSONLoader & json) : LoweredParserPrimitive(json) {
    json.load("source", source);
    json.load("dest", dest);
}
IR::Node * IR::BFN::LoweredSave::fromJSON(JSONLoader & json) { return new LoweredSave(json); }
bool IR::BFN::LoweredSave::operator==(IR::BFN::LoweredParserPrimitive const & a) const { return a == *this; }
bool IR::BFN::LoweredSave::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::LoweredSelect::operator==(IR::BFN::LoweredSelect const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::BFN::LoweredSelect::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::BFN::LoweredSelect::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::BFN::LoweredSelect::LoweredSelect(JSONLoader & json) : Node(json) {
}
IR::Node * IR::BFN::LoweredSelect::fromJSON(JSONLoader & json) { return new LoweredSelect(json); }
bool IR::BFN::LoweredSelect::operator==(IR::Node const & a) const { return a == *this; }
/// The match value: constant or a value set
/// Number of bytes to shift out of input buffer at current state
/// Final amount to add to header length
/// Extractions to perform
/// Save match word to registers
/// registers to scratch pad on write
/// Checksum calculations
/// Counter instructions
/// Priority set
/// The number of packet bytes which must be available in the input buffer
/// for the extractions to be performed successfully. If `std::nullopt`,
/// there is no particular requirement; the assembler will choose a default.

/// Next state to transition to (null means terminal state)
/// Increment destination offset on state re-entry
bool IR::BFN::LoweredParserMatch::operator==(IR::BFN::LoweredParserMatch const & a) const {
        return this->typeId() == a.typeId()
        && value == a.value
        && shift == a.shift
        && hdrLenIncFinalAmt == a.hdrLenIncFinalAmt
        && extracts == a.extracts
        && saves == a.saves
        && scratches == a.scratches
        && checksums == a.checksums
        && counters == a.counters
        && priority == a.priority
        && bufferRequired == a.bufferRequired
        && partialHdrErrProc == a.partialHdrErrProc
        && next == a.next
        && offsetInc == a.offsetInc;
    }
bool IR::BFN::LoweredParserMatch::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const LoweredParserMatch &>(a_);
        return (value ? a.value ? value->equiv(*a.value) : false : a.value == nullptr)
        && shift == a.shift
        && hdrLenIncFinalAmt == a.hdrLenIncFinalAmt
        && extracts.equiv(a.extracts)
        && saves.equiv(a.saves)
        && scratches == a.scratches
        && checksums.equiv(a.checksums)
        && counters.equiv(a.counters)
        && (priority ? a.priority ? priority->equiv(*a.priority) : false : a.priority == nullptr)
        && bufferRequired == a.bufferRequired
        && partialHdrErrProc == a.partialHdrErrProc
        && (next ? a.next ? next->equiv(*a.next) : false : a.next == nullptr)
        && offsetInc == a.offsetInc;
    }
void IR::BFN::LoweredParserMatch::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(value, "value");
    extracts.visit_children(v, "extracts");
    saves.visit_children(v, "saves");
    checksums.visit_children(v, "checksums");
    counters.visit_children(v, "counters");
    v.visit(priority, "priority");
    v.visit(next, "next");
}
void IR::BFN::LoweredParserMatch::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(value, "value");
    extracts.visit_children(v, "extracts");
    saves.visit_children(v, "saves");
    checksums.visit_children(v, "checksums");
    counters.visit_children(v, "counters");
    v.visit(priority, "priority");
    v.visit(next, "next");
}
void IR::BFN::LoweredParserMatch::validate() const {
#line 20979 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(value);
        extracts.validate();
        saves.validate();
        checksums.validate();
        counters.validate(); }
void IR::BFN::LoweredParserMatch::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " shift=" << shift;
        out << " partialHdrErrProc=" << partialHdrErrProc;
}
void IR::BFN::LoweredParserMatch::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("value", value);
    json.emit("shift", shift);
    json.emit("hdrLenIncFinalAmt", hdrLenIncFinalAmt);
    json.emit("extracts", extracts);
    json.emit("saves", saves);
    json.emit("scratches", scratches);
    json.emit("checksums", checksums);
    json.emit("counters", counters);
    if (priority != nullptr) json.emit("priority", priority);
    json.emit("bufferRequired", bufferRequired);
    json.emit("partialHdrErrProc", partialHdrErrProc);
    if (next != nullptr) json.emit("next", next);
    json.emit("offsetInc", offsetInc);
}
IR::BFN::LoweredParserMatch::LoweredParserMatch(JSONLoader & json) : Node(json) {
    json.load("value", value);
    json.load("shift", shift);
    json.load("hdrLenIncFinalAmt", hdrLenIncFinalAmt);
    json.load("extracts", extracts);
    json.load("saves", saves);
    json.load("scratches", scratches);
    json.load("checksums", checksums);
    json.load("counters", counters);
    json.load("priority", priority);
    json.load("bufferRequired", bufferRequired);
    json.load("partialHdrErrProc", partialHdrErrProc);
    json.load("next", next);
    json.load("offsetInc", offsetInc);
}
IR::Node * IR::BFN::LoweredParserMatch::fromJSON(JSONLoader & json) { return new LoweredParserMatch(json); }
bool IR::BFN::LoweredParserMatch::operator==(IR::Node const & a) const { return a == *this; }
#line 284 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
IR::BFN::LoweredParserState::LoweredParserState(cstring name, gress_t gress) : name(name), gress(gress),
          select(new LoweredSelect(ordered_set<MatchRegister>())) { }
#line 21027 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 287 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
int IR::BFN::LoweredParserState::stage() const { return -1; }
#line 21030 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 288 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
gress_t IR::BFN::LoweredParserState::thread() const { return gress; }
#line 21033 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 289 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
cstring IR::BFN::LoweredParserState::toString() const { return name; }
#line 21036 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 290 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredParserState::visit_children(Visitor & v, char const * n) {
        v.visit(select);
        transitions.parallel_visit_children(v, n); }
#line 290 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredParserState::visit_children(Visitor & v, char const * n) const {
        v.visit(select);
        transitions.parallel_visit_children(v, n); }
#line 21045 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The name of this state. This does not necessarily correspond to any
/// state name in the P4 program.
/// The thread to which this state is assigned.
/// The bytes from the input buffer which will be loaded into the match
/// registers and used to select a LoweredParserMatch transition to take.
/// The possible transitions out of this state.
bool IR::BFN::LoweredParserState::operator==(IR::BFN::LoweredParserState const & a) const {
        return Unit::operator==(static_cast<const Unit &>(a))
        && name == a.name
        && gress == a.gress
        && select == a.select
        && transitions == a.transitions;
    }
bool IR::BFN::LoweredParserState::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!Unit::equiv(a_)) return false;
        auto &a = static_cast<const LoweredParserState &>(a_);
        return name == a.name
        && gress == a.gress
        && (select ? a.select ? select->equiv(*a.select) : false : a.select == nullptr)
        && transitions.equiv(a.transitions);
    }
void IR::BFN::LoweredParserState::validate() const {
#line 21069 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(select);
        transitions.validate(); }
void IR::BFN::LoweredParserState::dump_fields(std::ostream & out) const {
    Unit::dump_fields(out);
        out << " name=" << name;
        out << " gress=" << gress;
}
void IR::BFN::LoweredParserState::toJSON(JSONGenerator & json) const {
    Unit::toJSON(json);
    json.emit("name", name);
    json.emit("gress", gress);
    json.emit("select", select);
    json.emit("transitions", transitions);
}
IR::BFN::LoweredParserState::LoweredParserState(JSONLoader & json) : Unit(json) {
    json.load("name", name);
    json.load("gress", gress);
    json.load("select", select);
    json.load("transitions", transitions);
}
IR::Node * IR::BFN::LoweredParserState::fromJSON(JSONLoader & json) { return new LoweredParserState(json); }
bool IR::BFN::LoweredParserState::operator==(IR::BFN::Unit const & a) const { return a == *this; }
bool IR::BFN::LoweredParserState::operator==(IR::Node const & a) const { return a == *this; }
#line 311 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
IR::BFN::BaseLoweredParser::BaseLoweredParser(gress_t gress) : AbstractParser(gress) {}
#line 21096 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(BFN::BaseLoweredParser, , )
#line 314 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
cstring IR::BFN::BaseLoweredParser::toString() const {
        return gress ? "lowered egress parser"_cs : "lowered ingress parser"_cs;
    }
#line 21102 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::BaseLoweredParser::operator==(IR::BFN::BaseLoweredParser const & a) const {
        return AbstractParser::operator==(static_cast<const AbstractParser &>(a));
    }
bool IR::BFN::BaseLoweredParser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!AbstractParser::equiv(a_)) return false;
        return true;
    }
void IR::BFN::BaseLoweredParser::toJSON(JSONGenerator & json) const {
    AbstractParser::toJSON(json);
}
IR::BFN::BaseLoweredParser::BaseLoweredParser(JSONLoader & json) : AbstractParser(json) {
}
IR::Node * IR::BFN::BaseLoweredParser::fromJSON(JSONLoader & json) { return new BaseLoweredParser(json); }
bool IR::BFN::BaseLoweredParser::operator==(IR::BFN::AbstractParser const & a) const { return a == *this; }
bool IR::BFN::BaseLoweredParser::operator==(IR::BFN::Unit const & a) const { return a == *this; }
bool IR::BFN::BaseLoweredParser::operator==(IR::Node const & a) const { return a == *this; }
#line 330 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
IR::BFN::LoweredParser::LoweredParser(gress_t gress, const IR::BFN::LoweredParserState* start, const IR::BFN::Phase0* phase0, cstring name, std::vector<int> portmap) : BaseLoweredParser(gress), start(start), phase0(phase0),
        name(name), portmap(portmap) {}
#line 21123 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(BFN::LoweredParser, , )
/// The entry point to the parser.
/// parser error
/// The set of PHV containers that are allowed to be written more than once (bitwise OR on rewrite)
/// The set of PHV containers that are allowed to be written more than once (clear on rewrite)
/// The set of PHV containers that are zero initialized
/// Num of bytes of metadata prepended to user packet



bool IR::BFN::LoweredParser::operator==(IR::BFN::LoweredParser const & a) const {
        return BaseLoweredParser::operator==(static_cast<const BaseLoweredParser &>(a))
        && start == a.start
        && parserError == a.parserError
        && bitwiseOrContainers == a.bitwiseOrContainers
        && clearOnWriteContainers == a.clearOnWriteContainers
        && initZeroContainers == a.initZeroContainers
        && hdrLenAdj == a.hdrLenAdj
        && phase0 == a.phase0
        && name == a.name
        && portmap == a.portmap;
    }
bool IR::BFN::LoweredParser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!BaseLoweredParser::equiv(a_)) return false;
        auto &a = static_cast<const LoweredParser &>(a_);
        return (start ? a.start ? start->equiv(*a.start) : false : a.start == nullptr)
        && (parserError ? a.parserError ? parserError->equiv(*a.parserError) : false : a.parserError == nullptr)
        && bitwiseOrContainers.equiv(a.bitwiseOrContainers)
        && clearOnWriteContainers.equiv(a.clearOnWriteContainers)
        && initZeroContainers.equiv(a.initZeroContainers)
        && hdrLenAdj == a.hdrLenAdj
        && (phase0 ? a.phase0 ? phase0->equiv(*a.phase0) : false : a.phase0 == nullptr)
        && name == a.name
        && portmap == a.portmap;
    }
void IR::BFN::LoweredParser::visit_children(Visitor & v, char const * n) {
    (void)n;
    BaseLoweredParser::visit_children(v, n);
    v.visit(start, "start");
    v.visit(parserError, "parserError");
    bitwiseOrContainers.visit_children(v, "bitwiseOrContainers");
    clearOnWriteContainers.visit_children(v, "clearOnWriteContainers");
    initZeroContainers.visit_children(v, "initZeroContainers");
    v.visit(phase0, "phase0");
}
void IR::BFN::LoweredParser::visit_children(Visitor & v, char const * n) const {
    (void)n;
    BaseLoweredParser::visit_children(v, n);
    v.visit(start, "start");
    v.visit(parserError, "parserError");
    bitwiseOrContainers.visit_children(v, "bitwiseOrContainers");
    clearOnWriteContainers.visit_children(v, "clearOnWriteContainers");
    initZeroContainers.visit_children(v, "initZeroContainers");
    v.visit(phase0, "phase0");
}
void IR::BFN::LoweredParser::validate() const {
#line 21181 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(start);
        bitwiseOrContainers.validate();
        clearOnWriteContainers.validate();
        initZeroContainers.validate(); }
void IR::BFN::LoweredParser::dump_fields(std::ostream & out) const {
    BaseLoweredParser::dump_fields(out);
        out << " hdrLenAdj=" << hdrLenAdj;
        out << " name=" << name;
}
void IR::BFN::LoweredParser::toJSON(JSONGenerator & json) const {
    BaseLoweredParser::toJSON(json);
    json.emit("start", start);
    if (parserError != nullptr) json.emit("parserError", parserError);
    json.emit("bitwiseOrContainers", bitwiseOrContainers);
    json.emit("clearOnWriteContainers", clearOnWriteContainers);
    json.emit("initZeroContainers", initZeroContainers);
    json.emit("hdrLenAdj", hdrLenAdj);
    if (phase0 != nullptr) json.emit("phase0", phase0);
    json.emit("name", name);
    json.emit("portmap", portmap);
}
IR::BFN::LoweredParser::LoweredParser(JSONLoader & json) : BaseLoweredParser(json) {
    json.load("start", start);
    json.load("parserError", parserError);
    json.load("bitwiseOrContainers", bitwiseOrContainers);
    json.load("clearOnWriteContainers", clearOnWriteContainers);
    json.load("initZeroContainers", initZeroContainers);
    json.load("hdrLenAdj", hdrLenAdj);
    json.load("phase0", phase0);
    json.load("name", name);
    json.load("portmap", portmap);
}
IR::Node * IR::BFN::LoweredParser::fromJSON(JSONLoader & json) { return new LoweredParser(json); }
bool IR::BFN::LoweredParser::operator==(IR::BFN::BaseLoweredParser const & a) const { return a == *this; }
bool IR::BFN::LoweredParser::operator==(IR::BFN::AbstractParser const & a) const { return a == *this; }
bool IR::BFN::LoweredParser::operator==(IR::BFN::Unit const & a) const { return a == *this; }
bool IR::BFN::LoweredParser::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::LoweredEmit::operator==(IR::BFN::LoweredEmit const & a) const {
        return this->typeId() == a.typeId();
    }
bool IR::BFN::LoweredEmit::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        return true;
    }
void IR::BFN::LoweredEmit::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
}
IR::BFN::LoweredEmit::LoweredEmit(JSONLoader & json) : Node(json) {
}
bool IR::BFN::LoweredEmit::operator==(IR::Node const & a) const { return a == *this; }
#line 373 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredEmitPhv::dbprint(std::ostream & out) const { out << "emit " << source << " if " << povBit; }
#line 21236 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The POV bit the emit is predicated on
/// The container whose contents will be written to the packet.
bool IR::BFN::LoweredEmitPhv::operator==(IR::BFN::LoweredEmitPhv const & a) const {
        return LoweredEmit::operator==(static_cast<const LoweredEmit &>(a))
        && povBit == a.povBit
        && source == a.source;
    }
bool IR::BFN::LoweredEmitPhv::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoweredEmit::equiv(a_)) return false;
        auto &a = static_cast<const LoweredEmitPhv &>(a_);
        return (povBit ? a.povBit ? povBit->equiv(*a.povBit) : false : a.povBit == nullptr)
        && (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr);
    }
void IR::BFN::LoweredEmitPhv::visit_children(Visitor & v, char const * n) {
    (void)n;
    LoweredEmit::visit_children(v, n);
    v.visit(povBit, "povBit");
    v.visit(source, "source");
}
void IR::BFN::LoweredEmitPhv::visit_children(Visitor & v, char const * n) const {
    (void)n;
    LoweredEmit::visit_children(v, n);
    v.visit(povBit, "povBit");
    v.visit(source, "source");
}
void IR::BFN::LoweredEmitPhv::validate() const {
#line 21264 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(povBit);
        CHECK_NULL(source); }
void IR::BFN::LoweredEmitPhv::toJSON(JSONGenerator & json) const {
    LoweredEmit::toJSON(json);
    json.emit("povBit", povBit);
    json.emit("source", source);
}
IR::BFN::LoweredEmitPhv::LoweredEmitPhv(JSONLoader & json) : LoweredEmit(json) {
    json.load("povBit", povBit);
    json.load("source", source);
}
IR::Node * IR::BFN::LoweredEmitPhv::fromJSON(JSONLoader & json) { return new LoweredEmitPhv(json); }
bool IR::BFN::LoweredEmitPhv::operator==(IR::BFN::LoweredEmit const & a) const { return a == *this; }
bool IR::BFN::LoweredEmitPhv::operator==(IR::Node const & a) const { return a == *this; }
#line 384 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredEmitChecksum::dbprint(std::ostream & out) const { out << "emit checksum " << unit << " if " << povBit; }
#line 21282 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The POV bit the emit is predicated on
/// The checksum unit whose result will be written to the packet.
bool IR::BFN::LoweredEmitChecksum::operator==(IR::BFN::LoweredEmitChecksum const & a) const {
        return LoweredEmit::operator==(static_cast<const LoweredEmit &>(a))
        && povBit == a.povBit
        && unit == a.unit;
    }
bool IR::BFN::LoweredEmitChecksum::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoweredEmit::equiv(a_)) return false;
        auto &a = static_cast<const LoweredEmitChecksum &>(a_);
        return (povBit ? a.povBit ? povBit->equiv(*a.povBit) : false : a.povBit == nullptr)
        && unit == a.unit;
    }
void IR::BFN::LoweredEmitChecksum::visit_children(Visitor & v, char const * n) {
    (void)n;
    LoweredEmit::visit_children(v, n);
    v.visit(povBit, "povBit");
}
void IR::BFN::LoweredEmitChecksum::visit_children(Visitor & v, char const * n) const {
    (void)n;
    LoweredEmit::visit_children(v, n);
    v.visit(povBit, "povBit");
}
void IR::BFN::LoweredEmitChecksum::validate() const {
#line 21308 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(povBit); }
void IR::BFN::LoweredEmitChecksum::dump_fields(std::ostream & out) const {
    LoweredEmit::dump_fields(out);
        out << " unit=" << unit;
}
void IR::BFN::LoweredEmitChecksum::toJSON(JSONGenerator & json) const {
    LoweredEmit::toJSON(json);
    json.emit("povBit", povBit);
    json.emit("unit", unit);
}
IR::BFN::LoweredEmitChecksum::LoweredEmitChecksum(JSONLoader & json) : LoweredEmit(json) {
    json.load("povBit", povBit);
    json.load("unit", unit);
}
IR::Node * IR::BFN::LoweredEmitChecksum::fromJSON(JSONLoader & json) { return new LoweredEmitChecksum(json); }
bool IR::BFN::LoweredEmitChecksum::operator==(IR::BFN::LoweredEmit const & a) const { return a == *this; }
bool IR::BFN::LoweredEmitChecksum::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::LoweredEmitClot::operator==(IR::BFN::LoweredEmitClot const & a) const {
        return LoweredEmit::operator==(static_cast<const LoweredEmit &>(a))
        && clot == a.clot;
    }
bool IR::BFN::LoweredEmitClot::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoweredEmit::equiv(a_)) return false;
        auto &a = static_cast<const LoweredEmitClot &>(a_);
        return (clot ? a.clot ? clot->equiv(*a.clot) : false : a.clot == nullptr);
    }
void IR::BFN::LoweredEmitClot::visit_children(Visitor & v, char const * n) {
    (void)n;
    LoweredEmit::visit_children(v, n);
    v.visit(clot, "clot");
}
void IR::BFN::LoweredEmitClot::visit_children(Visitor & v, char const * n) const {
    (void)n;
    LoweredEmit::visit_children(v, n);
    v.visit(clot, "clot");
}
void IR::BFN::LoweredEmitClot::validate() const {
#line 21348 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(clot); }
void IR::BFN::LoweredEmitClot::toJSON(JSONGenerator & json) const {
    LoweredEmit::toJSON(json);
    json.emit("clot", clot);
}
IR::BFN::LoweredEmitClot::LoweredEmitClot(JSONLoader & json) : LoweredEmit(json) {
    json.load("clot", clot);
}
IR::Node * IR::BFN::LoweredEmitClot::fromJSON(JSONLoader & json) { return new LoweredEmitClot(json); }
bool IR::BFN::LoweredEmitClot::operator==(IR::BFN::LoweredEmit const & a) const { return a == *this; }
bool IR::BFN::LoweredEmitClot::operator==(IR::Node const & a) const { return a == *this; }
#line 399 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredEmitConstant::dbprint(std::ostream & out) const {
        out << "emit " << (unsigned)constant << " if " << povBit;
    }
#line 21365 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The POV bit the emit is predicated on
bool IR::BFN::LoweredEmitConstant::operator==(IR::BFN::LoweredEmitConstant const & a) const {
        return LoweredEmit::operator==(static_cast<const LoweredEmit &>(a))
        && povBit == a.povBit
        && constant == a.constant;
    }
bool IR::BFN::LoweredEmitConstant::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!LoweredEmit::equiv(a_)) return false;
        auto &a = static_cast<const LoweredEmitConstant &>(a_);
        return (povBit ? a.povBit ? povBit->equiv(*a.povBit) : false : a.povBit == nullptr)
        && constant == a.constant;
    }
void IR::BFN::LoweredEmitConstant::visit_children(Visitor & v, char const * n) {
    (void)n;
    LoweredEmit::visit_children(v, n);
    v.visit(povBit, "povBit");
}
void IR::BFN::LoweredEmitConstant::visit_children(Visitor & v, char const * n) const {
    (void)n;
    LoweredEmit::visit_children(v, n);
    v.visit(povBit, "povBit");
}
void IR::BFN::LoweredEmitConstant::validate() const {
#line 21390 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(povBit); }
void IR::BFN::LoweredEmitConstant::dump_fields(std::ostream & out) const {
    LoweredEmit::dump_fields(out);
        out << " constant=" << constant;
}
void IR::BFN::LoweredEmitConstant::toJSON(JSONGenerator & json) const {
    LoweredEmit::toJSON(json);
    json.emit("povBit", povBit);
    json.emit("constant", constant);
}
IR::BFN::LoweredEmitConstant::LoweredEmitConstant(JSONLoader & json) : LoweredEmit(json) {
    json.load("povBit", povBit);
    json.load("constant", constant);
}
IR::Node * IR::BFN::LoweredEmitConstant::fromJSON(JSONLoader & json) { return new LoweredEmitConstant(json); }
bool IR::BFN::LoweredEmitConstant::operator==(IR::BFN::LoweredEmit const & a) const { return a == *this; }
bool IR::BFN::LoweredEmitConstant::operator==(IR::Node const & a) const { return a == *this; }
#line 412 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredDeparserParameter::dbprint(std::ostream & out) const {
        out << "parameter " << name << " = ";
        if (sources.size() > 1) {
            out << "[";
            std::string sep = "";
            for (const auto* source : sources) {
                out << sep << source->toString();
                sep = ", ";
            }
            out << "]";
        } else {
            out << sources.front()->toString();
        }
    }
#line 21424 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
#line 427 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LoweredDeparserParameter::validate() const {
#line 21427 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        sources.validate();
#line 427 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
{
        BUG_CHECK(sources.size() != 0, "LoweredDeparserParameter with 0 sources");
        for (const auto* source : sources) CHECK_NULL(source);
    } }
#line 21435 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The deparser parameter to configure. The name is the one used in the
/// assembly, which in turn derives from the CSR register name.
/// The container whose contents will be used for the parameter.
/// On JBay, each deparser parameter has a POV bit which determines whether
/// the container's contents are valid (and often implicitly enables or
/// disables the corresponding feature). On Tofino, the hidden per-container
/// validity bits serve the same role.
bool IR::BFN::LoweredDeparserParameter::operator==(IR::BFN::LoweredDeparserParameter const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && sources == a.sources
        && povBit == a.povBit;
    }
bool IR::BFN::LoweredDeparserParameter::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const LoweredDeparserParameter &>(a_);
        return name == a.name
        && sources.equiv(a.sources)
        && (povBit ? a.povBit ? povBit->equiv(*a.povBit) : false : a.povBit == nullptr);
    }
void IR::BFN::LoweredDeparserParameter::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    sources.visit_children(v, "sources");
    v.visit(povBit, "povBit");
}
void IR::BFN::LoweredDeparserParameter::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    sources.visit_children(v, "sources");
    v.visit(povBit, "povBit");
}
void IR::BFN::LoweredDeparserParameter::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
}
void IR::BFN::LoweredDeparserParameter::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    json.emit("sources", sources);
    if (povBit != nullptr) json.emit("povBit", povBit);
}
IR::BFN::LoweredDeparserParameter::LoweredDeparserParameter(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("sources", sources);
    json.load("povBit", povBit);
}
IR::Node * IR::BFN::LoweredDeparserParameter::fromJSON(JSONLoader & json) { return new LoweredDeparserParameter(json); }
bool IR::BFN::LoweredDeparserParameter::operator==(IR::Node const & a) const { return a == *this; }
/// The index for the entry
/// The PHV containers in the entry.
bool IR::BFN::DigestTableEntry::operator==(IR::BFN::DigestTableEntry const & a) const {
        return this->typeId() == a.typeId()
        && idx == a.idx
        && sources == a.sources;
    }
bool IR::BFN::DigestTableEntry::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const DigestTableEntry &>(a_);
        return idx == a.idx
        && sources.equiv(a.sources);
    }
void IR::BFN::DigestTableEntry::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    sources.visit_children(v, "sources");
}
void IR::BFN::DigestTableEntry::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    sources.visit_children(v, "sources");
}
void IR::BFN::DigestTableEntry::validate() const {
#line 21511 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        sources.validate(); }
void IR::BFN::DigestTableEntry::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " idx=" << idx;
}
void IR::BFN::DigestTableEntry::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("idx", idx);
    json.emit("sources", sources);
}
IR::BFN::DigestTableEntry::DigestTableEntry(JSONLoader & json) : Node(json) {
    json.load("idx", idx);
    json.load("sources", sources);
}
IR::Node * IR::BFN::DigestTableEntry::fromJSON(JSONLoader & json) { return new DigestTableEntry(json); }
bool IR::BFN::DigestTableEntry::operator==(IR::Node const & a) const { return a == *this; }
/// P4 Name of the digest field
/// The start byte in digest format for field location in network order
/// The start bit within byte in digest format for field location in network
/// order
/// The width of field within the phv container
/// The field offset within the phv container
bool IR::BFN::DigestField::operator==(IR::BFN::DigestField const & a) const {
        return this->typeId() == a.typeId()
        && fieldName == a.fieldName
        && startByte == a.startByte
        && startBit == a.startBit
        && fieldWidth == a.fieldWidth
        && fieldOffset == a.fieldOffset;
    }
bool IR::BFN::DigestField::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const DigestField &>(a_);
        return fieldName == a.fieldName
        && startByte == a.startByte
        && startBit == a.startBit
        && fieldWidth == a.fieldWidth
        && fieldOffset == a.fieldOffset;
    }
void IR::BFN::DigestField::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " fieldName=" << fieldName;
        out << " startByte=" << startByte;
        out << " startBit=" << startBit;
        out << " fieldWidth=" << fieldWidth;
        out << " fieldOffset=" << fieldOffset;
}
void IR::BFN::DigestField::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("fieldName", fieldName);
    json.emit("startByte", startByte);
    json.emit("startBit", startBit);
    json.emit("fieldWidth", fieldWidth);
    json.emit("fieldOffset", fieldOffset);
}
IR::BFN::DigestField::DigestField(JSONLoader & json) : Node(json) {
    json.load("fieldName", fieldName);
    json.load("startByte", startByte);
    json.load("startBit", startBit);
    json.load("fieldWidth", fieldWidth);
    json.load("fieldOffset", fieldOffset);
}
IR::Node * IR::BFN::DigestField::fromJSON(JSONLoader & json) { return new DigestField(json); }
bool IR::BFN::DigestField::operator==(IR::Node const & a) const { return a == *this; }
#line 485 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
void IR::BFN::LearningTableEntry::validate() const {
#line 21580 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

#line 485 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
{ CHECK_NULL(controlPlaneFormat); } }
#line 21584 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
/// The control plane name for this type of learn quanta.
bool IR::BFN::LearningTableEntry::operator==(IR::BFN::LearningTableEntry const & a) const {
        return DigestTableEntry::operator==(static_cast<const DigestTableEntry &>(a))
        && controlPlaneName == a.controlPlaneName;
    }
bool IR::BFN::LearningTableEntry::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!DigestTableEntry::equiv(a_)) return false;
        auto &a = static_cast<const LearningTableEntry &>(a_);
        return controlPlaneName == a.controlPlaneName;
    }
void IR::BFN::LearningTableEntry::dump_fields(std::ostream & out) const {
    DigestTableEntry::dump_fields(out);
        out << " controlPlaneName=" << controlPlaneName;
}
void IR::BFN::LearningTableEntry::toJSON(JSONGenerator & json) const {
    DigestTableEntry::toJSON(json);
    json.emit("controlPlaneName", controlPlaneName);
}
IR::BFN::LearningTableEntry::LearningTableEntry(JSONLoader & json) : DigestTableEntry(json) {
    json.load("controlPlaneName", controlPlaneName);
}
IR::Node * IR::BFN::LearningTableEntry::fromJSON(JSONLoader & json) { return new LearningTableEntry(json); }
bool IR::BFN::LearningTableEntry::operator==(IR::BFN::DigestTableEntry const & a) const { return a == *this; }
bool IR::BFN::LearningTableEntry::operator==(IR::Node const & a) const { return a == *this; }
/// The name of the digest, identifying a special buffer to be filled in by
/// the deparser hardware. The name is chosen to match the corresponding
/// assembly directive, which in turn is named based on definitions in the
/// CSRs.
/// The container which selects a digest table entry.
/// The available table entries; the index of each entry in the vector
/// corresponds to the selector value which chooses it.
bool IR::BFN::LoweredDigest::operator==(IR::BFN::LoweredDigest const & a) const {
        return this->typeId() == a.typeId()
        && name == a.name
        && selector == a.selector
        && entries == a.entries
        && povBit == a.povBit;
    }
bool IR::BFN::LoweredDigest::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const LoweredDigest &>(a_);
        return name == a.name
        && (selector ? a.selector ? selector->equiv(*a.selector) : false : a.selector == nullptr)
        && entries.equiv(a.entries)
        && (povBit ? a.povBit ? povBit->equiv(*a.povBit) : false : a.povBit == nullptr);
    }
void IR::BFN::LoweredDigest::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(selector, "selector");
    entries.visit_children(v, "entries");
    v.visit(povBit, "povBit");
}
void IR::BFN::LoweredDigest::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(selector, "selector");
    entries.visit_children(v, "entries");
    v.visit(povBit, "povBit");
}
void IR::BFN::LoweredDigest::validate() const {
#line 21648 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        entries.validate(); }
void IR::BFN::LoweredDigest::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " name=" << name;
}
void IR::BFN::LoweredDigest::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("name", name);
    if (selector != nullptr) json.emit("selector", selector);
    json.emit("entries", entries);
    if (povBit != nullptr) json.emit("povBit", povBit);
}
IR::BFN::LoweredDigest::LoweredDigest(JSONLoader & json) : Node(json) {
    json.load("name", name);
    json.load("selector", selector);
    json.load("entries", entries);
    json.load("povBit", povBit);
}
IR::Node * IR::BFN::LoweredDigest::fromJSON(JSONLoader & json) { return new LoweredDigest(json); }
bool IR::BFN::LoweredDigest::operator==(IR::Node const & a) const { return a == *this; }
/// A container whose contents will be included in the checksum.
/// On JBay, each checksum input has an independent POV bit, which makes it
/// easier to safely compute a checksum involving headers which may be
/// present in different combinations. On Tofino, this is always null.
bool IR::BFN::ChecksumPhvInput::operator==(IR::BFN::ChecksumPhvInput const & a) const {
        return this->typeId() == a.typeId()
        && source == a.source
        && swap == a.swap
        && povBit == a.povBit;
    }
bool IR::BFN::ChecksumPhvInput::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const ChecksumPhvInput &>(a_);
        return (source ? a.source ? source->equiv(*a.source) : false : a.source == nullptr)
        && swap == a.swap
        && (povBit ? a.povBit ? povBit->equiv(*a.povBit) : false : a.povBit == nullptr);
    }
void IR::BFN::ChecksumPhvInput::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(source, "source");
    v.visit(povBit, "povBit");
}
void IR::BFN::ChecksumPhvInput::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(source, "source");
    v.visit(povBit, "povBit");
}
void IR::BFN::ChecksumPhvInput::validate() const {
#line 21701 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(source); }
void IR::BFN::ChecksumPhvInput::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " swap=" << swap;
}
void IR::BFN::ChecksumPhvInput::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("source", source);
    json.emit("swap", swap);
    if (povBit != nullptr) json.emit("povBit", povBit);
}
IR::BFN::ChecksumPhvInput::ChecksumPhvInput(JSONLoader & json) : Node(json) {
    json.load("source", source);
    json.load("swap", swap);
    json.load("povBit", povBit);
}
IR::Node * IR::BFN::ChecksumPhvInput::fromJSON(JSONLoader & json) { return new ChecksumPhvInput(json); }
bool IR::BFN::ChecksumPhvInput::operator==(IR::Node const & a) const { return a == *this; }
bool IR::BFN::ChecksumClotInput::operator==(IR::BFN::ChecksumClotInput const & a) const {
        return this->typeId() == a.typeId()
        && clot == a.clot
        && invert == a.invert
        && povBit == a.povBit;
    }
bool IR::BFN::ChecksumClotInput::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const ChecksumClotInput &>(a_);
        return clot == a.clot
        && invert == a.invert
        && (povBit ? a.povBit ? povBit->equiv(*a.povBit) : false : a.povBit == nullptr);
    }
void IR::BFN::ChecksumClotInput::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    v.visit(povBit, "povBit");
}
void IR::BFN::ChecksumClotInput::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    v.visit(povBit, "povBit");
}
void IR::BFN::ChecksumClotInput::validate() const {
#line 21746 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        CHECK_NULL(povBit); }
void IR::BFN::ChecksumClotInput::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " clot=" << clot;
        out << " invert=" << invert;
}
void IR::BFN::ChecksumClotInput::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("clot", clot);
    json.emit("invert", invert);
    json.emit("povBit", povBit);
}
IR::BFN::ChecksumClotInput::ChecksumClotInput(JSONLoader & json) : Node(json) {
    json.load("clot", clot);
    json.load("invert", invert);
    json.load("povBit", povBit);
}
IR::Node * IR::BFN::ChecksumClotInput::fromJSON(JSONLoader & json) { return new ChecksumClotInput(json); }
bool IR::BFN::ChecksumClotInput::operator==(IR::Node const & a) const { return a == *this; }


/// The phv inputs to this checksum unit

bool IR::BFN::PartialChecksumUnitConfig::operator==(IR::BFN::PartialChecksumUnitConfig const & a) const {
        return this->typeId() == a.typeId()
        && unit == a.unit
        && phvs == a.phvs
        && povBit == a.povBit
        && invert == a.invert;
    }
bool IR::BFN::PartialChecksumUnitConfig::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const PartialChecksumUnitConfig &>(a_);
        return unit == a.unit
        && phvs.equiv(a.phvs)
        && (povBit ? a.povBit ? povBit->equiv(*a.povBit) : false : a.povBit == nullptr)
        && invert == a.invert;
    }
void IR::BFN::PartialChecksumUnitConfig::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    phvs.visit_children(v, "phvs");
    v.visit(povBit, "povBit");
}
void IR::BFN::PartialChecksumUnitConfig::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    phvs.visit_children(v, "phvs");
    v.visit(povBit, "povBit");
}
void IR::BFN::PartialChecksumUnitConfig::validate() const {
#line 21800 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        phvs.validate(); }
void IR::BFN::PartialChecksumUnitConfig::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " unit=" << unit;
        out << " invert=" << invert;
}
void IR::BFN::PartialChecksumUnitConfig::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("unit", unit);
    json.emit("phvs", phvs);
    if (povBit != nullptr) json.emit("povBit", povBit);
    json.emit("invert", invert);
}
IR::BFN::PartialChecksumUnitConfig::PartialChecksumUnitConfig(JSONLoader & json) : Node(json) {
    json.load("unit", unit);
    json.load("phvs", phvs);
    json.load("povBit", povBit);
    json.load("invert", invert);
}
IR::Node * IR::BFN::PartialChecksumUnitConfig::fromJSON(JSONLoader & json) { return new PartialChecksumUnitConfig(json); }
bool IR::BFN::PartialChecksumUnitConfig::operator==(IR::Node const & a) const { return a == *this; }



/// The unit number to configure; this is referenced by LoweredEmitChecksum
/// to actually write the checksum unit's output to the packet. On Tofino,
/// these unit numbers exist in a separate namespace for ingress and egress;
/// on JBay, both threads share the same checksum units.

/// clot inputs

bool IR::BFN::FullChecksumUnitConfig::operator==(IR::BFN::FullChecksumUnitConfig const & a) const {
        return this->typeId() == a.typeId()
        && unit == a.unit
        && partialUnits == a.partialUnits
        && clots == a.clots
        && zeros_as_ones == a.zeros_as_ones;
    }
bool IR::BFN::FullChecksumUnitConfig::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (this->typeId() != a_.typeId()) return false;
        auto &a = static_cast<const FullChecksumUnitConfig &>(a_);
        return unit == a.unit
        && partialUnits.equiv(a.partialUnits)
        && clots.equiv(a.clots)
        && zeros_as_ones == a.zeros_as_ones;
    }
void IR::BFN::FullChecksumUnitConfig::visit_children(Visitor & v, char const * n) {
    (void)n;
    Node::visit_children(v, n);
    partialUnits.visit_children(v, "partialUnits");
    clots.visit_children(v, "clots");
}
void IR::BFN::FullChecksumUnitConfig::visit_children(Visitor & v, char const * n) const {
    (void)n;
    Node::visit_children(v, n);
    partialUnits.visit_children(v, "partialUnits");
    clots.visit_children(v, "clots");
}
void IR::BFN::FullChecksumUnitConfig::validate() const {
#line 21862 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        partialUnits.validate();
        clots.validate(); }
void IR::BFN::FullChecksumUnitConfig::dump_fields(std::ostream & out) const {
    Node::dump_fields(out);
        out << " unit=" << unit;
        out << " zeros_as_ones=" << zeros_as_ones;
}
void IR::BFN::FullChecksumUnitConfig::toJSON(JSONGenerator & json) const {
    Node::toJSON(json);
    json.emit("unit", unit);
    json.emit("partialUnits", partialUnits);
    json.emit("clots", clots);
    json.emit("zeros_as_ones", zeros_as_ones);
}
IR::BFN::FullChecksumUnitConfig::FullChecksumUnitConfig(JSONLoader & json) : Node(json) {
    json.load("unit", unit);
    json.load("partialUnits", partialUnits);
    json.load("clots", clots);
    json.load("zeros_as_ones", zeros_as_ones);
}
IR::Node * IR::BFN::FullChecksumUnitConfig::fromJSON(JSONLoader & json) { return new FullChecksumUnitConfig(json); }
bool IR::BFN::FullChecksumUnitConfig::operator==(IR::Node const & a) const { return a == *this; }
#line 569 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
IR::BFN::LoweredDeparser::LoweredDeparser(gress_t gress) : AbstractDeparser(gress) { }
#line 21888 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
IRNODE_DEFINE_APPLY_OVERLOAD(BFN::LoweredDeparser, , )
#line 571 "/builddir/build/BUILD/p4c-1.2.5.8/backends/tofino/bf-p4c/ir/parde-lowered.def"
cstring IR::BFN::LoweredDeparser::toString() const { return gress ? "lowered egress deparser"_cs : "lowered ingress deparser"_cs; }
#line 21892 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"
bool IR::BFN::LoweredDeparser::operator==(IR::BFN::LoweredDeparser const & a) const {
        return AbstractDeparser::operator==(static_cast<const AbstractDeparser &>(a))
        && emits == a.emits
        && params == a.params
        && digests == a.digests
        && checksums == a.checksums;
    }
bool IR::BFN::LoweredDeparser::equiv(IR::Node const & a_) const {
        if (static_cast<const Node *>(this) == &a_) return true;
        if (!AbstractDeparser::equiv(a_)) return false;
        auto &a = static_cast<const LoweredDeparser &>(a_);
        return emits.equiv(a.emits)
        && params.equiv(a.params)
        && digests.equiv(a.digests)
        && checksums.equiv(a.checksums);
    }
void IR::BFN::LoweredDeparser::visit_children(Visitor & v, char const * n) {
    (void)n;
    AbstractDeparser::visit_children(v, n);
    emits.visit_children(v, "emits");
    params.visit_children(v, "params");
    digests.visit_children(v, "digests");
    checksums.visit_children(v, "checksums");
}
void IR::BFN::LoweredDeparser::visit_children(Visitor & v, char const * n) const {
    (void)n;
    AbstractDeparser::visit_children(v, n);
    emits.visit_children(v, "emits");
    params.visit_children(v, "params");
    digests.visit_children(v, "digests");
    checksums.visit_children(v, "checksums");
}
void IR::BFN::LoweredDeparser::validate() const {
#line 21926 "/builddir/build/BUILD/p4c-1.2.5.8/redhat-linux-build/ir/ir-generated.cpp"

        emits.validate();
        params.validate();
        digests.validate();
        checksums.validate(); }
void IR::BFN::LoweredDeparser::toJSON(JSONGenerator & json) const {
    AbstractDeparser::toJSON(json);
    json.emit("emits", emits);
    json.emit("params", params);
    json.emit("digests", digests);
    json.emit("checksums", checksums);
}
IR::BFN::LoweredDeparser::LoweredDeparser(JSONLoader & json) : AbstractDeparser(json) {
    json.load("emits", emits);
    json.load("params", params);
    json.load("digests", digests);
    json.load("checksums", checksums);
}
IR::Node * IR::BFN::LoweredDeparser::fromJSON(JSONLoader & json) { return new LoweredDeparser(json); }
bool IR::BFN::LoweredDeparser::operator==(IR::BFN::AbstractDeparser const & a) const { return a == *this; }
bool IR::BFN::LoweredDeparser::operator==(IR::BFN::Unit const & a) const { return a == *this; }
bool IR::BFN::LoweredDeparser::operator==(IR::Node const & a) const { return a == *this; }
